*==== BOF bufout =======================================================

*   Module data:

*    'network.inc'
*     MaxChannels - maximum number of channels.
*     NumCh - current number of channels.
*     Branch - current selected or active channel.
*     MaxLocations - maximum number of locations (computational or user).
*
*
*    'bufout.inc'
*     PlotType  - Destination of plot
*                 [1] Telegraf
*                 [2] Tab delimited (for micros)

*     MSS - Maximum number of space series.
*     UnitSpaceSeries  - file unit number for space series.
*     SpaceTimes(j) - time of space series.
*     j - space-series number.

*     MNS - Maximum number of points for time series plots.
*     UnitTimeSeries - file unit number for time series.
*     TimeLocations(k) - downstream location of time series.
*     k - time-series number.

*     TempX(i) - downstream coordinates.
*     TempQ(i) - streamflow.
*     TempZ(i) - water-surface elevation.
*     TempH(i) - depth of flow.
*      i - location number.

*     Flags:
*     TIMFLG - Time-series flag,
*               [.TRUE.]  - time series data is stored in buffer file.
*               [.FALSE.] - time-series data is not stored in buffer
*                           file.
*     SPCFLG - space-series flag,
*               [.TRUE.]  - space series data is stored in buffer file.
*               [.FALSE.] - space-series data is not stored in buffer
*                           file.

*== Public (InitSpaceTimeSeries) =======================================

      LOGICAL FUNCTION InitSpaceTimeSeries()

      IMPLICIT NONE

*   Purpose:  Initialize the module responsible for saving and reporting
*             time- and space- series of dependent variables.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'bufout.inc'

*   Local Variables:
      INTEGER   TimeSeries, SpaceSeries, Node, I
      REAL      Hours
      CHARACTER*12 InternalFilename1, InternalFilename2
      
*   Routines by module:

***** Channel flow status:
      REAL      GlobalStreamFlow, GlobalStreamDepth
      EXTERNAL  GlobalStreamFlow, GlobalStreamDepth

      REAL      GlobalStreamSurfaceElevation
      EXTERNAL  GlobalStreamSurfaceElevation

***** Channel schematic:
      INTEGER   TotalStreamLocations
      EXTERNAL  TotalStreamLocations

      REAL      GlobalStreamDistance
      EXTERNAL  GlobalStreamDistance

***** File utilities:
      LOGICAL   OpenNewBin
      EXTERNAL  OpenNewBin

***** Master file:
      INTEGER  GetFileUnit
      EXTERNAL GetFileUnit
      
***** Network control:
      INTEGER   NetworkTimeSeries, NetworkTimeSeriesLocation
      EXTERNAL  NetworkTimeSeries, NetworkTimeSeriesLocation

      INTEGER   NetworkSeconds
      EXTERNAL  NetworkSeconds
      
*   Intrinsics:
      REAL      FLOAT
      INTRINSIC FLOAT

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by: Barry L. Wicktom (use of master file names added)
*   Last modified: 1/8/92


*-- Implementation ------------------------------------------------------------


      InternalFilename1 = 'netspc1.out'
      InternalFilename2 = 'netts.dat '
      
      UnitSpaceSeries = GetFileUnit(InternalFilename1)
      UnitTimeSeries = GetFileUnit(InternalFilename2)
      
      IF (.NOT. OpenNewBin (UnitTimeSeries,  'TIME.TMP')   .OR.
     *    .NOT. OpenNewBin (UnitSpaceSeries, 'SPACE.TMP')) THEN
        PRINT *, 'Error(InitSpaceTimeSeries): Couldn''t open one of'
        PRINT *, ' the temporary files.'
        InitSpaceTimeSeries = .FALSE.

      ELSE

*------ Initialize time and space flags

        TIMFLG = .FALSE.
        SPCFLG = .FALSE.

*------ Write header and initial conditions to time-series scratch file.

        TimeSeries = NetworkTimeSeries()

        IF (TimeSeries .GE. 1) THEN

          Hours = FLOAT(NetworkSeconds()) / 3600.0

          DO 100 I = 1, TimeSeries
            Node      = NetworkTimeSeriesLocation(I)
            TempX(I)  = GlobalStreamDistance(Node)
            TempQ(I)  = GlobalStreamFlow(Node)
            TempZ(I)  = GlobalStreamSurfaceElevation(Node)
  100     CONTINUE

          WRITE (UnitTimeSeries) TimeSeries
          WRITE (UnitTimeSeries) (TempX(I), I=1, TimeSeries)
          WRITE (UnitTimeSeries) Hours, (TempQ(I),
     *                                   TempZ(I), I=1, TimeSeries)

        END IF

*------ Write header and initial conditions to spatial-series scratch file.

        SpaceSeries = TotalStreamLocations()

          DO 200 I = 1, SpaceSeries
            Node = I
            TempX(I)  = GlobalStreamDistance(Node)
            TempQ(I)  = GlobalStreamFlow(Node)
            TempH(I)  = GlobalStreamDepth(Node)
  200     CONTINUE

        WRITE(UnitSpaceSeries) SpaceSeries
        WRITE(UnitSpaceSeries) Hours, (TempQ(I),
     *                                 TempH(I), I=1, SpaceSeries)

        InitSpaceTimeSeries = .TRUE.

      END IF

      RETURN
      END



*== Public (WriteNetworkTimeSeries) ====================================

      LOGICAL FUNCTION WriteNetworkTimeSeries()

      IMPLICIT NONE

*   Purpose:  Write selected time series of network variables.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'bufout.inc'

*   Local Variables:
      INTEGER J, Node, TimeSeries
      REAL    ElapseHours

*   Routines by module:

***** Channel flow status:
      REAL     GlobalStreamFlow
      EXTERNAL GlobalStreamFlow

      REAL     GlobalStreamSurfaceElevation
      EXTERNAL GlobalStreamSurfaceElevation

***** Channel schematic:
      REAL     GlobalStreamDistance
      EXTERNAL GlobalStreamDistance

***** Network control:
      INTEGER  NetworkTimeSeries, NetworkTimeSeriesLocation
      EXTERNAL NetworkTimeSeries, NetworkTimeSeriesLocation

      INTEGER  NetworkSeconds
      EXTERNAL NetworkSeconds
      
*   Intrinsics:
      REAL      FLOAT
      INTRINSIC FLOAT

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:   Lew DeLong
*   Last modified: May 1991
*


*-- Implementation -----------------------------------------------------

      WriteNetworkTimeSeries = .FALSE.

      TimeSeries = NetworkTimeSeries()

      IF( TimeSeries .GE. 1 ) THEN

        ElapseHours = FLOAT(NetworkSeconds())/3600.0

        DO 100 J=1,TimeSeries

            Node = NetworkTimeSeriesLocation( J )
            TempQ(J) = GlobalStreamFlow(Node)
            TempZ(J) = GlobalStreamSurfaceElevation(Node)

 100    CONTINUE

*------- Write time-series results to scratch file.

         TIMFLG = .TRUE.

         WRITE(UnitTimeSeries) ElapseHours,
     #   ( TempQ(J), TempZ(J), J=1,TimeSeries )

      END IF


      WriteNetworkTimeSeries = .TRUE.

      RETURN
      END



*== Public (WriteNetworkSpatialSeries) =================================

      LOGICAL FUNCTION WriteNetworkSpatialSeries()

      IMPLICIT NONE

*   Purpose:  Write selected spatial series of network variables.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'bufout.inc'

*   Local Variables:
      INTEGER  J, Node, Locations
      REAL     ElapseHours

*   Routines by module:

***** Channel flow status:
      REAL     GlobalStreamFlow, GlobalStreamDepth
      EXTERNAL GlobalStreamFlow, GlobalStreamDepth

***** Channel schematic:
      INTEGER  TotalStreamLocations
      EXTERNAL TotalStreamLocations

***** Network control:
      INTEGER  NetworkSeconds
      EXTERNAL NetworkSeconds
      
*   Intrinsics:
      REAL      FLOAT
      INTRINSIC FLOAT

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      WriteNetworkSpatialSeries = .FALSE.

      ElapseHours = FLOAT(NetworkSeconds())/3600.0

      Locations = TotalStreamLocations()

      DO 100 J=1,Locations

        Node = J
        TempQ(J) = GlobalStreamFlow(Node)
        TempH(J) = GlobalStreamDepth(Node)

 100  CONTINUE

*---- Write spatial-series results to scratch file.

      SPCFLG = .TRUE.

      WRITE(UnitSpaceSeries) ElapseHours,
     #     ( TempQ(J), TempH(J), J=1,Locations )


      WriteNetworkSpatialSeries = .TRUE.

      RETURN
      END



*== Public (FlushSeries) ===============================================

      LOGICAL FUNCTION FlushSeries()

      IMPLICIT NONE

*   Purpose:  Read from scratch files and write space and time series.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'bufout.inc'

*   Local Variables:
      REAL DUM1(MSS*MaxLocations), DUM2(MSS*MaxLocations)
      REAL DUM3(MNS), DUM4(MNS), DUM5(MNS)

*   Routines by module:

***** Channel schematic:
      INTEGER  TotalStreamLocations
      EXTERNAL TotalStreamLocations

***** Locals:
      LOGICAL  SPACE, TIMES
      EXTERNAL SPACE, TIMES

*   Programmed by: Lew DeLong
*   Date:          April 1991
*   Modified by:   Barry Wicktom (use of master file names added)
*   Last modified: 1/8/92
*


*-- Implementation -----------------------------------------------------

      FlushSeries = .TRUE.

      IF (SPCFLG .AND. .NOT. SPACE
     I   ( UnitSpaceSeries, MSS, TotalStreamLocations(),
     I        TempX, PlotType,
     M        SpaceTimes, DUM1, DUM2))               THEN
        WRITE (*,*) ' '
        WRITE (*,*) ' No space series generated. '
        WRITE (*,*) ' '
        FlushSeries = .FALSE.
      END IF

      IF (TIMFLG .AND. .NOT. TIMES
     I                            (UnitTimeSeries, MNS, PlotType,
     M                             DUM3, DUM4, DUM5))        THEN
        WRITE (*,*) ' '
        WRITE (*,*) ' No time series generated. '
        WRITE (*,*) ' '
        FlushSeries = .FALSE.
      END IF

      RETURN
      END



*== Private (Times) ====================================================

      LOGICAL FUNCTION TIMES
     I                      (INFILE, MNS, TYPPLT,
     M                       Q, H, XORD)

      IMPLICIT NONE

*   Purpose:  Read time series from a binary file and prepare text output for
*             use by Telegraf (Prime), KaleidaGraph (Mac) or Grapher (PC).

*   Arguments:
      INTEGER INFILE, MNS, TYPPLT
      REAL    Q(MNS), H(MNS), XORD(MNS)

*   Argument definitions:
*     INFILE  - File in which time series are stored.
*     MNS     - Maximum number of points for time series plots.
*     Q       - Vector of discharges (feet^2/second).
*     H       - Vector of depths (feet).
*     TYPPLT  - Destination of plot (PARAMETER in MAIN)
*                 [1] Telegraf
*                 [2] Tab delimited (for micros)

*   Module data:

*   Local variables:
      INTEGER      I, NU(2),  NTMS, IERR
      CHARACTER*12 CFILE(2), InternalFileName1, InternalFileName2
      LOGICAL      OK

*   Routines by module:

***** File utilities:
      LOGICAL  DeleteFile, OpenNewText
      EXTERNAL DeleteFile, OpenNewText

***** Master file:
      INTEGER GetFileUnit
      CHARACTER*12 GetFileName
      EXTERNAL GetFileName, GetFileUnit
      
***** Locals:
      EXTERNAL TGFTIM, TABTIM

*   Programmed by: Dave Thompson
*   Date:          20 December 1989
*   Modified by: Barry Wicktom (use of master file names added)
*   Last modified: 1/8/92
*


*-- Implementation ------------------------------------------------------------

      InternalFileName1 = 'nettsq.dat'
      InternalFileName2 = 'nettsz.dat'
      
      CFILE(1) = GetFileName(InternalFileName1)
      CFILE(2) = GetFileName(InternalFileName2)
      NU(1) = GetFileUnit(InternalFileName1)
      NU(2) = GetFileUnit(InternalFileName2)
 
*---- Rewind the input file.
      REWIND (INFILE)

*---- Read the number of time series in the output file.
      READ (INFILE, IOSTAT=IERR) NTMS
      
*---- Check for range error.

****** kludge, this error occurrs because MNS is locally set rather than
*      being set equal to MaxTS in include file netcntrl.inc.  MaxTS
*      should be moved to network.inc and made available to all modules.

      IF( NTMS .LE. MNS ) THEN
      ELSE
        WRITE(*,*) ' ####Error(TIMES)'
        WRITE(*,*) ' Range error,',
     #             ' too many time-series values for dimension.'
        WRITE(*,*) ' NTMS...',NTMS,'  MNS...',MNS
        WRITE(*,*) ' Time-series output failed...'
        TIMES = .FALSE.
        RETURN
      END IF

*---- Check for an error (file probably hasn't been written to).
      IF (IERR .NE. 0) THEN

*------ An error occurred, probably the file has no data.
        WRITE (*,*) ' '
        WRITE (*,*) 'No data in time series file.'
        WRITE (*,*) ' '
      END IF

      IF (IERR .EQ. 0) THEN

*------ Read the vector of distances at which time series were put out.
        READ (INFILE) (XORD(I), I = 1, NTMS)

*------ Delete any old output files and open the files for output.
        OK = DeleteFile (CFILE(1))
        OK = DeleteFile (CFILE(2))

        IF ( OpenNewText(NU(1), CFILE(1)) .AND.
     *       OpenNewText(NU(2), CFILE(2)) ) THEN
          IF (TYPPLT .EQ. 1) THEN

*---------- Telegraf output.
            CALL TGFTIM
     I                 (INFILE, NTMS, NU, Q, H, XORD)
          ELSE IF (TYPPLT .EQ. 2) THEN

*---------- Tabbed output.
            CALL TABTIM
     I                 (INFILE, NTMS, NU, Q, H, XORD)
          END IF
        END IF

*------ Close the files.
        CLOSE (NU(1))
        CLOSE (NU(2))
        CLOSE (INFILE)

*------ Return all ok.
        TIMES = .TRUE.
      ELSE
*------ Otherwise, something messed up.
        WRITE(*,*) ' Failed to open files '
        WRITE(*,*) CFILE(1), CFILE(2)
        TIMES = .FALSE.
      END IF

      RETURN
      END



*== Private (TGFTIM) ===================================================

      SUBROUTINE TGFTIM
     I                 (INFILE, NTMS, NU, Q, H, XORD)

      IMPLICIT NONE

*   Purpose:  Write the TELEGRAF format file of output time series.

*   Arguments:
      INTEGER INFILE, NTMS, NU(2)
      REAL    Q(NTMS), H(NTMS), XORD(NTMS)

*   Argument definitions:
*     INFILE  - Unit number of input file.
*     NTMS    - Number of time series output stations.
*     NU      - Unit numbers of output files.
*     Q       - discharge (ft^2/sec).
*     H       - Stage (feet).
*     XORD    - Distance at which plot is produced (feet).

*   Module data:

*   Local Variables:
      INTEGER     NDUM, I,J
      REAL        TIME, DUMMY

*   Routines by module:

*   Programmed by: Dave Thompson
*   Date:          20 December 1989
*   Modified by:
*   Last modified:
*


*-- Implementation ------------------------------------------------------------

 2001 FORMAT (F15.2, 5X, F15.2)
 2002 FORMAT (10X, '"Cross section =', F10.1, '."')

      WRITE(NU(1),*) ' INPUT DATA. '
      WRITE(NU(2),*) ' INPUT DATA. '

      DO 110 I = 1, NTMS
        WRITE (NU(1), 2002) XORD(I)
        WRITE (NU(2), 2002) XORD(I)

*------ This is the beginning of a WHILE .NOT EOF structure.
   90   CONTINUE

*-------- Read the time series from the scratch file.
          READ(INFILE, END=100) TIME, (Q(J),H(J), J = 1, NTMS)

*-------- Write data pairs
          WRITE (NU(1), 2001) TIME, Q(I)
          WRITE (NU(2), 2001) TIME, H(I)
*-------- Repeat until EOF reached.
        GOTO 90

*------ Prepare to read the next time series.  (EOF comes here.)
  100   REWIND(INFILE)

*------ Clear the first two records (not necessary to re-read).
        READ (INFILE)  NDUM
        READ (INFILE) (DUMMY, J = 1, NTMS)
  110 CONTINUE
*---- Add ending text and return.
      WRITE(NU(1),*) ' END OF DATA. '
      WRITE(NU(2),*) ' END OF DATA. '

      WRITE (NU(1),*) 'X AXIS TEXT "TIME, IN HOURS".'
      WRITE (NU(2),*) 'X AXIS TEXT "TIME, IN HOURS".'
      WRITE (NU(1),*) 'Y AXIS TEXT "DISCHARGE".'
      WRITE (NU(2),*) 'Y AXIS TEXT "DEPTH".'

      WRITE(NU(1),*) 'GO.'
      WRITE(NU(2),*) 'GO.'

      RETURN
      END



*== Private (TABTIM) ===================================================

      SUBROUTINE TABTIM
     I                 (INFILE, NTMS, NU, Q, H, XORD)

      IMPLICIT NONE

*   Purpose:  Write the tab delimited columnar file of time series.

*   Arguments:
      INTEGER INFILE, NTMS, NU(2)
      REAL    Q(NTMS), H(NTMS), XORD(NTMS)

*   Argument Definitions:
*     INFILE  - Unit number of input file.
*     NTMS    - Number of time series stations to be put out.
*     NU      - Unit numbers of output files.
*     Q       - Vector of discharge.
*     H       - Vector of stage.
*     XORD    - Vector of distances (feet).

*   Module Data:

*   Local Variables:
      INTEGER      K
      CHARACTER*1  TAB
      REAL         TIME

*   Routines by module:

*   Intrinsics:
      CHARACTER*1 CHAR
      INTRINSIC   CHAR

*   Programmed by: Dave Thompson
*   Date:          20 December 1989
*   Modified by:
*   Last modified:
*


*-- Implementation ------------------------------------------------------------

 2001 FORMAT (F15.2, 10(A1, F15.2))
 2002 FORMAT ('Time', 10(A1, F15.2))
 2003 FORMAT (F15.2, 10(A1, F20.6))

      TAB    = CHAR(9)

*---- Write the header.
      WRITE (NU(1), 2002) (TAB, XORD(K), K = 1, NTMS)
      WRITE (NU(2), 2002) (TAB, XORD(K), K = 1, NTMS)

  120 CONTINUE
        READ(INFILE, END=130) TIME, (Q(K),H(K), K = 1, NTMS)
        WRITE (NU(1), 2001)   TIME, (TAB, Q(K), K = 1, NTMS)
        WRITE (NU(2), 2003)   TIME, (TAB, H(K), K = 1, NTMS)
      GOTO 120
  130 CONTINUE

      RETURN
      END



*== Private (SPACE) ====================================================

      LOGICAL FUNCTION SPACE
     I                      (INFILE, MSS, MCX, XORD, TYPPLT,
     M                       TIME, Q, H)

      IMPLICIT NONE

*   Purpose:  Read space series from a binary file and prepare text output
*             for use by Telegraf (Prime), KaleidaGraph (Mac) or Grapher (PC).

*   Arguments:
      INTEGER INFILE, MSS, MCX, TYPPLT
      REAL    Q(MSS*MCX), H(MSS*MCX), TIME(MSS), XORD(MCX)

*   Argument definitions:
*     INFILE  - File in which space series are stored.
*     MSS     - Maximum number of space series.
*     MXC     - Maximum number of cross sections.
*     H       - Vector of flow depths.
*     Q       - Vector of discharges.
*     TIME    - Vector of times (seconds).
*     TYPPLT  - Destination of plot
*                 [1] Telegraf
*                 [2] Tab delimited (for micros)
*     XORD    - Vector of distances (feet).

*   Module data:

*   Local Variables:
      INTEGER      J, NU(2), OFFSET, IERR
      INTEGER      NUMTIM, NUMSTA
      CHARACTER*12 CFILE(2), InternalFileName1, InternalFileName2
      LOGICAL      OK

*   Routines by module:

***** File utilities:
      LOGICAL  DeleteFile, OpenNewText
      EXTERNAL DeleteFile, OpenNewText
      
***** Master file:
      INTEGER GetFileUnit
      CHARACTER*12 GetFileName
      EXTERNAL GetFileName, GetFileUnit

***** Locals:
      EXTERNAL TABSPC, TGFSPC

*   Programmed by: Dave Thompson
*   Date:          20 December 1989
*   Modified by: Barry Wicktom (use of master file names added)
*   Last modified: 1/3/92
*


*-- Implementation ------------------------------------------------------------

      InternalFileName1 = 'netspcq.dat'
      InternalFileName2 = 'netspcz.dat'
      
      CFILE(1) = GetFileName(InternalFileName1)
      CFILE(2) = GetFileName(InternalFileName2)
      NU(1) = GetFileUnit(InternalFileName1)
      NU(2) = GetFileUnit(InternalFileName2)

*---- Rewind the input file.
      REWIND (INFILE)

*---- Read the number of cross sections from the file.
      READ (INFILE, IOSTAT=IERR) NUMSTA

*---- Check for an error (the file probably hasn't been written to).
      IF (IERR .NE. 0) THEN

*------ An error occurred (probably no file exists), so exit gracefully.
        WRITE (*,*) ' '
        WRITE (*,*) 'No data in space series file.'
        WRITE (*,*) ' '
      END IF

      IF (IERR .EQ. 0) THEN

*------ Delete any existing files and open the output files.
        OK = DeleteFile (CFILE(1))
        OK = DeleteFile (CFILE(2))
        IF ( OpenNewText(NU(1), CFILE(1) ) .AND.
     *       OpenNewText(NU(2), CFILE(2) )) THEN

*-------- Initialize variables.
          OFFSET = 0
          NUMTIM = 1

*-------- This is the beginning of a WHILE .NOT. EOF structure.
  100     CONTINUE
*---------- Read the space series from the temporary file.
            READ (INFILE, END=110) TIME(NUMTIM),
     *        ( Q(OFFSET+J), H(OFFSET+J), J = 1, NUMSTA)
            OFFSET = OFFSET + NUMSTA
            NUMTIM = NUMTIM + 1

*-------- If the allowable number of space series has not been exceeded,
*           then read another one into the buffers.
          IF (NUMTIM .LE. MSS) GOTO 100

*-------- Otherwise, decrement the counter (we've counted too many) and
*          continue.
  110     NUMTIM = NUMTIM - 1

          IF (TYPPLT .EQ. 1) THEN

*---------- Telagraf output.
            CALL TGFSPC
     I                 (NUMSTA, NUMTIM, MSS, MCX, TIME, NU, Q, H, XORD)
          ELSE IF (TYPPLT .EQ. 2) THEN

*---------- Tabbed output.
            CALL TABSPC
     I                 (NUMSTA, NUMTIM, MSS, MCX, TIME, NU, Q, H, XORD)
          END IF
        END IF

*------ Close the files.
        CLOSE (NU(1))
        CLOSE (NU(2))
        CLOSE (INFILE)

*------ Return all ok.
        SPACE = .TRUE.
      ELSE

*------ Otherwise, something messed up.
        WRITE(*,*) ' Failed to open files '
        WRITE(*,*) CFILE(1), CFILE(2)

        SPACE = .FALSE.
      END IF

      RETURN
      END



*== Private (TGFSPC) ===================================================

      SUBROUTINE TGFSPC
     I                 (NUMSTA, NUMTIM, MSS, MCX, TIME, NU, Q, H, XORD)

      IMPLICIT NONE

*   Purpose:  Write the TELEGRAF format file.

*   Arguments:
      INTEGER NUMSTA, NUMTIM, NU(2), MSS, MCX
      REAL    Q(MSS*MCX), H(MSS*MCX), TIME(MSS), XORD(MCX)

*   Argument definitions:
*     NUMSTA  - Number of stations in model.
*     NUMTIM  - Number of time steps.
*     MSS     - Maximum number of cross sections where output can be produced.
*     MCX     - Maximum number of cross section allowable in model.
*     TIME    - Vector of time steps at which space series are produced.
*     NU      - Unit number of output file.
*     Q       - Vector of discharge.
*     H       - Vector of stage.
*     XORD    - Vector of distances.

*   Module data:

*   Local Variables:
      INTEGER K, I, OFFSET

*   Routines by module:

*   Programmed by: Dave Thompson
*   Date:          20 December 1989
*   Modified by:
*   Last modified:
*


*-- Implementation ------------------------------------------------------------

 2001 FORMAT (F15.2, 5X, F15.2)
 2002 FORMAT (10X, '"Time =', F10.1, ' hours."')

      OFFSET = 0

      WRITE(NU(1),*) ' INPUT DATA. '
      WRITE(NU(2),*) ' INPUT DATA. '

*     Write data pairs
      DO 200 K = 1, NUMTIM
        WRITE (NU(1), 2002) TIME(K)
        WRITE (NU(2), 2002) TIME(K)
        DO 100 I = 1, NUMSTA
          WRITE (NU(1), 2001) XORD(I), Q(OFFSET+I)
          WRITE (NU(2), 2001) XORD(I), H(OFFSET+I)
 100    CONTINUE
        OFFSET = OFFSET + NUMSTA
 200  CONTINUE


      WRITE(NU(1),*) ' END OF DATA. '
      WRITE(NU(2),*) ' END OF DATA. '

      WRITE (NU(1),*) 'X AXIS TEXT "DISTANCE".'
      WRITE (NU(2),*) 'X AXIS TEXT "DISTANCE".'
      WRITE (NU(1),*) 'Y AXIS TEXT "DISCHARGE".'
      WRITE (NU(2),*) 'Y AXIS TEXT "DEPTH".'

      WRITE(NU(1),*) 'GO.'
      WRITE(NU(2),*) 'GO.'
      RETURN
      END



*== Private (TABSPC) ===================================================

      SUBROUTINE TABSPC
     I                 (NUMSTA, NUMTIM, MSS, MCX, TIME, NU, Q, H, XORD)

      IMPLICIT NONE

*   Purpose:  Write the tab delimited columnar file.

*   Arguments:
      INTEGER NUMSTA, NUMTIM, NU(2), MSS, MCX
      REAL    Q(MSS*MCX), H(MSS*MCX), TIME(MSS), XORD(MCX)

*   Argument definitions:
*     NUMSTA  - Number of stations in model.
*     MSS     - Maximum number of cross sections where output can be produced.
*     MCX     - Maximum number of cross section allowable in model.
*     NUMTIM  - Number of time steps.
*     TIME    - Vector of time steps at which space series are produced.
*     NU      - Unit number of output file.
*     Q       - Vector of discharge.
*     H       - Vector of stage.
*     XORD    - Vector of distances.

*   Module data:

*   Local Variables:
      INTEGER      K, I, OFFSET, M, J
      REAL         Channel
      CHARACTER*1  TAB
      LOGICAL      OK

*   Routines by module:

****Schematic data:
      INTEGER  NumberOfChannels, NumberOfStreamLocations
      EXTERNAL NumberOfChannels, NumberOfStreamLocations
      LOGICAL  OpenChannel, CloseChannel
      EXTERNAL OpenChannel, CloseChannel
      INTEGER  CurrentChannel
      EXTERNAL CurrentChannel
      
*   Intrinsics:
      CHARACTER*1 CHAR
      REAL        FLOAT
      INTRINSIC   CHAR, FLOAT

*   Programmed by: Dave Thompson
*   Date:          20 December 1989
*   Modified by:   Lew DeLong
*   Last modified: October, 1993
*


*-- Implementation ------------------------------------------------------------

 2001 FORMAT (F15.2, 10(A1, F15.2))
 2002 FORMAT ('Distance', 10(A1, F15.2))

      TAB    = CHAR(9)
      OFFSET = 0

*---- Write the header.
      WRITE (NU(1), 2002) (TAB, TIME(K), K = 1, NUMTIM)
      WRITE (NU(2), 2002) (TAB, TIME(K), K = 1, NUMTIM)
*     WRITE (NU(1), *) ' '
*     WRITE (NU(2), *) ' '


*---- Write the discharges and depths.
      I = 0
      DO 200 M=1,NumberOfChannels()
        OK = OpenChannel(M)
        Channel = FLOAT(CurrentChannel())
        DO 100 J = 1, NumberOfStreamLocations()
          I = I + 1
          WRITE (NU(1), 2001) XORD(I),
     *                     (TAB, Q(I+NUMSTA*(K-1)), K = 1, NUMTIM),
     #                                               TAB, Channel
          WRITE (NU(2), 2001) XORD(I),
     *                     (TAB, H(I+NUMSTA*(K-1)), K = 1, NUMTIM),
     #                                               TAB, Channel
  100   CONTINUE
        OK = CloseChannel()
  200 CONTINUE

      RETURN
      END

*==== EOF bufout ==============================================================
