*===== BOF chcxtbl ======================================================

************************************************************************
*
*     This file is a FORTRAN module for estimating hydraulic properties
*     of multiple non-prismatic channels.  Width, Beta, 1/effective Manning's n,  
*     area-weighted sinuosity, discharge-weighted sinuosity, and wetted
*     perimeter are interpolated linearly with respect to depth and downstream
*     distance.  Area and sinuosity-weighted conveyance at a location are 
*     computed from the interpolated values.  For the purposes of this module,
*     a cross section may be viewed as a sequence of one or more adjoining
*     trapezoids.  Lines delimiting the trapezoids correspond to data lines
*     in the tabular input to the module.  Conceptually, the channel is formed
*     by the straight-line connection (from cross section to cross section
*     within a particular branch) of corresponding extremities of lines
*     delimiting trapeziods within individual cross sections.  Thus, each
*     cross section within a branch must be constructed of the same number
*     of trapezoids.  Branches are independent.  Data input to this module
*     are formatted according to the table format obtained from the hydraulic
*     information exchange program (HYDIE) when the HYDRAUX-output-table
*     option is specified.
*
*     Program note: Only functions or subroutines marked "public" should be
*                   used outside of this module as those marked "private"
*                   may not be supported by future revisions of this module
*                   or replacement modules.  Likewise, no data or common
*                   blocks contained within this module should be accessed
*                   by routines outside of this module accept through the
*                   use of "public" functions or subroutines contained 
*                   within this module.
*
*
*
*   Programmed by:   Lew DeLong
*   Date programmed: January, 1990
*   Modified by:     Lew DeLong
*   Last modified:   July, 1991
*
*
*   
*   Public routines:
*
*     LOGICAL FUNCTION InitializeChannelProperties (DUNIT,FILDAT)
*            - initializes the module (reads properties file).
*
*     LOGICAL FUNCTION EchoChannelProperties (DUNIT,FILDAT)
*            - write properties data to a file.
*
*     LOGICAL FUNCTION AdjustChannelWidths ()
*            - adjust widths consistent with areas.
*
*     REAL FUNCTION BtmElev (X)
*            - returns channel-bottom elevation.
*
*     REAL FUNCTION BtmSlope (X)
*            - returns channel-bottom slope.
*
*     REAL FUNCTION ChannelWidth (X,H)
*            - returns channel width.
*
*     REAL FUNCTION dChannelWidth (X,H)
*            - returns d(Width)/dH.
*
*     REAL FUNCTION CxArea (X,H)
*            - returns cross-sectional area.
*
*     REAL FUNCTION Conveyance (X,H)
*            - returns sinuosity-weighted conveyance.
*
*     REAL FUNCTION dConveyance (X,H)
*            - returns d(Conveyance)/dH.
*
*     REAL FUNCTION Beta (X,H)
*            - returns momentum coefficient.
*
*     REAL FUNCTION dBeta (X,H)
*            - returns d(Beta)/dH
*
*     REAL FUNCTION AreaWtSinuosity (X,H)
*            - returns area-weighted sinuosity.
*
*     REAL FUNCTION dAreaWtSinuosity (X,H)
*            - returns d(AreaWtSinuosity)/dH.
*
*     REAL FUNCTION QWtSinuosity (X,H)
*            - returns discharge-weighted sinuosity.
*
*     REAL FUNCTION dQWtSinuosity (X,H)
*            - returns d(QWtSinuosity)/dH.
*
*     REAL FUNCTION dCxAreadX (X,H)
*            - returns d(CxArea)/dX, with constant water-surface.
*
*     REAL FUNCTION CxCentroid (X,H)
*            - returns distance from water surface to centroid.
*
*     REAL FUNCTION dCxCentroid (X,H)
*            - returns d(CxCentroid)/dH
*
*     REAL FUNCTION dChannelWidthdX (X,H)
*            - returns d(Width/dX), with constant elevation.
*
*
*     SUBROUTINE GXDistance( in: NumberOfLocations  out: UserLocation )
*            - gets an array of user cross-section downstream coordinates.
*
*     + + + Public arguments + + +
*
*     X      - current downstream distance
*     H      - current depth of flow
*
*     NumberOfLocations - maximum number of elements in a requested list
*                         of user cross-section coordinates.
*     UserLocation - an array of user cross-section coordinates.
*
*   Module data:
*
*    'network.inc'
*     MaxChannels - maximum number of channels.
*     NumCh - current number of channels.
*     Branch - current selected or active channel.
*     MaxLocations - maximum number of locations (computational or user).
*
*    'chcxtbl.inc'
*
*     MaxTables        - maximum number of tables.
*     MaxLinesPerTable - maximum number of lines per table.
*     MaxLines         - maximum total number of lines in tables.
*
*     FirstTable(m) - table number of first table for a branch.
*     LastTable(m)  - table number of last table of a branch.
*     Lines(m) - number of lines per table for the current branch.
*     m - branch number.
*
*     Offset(i) - pointer to first line of a table in properties arrays.
*     XDistance(i) - downstream-distance coordinate.
*     Datum(i) - elevation of bottom of the channel.
*     i - table number, same as user cross-section sequence number.
*
*     Depth(j) - depth of flow.
*     Width(j) - width of channel.
*     A(j)     - cross-sectional area.
*     K(j)     - sinuosity-weighted conveyance.
*     P(j)     - wetted perimeter.
*     N(j)     - 1/effective Manning's "n".
*     Bta(j)   - momentum coefficient.
*     MA(j)    - area-weighted sinuosity.
*     MQ(j)    - discharge-weighted sinuosity.
*     j - line number.    
*
*     N1 - adjacent upstream cross section. 
*     N2 - adjacent downstream cross section.
*
*     NP(k)       - pointers to table positions in adjacent upstream and downstream
*                   property tables bracketing the estimation point.
*     Shape(k)    - local bilinear shape function for the estimation point.
*     dShapeDX(k) - derivative of Shape with respect to the local depth coordinate.
*     k = degree of freedom.
*
*     DegreesOfFreedom - maximum number of degrees of freedom,
*                        four for bilinear interpolation.
*
*     Xi - stream-wise local coordinate of the estimation point.
*     Eta - depth-wise local coordinate of the estimation point.
*     dH - global depth corresponding to a local depth of 1 at the
*          estimation point.
*
*     PreviousBranch - branch of previous estimation point.
*     PreviousX      - int( X location ) at previous estimation point.
*     PreviousH      - depth of flow at previous estimation point.
*
*
*
*
*     Program Note:
*
*     By convention, the order for addressing known points bracketing
*     the interpolation point is the following:
*
*              4      <------------       3
*                                         ^
*                         *               |         downstream ----->
*                                         |
*                                         
*              1      ------------>       2
*
*
*              ^                          ^
*             (N1)                       (N2)
*
************************************************************************





*=======================================================================
*   Public: InitializeChannelProperties
*=======================================================================

      LOGICAL FUNCTION InitializeChannelProperties()

      IMPLICIT NONE

*   Purpose:  
*     Initialize hydraulic properties for the computation of 
*     open-channel flow.

*   Arguments:

*   Argument definitions:
*     DUNIT - FORTRAN unit number.
*     FILDAT - file name.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      CHARACTER*2  ICARD
      CHARACTER*80 DUM
      CHARACTER*12 FILDAT, InternalFileName
      INTEGER   LNUM,USR,MC,M,NPRT
      INTEGER DUNIT
      LOGICAL OK

*   Routines by module:

***** Local:
      LOGICAL  AdjustChannelWidths
      EXTERNAL AdjustChannelWidths
      LOGICAL  Read1stCxLine, ReadCxProperties, RiverFtToDownStream
      EXTERNAL Read1stCxLine, ReadCxProperties, RiverFtToDownStream
      LOGICAL  SingleTrap, InitChnlBtmHermites
      EXTERNAL SingleTrap, InitChnlBtmHermites

***** Master file:
      INTEGER GetFileUnit
      CHARACTER*12 GetFileName
      
*   Programmed by: Lew DeLong
*   Date:          July 1991
*   Modified by:  Barry Wicktom (use of master file names added)
*   Last modified: 1/7/92
*

*-- Implementation -----------------------------------------------------

      InitializeChannelProperties = .FALSE.
      
      InternalFileName = 'cxgeom.dat'
      
      FILDAT = GetFileName(InternalFileName)
      DUNIT = GetFileUnit(InternalFileName)
      
      PreviousX   = -9
      PreviousBranch = 999
      PreviousH      = 0.0

*---- Begin branch loop, M = branch counter.

      IF ( Read1stCxLine
     I          (
     O                        NPRT,DUM) ) THEN
        IF( IABS(NPRT) .GT. 0 ) THEN
          WRITE(*,*) 'Reading cross-section properties...'
        END IF
      ELSE
        WRITE(*,*) 'Read1stCxLine failed...'
        RETURN
      END IF
      
      IF( IABS(NPRT) .EQ. 1 ) THEN
        Print = .TRUE.
      ELSE
        Print = .FALSE.
      END IF
      
      IF( IABS(NPRT) .EQ. 4 ) THEN
        WriteIntrp = .TRUE.
      ELSE
        WriteIntrp = .FALSE.
      END IF
      
      IF( NPRT .LT. 0 ) THEN
        HermiteBtm = .TRUE.
      ELSE
        HermiteBtm = .FALSE.
      END IF
      

      USR = 0
      LNUM = 0
      DO 500 MC=1,NumCh
        M = MC

*------ Read cross-sectional hydraulic properties computed by HYDIE.

 100    CONTINUE
          IF ( ReadCxProperties
     I              (DUNIT,M,NPRT,
     M               DUM,LNUM,USR,
     O               ICARD        )  ) THEN
          ELSE
            WRITE(*,*) 'ReadCxProperties failed...'
            RETURN
          END IF
        IF(ICARD.EQ.'HY') GO TO 100

*------ Convert river distance to down-stream distance, assign
*       internal stream coordinates and datum.

        IF (RiverFtToDownStream(M) ) THEN
        ELSE
          RETURN
        END IF
        IF(ICARD.EQ.'EN') GO TO 502
 500  CONTINUE
 502  CONTINUE
      
*---- Check for sufficient number of cross sections.

      IF(M.LT.NumCh) THEN
        WRITE(*,*) 'Last branch = ',M,'.  Expected',NumCh,'.'
        RETURN
      END IF
      
*---- Adjust channel widths to be consistent with areas
*     computed by HYDIE.

      IF( AdjustChannelWidths() ) THEN
      ELSE
        WRITE(*,*) '#### error (InitializeChannelProperties)'
        WRITE(*,*) 'Channel widths not adjusted ...'
      END IF
      
*---- Write single trapezoid rough approximation of properties,
*     and stop program if NPRT = -1.

      IF( IABS(NPRT) .EQ. 5 ) THEN
      
        IF( SingleTrap() ) THEN
          WRITE(*,*) ' Single trapezoidal approximation completed.'
        END IF
        
        WRITE(*,*) ' Program stopped, print level = -1.'
        STOP
        
      END IF
      
*---- Initialize for hermite interpolation of channel bottom.

      IF( HermiteBtm ) THEN
        WRITE(*,*) ' Channel bottom is interpolated using',
     #             ' Hermite cubics...'
        OK = InitChnlBtmHermites()
      END IF
      

      InitializeChannelProperties = .TRUE.

      CLOSE(DUNIT)

      RETURN
      END




*=======================================================================
*   Public: EchoChannelProperties
*=======================================================================

      LOGICAL FUNCTION EchoChannelProperties()

      IMPLICIT NONE

*   Purpose:  
*     Write hydraulic properties to a file.

*   Arguments:
      INTEGER DUNIT
      CHARACTER*64 FILDAT, InternalFileName

*   Argument definitions:
*     DUNIT  - FORTRAN unit number
*     FILDAT - file name

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER   LNUM,USR,MC,M,TBL,I

*   Routines by module:

***** Master file:
      INTEGER GetFileUnit
      CHARACTER*12 GetFileName

***** File utilities:
      LOGICAL  OpenNewText
      EXTERNAL OpenNewText


*   Output formats:
 2000 FORMAT('DP',1X,F10.2,2(1X,E13.6),3(1X,F5.2),2(1X,F7.1))
 2001 FORMAT('HY',1X,A16,1X,F10.1,1X,F10.2)
 2002 FORMAT('CH',I5)

*   Programmed by: Lew DeLong
*   Date:          July 1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      EchoChannelProperties = .FALSE.
      
      InternalFileName = 'cxgeom.dat'

      DUNIT = GetFileUnit(InternalFileName)
      FILDAT = GetFileName(InternalFileName)
      
*---- Open properties file.
      IF(OpenNewText(DUNIT,FILDAT)) THEN
      ELSE
        WRITE(*,'(''Unable to open properties file...'',A64)') FILDAT
        RETURN
      END IF

*---- Write screen-output index.
      WRITE(DUNIT,'(''0'')') 

*     Begin branch loop, M = branch counter.

      USR = 0
      LNUM = 0
      DO 500 MC=1,NumCh
        M = MC

*------ Write channel number.
        WRITE(DUNIT,2002) M

*-----  Begin table loop, TBL = Table number.
        DO 400 TBL=FirstTable(M),LastTable(M)
          USR = USR+1

*-------- Write table header.
          WRITE(DUNIT,2001) ID(USR),XDistance(USR),Datum(USR)
          DO 300 I=1,Lines(M)
            LNUM = LNUM+1

*---------- Write properties.
            WRITE(DUNIT,2000) Depth(LNUM),A(LNUM),K(LNUM),Bta(LNUM),
     #                        MA(LNUM),MQ(LNUM),Width(LNUM),P(LNUM)
 300      CONTINUE
 400    CONTINUE
 500  CONTINUE

*---- Close properties file.
      CLOSE(DUNIT)

      EchoChannelProperties = .TRUE.

      RETURN
      END




*=======================================================================
*   Public: WriteInterveningProperties
*=======================================================================

      LOGICAL FUNCTION WriteInterveningProperties()

      IMPLICIT NONE

*   Purpose:  
*     Interpolate channel properties at computational locations
*     and write them to a file.

*   Arguments:
      INTEGER DUNIT
      CHARACTER*64 FILDAT, InternalFileName

*   Argument definitions:
*     DUNIT  - FORTRAN unit number
*     FILDAT - file name

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER   MC, M, I, J, L
      INTEGER CXNo, LocNo, IX, Dif
      REAL    X, H(MaxLinesPerTable), Bottom
      CHARACTER*16 NewID, Temp
      CHARACTER*1 UnderScore
      LOGICAL OK

*   Routines by module:

***** Master file:
      INTEGER GetFileUnit
      CHARACTER*12 GetFileName

***** File utilities:
      LOGICAL  OpenNewText
      EXTERNAL OpenNewText
      
***** Local:
      REAL     BtmElev, CxArea, ChannelWidth, WetPerimeter
      EXTERNAL BtmElev, CxArea, ChannelWidth, WetPerimeter
      REAL     AreaWtSinuosity, QWtSinuosity
      EXTERNAL AreaWtSinuosity, QWtSinuosity
      REAL     Beta, Conveyance
      EXTERNAL Beta, Conveyance
      
***** Schematic data:
      INTEGER  NumberOfStreamLocations
      EXTERNAL NumberOfStreamLocations
      REAL     StreamDistance
      EXTERNAL StreamDistance
      LOGICAL  OpenChannel, CloseChannel
      EXTERNAL OpenChannel, CloseChannel

*   Intrinsics:
      INTEGER   INT
      INTRINSIC INT


*   Output formats:
 2000 FORMAT('DP',1X,F10.2,2(1X,E13.6),3(1X,F5.2),2(1X,F7.1))
 2001 FORMAT('HY',1X,A16,1X,F10.1,1X,F10.2)
 2002 FORMAT('CH',I5)

*   Programmed by: Lew DeLong
*   Date:          October 1993
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      WriteInterveningProperties = .FALSE.

      IF( WriteIntrp ) THEN
        WRITE(*,*) ' Writing channel properties for computational',
     #             ' locations to file...'
      ELSE
        WriteInterveningProperties = .TRUE.
        RETURN
      END IF
      
      UnderScore = '_'
      
      InternalFileName = 'intpgeom.dat'
      WRITE(*,*) FILDAT

      DUNIT = GetFileUnit(InternalFileName)
      FILDAT = GetFileName(InternalFileName)
      WRITE(*,*) FILDAT
      
*---- Open properties file.
      IF(OpenNewText(DUNIT,FILDAT)) THEN
      ELSE
        WRITE(*,'(''Unable to open properties file...'',A64)') FILDAT
        RETURN
      END IF

*---- Write screen-output index.
      WRITE(DUNIT,'(''0'')') 

*     Begin branch loop, M = branch counter.

      CXNo = 1
      DO 500 MC=1,NumCh
        M = MC
        
        OK = OpenChannel(M)
        

*------ Write channel number.
        WRITE(DUNIT,2002) M

*-----  Begin computational-location loop.
        
        DO 400 I=1,NumberOfStreamLocations()
        
          LocNo = I
C         WRITE(*,*) ' LocNo...',LocNo

*-------- Set ID.
          
          X = StreamDistance(LocNo)
          IX = INT(X)
 100      CONTINUE
 
          Dif = IX - INT(XDistance(CXNo))
          
C         WRITE(*,*) ' Dif, CXNo, IX...',Dif, CXNo, IX
C         WRITE(*,*) ' X, XDistance(CXNo)...',X, XDistance(CXNo)
          
          IF( Dif .EQ. 0 ) THEN
            NewID = ID(CXNo)
            L = Offset(CXNo) - 1
            DO 120 J=1,Lines(M)
              H(J) = Depth(L+J)
 120        CONTINUE
          ELSE IF( Dif .LT. 0 ) THEN
            IF( M .LT. 9 ) THEN
              WRITE(Temp,'(I1,A1,I14)')
     #          M, UnderScore, IX
            ELSE IF( M .GT. 9 .AND. M .LT. 99 ) THEN
              WRITE(Temp,'(I2,A1,I13)')
     #          M, UnderScore, IX
            ELSE IF( M .GT. 99 .AND. M .LT. 999 ) THEN
              WRITE(Temp,'(I3,A1,I12)')
     #          M, UnderScore, IX
            ELSE IF( M .GT. 999 .AND. M .LT. 9999 ) THEN
              WRITE(Temp,'(I4,A1,I11)')
     #          M, UnderScore, IX
            ELSE
              WRITE(*,*) ' ####error(WriteInterveningProperties)'
              WRITE(*,*) ' exceeds format, channel...',M
            END IF
              READ(Temp,'(A16)') NewID
          ELSE
            CXNo = CXNo + 1
            L = Offset(CXNo) - 1
            DO 130 J=1,Lines(M)
              H(J) = Depth(L+J)
 130        CONTINUE
            GO TO 100
          END IF

*-------- Determine bottom elevation.

          Bottom = BtmElev(X)
          
          
*-------- Write table header.

          WRITE(DUNIT,2001) NewID,X,Bottom

          DO 300 L=1,Lines(M)

*---------- Write properties.
            WRITE(DUNIT,2000) H(L),CxArea(X,H(L)),Conveyance(X,H(L)),
     #                        Beta(X,H(L)), AreaWtSinuosity(X,H(L)),
     #                        QWtSinuosity(X,H(L)),ChannelWidth(X,H(L)),
     #                        WetPerimeter(X,H(L))
 300      CONTINUE
 400    CONTINUE
        OK = CloseChannel()
 500  CONTINUE

*---- Close properties file.
      WRITE(DUNIT,*) ' '
      CLOSE(DUNIT)

      WriteInterveningProperties = .TRUE.

      RETURN
      END



*=======================================================================
*   Public: UserStreamLocations
*=======================================================================

      SUBROUTINE UserStreamLocations
     I                ( NumberOfLocations, LocationID,
     O                   UserLocation                  )

      IMPLICIT NONE

*   Purpose:  
*     Get an array of downstream coordinates for the user cross sections
*     in the current channel.

*   Arguments:
      INTEGER NumberOfLocations
      REAL    UserLocation( NumberOfLocations )
      CHARACTER*16 LocationID( NumberOfLocations )
      
*   Argument definitions:
*     NumberOfLocations - dimension of UserLocation().
*     UserLocation(i) - downstream user-supplied cross-section location.
*     LocationID(i) - location identifiers.
*     i - user cross-section sequence number.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I, J, Locations, L
      LOGICAL Match
      CHARACTER*255 String1, String2

*   Routines by module:

***** Channel-schematic module:
      INTEGER  NumberOfUserStreamLocations
      EXTERNAL NumberOfUserStreamLocations
      EXTERNAL GetUserStreamLocationIDs
      
***** String manipulation:
      CHARACTER*255   LeftTrim, RightTrim
      EXTERNAL        LeftTrim, RightTrim

*   Programmed by: Lew DeLong
*   Date:          July 1991
*   Modified by:   Lew DeLong
*   Last modified: July 1993
*

*-- Implementation -----------------------------------------------------

      Match = .TRUE.
      
      Locations = NumberOfUserStreamLocations()
      
      IF( Locations .LE. NumberOfLocations ) THEN
      
*------ Search for IDs and get corresponding locations.

        CALL GetUserStreamLocationIDs(Locations, LocationID)

        DO 200 I=1,Locations
  
*-------- Check for range error.

          IF( LastTable(Branch) .GT. MaxTables ) THEN
            WRITE(*,*) ' #### error{1}(UserStreamLocations)'
            WRITE(*,*) ' Branch...',Branch
            WRITE(*,*) ' MaxTables exceeded...'
            WRITE(*,*) ' ',LastTable(Branch),'>',MaxTables
          END IF
          

*-------- Look for cross-section ID.

          DO 100 J=FirstTable(Branch),LastTable(Branch)
            
            L = J
            IF( LocationID(I) .EQ. ID(J) ) GO TO 102

 100      CONTINUE     

*-------- Failed to find ID, remove leading and trailing
*         blanks and try again.

          String1 = LeftTrim( LocationID(I) )
          String1 = RightTrim( String1 )
          DO 101 J=FirstTable(Branch),LastTable(Branch)
            
            L = J
            String2 = LeftTrim( ID(J) )
            String2 = RightTrim( String2 )
            IF( String1 .EQ. String2 ) GO TO 102

 101      CONTINUE     

          WRITE(*,*) ' #### error{2}(UserStreamLocations)'
          WRITE(*,*) ' Branch...',Branch
          WRITE(*,*) ' No match for...',LocationID(I)
          Match = .FALSE.
          UserLocation(J) = -9999999999.99
          GO TO 200

 102      CONTINUE
        
*-------- Match for ID found, get location.

          UserLocation(I) = XDistance(L)

 200    CONTINUE
 
        IF( Match ) THEN
        ELSE

          WRITE(*,*) ' '
          WRITE(*,*) '            ID                Location'
          DO 300 I=1,Locations

            WRITE(*,'(I5,2X,A16,2X,F14.2)')
     #             I, LocationID(I), UserLocation(I) 

 300      CONTINUE
          WRITE(*,*) ' '
       
          WRITE(*,*) 'Abnormal program end.'
          STOP
        END IF
        
      ELSE
          WRITE(*,*) ' Locations...', Locations
          WRITE(*,*) ' NumberOfLocations...', NumberOfLocations
          WRITE(*,*) ' #### error{3}(UserStreamLocations)'
          WRITE(*,*) ' Branch...',Branch
          WRITE(*,*) 
     #       ' Number of user locations exceeds passed dimension.'
          WRITE(*,*) ' MaxLocations too small...'
          WRITE(*,*) ' Abnormal program end.'
          STOP
      
      END IF

      RETURN
      END



*=======================================================================
*   Public: BtmElev
*=======================================================================

      REAL FUNCTION BtmElev(X)

      IMPLICIT NONE

*   Purpose:  
*     Estimate channel-bottom elevation in the current channel
*     at X downstream distance.

*   Arguments:
      REAL    X

*   Argument definitions:
*     X - downstream distance in current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction, SetXi
      EXTERNAL CxShapeFunction,SetXi
      REAL     HermBtmElev
      EXTERNAL HermBtmElev


*   Programmed by: Lew DeLong
*   Date:          OCT   1993
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF(Branch.NE.PreviousBranch.OR.INT(X).NE.PreviousX) THEN
        PreviousBranch = Branch
        PreviousX   = INT(X)
        
*       Determine relative longitudinal location of a point,
*       within a branch.

        IF(SetXi(X)) THEN
        ELSE
          WRITE(*,*) '*** error(BtmElev)..branch,X',Branch,X
          WRITE(*,*) 'Could not determine relative x location.'
          WRITE(*,*) 'Stopping...'
          STOP
        END IF
      END IF  

      IF( .NOT. HermiteBtm ) THEN
        BtmElev = (1.0-Xi)*Datum(N1)+Xi*Datum(N2)
      ELSE
        BtmElev = HermBtmElev()
      END IF

      RETURN
      END



*=======================================================================
*   Public: BtmSlope
*=======================================================================

      REAL FUNCTION BtmSlope(X)

      IMPLICIT NONE

*   Purpose:  
*     Estimate channel-bottom slope in the current channel
*     at X downstream distance.

*   Arguments:
      REAL    X

*   Argument definitions:
*     X - downstream distance in current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction, SetXi
      EXTERNAL CxShapeFunction, SetXi
      REAL     HermBtmSlope
      EXTERNAL HermBtmSlope


*   Intrinsics:
      INTEGER   INT
      INTRINSIC INT

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   Lew Delong
*   Last modified: Oct   1993
*

*-- Implementation -----------------------------------------------------

      IF(Branch.NE.PreviousBranch.OR.INT(X).NE.PreviousX) THEN
        PreviousBranch = Branch
        PreviousX   = INT(X)
       
*------ Determine relative longitudinal location of a point,
*       within a branch.

        IF(SetXi(X)) THEN
        ELSE
          WRITE(*,*) '*** error(BtmSlope)..branch,X',Branch,X
          WRITE(*,*) 'Could not determine relative x location.'
          WRITE(*,*) 'Stopping...'
          STOP
        END IF
      END IF  

      IF( .NOT. HermiteBtm ) THEN
        BtmSlope = (Datum(N2)-Datum(N1))/(XDistance(N2)-XDistance(N1))
      ELSE
        BtmSlope = HermBtmSlope()
      END IF

      RETURN
      END






*=======================================================================
*   Public: ChannelWidth
*=======================================================================

      REAL FUNCTION ChannelWidth(X,H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate channel width in the current channel at X downstream distance and
*     at H distance above the lowest point in the cross section.

*   Arguments:
      REAL    X,H

*   Argument definitions:
*     X      - downstream distance in current channel.
*     H      - distance, above lowest point in channel, at which the
*              channel width is to be computed.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I

*   Functions:
      LOGICAL  CxShapeFunction
      EXTERNAL CxShapeFunction

*   Subroutines:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF(CxShapeFunction(X,H)) THEN
        ChannelWidth = 0.0
        DO 100 I=1,DegreesOfFreedom
          ChannelWidth = ChannelWidth+Shape(I)*Width(NP(I))
 100    CONTINUE
      ELSE
        WRITE(*,*) '*** error(ChannelWidth)',Branch,X,H
        STOP
      END IF  

      RETURN
      END






*=======================================================================
*   Public: dChannelWidth
*=======================================================================

      REAL FUNCTION dChannelWidth(X,H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate D(width)/dH in the current channel at X downstream distance
*      and at H distance above the lowest point in the cross section.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance in current channel.
*     H - distance, above lowest point in channel, at which the
*         channel width is to be computed.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      EXTERNAL CxShapeFunction

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF(CxShapeFunction(X,H)) THEN
        dChannelWidth = (1.0-Xi)*( Width(NP(4))-Width(NP(1)) ) / 
     #                  ( Depth(NP(4))-Depth(NP(1)) )
     #          +    Xi*( Width(NP(3))-Width(NP(2)) ) / 
     #                  ( Depth(NP(3))-Depth(NP(2)) )
      ELSE
        WRITE(*,*) '*** error(dChannelWidthH)',Branch,X,H
        STOP
      END IF  

      RETURN
      END



*=======================================================================
*   Public: CxArea
*=======================================================================

      REAL FUNCTION CxArea(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate cross-sectional area in the current channel, at X downstream
*     distance, limited by the lowest point in the channel and a
*     distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I, L1, L2
      REAL    XiM

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      REAL     ChannelWidth
      EXTERNAL CxShapeFunction, ChannelWidth

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------
      
      IF(CxShapeFunction(X,H)) THEN
        XiM = 1.0-Xi
        CxArea = 0.0
        L1 = Offset(N1)
        L2 = Offset(N2)
        DO 100 I=2,NP(1)-Offset(N1)+1
          L1 = L1+1
          L2 = L2+1
          CxArea = CxArea+0.5*( XiM*(Width(L1)+Width(L1-1))
     #                     +Xi*(Width(L2)+Width(L2-1))  )
     #                   *(  XiM*Depth(L1)+Xi*Depth(L2)
     #                     -XiM*Depth(L1-1)-Xi*Depth(L2-1) )
 100    CONTINUE

        CxArea = CxArea+0.5*((XiM)*Width(L1)+Xi*Width(L2) 
     #                    +ChannelWidth(X,H)             )
     #                 *( H-XiM*Depth(L1)-Xi*Depth(L2) )
*     
      ELSE
        WRITE(*,*) '*** error(CxArea)',Branch,X,H
        STOP
      END IF  

      RETURN
      END



*=======================================================================
*   Public: dCxAreadX
*=======================================================================

      REAL FUNCTION dCxAreadX(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate d(CxArea)/dX, water-surface elevation held constant,
*     in the current channel, at X downstream distance,
*     limited by the lowest point in the channel and a
*     distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I, L1, L2
      REAL    XiM, DX

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      REAL     ChannelWidth, BtmSlope, dChannelWidthdX
      EXTERNAL CxShapeFunction, ChannelWidth, BtmSlope, dChannelWidthdX

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF(CxShapeFunction(X,H)) THEN
        XiM = 1.0-Xi
        dCxAreadX = 0.0
        DX = XDistance(N2)-XDistance(N1)
        L1 = Offset(N1)
        L2 = Offset(N2)
        DO 100 I=2,NP(1)-Offset(N1)+1
          L1 = L1+1
          L2 = L2+1

*-------- Estimate d(CxArea)/dX for all lower trapezoidal sections.

          dCxAreadX = dCxAreadX + 0.5 * (

     #          ( Width(L2)-Width(L1) + Width(L2-1)-Width(L1-1) )/DX
     #               * (  XiM*(Depth(L1)-Depth(L1-1))
     #                    +Xi*(Depth(L2)-Depth(L2-1))  )

     #          + ( XiM*(Width(L1)+Width(L1-1))
     #             + Xi*(Width(L2)+Width(L2-1)) )
     #          * ( Depth(L2)-Depth(L2-1) - Depth(L1)+Depth(L1-1) )/DX

     #                                   )
 100    CONTINUE

*------ Estimate contribution to d(CxArea)/dX, with constant
*       water-surface elevation, in the partial trapezoid terminated
*       by the water surface.

        dCxAreadX = dCxAreadX+0.5*(
     #            ( dChannelWidthdX(X,H) + (Width(L2)-Width(L1))/DX )
     #          * ( H - (Xi*Depth(L2)+XiM*Depth(L1)))
     #      +
     #            ( ChannelWidth(X,H) + Xi*Width(L2)+XiM*Width(L1) )
     #          * ( -(Depth(L2)-Depth(L1))/DX - BtmSlope(X)    )
     #                   )
*     
      ELSE
        WRITE(*,*) '*** error(dCxAreadX)',Branch,X,H
        STOP
      END IF  

      RETURN
      END






*=======================================================================
*   Public: CxCentroid
*=======================================================================

      REAL FUNCTION CxCentroid(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate distance from the water surface to the cross-section
*     centroid, in the current channel, at X downstream distance,
*     limited by the lowest point in the channel and a
*     distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local variables:
      REAL Small
      PARAMETER (Small = 1.00E-6)
      INTEGER I, L1, L2
      REAL    XiM, DX 
      REAL    Z1, Z2, WZ1, WZ2, Z2MZ1, Z2M

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      REAL     ChannelWidth, CxArea
      EXTERNAL CxShapeFunction, ChannelWidth, CxArea

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   Barry Wicktom
*   Last modified: January, 1993
*

*-- Implementation -----------------------------------------------------
           
      IF (CxArea(X,H).GT.Small) THEN
       
       
      IF(CxShapeFunction(X,H)) THEN
        XiM = 1.0-Xi
        CxCentroid = 0.0
        DX = XDistance(N2)-XDistance(N1)
       
         L1 = Offset(N1)
         L2 = Offset(N2)
        
        
        Z1 =  H - XiM*Depth(L1) - Xi*Depth(L2)
        WZ1 =  XiM*Width(L1) + Xi*Width(L2)
        

        IF( NP(1) .GE. L1 ) THEN
         DO 100 I=2,NP(1)-Offset(N1)+1
          L1 = L1+1
          L2 = L2+1
           
*-------- Accumulate first moments for each lower trapezoidal section.
          Z2 = Z1
          WZ2 =    WZ1
          Z1 =     H - XiM*Depth(L1) - Xi*Depth(L2)
          Z2MZ1 =  Z2-Z1 
          WZ1 =    XiM*Width(L1) + Xi*Width(L2)
          CxCentroid = CxCentroid + 0.5 * (Z2-Z1) 
     #                            * (  WZ2*(Z2+Z1)
     #                           +  (WZ1-WZ2)*(2*Z1+Z2)/3.0 ) 
 100     CONTINUE
       END IF

*------ Add first moment for the trapezoid ending at the water surface.
        Z2M = Z1
        WZ2 = WZ1
        WZ1 = ChannelWidth(X,H)
        CxCentroid= CxCentroid + Z2M*(Z2M*(WZ1-WZ2)/6.0 + Z2M*WZ2/2.0)
        
*------ Divide moment by area to locate distance of centroid
*       below surface.
        
        CxCentroid = CxCentroid/CxArea(X,H)
        
      ELSE
        WRITE(*,*) '*** error(CxCentroid)',Branch,X,H
        STOP
      END IF  

      ELSE
       CxCentroid = 0.0
      ENDIF

      RETURN
      END



*=======================================================================
*   Public: dCxCentroid
*=======================================================================

      REAL FUNCTION dCxCentroid(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate d(centroid distance from surface)/dH,
*     in the current channel, at X downstream distance,
*     limited by the lowest point in the channel and a
*     distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local variables:
      REAL Small
      PARAMETER (Small = 1.00E-6)
      INTEGER I, L1, L2
      REAL    XiM, DX
      REAL    Z1, Z2, WZ1, WZ2, Z2MZ1

*   Routines by module:

***** Local:
      LOGICAL CxShapeFunction
      REAL    ChannelWidth, CxArea, CxCentroid, dChannelWidth
      EXTERNAL CxShapeFunction, ChannelWidth, CxArea
      EXTERNAL CxCentroid, dChannelWidth

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   Barry Wicktom
*   Last modified: January, 1993
*

*-- Implementation -----------------------------------------------------

      IF (CxArea(X,H).GT.Small) THEN

      IF(CxShapeFunction(X,H)) THEN
        XiM = 1.0-Xi
        dCxCentroid = 0.0
        DX = XDistance(N2)-XDistance(N1)
        L1 = Offset(N1)
        L2 = Offset(N2)
        Z1 =     H - XiM*Depth(L1) - Xi*Depth(L2)
        WZ1 =    XiM*Width(L1) + Xi*Width(L2)
        DO 100 I=2,NP(1)-Offset(N1)+1
          L1 = L1+1
          L2 = L2+1

*-------- Accumulate derivative of the first moment
*         for each lower trapezoidal section.

          Z2 =     Z1
          WZ2 =    WZ1
          Z1 =     H - XiM*Depth(L1) - Xi*Depth(L2)
          Z2MZ1 =  Z2 - Z1
          WZ1 =    XiM*Width(L1) + Xi*Width(L2)
          dCxCentroid= dCxCentroid + (.5) *(Z2-Z1) 
     #      * (WZ1+WZ2)
        
 100    CONTINUE

*------ Add derivative of the first moment 
*       for the trapezoid ending at the water surface.

        Z2 = Z1
        WZ2 = WZ1
        WZ1 = ChannelWidth(X,H)
        dCxCentroid = dCxCentroid + Z2*(
     #         4.0 * WZ2+2.0 * WZ1 + Z2 * dChannelWidth(X,H)
     #                                 ) /6.0

*------ Estimate the derivative of the distance from the centroid
*       to the water surface, with respect to depth of flow.
        dCxCentroid = (dCxCentroid-WZ1*CxCentroid(X,H))/CxArea(X,H)

      ELSE
        WRITE(*,*) '*** error(dCxCentroid)',Branch,X,H
        STOP
      END IF  
*       WRITE(*,*) 'dCxCentroid =',dCxCentroid
      
      ELSE
       dCxCentroid = 0.0
      ENDIF

      RETURN
      END



*=======================================================================
*   Public: dChannelWidthdX
*=======================================================================

      REAL FUNCTION dChannelWidthdX(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate d(Width)/dX, water-surface elevation held constant,
*     in the current channel, at X downstream distance,
*     at a distance H above the lowest point:
*
*     (d(Width)/dX)y    = d(Width)/dX     - (d(Width)/dY)x * dY/dX 
*
*       dChannelWidthdX = dChannelWidthdX -     DWDY       * SLOPE

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      REAL    XiM, EtaM, SLOPE, DWDY, DX

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      REAL     BtmSlope
      EXTERNAL CxShapeFunction, BtmSlope

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF(CxShapeFunction(X,H)) THEN
        XiM  = 1.0-Xi
        EtaM = 1.0-Eta
        DX = XDistance(N2)-XDistance(N1)

        dChannelWidthdX = (  EtaM*(Width(NP(2))-Width(NP(1)))
     #            +Eta *(Width(NP(3))-Width(NP(4)))  )/DX

        SLOPE = (  EtaM*(Depth(NP(2))-Depth(NP(1)))
     #            +Eta *(Depth(NP(3))-Depth(NP(4)))  )/DX
     #            + BtmSlope(X)

        DWDY  =    XiM*(Width(NP(4))-Width(NP(1)))
     #                /(Depth(NP(4))-Depth(NP(1)))
     #                             +
     #             Xi *(Width(NP(3))-Width(NP(2)))
     #                /(Depth(NP(3))-Depth(NP(2)))

        dChannelWidthdX = dChannelWidthdX-DWDY*SLOPE

      ELSE
        WRITE(*,*) '*** error(dChannelWidthdX)',Branch,X,H
        STOP
      END IF  

      RETURN
      END



*=======================================================================
*   Public: Conveyance
*=======================================================================

      REAL FUNCTION Conveyance(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate sinuosity-weighted conveyance in the current channel,
*     at X downstream distance, limited by the lowest point in the channel
*     and a distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      REAL R53,R23
      PARAMETER (R53 = 5.0/3.0, R23 = 2.0/3.0)
      REAL Small
      PARAMETER (Small = 1.00E-6)

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      REAL     CxArea, WetPerimeter, EffectiveN
      EXTERNAL CxArea, WetPerimeter, EffectiveN, CxShapeFunction

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          August 1994

*-- Implementation -----------------------------------------------------

      IF (WetPerimeter(X,H).GT.Small) THEN
      
       IF(CxShapeFunction(X,H)) THEN
        Conveyance = 1.486*(CxArea(X,H))**R53 *
     #         (EffectiveN(X,H)/(WetPerimeter(X,H))**R23)
       ELSE
        WRITE(*,*) '*** error(Conveyance)',Branch,X,H
        STOP
       END IF
      
      ELSE
       Conveyance = 0.0
      ENDIF

      RETURN
      END



*=======================================================================
*   Public: dConveyance
*=======================================================================

      REAL FUNCTION dConveyance(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate d(sinuosity-weighted conveyance)/dH in the current channel,
*     at X downstream distance, limited by the lowest point in the channel
*     and a distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      REAL R53,R23
      PARAMETER (R53 = 5.0/3.0, R23 = 2.0/3.0)
      REAL Small
      PARAMETER (Small = 1.00E-6)

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      REAL     Conveyance, CxArea, ChannelWidth
      REAL     EffectiveN, dEffectiveN
      REAL     WetPerimeter, dWetPerimeter
      EXTERNAL CxShapeFunction, Conveyance, CxArea, ChannelWidth
      EXTERNAL EffectiveN, dEffectiveN, WetPerimeter, dWetPerimeter
      
***** Schematic data:
      INTEGER  CurrentChannel
      EXTERNAL CurrentChannel

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          August 1994

*-- Implementation -----------------------------------------------------

      IF (WetPerimeter(X,H).GT.Small) THEN
      
       IF(CxShapeFunction(X,H)) THEN
       dConveyance = 1.486 * (
     #             R53 * ChannelWidth(X,H) * EffectiveN(X,H) 
     #               * ( CxArea(X,H) / WetPerimeter(X,H) )**R23
     #            
     #            - R23 * dWetPerimeter(X,H) * EffectiveN(X,H)
     #            * ( CxArea(X,H)  / WetPerimeter(X,H) )**R53
     #
     #            + dEffectiveN(X,H) * CxArea(X,H)**R53
     #            /  WetPerimeter(X,H)**R23
     #                       )

       ELSE
         WRITE(*,*) '*** error(dConveyance)',Branch,X,H
         STOP
       END IF
       
      ELSE
       dConveyance = 0.0
      ENDIF
      
      IF( dConveyance .LT. 0 ) THEN
        IF( H .GT. .5 ) THEN
          dConveyance = Conveyance(X,H+.5) - Conveyance(X,H-.5)
        ELSE
          dConveyance = Conveyance(X,H+1.) - Conveyance(X,H)
        END IF
        IF( dConveyance .LT. 0 ) THEN
          WRITE(*,*) ' '
          WRITE(*,*) ' ####error(dConveyance)'
          WRITE(*,*) ' negative gradient with depth,'
          WRITE(*,*) ' suggest subdividing adjacent',
     #               ' cross section(s).'
          WRITE(*,*) ' Channel...',CurrentChannel()
          WRITE(*,*) ' X, H...',X,H
          WRITE(*,*) ' Width, Area, Conveyance...',
     #               ChannelWidth(X,H),CxArea(X,H),Conveyance(X,H)
          WRITE(*,*) ' one over effective n, 1/(dn/dZ)...',
     #               EffectiveN(X,H),dEffectiveN(X,H)
          WRITE(*,*) ' P, dP/dZ...',
     #               WetPerimeter(X,H), dWetPerimeter(X,H)
        END IF
      END IF

      RETURN
      END



*=======================================================================
*   Public: EffectiveN
*=======================================================================

      REAL FUNCTION EffectiveN(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate one over effective Manning's "n" in the current channel,
*     at X downstream distance,  for the area limited by the lowest point
*     in the channel and a distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      EXTERNAL CxShapeFunction

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF(CxShapeFunction(X,H)) THEN
        EffectiveN = 0.0
        DO 100 I=1,DegreesOfFreedom
          EffectiveN = EffectiveN+Shape(I)*N(NP(I))
 100    CONTINUE
      ELSE
        WRITE(*,*) '*** error(EffectiveN)',Branch,X,H
        STOP
      END IF  

      RETURN
      END



*=======================================================================
*   Public: dEffectiveN
*=======================================================================

      REAL FUNCTION dEffectiveN(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate d(EffectiveN)/dH in the current channel,
*     at X downstream distance,  for the area limited by the lowest point
*     in the channel and a distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      EXTERNAL CxShapeFunction

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF(CxShapeFunction(X,H)) THEN
        dEffectiveN = 0.0
        DO 100 I=1,DegreesOfFreedom
          dEffectiveN = dEffectiveN+dShapeDX(I)*N(NP(I))
 100    CONTINUE
      ELSE
        WRITE(*,*) '*** error(dEffectiveN)',Branch,X,H
        STOP
      END IF
      dEffectiveN = dEffectiveN/dH
     
      RETURN
      END



*=======================================================================
*   Public: WetPerimeter
*=======================================================================

      REAL FUNCTION WetPerimeter(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate wetted perimeter of a cross section in the current channel,
*     at X downstream distance, limited by the lowest point in the channel
*     and a distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      EXTERNAL CxShapeFunction

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------
     
      IF(CxShapeFunction(X,H)) THEN
        WetPerimeter = 0.0
        DO 100 I=1,DegreesOfFreedom
          WetPerimeter = WetPerimeter+Shape(I)*P(NP(I))
 100    CONTINUE
      ELSE
        WRITE(*,*) '*** error(WetPerimeter)',Branch,X,H
        STOP
      END IF  
     
      RETURN
      END



*=======================================================================
*   Public: dWetPerimeter
*=======================================================================

      REAL FUNCTION dWetPerimeter(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate d(WetPerimeter)/dH of a cross section in the current channel,
*     at X downstream distance, limited by the lowest point in the channel
*     and a distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      EXTERNAL CxShapeFunction

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF(CxShapeFunction(X,H)) THEN
        dWetPerimeter = 0.0
        DO 100 I=1,DegreesOfFreedom
          dWetPerimeter = dWetPerimeter+dShapeDX(I)*P(NP(I))
 100    CONTINUE
      ELSE
        WRITE(*,*) '*** error(dWetPerimeter)',Branch,X,H
        STOP
      END IF
      dWetPerimeter = dWetPerimeter/dH

      RETURN
      END



*=======================================================================
*   Public: Beta
*=======================================================================

      REAL FUNCTION Beta(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate the momentum coefficient in the current channel,
*     at X downstream distance, limited by the lowest point in the channel
*     and a distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      EXTERNAL CxShapeFunction

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF(CxShapeFunction(X,H)) THEN
        Beta = 0.0
        DO 100 I=1,DegreesOfFreedom
          Beta = Beta+Shape(I)*Bta(NP(I))
 100    CONTINUE
      ELSE
        WRITE(*,*) '*** error(Beta)',Branch,X,H
        STOP
      END IF  

      RETURN
      END



*=======================================================================
*   Public: dBeta
*=======================================================================

      REAL FUNCTION dBeta(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate d(Beta)/dH in the current channel,
*     at X downstream distance, limited by the lowest point in the channel
*     and a distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      EXTERNAL CxShapeFunction

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF(CxShapeFunction(X,H)) THEN
        dBeta = 0.0
        DO 100 I=1,DegreesOfFreedom
          dBeta = dBeta+dShapeDX(I)*Bta(NP(I))
 100    CONTINUE
      ELSE
        WRITE(*,*) '*** error(dBeta)',Branch,X,H
        STOP
      END IF

      dBeta = dBeta/dH

      RETURN
      END



*=======================================================================
*   Public: AreaWtSinuosity
*=======================================================================

      REAL FUNCTION AreaWtSinuosity(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate area-weighted sinuosity in the current channel,
*     at X downstream distance,  for the area limited by the lowest point
*     in the channel and a distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      EXTERNAL CxShapeFunction

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF(CxShapeFunction(X,H)) THEN
        AreaWtSinuosity = 0.0
        DO 100 I=1,DegreesOfFreedom
          AreaWtSinuosity = AreaWtSinuosity+Shape(I)*MA(NP(I))
 100    CONTINUE
      ELSE
        WRITE(*,*) '*** error(AreaWtSinuosity)',Branch,X,H
        STOP
      END IF  

      RETURN
      END



*=======================================================================
*   Public: dAreaWtSinuosity
*=======================================================================

      REAL FUNCTION dAreaWtSinuosity(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate d(area-weighted sinuosity)/dH in the current channel,
*     at X downstream distance,  for the area limited by the lowest point
*     in the channel and a distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      EXTERNAL CxShapeFunction

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF(CxShapeFunction(X,H)) THEN
        dAreaWtSinuosity = 0.0
        DO 100 I=1,DegreesOfFreedom
          dAreaWtSinuosity = dAreaWtSinuosity+dShapeDX(I)*MA(NP(I))
 100    CONTINUE
      ELSE
        WRITE(*,*) '*** error(dAreaWtSinuosity)',Branch,X,H
        STOP
      END IF 

      dAreaWtSinuosity = dAreaWtSinuosity/dH

      RETURN
      END



*=======================================================================
*   Public: QWtSinuosity
*=======================================================================

      REAL FUNCTION QWtSinuosity(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate discharge-weighted sinuosity in the current channel,
*     at X downstream distance,  for the area limited by the lowest point
*     in the channel and a distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      EXTERNAL CxShapeFunction

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF(CxShapeFunction(X,H)) THEN
        QWtSinuosity = 0.0
        DO 100 I=1,DegreesOfFreedom
          QWtSinuosity = QWtSinuosity+Shape(I)*MQ(NP(I))
 100    CONTINUE
      ELSE
        WRITE(*,*) '*** error(QWtSinuosity)',Branch,X,H
        STOP
      END IF  

      RETURN
      END



*=======================================================================
*   Public: dQWtSinuosity
*=======================================================================

      REAL FUNCTION dQWtSinuosity(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate d(discharge-weighted sinuosity)/dH in the current channel,
*     at X downstream distance,  for the area limited by the lowest point
*     in the channel and a distance H above the lowest point.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I

*   Routines by module:

***** Local:
      LOGICAL  CxShapeFunction
      EXTERNAL CxShapeFunction

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF(CxShapeFunction(X,H)) THEN
        dQWtSinuosity = 0.0
        DO 100 I=1,DegreesOfFreedom
          dQWtSinuosity = dQWtSinuosity+dShapeDX(I)*MQ(NP(I))
 100    CONTINUE
      ELSE
        WRITE(*,*) '*** error(dQWtSinuosity)',Branch,X,H
        STOP
      END IF
      dQWtSinuosity = dQWtSinuosity/dH

      RETURN
      END



*=======================================================================
*   Private: CxShapeFunction
*=======================================================================

      LOGICAL FUNCTION CxShapeFunction(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Compute common factors governing the interpolation of hydraulic
*     properties at a point. 

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:

      LOGICAL ok

*   Routines by module:

***** Local:
      LOGICAL  SetXi, SetEta, BiLinearShapeFunction

***** Buffered output:
      LOGICAL  FlushSeries
      EXTERNAL FlushSeries
      EXTERNAL SetXi, SetEta, BiLinearShapeFunction
      
***** Channel status:
      LOGICAL  WriteNetworkRestartFile
      EXTERNAL WriteNetworkRestartFile

***** Schematic data:
      LOGICAL  CloseChannel
      EXTERNAL CloseChannel

*   Intrinsics:
      REAL      ABS
      INTRINSIC ABS

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      CxShapeFunction = .FALSE.
      
      IF(     Branch .NE. PreviousBranch
     #               .OR.
     #        INT(X) .NE. PreviousX
     #               .OR.
     #  ABS(H-PreviousH) .GT. 1.0E-20   ) THEN

        PreviousBranch = Branch
        PreviousX      = INT(X)
        PreviousH      = H
       
*------ Determine relative longitudinal location of a point
*       within a branch.

        IF(SetXi(X)) THEN

*-------- Determine relative vertical location of the point.
          IF(SetEta(X,H)) THEN

*---------- Compute bi-linear shape functions.
            IF(BiLinearShapeFunction()) THEN
            END IF

          ELSE

            ok = CloseChannel()
            WRITE(*,*) ' Flushing series buffer...'
*---------- Write space- and time-series network results.
            ok = FlushSeries()
     
*---------- Write network restart file.
            WRITE(*,*) ' Writing restart file if requested...'
            ok = WriteNetworkRestartFile()

            WRITE(*,*) '*** error(CxShapeFunction)..branch,X',Branch,X
            WRITE(*,*) ' '
            WRITE(*,*) ' Abnormal program end.'
            STOP
          END IF
        ELSE
          WRITE(*,*) '*** error(CxShapeFunction)..branch,X',Branch,X
          WRITE(*,*) ' '
          WRITE(*,*) ' Abnormal program end.'
          STOP
        END IF
      END IF
     
      CxShapeFunction = .TRUE.

      RETURN
      END



*=======================================================================
*   Private: SetXi
*=======================================================================

      LOGICAL FUNCTION SetXi(X)

      IMPLICIT NONE

*   Purpose:  
*     Locate current point in relation to X locations of existing tables.

*   Arguments:
      REAL    X

*   Argument definitions:
*     X - downstream distance.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I, II

*   Routines by module:

***** Local:

*   Intrinsics:
      INTEGER   INT
      INTRINSIC INT

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: Sept  1993
*

*-- Implementation -----------------------------------------------------

      SetXi = .FALSE.

      DO 100 II=FirstTable(Branch),LastTable(Branch)
        I = II
        IF( X .LT. XDistance(I) ) GO TO 200
 100  CONTINUE       

      IF(INT(X).NE.INT(XDistance(LastTable(Branch)))) THEN
        WRITE(*,*) ' *** warning(SetXi)'
        WRITE(*,*) X,' > maximum XDistance = ',
     #             XDistance( LastTable(Branch) )
      END IF

 200  CONTINUE

      IF( INT(X) .NE. INT(XDistance(FirstTable(Branch))) ) THEN
        IF( I .NE. FirstTable(Branch) ) THEN
          N1 = I-1
          N2 = I
          Xi = (X-XDistance(N1))/(XDistance(N2)-XDistance(N1))
        ELSE
          WRITE(*,*) '*** error(SetXi)'
          WRITE(*,*) X, '< minimum XDistance = ',
     #               XDistance(FirstTable(Branch))
          Xi = 0.0
        END IF
      ELSE
      
        N1 = FirstTable(Branch)
        N2 = N1 + 1
        Xi = 0.0
        
      END IF
      
      DegreesOfFreedom = 4

      SetXi = .TRUE.

      RETURN
      END



*=======================================================================
*   Private: SetEta
*=======================================================================

      LOGICAL FUNCTION SetEta(X, H)

      IMPLICIT NONE

*   Purpose:  
*     Locate current point (H) in relation to depths tabulated at 
*     XDistance(N1) and XDistance(N2).  N1 and N2 must be current.

*   Arguments:
      REAL    X, H

*   Argument definitions:
*     X - downstream distance.
*     H - distance above lowest point in cross section.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I, L1, L2
      REAL    XiM

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      SetEta = .FALSE.
      XiM = 1.0-Xi

*---- Determine which pair of upstream-downstream points bracket the
*     interpolation point on the top and then exit the loop.

        L1 = Offset(N1)-1
        L2 = Offset(N2)-1
        DO 100 I=1,Lines(Branch)
          L1 = L1+1
          L2 = L2+1
                    
          IF(H .LT. (XiM*Depth(L1)+Xi*Depth(L2))) GO TO 200
 100    CONTINUE       

          IF(H .GT. (XiM*Depth(L1)+Xi*Depth(L2) + 1.0) ) THEN
            WRITE(*,*) ' '
            WRITE(*,*) ' *** warning(SetEta)'
            WRITE(*,*) ' Channel =',Branch,' X = ',X
            WRITE(*,*) ' ',H,' > maximum tabulated depth,'
            WRITE(*,*) ' (the line connecting ',Depth(L1),
     #                 ' and ',Depth(L2),') ...'
          END IF

 200    CONTINUE

        IF(I.NE.1) THEN
          NP(1) = L1-1
          NP(2) = L2-1
          NP(3) = L2
          NP(4) = L1
          dH =      XiM*(Depth(NP(4))-Depth(NP(1)))
     #              +Xi*(Depth(NP(3))-Depth(NP(2)))
          Eta = (H-XiM*Depth(NP(1))-Xi*Depth(NP(2)))/dH

          IF(Eta.LT.-1.0E-06) THEN
            WRITE(*,*) ' '
            WRITE(*,*) 'SetEta...'
            WRITE(*,*) 'eta = ',Eta
            WRITE(*,*) 'depths = '
            WRITE(*,'(4F12.2)') (Depth(NP(I)),I=1,4)
            WRITE(*,*) 'dH = ',dH,'  Xi = ',Xi
          END IF

        ELSE
          WRITE(*,*) ' '
          WRITE(*,*) '*** error(SetEta)'
          WRITE(*,*) ' Channel =',Branch,' X = ',X
          WRITE(*,*) ' ',H, '< minimum depth,'
          WRITE(*,*) ' (the average of ',Depth(Offset(N1)),' and ',
     #                Depth(Offset(N2)),') ...'

     
          RETURN
        END IF

      SetEta = .TRUE.

      RETURN
      END



*=======================================================================
*   Private: BiLinearShapeFunction
*=======================================================================

      LOGICAL FUNCTION BiLinearShapeFunction()

      IMPLICIT NONE

*   Purpose:  
*     Compute bi-linear shape functions in local coordinates.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      Shape(1)    = (1.0-Xi)*(1.0-Eta)
      dShapeDX(1) = -1.0+Xi
      Shape(2)    = Xi*(1.0-Eta)
      dShapeDX(2) = -Xi

      IF(DegreesOfFreedom.EQ.4) THEN
        Shape(3)    = Xi*Eta
        dShapeDX(3) = Xi
        Shape(4)    = (1.0-Xi)*Eta
        dShapeDX(4) = 1.0-Xi
      END IF

      BiLinearShapeFunction = .TRUE.
      
      RETURN
      END



*=======================================================================
*   Private: ReadCxProperties
*=======================================================================

      LOGICAL FUNCTION ReadCxProperties
     I                 ( INUNIT,M,NPRT,
     M                   DUM,LNUM,USR,
     O                   ICARD         )

      IMPLICIT NONE

*   Purpose:  
*     Read hydraulic properties and geometry for 1 cross section.

*   Arguments:
      INTEGER INUNIT, USR, LNUM, M, NPRT
      CHARACTER*80 DUM
      CHARACTER*2 ICARD

*   Argument definitions:
*     INUNIT - FORTRAN unit number
*     M      - current branch number
*     NPRT   - screen output index
*              [0] no screen output
*              [1] write to screen
*     DUM    - a line read from properties input file
*     LNUM   - current line number
*     USR    - current cross-section number
*     ICARD  - record index
*              [CH] branch header
*              [HY] cross-section header
*              [DP] properties record

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I, J, NPT, BRN
      REAL    R(8), R23, R53
      PARAMETER (R23 = 2.0/3.0, R53 = 5.0/3.0)
      LOGICAL FIRST, FirstLine

*   Variable definitions:
*     R      - hydraulic properties of channel
*              [1] depth
*              [2] area
*              [3] conveyance
*              [4] beta
*              [5] area-weighted sinuosity
*              [6] conveyance-weighted sinuosity
*              [7] width
*              [8] wetted perimeter

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      ReadCxProperties = .FALSE.

      NPT=0
      READ(DUM,'(A2)') ICARD

      IF(ICARD.EQ.'CH') THEN
        READ(DUM,'(2X,I10)') BRN
        FIRST = .TRUE.

        IF(BRN.NE.M) THEN
          WRITE(*,*) 'Branch numbers are not in correct sequence.'
          WRITE(*,*) BRN,' changed to ',M,'....'
          BRN = M
        END IF

        Lines(BRN) = MaxLinesPerTable
        IF(NPRT .GT. 0 ) THEN
          WRITE(*,*) ' '
          WRITE(*,*) ' '
          WRITE(*,*) 'Currently reading...'
          WRITE(*,'(A80)') DUM
        END IF
        FirstTable(BRN) = USR+1
        READ(INUNIT,'(80A)',END=50) DUM
        READ(DUM,'(A2)') ICARD
        GO TO 60
50      CONTINUE

          WRITE(*,*)'***error(ReadCxProperties) unexpected end of file.'
          WRITE(*,*) 'Branch',BRN
          ICARD = 'EN'

60      CONTINUE
      END IF

      IF(ICARD.EQ.'HY') THEN
        USR = USR+1
        
        FirstLine = .TRUE.

        IF(USR.GT.MaxTables) THEN
          WRITE(*,*) '***error (ReadCxProperties)'
          WRITE(*,*) 'Maximum number of tables (',
     #                 MaxTables,') exceeded.'
          WRITE(*,*) 'Returning...'
        END IF

        READ(DUM,'(3X,A16,1X,F10.0,1X,F10.0)') 
     #            ID(USR),XDistance(USR),Datum(USR)

        IF(NPRT.GT.0) THEN
          WRITE(*,'(I10,F16.2)') USR,XDistance(USR)
        END IF

        Offset(USR) = LNUM+1

        IF((LNUM+Lines(BRN)) .GT. MaxLines) THEN
          WRITE(*,*) '***error (ReadCxProperties)'
          WRITE(*,*) 'Maximum number of lines exceeded...'
          WRITE(*,*) 'Cross section ',USR
          WRITE(*,*) 'Current line number = ',LNUM
          WRITE(*,*) 'Returning...'
          RETURN
        END IF

      ELSE
        WRITE(*,*) '***ERROR (ReadCxProperties) expecting HY record.'
        WRITE(*,*) 'Returning...'
        ReadCxProperties = .FALSE.
        RETURN
      END IF

      DO 100 I =1,Lines(BRN)+1
        READ(INUNIT,'(80A)',END=102) DUM
        READ(DUM,'(A2)') ICARD

        IF(ICARD.NE.'DP') THEN

          IF(FIRST) THEN
            FIRST = .FALSE.
            Lines(BRN) = NPT
          ELSE

            IF(Lines(BRN).GT.MaxLinesPerTable) THEN
              WRITE(*,*) '***error (ReadCxProperties)'
              WRITE(*,*) 'Lines per table exceeded.'
              WRITE(*,*) 'Current = ',Lines(BRN),
     #                    ', MaxLinesPerTable = ',MaxLinesPerTable
              WRITE(*,*) 'Returning...'
              RETURN
            END IF

          END IF
          GO TO 102

        ELSE IF(ICARD.EQ.'DP') THEN
          LNUM = LNUM+1
          NPT = NPT+1

          IF(NPT.LE.Lines(BRN)) THEN
            READ(DUM,
     #     '(3X,F10.0,2(1X,E13.6),  3(1X,F5.0),   2(1X,F7.0))')
*               depth,   area,K     beta,Ma,Mq      width,Wp
     #                     (R(J),J=1,8)

            IF(NPRT.GT.1) THEN
              WRITE(*,
     #        '(3X,F10.2,2(1X,E13.6),  3(1X,F5.2),   2(1X,F7.2))')
     #                     (R(J),J=1,8)
            END IF

*---------- Assign hydraulic properties.
            Depth(LNUM) = R(1)
            A(LNUM)     = R(2)
            K(LNUM)     = R(3)
            Bta(LNUM)   = R(4)
            MA(LNUM)    = R(5)
            MQ(LNUM)    = R(6)
            Width(LNUM) = R(7)
            P(LNUM)     = R(8)

*---------- Check for adverse conveyance gradient with depth.
            IF( .NOT. FirstLine ) THEN
            
              IF( K(LNUM) .LE. K(LNUM-1) ) THEN
                WRITE(*,*) ' ***error (ReadCxProperties) adverse ',
     #                     'conveyance gradient,'
                WRITE(*,*) ' Channel...',M, ' Cross section...',ID(USR),
     #                     ' conveyance...',K(LNUM)
              END IF
            
            ELSE
              FirstLine = .FALSE.
            END IF


*---------- Compute one over effective n.
            IF(A(LNUM).GT.0.1) THEN
              N(LNUM) = (P(LNUM)**R23*K(LNUM))/(1.486*A(LNUM)**R53)
            ELSE
              N(LNUM) = 0.0
            END IF

          ELSE
            WRITE(*,*) 'Too many lines...'
            WRITE(*,*) 'I = ',I,'  Lines = ',Lines(BRN)
          END IF

        END IF

 100  CONTINUE
        WRITE(*,*) '***ERROR (ReadCxProperties)' 
        WRITE(*,*) 
     #  'Maximum number of lines per table (',Lines(BRN),') exceeded.'
        WRITE(*,*) 'Last line read...'
        WRITE(*,'(A80)') DUM
        WRITE(*,*) 'I = ',I,' NPT = ',NPT,'LNUM = ',LNUM
        WRITE(*,*) 'Returning ...'
        ReadCxProperties = .FALSE.
        RETURN
 102  CONTINUE

      IF(ICARD.EQ.'DP') THEN
        ICARD = 'EN'
      END IF

      LastTable(BRN) = USR
      IF(NPT.EQ.Lines(BRN)) THEN

*-------- The following statements correct for zero values of
*         momentum & sinuosity coefficients when the first 
*         value in the input table is at zero area.

        IF(Bta(Offset(USR)).LT.1.0E-06) 
     #     Bta(Offset(USR)) = Bta(Offset(USR)+1)
        IF(MA(Offset(USR)).LT.1.0E-06) 
     #     MA(Offset(USR)) = MA(Offset(USR)+1)
        IF(MQ(Offset(USR)).LT.1.0E-06) 
     #     MQ(Offset(USR)) = MQ(Offset(USR)+1)
        IF(N(Offset(USR)).LT.1.0E-06) 
     #     N(Offset(USR)) = N(Offset(USR)+1)

        ReadCxProperties = .TRUE.

      ELSE IF(NPT.LT.Lines(BRN)) THEN
        WRITE(*,*) '***ERROR (ReadCxProperties)'
        WRITE(*,*) 'Fewer than expected (',Lines(BRN),
     #             ') lines per cross section.'
      END IF

      RETURN
      END



*=======================================================================
*   Private: Read1stCxLine
*=======================================================================

      LOGICAL FUNCTION Read1stCxLine
     I          (  
     O                         NPRT,DUM)

      IMPLICIT NONE

*   Purpose:  
*     Open a flat file and read an 80-character record.

*   Arguments:
      INTEGER NPRT
      CHARACTER*80  DUM

*   Argument definitions:
*     NPRT - index for output,
*            [0] no output,
*            [1] echo input.
*     DUM  - an 80-character record read from the file INAME.

*   Module data:

*   Local Variables:
      INTEGER INUNIT 
      CHARACTER*12 INNAME, InternalFileName
      
*   Routines by module:

***** Master File:
      INTEGER GetFileUnit
      CHARACTER*12 GetFileName

***** File utilities:
      LOGICAL  OpenOldText
      EXTERNAL OpenOldText

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      InternalFileName = 'cxgeom.dat'
      
      INUNIT = GetFileUnit(InternalFileName)
      INNAME = GetFileName(InternalFileName)
      
      IF(OpenOldText(INUNIT,INNAME)) THEN
        READ(INUNIT,'(A80)') DUM
        READ(DUM,*)     NPRT
        READ(INUNIT,'(80A)') DUM
        Read1stCxLine = .TRUE.
      ELSE
        WRITE(*,*) '***ERROR (Read1stCxLine) file not opened.'
        WRITE(*,'(A)') INNAME
        WRITE(*,*) 'Returning...'
        Read1stCxLine = .FALSE.
      END IF

      RETURN
      END



*=======================================================================
*   Private: RiverFtToDownStream
*=======================================================================

      LOGICAL FUNCTION RiverFtToDownStream(M)

      IMPLICIT NONE

*   Purpose:  
*     Convert river feet to downstream distance.

*   Arguments:
      INTEGER M

*   Argument definitions:
*     M - branch number.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I, J

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF( (XDistance(LastTable(M))-XDistance(FirstTable(M))) 
     #                                        .LT. 1.0E-06 ) THEN

        DO 100 I=FirstTable(M),LastTable(M)
          XDistance(I) = -XDistance(I)
 100    CONTINUE

      END IF

      DO 200 I=FirstTable(M)+1,LastTable(M)
        IF( (XDistance(I)-XDistance(I-1)) .LT. 1.0E-06 ) THEN
          WRITE(*,*) '***ERROR (RiverFtToDownStream)'
          WRITE(*,*) 'X coordinates out of order...'
          WRITE(*,*) 'Branch',Branch
          WRITE(*,'(I5,F15.2)') 
     #      (J,XDistance(J),J=FirstTable(M),LastTable(M))
          RiverFtToDownStream = .FALSE.
          RETURN
        END IF

 200  CONTINUE

      RiverFtToDownStream = .TRUE.

      RETURN
      END



*======================================================================
*   Public: AdjustChannelWidths
*=======================================================================

      LOGICAL FUNCTION AdjustChannelWidths()

      IMPLICIT NONE

*   Purpose:  
*     Compute and tabulate cross-section widths consistent with
*     cross-sectional tabulated areas.
*     

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER I, II, ONE, TWO, M, END, CxCount, LineCount
      INTEGER J
      REAL    dB( MaxLinesPerTable )

*   Routines by module:

***** Local:

*   Intrinsics:
      INTEGER   INT
      INTRINSIC INT

*   Programmed by: Lew DeLong
*   Date:          Jan   1994

*-- Implementation -----------------------------------------------------

      AdjustChannelWidths = .FALSE.
      CxCount = 0
      LineCount = 0
      
      DO 300 M=1,NumCh
        DO 200 II=FirstTable(M),LastTable(M)
        
          CxCount = CxCount + 1

*-------- Compute change in bottom width.
          ONE = Offset(II)
          TWO = ONE+1
          dB(1) = ( A(TWO) - A(ONE) 
     #            - .5*(Width(ONE)+Width(TWO))*(Depth(TWO)-Depth(ONE))
     #            ) / ( 2.0*(Depth(TWO)-Depth(ONE)) )

*-------- Check for negative widths and reset if necessary.

          IF( Width(ONE)+dB(1) .GT. 0.0 ) THEN
          ELSE
          
            dB(1) = 0.0
            
          END IF
                      

*-------- Equate wetted perimeter, at the channel bottom, to bottom width.
          P(ONE) = Width(ONE) + dB(1)

*-------- Equate values of sinuosity and beta at the bottom of the channel
*         to those at the top of the lowest trapezoid.
          Bta(ONE) = Bta(TWO)
          MA(ONE)  = MA(TWO)
          MQ(ONE)  = MQ(TWO)

*-------- Compute change in widths at the top of second and higher trapezoids.
          IF(Lines(M).GE.3) THEN
            END   = ONE+Lines(M)-2
            J = 1
            DO 100 I=TWO,END
              
              J = J + 1
              dB(J) = (A(I+1)-A(I-1))/(Depth(I+1)-Depth(I-1))
     #                -.25*(Width(I-1)+2.0*Width(I)+Width(I+1))
     
 100        CONTINUE
          END IF
          
          J = J + 1
          I = END + 1
          dB(J) = ( A(I) - A(I-1) 
     #            - .5*(Width(I)+Width(I-1))*(Depth(I)-Depth(I-1))
     #            ) / ( 2.0*(Depth(I)-Depth(I-1)) ) 
     
          J = 0
          DO 110 I=ONE,END+1
            J = J + 1
            Width(I) = Width(I) + dB(J)
 110      CONTINUE


        IF( Print ) THEN
          WRITE(*,*) ' '
          WRITE(*,*) ' Cross section...',ID(CxCount)
          WRITE(*,*) '  Depth     Adjusted Width 1/Effective n'
          DO 150 I=1,Lines(M)
            LineCount = LineCount + 1
            WRITE(*,'(F6.2,6X,F10.2,4X,F6.3)')
     #       Depth(LineCount), Width(LineCount), N(LineCount)
 150      CONTINUE
        END IF

 200    CONTINUE       
 300  CONTINUE
 

      AdjustChannelWidths = .TRUE.

      RETURN
      END



*=======================================================================
*   Public: SingleTrap
*=======================================================================

      LOGICAL FUNCTION SingleTrap()

      IMPLICIT NONE

*   Purpose:  
*     Roughly approximate tabulated properties with a single trapezoid
*     write resulting cross-section properties in a form suitable for
*     use with the simple trapezoidal-properties module.
*     

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'

*   Local Variables:
      INTEGER II, M, UN, TopLine
      REAL    MaxArea, MaxDepth, MaxWidth, BottomWidth, dWdH
      REAL    ApproxN
      CHARACTER*12 FileName

*   Routines by module:

***** Local:

***** File utilities:
      LOGICAL  OpenNewText
      EXTERNAL OpenNewText

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          July  1993
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      SingleTrap = .FALSE.
      
      UN = 57
      FileName = 'geomtrap.dat'

      IF( OpenNewText( UN, FileName ) ) THEN
        WRITE(UN,*) '0', ' / output index ( 0 - no echo, 1 - echo ).'
      ELSE
        WRITE(*,*) ' Could not open ', FileName
        WRITE(*,*) ' Approximate single trapezoids not computed.'
        RETURN
      END IF
      
      DO 300 M=1,NumCh
        WRITE(UN,*) ' '
        WRITE(UN,*) M,' / channel number.'
        II = FirstTable(M)
        ApproxN = N( Offset(II) + Lines(II) - 1 )
        II = LastTable(M)
        ApproxN =  0.5 * (
     #        ApproxN + N( Offset(II) + Lines(II) - 1 )
     #                   )
        
        WRITE(UN,*) ApproxN,' / 1/effective n.'
        
        DO 200 II=FirstTable(M),LastTable(M),LastTable(M)-FirstTable(M)

*-------- Compute bottom width.
          
          TopLine = Offset(II) + Lines(II) - 1
          MaxArea = A( TopLine )
          MaxWidth = Width( TopLine )
          MaxDepth = Depth( TopLine )
          BottomWidth = 2.0 * MaxArea / MaxDepth - MaxWidth

*-------- Check for negative widths and recompute if necessary.

          IF( BottomWidth .GE. 0.0 ) THEN
          
            dWdH = (MaxWidth - BottomWidth) / MaxDepth
            
          ELSE

            BottomWidth = 0.0
            dWdH = 2.0 * MaxArea / ( MaxDepth ** 2 ) 
            
          END IF
            
*-------- Write properties.
          
          WRITE(UN,*) ID(II), ' / ID.'
          WRITE(UN,*) XDistance(II), BottomWidth, Datum(II), dWdH

 200    CONTINUE       


 300  CONTINUE

      SingleTrap = .TRUE.

      RETURN
      END




*== Public (InitChnlBtmHermites) ================================================

      LOGICAL FUNCTION InitChnlBtmHermites()

      IMPLICIT NONE

*   Purpose:  Initialize values necessary for interpolating
*             channel bottom elevation and slope using
*             hermite cubics.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'
      INCLUDE 'herman.inc'

*   Local Variables:
      INTEGER I, J, M
      INTEGER Points, FirstPoint

*   Routines by module:

***** Local:

      EXTERNAL HMMM, HGRAD

***** Schematic data:
      INTEGER  NumberOfChannels
      EXTERNAL NumberOfChannels

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Oct   1993
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      InitChnlBtmHermites = .FALSE.
      
*---- Set known values.

      J = -1
      DO 100 I=1,MaxTables
        J = J + 2
        XHerm(J) = XDistance(I)
        ZHerm(J) = Datum(I)
 100  CONTINUE
 
*---- Estimate gradients.

      DO 500 M=1,NumberOfChannels()
      
        Points = 2 * (LastTable(M) - FirstTable(M) + 1)
        FirstPoint = 2 * FirstTable(M) - 1
        CALL HMMM( Points, XHerm(FirstPoint) )
*       write(*,'(I4,2F12.4)') (I,XHerm(I),XHerm(I+1),I=1,Points,2)
        CALL HMMM( Points, ZHerm(FirstPoint) )
*       write(*,'(I4,2F12.4)') (I,ZHerm(I),ZHerm(I+1),I=1,Points,2)
        CALL HGRAD( Points,XHerm(FirstPoint),ZHerm(FirstPoint) )
*       write(*,'(I4,2F12.4)') (I,XHerm(I),XHerm(I+1),I=1,Points,2)
      
 500  CONTINUE
 
      
*     write(*,'(I4,4F12.4)') 
*    #     ( I,XHerm(I),XHerm(I+1),ZHerm(I),ZHerm(I+1),
*    #      I=1,Points,2 )
      
      InitChnlBtmHermites = .TRUE.

      RETURN
      END



*== Public (HermBtmElev) ================================================

      REAL FUNCTION HermBtmElev()

      IMPLICIT NONE

*   Purpose:  Compute bottom elevation using Hermite interpolation.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'
      INCLUDE 'herman.inc'

*   Local Variables:
      INTEGER m
      REAL    dX

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Oct   1993
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      dX = XDistance(N2) - XDistance(N1)
      m = 2 * N1 - 1
      
      HermBtmElev =
     #     (Xi-1.0)**2 * (2.0*Xi+1.0) * ZHerm(m  )
     #   +  Xi * (Xi-1.0)**2 * dX     * ZHerm(m+1)
     #   -  Xi**2 * (2.0*Xi-3.0)      * ZHerm(m+2)
     #   +  Xi**2 * (Xi-1.0) * dX     * ZHerm(m+3)
          
      RETURN
      END



*== Public (HermBtmSlope) ================================================

      REAL FUNCTION HermBtmSlope()

      IMPLICIT NONE

*   Purpose:  Compute bottom slope using Hermite interpolation.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtbl.inc'
      INCLUDE 'herman.inc'

*   Local Variables:
      INTEGER m
      REAL    dX

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Oct   1993
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      dX = XDistance(N2) - XDistance(N1)
      m = 2 * N1 - 1
      
      HermBtmSlope =
     #      6.0*Xi * (Xi-1.0) / dX    * ZHerm(m  )
     #   +  (Xi-1.0) * (3.0*Xi-1.0)   * ZHerm(m+1)
     #   -  6.0*Xi * (Xi-1.0) / dX    * ZHerm(m+2)
     #   +  Xi * (3.0*Xi-2.0)         * ZHerm(m+3)

      RETURN
      END


      SUBROUTINE   HMMM
     I          (NPT,
     M           HDATA)
C
C     + + + PURPOSE + + +
C
C     Compute mean incremental changes in a sequence of numbers.
C     The numbers are input as the odd elements of HDATA(). Mean
C     incremental changes are computed and stored as even elements.
C
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   NPT
      REAL   HDATA(NPT)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NPT    - total number of elements in HDATA()
C     HDATA  - floating point array, odd elements of which
C              are the sequence of numbers from which mean incremental
C              changes will be computed
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I,MPT
      REAL   D,E,XP,XM,TEST
C
C     + + + INTRINSICS + + +
      INTRINSIC ABS
C
C     + + + END SPECIFICATIONS + + +
C
      HDATA(2)=HDATA(3)-HDATA(1)
      XM=HDATA(2)
      MPT=NPT-2
      IF(MPT.GT.3) THEN
        DO 100 I=3,MPT,2
          XP=HDATA(I+2)-HDATA(I)
          D=XP+XM
          E=XP*XM
          TEST=ABS(D)
          IF(TEST.GT.1.0E-30.AND.E.GT.0.0) THEN
            HDATA(I+1)=2.0*E/D
          ELSE
            HDATA(I+1)=0.0
          END IF
            XM=XP
 100    CONTINUE
        HDATA(NPT)=XP
      ELSE
        HDATA(4)=HDATA(2)
      END IF
C
      RETURN
      END



      SUBROUTINE  HGRAD
     I                  (NPT,
     M                   HX,HY)
C
C     + + + PURPOSE + + +
C     Compute an array of gradients from previously
C     computed incremental changes. Mean incremental changes in the values
C     stored in odd elements of the arrays are input in corresponding
C     even elements of the arrays. Thus:
C     .
C                     Gradient = dY/dX = HY(even)/HX(even),
C     .
C                     and are stored as 
C     .
C                     HY(even) = gradient.
C
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   NPT
      REAL   HX(NPT),HY(NPT)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NPT    - number of elements in HX() & HY() arrays
C     HX     - array of floating-point values, even elements are mean
C              mean incremental change in values stored in odd elements.
C     HY     - array of floating-point values similar to HX()
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I
C
C     + + + END SPECIFICATIONS + + +
C
      DO 100 I=2,NPT,2
      HY(I)=HY(I)/HX(I)
 100  CONTINUE
C
      RETURN
      END

*===== BOF chcxtbl ======================================================

