***** BOF TRAPGEOM *****************************************************
*
*     This file is a FORTRAN module for the computation of the geometric
*     and hydraulic properties of multiple, trapezoidal, non-prismatic
*     channels.   Manning's "n" is assumed constant with depth and
*     downstream distance within a branch.    A branch is represented by
*     two trapezoidal cross sections at its extremities.  Width, depth,
*     and wetted perimeter are interpolated linearly.  Area and
*     conveyance are subsequently computed from these interpolated
*     values.  Five data lines are input for each branch.  All input is
*     free-field.
*
*     line 0:
*       PrintOption (integer)
*
*     line 1:
*       Eta (real)

*     line 2:
*       UpStreamID (character*16)
*
*     line 3:
*       X1 , Width1, Btm1, dWdH1  
*
*     line 4:
*       DownStreamID (character*16)
*
*     line 5:
*       X2 , Width2, Btm2, dWdH2 
*
*          (Repeat above lines 1 and 2 for additional branches.)
*
*     where,
*
*       PrintOption - index for writing to standard output device.
*          [0] input data will not be written to standard output.
*          [1] input data will be echoed to standard output.
*
*       ChannelNumber - sequence number of channel, monotonically
*          increasing, beginning with 1.
*
*       X1      - downstream reference distance for upstream extent of
*                 branch
*       Width1  - channel width corresponding to X1
*       Btm1    - channel-bottom elevation corresponding to X1
*       dWdH1   - d(Width1)/d(depth)
*       X2      - downstream reference distance for downstream extent of
*                 branch
*       Width2  - channel width corresponding to X2
*       Btm2    - channel-bottom elevation corresponding to X2
*       dWdH2   - d(Width2)/d(depth)

*       Eta     - effective Manning's n for the branch
*
*       Note that for the purpose of determining wetted perimiter, the
*       channel is assumed symmetric.
*       The equations for a non-symmetric channel with one flat side
*       are found within the code as well in wetted-perimeter routines,
*       but are commented out.
*
*     Module note: Only functions or subroutines marked "public" should
*                  be used outside of this module as those marked
*                  "private" may not be supported by future revisions of
*                  this module or replacement modules.  Likewise, no data
*                  or common blocks contained within this module should
*                  be accessed by routines outside of this module accept
*                  through the use of "public" functions or subroutines
*                  contained within this module.
*
*     Non-standard usage: Symbolic names in this module may be
*                         represented by as many as 31 characters in
*                         order to provide better definition directly in
*                         the code.  Standard FORTRAN allows only 6
*                         characters, but this restriction is generally
*                         extended to 32 characters by most compilers.
*
*
*     Public functions:
*
*     LOGICAL FUNCTION InitializeChannelProperties()
*            - begin initialization of module.
*
*     REAL FUNCTION BtmSlope(X)
*            - returns channel-bottom slope.
*
*     REAL FUNCTION ChannelWidth(,X,H)
*            - returns channel width.
*
*     REAL FUNCTION dChannelWidth(,X,H)
*            - returns channel side slope.
*
*     REAL FUNCTION CxArea(X,H)
*            - returns cross-sectional area.
*
*     REAL FUNCTION Conveyance(X,H)
*            - returns sinuosity-weighted conveyance.
*
*     REAL FUNCTION dConveyance(X,H)
*            - returns d(Conveyance)/dH.
*
*     REAL FUNCTION WetPerimeter(X,H)
*            - returns wetted perimeter.
*
*     REAL FUNCTION dWetPerimeter(X,H)
*            - returns d(WetPerimeter)/dH.
*
*     Arguments:
*       fUnit  - FORTRAN unit number for proerties input data file
*       FileName - properties input data file name
*       NumCh   - number of branches in the current network
*       X      - current downstream distance
*       H      - current depth of flow
*
*
*     Module data:
*       X1(i)     - downstream reference distance,
*                   at upstream extent of channel "i".
*       Width1(i) - channel width at X(i).
*       Btm1(i)   - channel-bottom elevation at X1(i).
*       Z1(i)     - d(Width1)/d(depth) at X1(i).
*       X2(i)     - downstream reference distance,
*                   at downstream extent of channel "i".
*       Width2(i) - channel width at X(i).
*       Btm2(i)   - channel-bottom elevation at X2(i).
*       Z2(i)     - d(Width2)/d(depth) at X2(i).
*       Eta(i)    - effective Manning's n for channel "i".
*       i         - channel sequence number.
*
*       FileName  - name of a file containing hydraulic properties
*                     appropriate to his module.
*       fUnit     - FORTRAN unit number for the FileName.
*
********************************************************************************


*=====Public (InitializeChannelProperties)======================================

        LOGICAL FUNCTION InitializeChannelProperties()

*     Purpose:  Begin initialization of hydraulic properties module.

        IMPLICIT NONE

*     Arguments:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*     Local variables:
        INTEGER ChannelsInitialized, ChannelNumber, I
        CHARACTER*16 InternalFileName

*     Functions:
        LOGICAL  ReadHydraulicProperties, OpenOldText
        EXTERNAL ReadHydraulicProperties, OpenOldText

***** Master file:
        INTEGER GetFileUnit
        CHARACTER*12 GetFileName
        EXTERNAL GetFileName, GetFileUnit


*-- Implementation -----------------------------------------------------

      InitializeChannelProperties = .FALSE.

*---- Set properties file name and unit number.
      InternalFileName = 'cxgeom.dat  '
      
*---- retrieve file name and unit number from master file

      fUnit = GetFileUnit(InternalFileName)
      FileName = GetFileName(InternalFileName)

        IF(OpenOldText(fUnit,FileName)) THEN
          READ(fUnit,*) PrintOption
        ELSE
          WRITE(*,*) ' Could not open ', FileName
          RETURN
        END IF

        ChannelsInitialized = 0

       
        DO 100 I = 1,NumCh
          ChannelNumber = I

          IF( ReadHydraulicProperties(ChannelNumber) ) THEN
            ChannelsInitialized = ChannelsInitialized + 1
          END IF
 100    CONTINUE

      IF(ChannelsInitialized .EQ. NumCh) THEN
        InitializeChannelProperties = .TRUE.
      ELSE
        WRITE(*,*) ' Failed to initialize',
     #      NumCh - ChannelsInitialized,' channels...'
      END IF

      CLOSE(fUnit)

      RETURN
      END



*=====Public: (BtmSlope)====================================================

        REAL FUNCTION BtmSlope(X)

*     Purpose:   Compute bottom slope.

        IMPLICIT NONE

*     Arguments:
        REAL X

*     Definitions:
*       X      - downstream distance within channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*-- Implementation -----------------------------------------------------

      BtmSlope  = (Btm2(Branch) - Btm1(Branch))
     #          / (X2(Branch)   - X1(Branch))

      RETURN
      END

*== Public (BtmElev) ===================================================

      REAL FUNCTION BtmElev(X)

      IMPLICIT NONE

*   Purpose:  Return bottom elevation.

*   Arguments:
      REAL X

*   Argument definitions:
*     X      - downstream distance within channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*   Local variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation ------------------------------------------------------------

      BtmElev = (
     #     (X2(Branch)-X)*Btm1(Branch)+(X-X1(Branch))*Btm2(Branch)
     #               ) / (X2(Branch)-X1(Branch))

      RETURN
      END



*=====Public (Conveyance)========================================================

        REAL FUNCTION Conveyance(X,H)

*     Purpose:   Compute conveyance.

        IMPLICIT NONE

*     Arguments:
        REAL X,H

*     Definitions:
*       X      - downstream distance.
*       H      - depth of flow.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*     Local variables:
        REAL R53,R23
        PARAMETER (R53 = 5.0/3.0, R23 = 2.0/3.0)
        REAL Small
        PARAMETER (Small = 1.00E-6)

*     Functions:
        REAL     CxArea,WetPerimeter,ManningN
        EXTERNAL CxArea,WetPerimeter,ManningN

*-- Implementation -----------------------------------------------------

      IF (WetPerimeter(X,H).GT.Small) THEN
    
       Conveyance = 1.486 / ManningN(X,H)
     #           * (CxArea(X,H)**R53)
     #           / (WetPerimeter(X,H)**R23)
     
      ELSE
       Conveyance = 0.0
      ENDIF

      RETURN
      END



*=====Public (dConveyance)========================================================
 
        REAL FUNCTION dConveyance(X,H)

*     Purpose:   Compute d(Conveyance)/dH.

        IMPLICIT NONE

*     Arguments:
        REAL X,H

*     Definitions:
*       X      - downstream distance.
*       H      - depth of flow.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*     Local variables:
        REAL R53,R23, Zero
        PARAMETER (R53 = 5.0/3.0, R23 = 2.0/3.0,  Zero = 0.0)
        REAL Small
        PARAMETER (Small = 1.00E-6)

*     Functions:
        REAL     ChannelWidth,CxArea,WetPerimeter,dWetPerimeter
        REAL     ManningN,dManningN,Conveyance
        EXTERNAL ChannelWidth,CxArea,WetPerimeter,dWetPerimeter
        EXTERNAL ManningN,dManningN,Conveyance

*-- Implementation -----------------------------------------------------

      IF (WetPerimeter(X,H).GT.Small) THEN
      
       dConveyance = R53 * 1.486 * ChannelWidth(X,H) * CxArea(X,H)**R23
     #            / ManningN(X,H) / WetPerimeter(X,H)**R23
     #            - 1.486 * R23 * dWetPerimeter(X,H) * CxArea(X,H)**R53
     #            / ManningN(X,H) / WetPerimeter(X,H)**R53
      
      ELSE
       dConveyance = 0.0
      ENDIF

      RETURN
      END



*=====Public (ChannelWidth)========================================================

        REAL FUNCTION ChannelWidth(X,H)

*     Purpose:   Compute width of channel at water surface of cross section.

        IMPLICIT NONE

*     Arguments:
        REAL X,H

*     Definitions:
*       X      - downstream distance.
*       H      - depth of flow.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*     Functions:
        REAL     dChannelWidth
        EXTERNAL dChannelWidth
	
*-- Implementation -----------------------------------------------------

      ChannelWidth = (((X2(Branch)-X) * Width1(Branch)
     #             +  (X-X1(Branch)) * Width2(Branch))
     #             / (X2(Branch)-X1(Branch))) + H * dChannelWidth(X,H)

      RETURN
      END


*=====Public (dChannelWidth)========================================================

        REAL FUNCTION dChannelWidth(X,H)

*     Purpose:   Compute side slope of channel.

        IMPLICIT NONE

*     Arguments:
        REAL X,H

*     Definitions:
*       X      - downstream distance.
*       H      - depth of flow.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*-- Implementation -----------------------------------------------------

      dChannelWidth = ((X2(Branch)-X) * dWdH1(Branch)
     #         +  (X-X1(Branch)) * dWdH2(Branch))
     #         / (X2(Branch)-X1(Branch))

      RETURN
      END


*=====Public (CxArea)========================================================

        REAL FUNCTION CxArea(X,H)

*     Purpose:   Compute cross-sectional area.

        IMPLICIT NONE

*     Arguments:
        REAL X,H
	
*     Parameters:
        REAL Zero
	PARAMETER (Zero = 0.0)

*     Definitions:
*       X      - downstream distance.
*       H      - depth of flow.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*     Functions:
        REAL     ChannelWidth, dChannelWidth
        EXTERNAL ChannelWidth, dChannelWidth

*-- Implementation -----------------------------------------------------

      CxArea = H * ChannelWidth(X,Zero)
     #       + 0.5 * H * H * dChannelWidth(X,H)

      RETURN
      END



*=====Public (WetPerimeter)========================================================

        REAL FUNCTION WetPerimeter(X,H)

*     Purpose:   Compute wetted perimeter.

        IMPLICIT NONE

*     Arguments:
        REAL X,H

*     Definitions:
*       X      - downstream distance.
*       H      - depth of flow.

*     Parameters:
        REAL Zero
	PARAMETER (Zero = 0.0)

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*     Functions:
        REAL     ChannelWidth, dChannelWidth
        EXTERNAL ChannelWidth, dChannelWidth

*-- Implementation -----------------------------------------------------

*-- this equation can be used when assuming all slope is on one side.
*     WetPerimeter = ChannelWidth(X,Zero) + H 
*    #             + H * (1.0 + dChannelWidth(X,H)**2)**0.5
     
*-- this equation assumes symmetric slopes on both sides of the channel.      
      WetPerimeter = ChannelWidth(X,Zero)  
     #             + 2.0 * H * (1.0 + (dChannelWidth(X,H)/2)**2)**0.5

      RETURN
      END



*=====Public (dWetPerimeter)========================================================

        REAL FUNCTION dWetPerimeter(X,H)

*     Purpose:   Compute d(wp)/dH.
        IMPLICIT NONE

*     Arguments:
        REAL X,H
        
*     Definitions:
*       X      - downstream distance.
*       H      - depth of flow.
        
*     Parameters:
        REAL Zero
	PARAMETER (Zero = 0.0)

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*     Functions:
        REAL     dChannelWidth
        EXTERNAL dChannelWidth

*-- Implementation -----------------------------------------------------

*-- this equation can be used when assuming all slope is on one side.
*     dWetPerimeter = 1.0 + (1.0 + dChannelWidth(X,H)**2)**0.5
      
*-- this equation assumes symmetric slopes on both sides of the channel.      
      dWetPerimeter = 2.0 * (1.0 + (dChannelWidth(X,H)/2)**2)**0.5

      RETURN
      END



*=====Public (ManningN)========================================================

        REAL FUNCTION ManningN(X,H)

*     Purpose:   Compute effective flow-resistance coefficient.

        IMPLICIT NONE

*     Arguments:
        REAL X,H

*     Definitions:
*       X      - downstream distance.
*       H      - depth of flow.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*-- Implementation -----------------------------------------------------

      ManningN = Eta(Branch)

      RETURN
      END



*=====Public (dManningN)========================================================

        REAL FUNCTION dManningN(X,H)

*     Purpose:   Compute d(ManningN)/dH.

        IMPLICIT NONE

*     Arguments:
        REAL X,H

*     Definitions:
*       X      - downstream distance.
*       H      - depth of flow.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*-- Implementation -----------------------------------------------------

      dManningN = 0.0

      RETURN
      END



*== Public (ReadHydraulicProperties) ===================================

      LOGICAL FUNCTION ReadHydraulicProperties(ChannelNumber)

      IMPLICIT NONE

*   Purpose:  Read geometric and hydraulic properties.

*   Arguments:
      INTEGER ChannelNumber

*   Argument definitions:
*     ChannelNumber - channel sequence number.  

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*   Local variables:
      INTEGER Brn, I

*   Routines by module:

*   Programmed by: Barry Wicktom
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation ------------------------------------------------------------

 2001 FORMAT(/'             Cross-sectional properties...'/)
 2002 FORMAT('               Distance     Width   Btm_elev      dWdH'/
     #' Branch',I3/
     #' Upstream.....',4F10.2/
     #' Downstream...',4F10.2/
     #' Effective n..',F6.3/)

      ReadHydraulicProperties = .FALSE.

C     WRITE(*,*) 'BEGINNING READ...'
      
        IF(PrintOption.GT.0) THEN
          WRITE(*,*) ' '
          WRITE(*,*) ' Channel ',ChannelNumber,'...'
          WRITE(*,2001)
        END IF

          Brn = ChannelNumber
          READ(fUnit,*,END=200) I

          IF(I .NE. Brn ) THEN
            WRITE(*,*) ' Channel sequence error...'
            WRITE(*,*) ' Read number', I
            WRITE(*,*) ' Changed to ',Brn
          END IF

          READ(fUnit,*,END=200) Eta(Brn)

          READ(fUnit,'(A16)',END=200) UpStreamID(Brn)
          READ(fUnit,*,END=200)
     #    X1(Brn),Width1(Brn),Btm1(Brn),dWdH1(Brn)
          READ(fUnit,'(A16)',END=200) DownStreamID(Brn)
          READ(fUnit,*,END=200)
     #    X2(Brn),Width2(Brn),Btm2(Brn),dWdH2(Brn)

          IF(PrintOption.GT.0) THEN
            WRITE(*,2002) Brn,
     #      X1(Brn),Width1(Brn),Btm1(Brn),dWdH1(Brn),
     #      X2(Brn),Width2(Brn),Btm2(Brn),dWdH2(Brn),Eta(Brn)
          END IF

      ReadHydraulicProperties = .TRUE.

      RETURN

 200  CONTINUE

      WRITE(*,*) ' End of file...reading Channel',Brn

      RETURN
      END


*== Public (AreaWtSinuosity) ================================================

      REAL FUNCTION AreaWtSinuosity(X,H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate area-weighted sinuosity in the current channel,
*     at X downstream distance, for the area limited by the lowest point
*     in the channel and a distance H above the lowest point.
*     This routine is a stub, returning only 1.0.

*   Arguments:
      REAL X, H

*   Argument definitions:
*     X      - downstream distance within channel.
*     H      - depth of flow.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*   Local Variables:

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Barry Wicktom
*   Date:          October 1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      AreaWtSinuosity = 1.0

      RETURN
      END



*== Public ( dAreaWtSinuosity) ================================================

      REAL FUNCTION  dAreaWtSinuosity(X,H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate d(area-weighted sinuosity)/dH in the current channel,
*     at X downstream distance, for the area limited by the lowest point
*     in the channel and a distance H above the lowest point.
*     This routine is a stub, returning only 0.0.

*   Arguments:
      REAL X, H

*   Argument definitions:
*     X      - downstream distance within channel.
*     H      - depth of flow.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*   Local Variables:

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Barry Wicktom
*   Date:          October 1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

       dAreaWtSinuosity = 0.0

      RETURN
      END



*== Public ( QWtSinuosity) ================================================

      REAL FUNCTION  QWtSinuosity(X,H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate Q-weighted sinuosity in the current channel,
*     at X downstream distance, for the area limited by the lowest point
*     in the channel and a distance H above the lowest point.
*     This routine is a stub, returning only 1.0.

*   Arguments:
      REAL X, H

*   Argument definitions:
*     X      - downstream distance within channel.
*     H      - depth of flow.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*   Local Variables:

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Barry Wicktom
*   Date:          October 1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

       QWtSinuosity = 1.0

      RETURN
      END



*== Public ( dQWtSinuosity) ================================================

      REAL FUNCTION  dQWtSinuosity(X,H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate d(Q-weighted sinuosity)/dH in the current channel,
*     at X downstream distance, for the area limited by the lowest point
*     in the channel and a distance H above the lowest point.
*     This routine is a stub, returning only 0.0.

*   Arguments:
      REAL X, H

*   Argument definitions:
*     X      - downstream distance within channel.
*     H      - depth of flow.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*   Local Variables:

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Barry Wicktom
*   Date:          October 1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

       dQWtSinuosity = 0.0

      RETURN
      END



*== Public (Beta) ======================================================

      REAL FUNCTION Beta(X,H)

      IMPLICIT NONE

*   Purpose:  Return momentum coefficient, assumed equal to 1.0.

*   Arguments:
      REAL X, H

*   Argument definitions:
*     X      - downstream distance within channel.
*     H      - depth of flow.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*   Local variables:

*   Routines by module:

*   Programmed by: Barry Wicktom
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation ------------------------------------------------------------

      Beta = 1.0

      RETURN
      END


*== Public (dBeta) =====================================================

      REAL FUNCTION dBeta(X,H)

      IMPLICIT NONE

*   Purpose:  Return d(Beta)/dX, assumed equal to 0.0.

*   Arguments:
      REAL X, H

*   Argument definitions:
*     X      - downstream distance within channel.
*     H      - depth of flow.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*   Local variables:

*   Routines by module:

*   Programmed by: Barry Wicktom
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation ------------------------------------------------------------

      dBeta = 0.0

      RETURN
      END

*== Public (UserStreamLocations) =======================================

      SUBROUTINE UserStreamLocations(
     I                                NumberOfCoordinates,
     O                                ID, Coordinate
     #                               )

      IMPLICIT NONE

*   Purpose:  Return coordinates of user-supplied location IDs
*             and locations in current channel.

*   Arguments:
      INTEGER NumberOfCoordinates
      REAL    Coordinate(NumberOfCoordinates)
      CHARACTER*16 ID(NumberOfCoordinates)

*   Argument definitions:
*     NumberOfCoordinates - dimension of following array.
*     Coordinates(i) - downstream distances of locations in current channel.
*     ID(i) - location identifiers.
*     i - location index, 1 to NumberOfLocations, increasing downstream.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*   Local Variables:

*   Routines by module:

***** Channel schematic:
      EXTERNAL GetUserStreamLocationIDs
      
***** String utilities:
      CHARACTER*16 LeftTrim
      EXTERNAL     LeftTrim

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      CALL GetUserStreamLocationIDs(NumberOfCoordinates, ID)

      IF( Branch .GT. 0 ) THEN
        IF( ID(1) .EQ. UpStreamID(Branch) ) THEN
          Coordinate(1) = X1(Branch)
C         WRITE(*,*) '  Upstream coordinate...',X1(Branch)
        ELSE IF(
     #    LeftTrim(ID(1)) .EQ. LeftTrim(UpStreamID(Branch))
     #         ) THEN
          Coordinate(1) = X1(Branch)
        ELSE
          WRITE(*,*) ' ***ERROR (UserStreamLocations)...'
          WRITE(*,*) ' ID mismatch...upstream...channel ',Branch
          WRITE(*,'(A16,2X,A16)') ID(1), UpStreamID(Branch)
        END IF

        IF( ID(2) .EQ. DownStreamID(Branch) ) THEN
          Coordinate(2) = X2(Branch)
C         WRITE(*,*) '  Downstream coordinate...',X2(Branch)
        ELSE IF(
     #    LeftTrim(ID(2)) .EQ. LeftTrim(DownStreamID(Branch))
     #         ) THEN
          Coordinate(2) = X2(Branch)
        ELSE
          WRITE(*,*) ' ***ERROR (UserStreamLocations)...'
          WRITE(*,*) ' ID mismatch...downstream...channel ',Branch
          WRITE(*,'(A16,2X,A16)') ID(2), DownStreamID(Branch)
        END IF

      ELSE
        WRITE(*,*) ' CurrentChannel not set (UserStreamLocations)...'
        STOP
      END IF


      RETURN
      END


*== Public (UserStreamDistance) ========================================

      REAL FUNCTION UserStreamDistance(LocationNumber)

      IMPLICIT NONE

*   Purpose:  Return the downstream distance of the user-supplied
*             location corresponding to the "LocationNumber"
*             location counted from the upstream end of the current
*             channel.

*   Arguments:
      INTEGER LocationNumber

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*   Local variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      IF( LocationNumber .EQ. Branch*2 ) THEN

*------ Requested location is at downstream end of current branch.
        UserStreamDistance = X2(Branch)

      ELSE IF( LocationNumber .EQ. Branch*2-1 ) THEN

*------ Requested location is at upstream end of current branch.
        UserStreamDistance = X1(Branch)

      ELSE

*------ Requested location does not fall within current branch,
*       return upstream end with warning.

        WRITE(*,*) ' Requested user-supplied location does not'
        WRITE(*,*) ' fall within current branch...(',Branch,')'
        WRITE(*,*) ' (UserStreamDistance)'
        WRITE(*,*) ' Upstream most location returned...'
        UserStreamDistance = X1(Branch)

      END IF

      RETURN
      END

*== Public (CxCentroid) ====================================================

      REAL FUNCTION CxCentroid(X,H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate distance from the water surface to the cross-section
*     centroid, in the current channel, at X downstream distance,
*     limited by the lowest point in the channel and a
*     distance H above the lowest point.

*   Arguments:
      REAL X, H

*   Argument definitions:
*     X      - downstream distance.
*     H      - depth of flow.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*   Local variables:
      REAL Small
      PARAMETER (Small = 1.00E-6)

*   Routines by module:

***** Local:
      REAL ChannelWidth, dChannelWidth, CxArea
      EXTERNAL ChannelWidth, dChannelWidth, CxArea

*   Programmed by: Barry Wicktom
*   Date:          November 1990
*   Modified by:   Barry Wicktom
*   Last modified: Dec 1992
*


*-- Implementation ------------------------------------------------------------
      
      IF (CxArea(X,H).GT.Small) THEN
      CxCentroid = (((H**2)/6) * (3 * ChannelWidth(X,H)
     #            - 2 * H * dChannelWidth(X,H)))
     #            / CxArea(X,H)
      ELSE
       CxCentroid = 0.0
      ENDIF

      RETURN
      END



*== Public (dCxCentroid) ====================================================

      REAL FUNCTION dCxCentroid(X,H)

      IMPLICIT NONE

*   Purpose:  
*     Estimate d(CxCentroid)/dH, in the current channel,
*     at X downstream distance, limited by the lowest point
*     in the channel and a distance H above the lowest point.

*   Arguments:
      REAL X, H

*   Argument definitions:
*     X      - downstream distance.
*     H      - depth of flow.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcxtrap.inc'

*   Local variables:
      REAL Small
      PARAMETER (Small = 1.00E-6)

*   Routines by module:

***** Local:
      REAL ChannelWidth, dChannelWidth, CxCentroid, CxArea 
      EXTERNAL ChannelWidth, dChannelWidth, CxCentroid, CxArea 

*   Programmed by: Barry Wicktom
*   Date:          November 1990
*   Modified by:   Barry Wicktom 
*   Last modified: Dec 1992
*


*-- Implementation ------------------------------------------------------------
     
      IF (CxArea(X,H).GT.Small) THEN
      dCxCentroid = (H * (ChannelWidth(X,H) - (H/2)*dChannelWidth(X,H)) 
     #            - CxCentroid(X,H) * ChannelWidth(X,H)) /
     #            CxArea(X,H)
      ELSE
       dCxCentroid = 0.0
      ENDIF

      RETURN
      END

      LOGICAL FUNCTION WriteInterveningProperties()
      WriteInterveningProperties = .TRUE.
      RETURN
      END


***** EOF TRAPGEOM *****************************************************
