*==== BOF chschmt ======================================================



*****************************************************************************
*
*     This file is a FORTRAN module containing schematic data associated with
*     networks of 1-D channels.
*
*
*
*     Module note: Only functions or subroutines marked "public" should be
*                   used outside of this module as those marked "private"
*                   may not be supported by future revisions of this module
*                   or replacement modules.  Likewise, no data or common
*                   blocks contained within this module should be accessed
*                   by routines outside of this module accept through the
*                   use of "public" functions or subroutines contained
*                   within this module.
*
*     Non-standard usage: Symbolic names in this module may be represented by
*                         as many as 31 characters in order to provide better
*                         definition directly in the code.  Standard FORTRAN
*                         allows only 6 characters, but this restriction is
*                         generally extended to 32 characters by most compilers.
*
*
*
*   Public functions:
*     INTEGER FUNCTION NumberOfChannels()
*            - return number of channels in the network.
*
*     INTEGER FUNCTION CurrentChannel()
*            - return the current channel number.
*
*     LOGICAL FUNCTION OpenChannel(ChannelNumber)
*            - open a channel making it the current channel.
*
*     LOGICAL FUNCTION CloseChannel()
*            - close the current channel.
*
*     INTEGER FUNCTION NumberOfUserStreamLocations()
*            - return number of user-supplied locations in current channel.
*
*     LOGICAL FUNCTION InitializeChannelNetwork()
*            - initialize a network of open channels.
*
*     INTEGER FUNCTION NumberOfStreamLocations()
*            - return number of computational locations in current channel.
*
*     INTEGER FUNCTION UpstreamPointer()
*            - return global sequence number for first computational
*              location in current channel.
*
*     INTEGER FUNCTION DownstreamPointer()
*            - return global sequence number for last computational
*              location in current channel.
*
*     REAL FUNCTION StreamDistance(LocationNumber)
*            - return downstream distance of a computational location.
*
*     INTEGER FUNCTION UpstreamCode()
*            - return upstream boundary condition code.
*
*     INTEGER FUNCTION DownstreamCode()
*            - return downstream boundary condition code.
*
*     INTEGER FUNCTION UpstreamConnections()
*            - return number of upstream connections.
*
*     INTEGER FUNCTION DownstreamConnections()
*            - return number of downstream connections.
*
*     INTEGER FUNCTION UpstreamConnect1()
*            - return channel number of first upstream connection.
*
*     INTEGER FUNCTION UpstreamConnect2()
*            - return channel number of second upstream connection.
*
*     INTEGER FUNCTION UpstreamConnect3()
*            - return channel number of third upstream connection.
*
*     INTEGER FUNCTION DownstreamConnect1()
*            - return channel number of first downstream connection.
*
*     INTEGER FUNCTION DownstreamConnect2()
*            - return channel number of second downstream connection.
*
*     INTEGER FUNCTION DownstreamConnect3()
*            - return channel number of third downstream connection.
*
*   Public Subroutines:
*
*     SUBROUTINE GetUserStreamLocationIDs(in: Num, out: CurrentUserIDs)
*            - returns an array of user location IDs for current channel.
*
*     SUBROUTINE GetUserStreamFlow(in: Num, out: StreamFlow)
*            - return an array of streamflow values for user locations
*              in current channel.
*
*     SUBROUTINE GetUserStreamSurfaceElevation(in: Num, out: StreamElevation)
*            - return an array of watresurface elevations at user locations
*              in current channel.
*
*   Arguments:
*     ChannelNumber - channel sequence number.
*     LocationNumber - sequential location number within current channel,
*                      1 beginning at upstream most location.
*     Num - an integer count of the elements in an array.
*
*         corresponding to user-supplied locations, arrays of user-supplied ...
*     CurrentUserIDs - identifiers.
*     StreamFlow - discharges.
*     StreamElevation - watersurface elevations.
*
*
*   Module data:
*
*    'network.inc'
*     MaxChannels - maximum number of channels.
*     NumCh - current number of channels.
*     Branch - current selected or active channel.
*     MaxLocations - maximum number of locations (computational or user).
*
*    'chnluser.inc'
*     NumUserLoc - current total number of user locations.
*     UpUserPointer(i) - pointer to upstream most user-supplied
*                        location for channel "i".
*     DownUserPointer(i) - pointer to downstream most user-supplied
*                          location for channel "i".
*     NumberOfUserLocations(i) - number of user-supplied locations
*                              for channel "i".
*     UserWS(i) - water surface elevation at user location "i".
*     UserQ(i) - volumetric discharge at user location "i".
*     UserH(i) - depth of flow at user location "i".
*
*    'chconnec.inc'
*        for the current channel i ...
*     UpBoundaryCode(i) - upstream boundary-condition code.
*     UpNumberOfConnections(i) - number of channels connected to upstream end.
*     UpConnection((i-1)*J+k) - number of kth channel connected to
*                               upstream end,
*                               where j = MaxConnectingChannels.
*     DownBoundaryCode(i) - downstream boundary-condition code.
*                           (See codes below.)
*     DownNumberOfConnections(i) - number of channels connected to
*                                  downstream end.
*     DownConnection((i-1)*J+k) - number of kth channel connected to
*                                 downstream end,
*                                 where j = MaxConnectingChannels.
*     InitialApprox(i) - initial approximation index, indicating approximation
*                        of intial conditions from:
*              [0] user input (schematic data file),
*              [1] normal depth,
*              [2] steady state,
*              [3] normal depth w/adverse slopes removed,
*              [4] maximum ws elevation for branch,
*              [5] model dependent file, or
*              [6] no approximation attempted.
*
*      AdditionalVariables(i) - number of variables in addition to watersurface
*                               elevation and discharge (not currently used).
*      dX(i) - desired spacing of computational locations.
*      KeepAllCx(i) - index indicating,
*               [.TRUE.] - keep all user-supplied locations as computational locations.
*               [.FALSE.] - do not keep intermediate user-supplied locations as
*                           computational locations.
*
*
*     'chnlcomp.inc'
*           for the current channel, i,
*     NumberOfCompLocations(i) - number of computational locations.
*     UpCompPointer(i) - global sequence number of most upstream location.
*     DownCompPointer(i) - global sequence number of most downstream location.
*            for the global sequence number, j,
*     CompLocation(j) - downstream distance coordinate.
*     DummyArray(j) - a REAL value dependent upon context.
*     DummyArray2(j) - a REAL value dependent upon context.
*     DummyCharArray(j) - a CHARACTER value dependent upon context.
*
*
************************************************************************



*== Public (NumberOfChannels) ==========================================

      INTEGER FUNCTION NumberOfChannels()

      IMPLICIT NONE

*   Purpose:  Return number of channels in current channel network.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'

*   Local Variables:

*   Routines by modules:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation ---------------------------------------------------

      IF( NumCh .GT. 0 ) THEN
        NumberOfChannels = NumCh
      ELSE
        WRITE(*,*) ' Network not initialized (NumberOfChannels)...'
        STOP
      END IF

      RETURN
      END



*== Public (CurrentChannel) ============================================

      INTEGER FUNCTION CurrentChannel()

      IMPLICIT NONE

*   Purpose:  Return current channel number.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation ---------------------------------------------------

      IF( Branch .GT. 0 ) THEN
        CurrentChannel = Branch
      ELSE
        WRITE(*,*) ' CurrentChannel not set (CurrentChannel)...'
        STOP
      END IF

      RETURN
      END


*== Public (OpenChannel) ===============================================

      LOGICAL FUNCTION OpenChannel(ChannelNumber)

      IMPLICIT NONE

*   Purpose:  Set current channel number.

*   Arguments:
      INTEGER ChannelNumber

*   Argument definitions:
*      ChannelNumber - channel sequence number. 

*   Module data:
      INCLUDE 'network.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation ---------------------------------------------------

      IF( Branch .EQ. 0 ) THEN

        IF( ChannelNumber .LE. NumCh .AND. ChannelNumber .GT. 0 ) THEN
          OpenChannel = .TRUE.
          Branch = ChannelNumber
        ELSE
          OpenChannel = .FALSE.
          WRITE(*,*) ' Can not open channel', ChannelNumber
          WRITE(*,*) ' NumberOfChannels = ', NumCh
        END IF

      ELSE
        WRITE(*,*) ' *** Error (OpenChannel)'
        WRITE(*,*) ' Channel ', Branch, ' still open ???'
        OpenChannel = .FALSE.
      END IF

      RETURN
      END



*== Public (CloseChannel) ==============================================

      LOGICAL FUNCTION CloseChannel()

      IMPLICIT NONE

*   Purpose:  Close current channel, set CurrentChannel = 0.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation ---------------------------------------------------

      IF( Branch .GT. 0 ) THEN
        CloseChannel = .TRUE.
        Branch = 0
      ELSE
        CloseChannel = .FALSE.
        Branch = 0
      END IF

      RETURN
      END



*== Public (NumberOfUserStreamLocations) ===============================

      INTEGER FUNCTION NumberOfUserStreamLocations()

      IMPLICIT NONE

*   Purpose:  Return number of user-supplied stream locations.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chconnec.inc'
      INCLUDE 'chnluser.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      IF( Branch .GT. 0 ) THEN
        NumberOfUserStreamLocations = NumberOfUserLocations(Branch)
      ELSE
        WRITE(*,*)
     #       ' CurrentChannel not set (NumberOfUserStreamLocations)...'
      STOP
      END IF

      RETURN
      END



*== Public (InitializeChannelNetwork) ==================================

      LOGICAL FUNCTION InitializeChannelNetwork()

      IMPLICIT NONE

*   Purpose:  Initialize a network of channels.

*   Arguments:

*   Argument definitions:

*   Module data:

*   Local Variables:
      LOGICAL OK

*   Routines by module:

***** Network control:
      LOGICAL  VariableStreamDensity, VariableStreamSinuosity
      INTEGER  Terms1D
      EXTERNAL VariableStreamDensity, VariableStreamSinuosity, Terms1D

***** Channel flow status:
      LOGICAL  InitializeNetworkFlowValues, InitializeNetworkDensity
      LOGICAL  SetConstantStreamDensity
      EXTERNAL InitializeNetworkFlowValues, InitializeNetworkDensity
      EXTERNAL SetConstantStreamDensity

***** Channel properties:
      LOGICAL  InitializeChannelProperties
      EXTERNAL InitializeChannelProperties
      LOGICAL  WriteInterveningProperties
      EXTERNAL WriteInterveningProperties

***** Locals:
      LOGICAL  InitializeNetworkSchematic, SetCompLocations
      EXTERNAL InitializeNetworkSchematic, SetCompLocations

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      InitializeChannelNetwork = .FALSE.

*---- Read network schematic data.
      IF( InitializeNetworkSchematic() ) THEN

*------ Initialize geometric and hydraulic properties of the channels.
        IF( InitializeChannelProperties() ) THEN

*-------- Determine computational locations and set channel pointers.
          IF( SetCompLocations() ) THEN
          
*---------- Write channel properties for computational locations...
            OK = WriteInterveningProperties()

*---------- Set initial channel flow values.
            IF( InitializeNetworkFlowValues() ) THEN

            ELSE
              WRITE(*,*) ' Attempt to set initial flow channel',
     #                   ' values failed...'
              RETURN
            END IF

*---------- Set initial water density.

            IF( VariableStreamDensity() ) THEN
            
              IF( InitializeNetworkDensity() ) THEN
              ELSE
                
              END IF
            
            
            ELSE IF( Terms1D() .EQ. 1
     #                         .AND.
     #               VariableStreamSinuosity() ) THEN
              OK = SetConstantStreamDensity()
            
            END IF




          ELSE
            WRITE(*,*)
     #      ' Attempt to set computational locations failed...'
            RETURN
          END IF
        ELSE
          WRITE(*,*) ' Initialization of channel properties failed...'
          RETURN
        END IF
      ELSE
        WRITE(*,*) ' Attempt to read network schematic data failed...'
        RETURN
      END IF

      InitializeChannelNetwork = .TRUE.

      RETURN
      END



*== Public (InitializeNetworkSchematic) ================================

      LOGICAL FUNCTION InitializeNetworkSchematic()

      IMPLICIT NONE

*   Purpose:  Read schematic data for a network of channels.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chnluser.inc'
      INCLUDE 'chconnec.inc'

*   Local Variables:
      INTEGER Unit, M
      CHARACTER*12 FileName, InternalFileName

*   Routines by module:

***** Master file:
      INTEGER GetFileUnit
      CHARACTER*12 GetFileName
      EXTERNAL GetFileName, GetFileUnit

***** File utilities:
      LOGICAL  OpenOldText
      EXTERNAL OpenOldText

***** Locals:
      LOGICAL  ReadChannelCxSequence
      EXTERNAL ReadChannelCxSequence

      LOGICAL  ReadChannelConnections
      EXTERNAL ReadChannelConnections
      
***** Network control:
      INTEGER  NetworkPrintLevel
      EXTERNAL NetworkPrintLevel
      
***** Channel constraints:
      LOGICAL  InitStreamConstraint
      EXTERNAL InitStreamConstraint

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      InitializeNetworkSchematic = .FALSE.
      
*                         123456789012
      InternalFileName = 'schemat.dat '
      
      Unit = GetFileUnit(InternalFileName)
      FileName = GetFileName(InternalFileName)

*---- Open schematic data file for reading.
      IF( OpenOldText( Unit, FileName ) ) THEN
        IF( NetworkPrintLevel() .GT. 0 ) THEN
          WRITE(*,*) ' Reading schematic data...'  
        END IF

*------ Begin channel loop, jump out when EOF is encountered.
        NumCh = 0
        NumUserLoc = 0
        DO 100 M = 1, MaxChannels


*-------- Read user-supplied channel-cross-section sequence
*         and approximate initial-conditions.
          IF(ReadChannelCxSequence(Unit)) THEN

*---------- Read channel connections.
            IF( ReadChannelConnections(Unit) ) THEN

            ELSE
              WRITE(*,*) ' Attempt to read Channel connections failed',
     #                 '   (InitializeNetworkSchematic)...'
              RETURN
            END IF
          ELSE

*---------- EOF encountered routinely, continue...
            GO TO 110

          END IF

 100    CONTINUE

*------ Report attemt to exceed maximum channels.
        WRITE(*,*) ' Warning, reading of schematic channel data stopped'
        WRITE(*,*) ' at maximum number of channels...',MaxChannels

 110    CONTINUE


      ELSE
        WRITE(*,*)
     #  ' An error occurred attemping to open the schematic data file.'
        WRITE(*,*) ' File name : ', FileName
        RETURN
      END IF

      CLOSE(Unit)
      Branch = 0
      
      
      IF( InitStreamConstraint() ) THEN
      ELSE
        WRITE(*,*)' Attempt to initialize network constraints failed...'
        RETURN
      END IF
      

      InitializeNetworkSchematic = .TRUE.

      RETURN
      END



*== Private (ReadChannelCxSequence) ====================================

      LOGICAL FUNCTION ReadChannelCxSequence(Unit)

      IMPLICIT NONE

*   Purpose:  Read user-supplied schematic data
*             and approximate initial conditions.

*   Arguments:
      INTEGER Unit

*   Argument definitions:
*     Unit - FORTRAN unit number for input.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chnluser.inc'
      INCLUDE 'chconnec.inc'

*   Local Variables:
      INTEGER BRN, M, N, I, CxNum
      INTEGER Keep, Approx
      LOGICAL Out

*   Routines by module:

***** Network control:
      INTEGER  NetworkPrintLevel
      EXTERNAL NetworkPrintLevel

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      ReadChannelCxSequence = .FALSE.

*------ Read channel number, number of user-defined cross sections,
*       desired spacing for computational cross sections,
*       insertion index, initial-condition approximation index, and
*       the number of initial-value types in addition to discharge and
*       water-surface elevation (perhaps density or constituent
*       concentrations to be simulated).

      N = Unit

*---- Set output index.

      IF( NetworkPrintLevel() .GT. 1 ) THEN
        Out = .TRUE.
      ELSE
        Out = .FALSE.
      END IF

*---- Check for out-of-range on number of channels.
      IF( NumCh .LT. MaxChannels ) THEN
        M = NumCh + 1
        Keep  = 1
        InitialApprox(M) = 0
        AdditionalVariables(M) = 0

*------ Read channel number, jump out if EOF.
        READ(N,*, END=200 ) BRN
        IF(BRN.NE.M) THEN
          WRITE(*,*) ' ****Channel-number-sequence error...'
          WRITE(*,*) BRN,' changed to ',M,'...'
        END IF
        READ(N,*) NumberOfUserLocations(M),dX(M),Keep,InitialApprox(M),
     #            AdditionalVariables(M)
        IF( Out ) THEN
          WRITE(*,*) ' '
          WRITE(*,*) ' Channel ',M,'...'
          WRITE(*,*) '  ',NumberOfUserLocations(M),
     #                    ' stream locations...'
          WRITE(*,*) '   dX = ',dX(M)
        END IF
        IF(Keep.EQ.0) THEN
          KeepAllCx(M) = .FALSE.

          IF( Out ) THEN
            WRITE(*,*) '   Intermediate user locations will not be ',
     #                     'included as computational locations...'
          END IF

        ELSE
          KeepAllCx(M) = .TRUE.

          IF( Out ) THEN
            WRITE(*,*) '   All user locations will be included as',
     #               ' computational locations...'
          END IF
        END IF

        Approx = InitialApprox(M)

        IF(Approx .EQ. 0) THEN
          IF( Out ) THEN
            WRITE(*,*) '   Initial conditions are to be approximated ',
     #               'from user input...'
          END IF
        ELSE IF(Approx .EQ. 1) THEN
          IF( Out ) THEN
            WRITE(*,*) '   Initial ws_elev is to be approximated',
     #                 ' by normal depth...'
          END IF
        ELSE IF(Approx .EQ. 2) THEN
          IF( Out ) THEN
            WRITE(*,*) '   Initial ws_elev is to be computed',
     #                  ' assuming steady state...'
            WRITE(*,*) '    (not implemented)...'
          END IF
        ELSE IF(Approx .EQ. 3) THEN
          IF( Out ) THEN
            WRITE(*,*) '  Initial ws_elev is to be approximated',
     #                 ' by normal depth...'
            WRITE(*,*) '   Adverse slopes are to be flattened...'
          END IF
        ELSE IF(Approx .EQ. 4) THEN
          IF( Out ) THEN
            WRITE(*,*) '   Initial ws_elev is to be approximated',
     #                 ' equal to maximum for channel...'
          END IF
        ELSE IF(Approx .EQ. 5) THEN
          IF( Out ) THEN
            WRITE(*,*) '   Initial conditions are to be approximated ',
     #               'from model-dependent file...'
          END IF
        ELSE IF(Approx .EQ. 6) THEN
          IF( Out ) THEN
           WRITE(*,*) '   Initial conditions are',
     #              'not to be approximated... '
          END IF
          
        ELSE IF( Approx .EQ. 7 ) THEN
          IF( Out ) THEN
           WRITE(*,*) '   Initial conditions are to be approximated',
     #                ' assuming steady state...'
          END IF
        ELSE
          IF( Out ) THEN
            WRITE(*,*) '   Initial-condition index ',Approx,
     #               ' not supported...'
          END IF
        END IF

*------ Begin cross-section loop, stop if EOF.
        UpUserPointer(M) = NumUserLoc + 1
        DO 100 I = 1, NumberOfUserLocations(M)
          NumUserLoc = NumUserLoc + 1
          CxNum = NumUserLoc

*-------- Read cross-section ID.
          READ(N,'(A16)', END=150) UserLocationID(CxNum)

*-------- Read user cross-section number, reference distance,
*         initial ws_elev, and initial flow.
          UserWS(CxNum) = 0.0
          UserQ(CxNum)  = 0.0
          UserH(CxNum)  = 1.0E-08
          READ(N,*, END=150)  UserWS(CxNum), UserQ(CxNum), UserH(CxNum)

 100    CONTINUE
        DownUserPointer(M) = CxNum

        NumCh = NumCh + 1
        Branch = NumCh

        ReadChannelCxSequence = .TRUE.

        RETURN

      ELSE
        WRITE(*,*) ' Too many channels for dimension...',
     #  '(ReadChannelCxSequence)'
        WRITE(*,*) ' Attempt to read ',NumCh+1
        WRITE(*,*) ' Dimensioned for',MaxChannels
        STOP
      END IF

 150  CONTINUE
        WRITE(*,*) ' End of file encountered prematurely...',
     #             ' (ReadChannelCxSequence)'
        WRITE(*,*) ' Channel',M,'  location',I
        STOP
 200  CONTINUE

      RETURN
      END



*== Private (ReadChannelConnections) ===================================

      LOGICAL FUNCTION ReadChannelConnections(Unit)

      IMPLICIT NONE

*   Purpose:  Read user-supplied schematic data
*             and approximate initial conditions.

*   Arguments:
      INTEGER Unit

*   Argument definitions:
*     Unit - FORTRAN unit number for input.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chnluser.inc'
      INCLUDE 'chconnec.inc'

*   Local Variables:
      INTEGER I, J, K, Connections
      LOGICAL Out

*   Routines by module:

***** Network control:
      INTEGER  NetworkPrintLevel
      EXTERNAL NetworkPrintLevel

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      ReadChannelConnections = .FALSE.

*---- Set output index.

      IF( NetworkPrintLevel() .GT. 1 ) THEN
        Out = .TRUE.
      ELSE
        Out = .FALSE.
      END IF

*---- Read connection and condition codes.
             I = Branch
             K = (I - 1) * MaxConnectingChannels

*-------- Upstream...

             READ(Unit,*,END=999) UpBoundaryCode(I)
             READ(Unit,*,END=999) Connections
             IF( Out ) THEN
               WRITE(*,*) '   Upstream condition code = ',
     #                 UpBoundaryCode(I),
     #              ' with',Connections,' connection(s) ...'
             END IF
             UpNumberOfConnections(I) = Connections
             IF(Connections .EQ. 0) THEN
             ELSE IF(Connections .LE. MaxConnectingChannels) THEN

               DO 100 J=1,Connections
                 READ(Unit,*,END=999) UpConnection(K+J)
                 IF( Out ) THEN
                   WRITE(*,*) '    ...  ',UpConnection(K+J)
                 END IF
 100           CONTINUE

             ELSE
               WRITE(*,*)
     #         ' Number of upstream connections (',Connections,'),'
               WRITE(*,*) ' for channel',I
               WRITE(*,*) ' are <0 or >',MaxConnectingChannels,' .'
               STOP
             END IF


*-------- Downstream...

             READ(Unit,*,END=999) DownBoundaryCode(I)
             READ(Unit,*,END=999) Connections
             IF( Out ) THEN
               WRITE(*,*) '   Downstream condition code = ',
     #                 DownBoundaryCode(I),
     #              ' with',Connections,' connection(s) ...'
             END IF
             DownNumberOfConnections(I) = Connections
             IF(Connections .EQ. 0) THEN
             ELSE IF(Connections .LE. MaxConnectingChannels) THEN
               DO 150 J=1,Connections
                 READ(Unit,*,END=999) DownConnection(K+J)
                 IF( Out ) THEN
                   WRITE(*,*) '    ...  ',DownConnection(K+J)
                 END IF
 150           CONTINUE
             ELSE
               WRITE(*,*)
     #         ' Number of downstream connections (',Connections,'),'
               WRITE(*,*) ' for channel',I
               WRITE(*,*) ' are <0 or >',MaxConnectingChannels,' .'
               STOP
             END IF
  220      CONTINUE

      IF( Out ) THEN
        WRITE(*,*) ' '
      END IF

      ReadChannelConnections = .TRUE.

      RETURN

 999  CONTINUE
      WRITE(*,*) ' Reached end of channel schematic data file too soon!'

      RETURN
      END



*== Public (SetCompLocations) ==========================================

      LOGICAL FUNCTION SetCompLocations()

      IMPLICIT NONE

*   Purpose:

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chnluser.inc'
      INCLUDE 'chnlcomp.inc'
      INCLUDE 'chconnec.inc'

*   Local Variables:
      INTEGER I, ChNum, MaxCx, ModelIndex
      LOGICAL OK, Out

*   Routines by module:

***** Channel properties:
      EXTERNAL UserStreamLocations

***** Linear interpolation utilities:
      LOGICAL  XINSRT
      EXTERNAL XINSRT

***** Network control:
      LOGICAL  SetNetworkTimeSeriesLocations
      EXTERNAL SetNetworkTimeSeriesLocations

      INTEGER  NetworkPrintLevel
      EXTERNAL NetworkPrintLevel

***** Local:
      LOGICAL  OpenChannel, CloseChannel
      EXTERNAL OpenChannel, CloseChannel

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      SetCompLocations = .FALSE.
      IF( NetworkPrintLevel() .GT. 0) THEN
        WRITE(*,*) ' Setting computational locations...'
      END IF

*---- Set output index.

      IF( NetworkPrintLevel() .GT. 1 ) THEN
        Out = .TRUE.
      ELSE
        Out = .FALSE.
      END IF

      ModelIndex = 1
      MaxCx = MaxLocations
      TotalCompLocations = 0

*---- Begin channel loop.
      DO 100 I = 1, NumCh
        ChNum = I

        IF( OpenChannel( ChNum ) ) THEN

*-------- Load user downstream distances in DummyArray.
          CALL UserStreamLocations(
     I          MaxCx,
     O                DummyCharArray, DummyArray
     #                            )


*-------- Insert computational locations if needed.
          IF( ChNum .GT. 1 ) THEN
            UpCompPointer(ChNum) = DownCompPointer(ChNum-1) + 1
          ELSE
            UpCompPointer(ChNum) = 1
          END IF
          IF( XINSRT(
     I      MaxCx, NumberOfUserLocations(ChNum),
     I      DummyArray,
     I      dX(ChNum), KeepAllCx(ChNum), ModelIndex,
     O      NumberOfCompLocations(ChNum),
     O      CompLocation( UpCompPointer(ChNum) )
     #            )  )  THEN

            IF( Out ) THEN
              WRITE(*,*) '  Channel number ',ChNum,' ...',
     #              NumberOfCompLocations(ChNum)
            END IF

            TotalCompLocations = TotalCompLocations
     #                            + NumberOfCompLocations(ChNum)

          ELSE
            WRITE(*,*) ' ####error(SetCompLocations)'
            WRITE(*,*) ' Maximum number of computational',
     #                 'locations eceeded...'
            WRITE(*,*) ' Insertion of computational locations failed...'
            WRITE(*,*) ' Attempting channel...',ChNum
            RETURN
          END IF

          DownCompPointer(ChNum) = UpCompPointer(ChNum)
     #            + NumberOfCompLocations(ChNum) - 1


*-------- Reduce maximum number of remaining locations.
          MaxCx = MaxCx - NumberOfCompLocations(ChNum)
          IF( MaxCx .LT. 0 ) THEN
            WRITE(*,*) ' Max number of computational locations',
     #               ' exceeded...(SetCompLocations)'
            WRITE(*,*) ' Channel number...',ChNum
          END IF

          OK = CloseChannel()

        ELSE

          WRITE(*,*) ' Attempt to open channel',ChNum,' failed...'
          WRITE(*,*) ' (SetCompLocations)'
          RETURN

        END IF

 100  CONTINUE


*---- Determine global, computational-cross-section number
*       of locations at which output of time-series results
*       is required.
      OK = SetNetworkTimeSeriesLocations()


      SetCompLocations = .TRUE.

      RETURN
      END



*== Public (NumberOfStreamLocations) ===================================

      INTEGER FUNCTION NumberOfStreamLocations()

      IMPLICIT NONE

*   Purpose:  Return the number of computational locations in the
*             current channel.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chnlcomp.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      NumberOfStreamLocations = NumberOfCompLocations(Branch)

      RETURN
      END



*== Public (TotalStreamLocations) ======================================

      INTEGER FUNCTION TotalStreamLocations()

      IMPLICIT NONE

*   Purpose:  Return total number of computational locations in network.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chnlcomp.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      TotalStreamLocations = TotalCompLocations

      RETURN
      END



*== Public (StreamEndNode) =============================================

      INTEGER FUNCTION StreamEndNode( ChannelNumber )

      IMPLICIT NONE

*   Purpose:  Return a global location number delimiting the channel,
*             ChannelNumber.  The location number will be for the
*             upstream end of the channel if ChannelNumber is
*             positive and downstream if negative.  The returned
*             location number will carry the same sign as ChannelNumber.

*   Arguments:
      INTEGER ChannelNumber

*   Argument definitions:
*      ChannelNumber - channel sequence number. 

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chnlcomp.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      IF( ChannelNumber .GT. 0 ) THEN

*------ Upstream end of channel.

        StreamEndNode = UpCompPointer( ChannelNumber )

      ELSE IF( ChannelNumber .LT. 0 ) THEN

*------ Downstream end of channel.

        StreamEndNode = - DownCompPointer(  - ChannelNumber )

      ELSE

*------ No channel number.

        StreamEndNode = 0

      END IF

      RETURN
      END



*== Public (UpstreamPointer) ===========================================

      INTEGER FUNCTION UpstreamPointer()

      IMPLICIT NONE

*   Purpose:  Return pointer to upstream computational location number
*             for current channel.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chnlcomp.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      UpstreamPointer = UpCompPointer(Branch)

      RETURN
      END



*== Public (DownstreamPointer) =========================================

      INTEGER FUNCTION DownstreamPointer()

      IMPLICIT NONE

*   Purpose:  Return pointer to downstream computational location number
*             for current channel.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chnlcomp.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      DownstreamPointer = DownCompPointer(Branch)

      RETURN
      END



*== Public (StreamDistance) ============================================

      REAL FUNCTION StreamDistance(LocationNumber)

      IMPLICIT NONE

*   Purpose:  Return downstream distance to the computational 
*             LocationNumber, within the current channel.

*   Arguments:
      INTEGER LocationNumber

*   Argument definitions:
*     LocationNumber - computational location number, begining with 1
*                       at upstream end of current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chnlcomp.inc'

*   Local Variables:

*   Routines by module:

***** Locals:
      LOGICAL  CheckChannelCompLocationRange
      EXTERNAL CheckChannelCompLocationRange

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      IF( CheckChannelCompLocationRange(LocationNumber) ) THEN

        StreamDistance = CompLocation(
     #            UpCompPointer(Branch) + LocationNumber - 1
     #                             )

      ELSE

        WRITE(*,*) ' Failed to determine correct stream distance...'
        WRITE(*,*) ' Incorrectly returned last location in channel...'
        WRITE(*,*) ' (StreamDistance)'
        StreamDistance = CompLocation(
     #                     DownCompPointer(Branch)
     #                               )

      END IF

      RETURN
      END



*== Public (GlobalStreamDistance) ======================================

      REAL FUNCTION GlobalStreamDistance(GlobalLocationNumber)

      IMPLICIT NONE

*   Purpose:  Return downstream distance to the GlobalLocationNumber.

*   Arguments:
      INTEGER GlobalLocationNumber

*   Argument definitions:
*     GlobalLocationNumber - global location number, sequential
*                             with out regard to channel number.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chnlcomp.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          March 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      IF( GlobalLocationNumber .GT. 0
     #                         .AND.
     #    GlobalLocationNumber .LE. MaxLocations ) THEN
        GlobalStreamDistance = CompLocation( GlobalLocationNumber )
      ELSE
        WRITE(*,*) ' ####error( GlobalStreamDistance )'
        WRITE(*,*) ' Global location number out of range...'
        GlobalStreamDistance = 0.0
      END IF
      RETURN
      END



*== Private (CheckChannelCompLocationRange) ============================

      LOGICAL FUNCTION CheckChannelCompLocationRange(LocationNumber)

      IMPLICIT NONE

*   Purpose:  Check to see if LocationNumber is in the range of
*             location numbers available to the current channel.

*   Arguments:
      INTEGER LocationNumber

*   Argument definitions:
*     LocationNumber - computational-location number (index) within
*                      current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chnlcomp.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      IF( LocationNumber .LE. NumberOfCompLocations(Branch)
     #                   .AND.
     #    LocationNumber .GT. 0 ) THEN

        CheckChannelCompLocationRange = .TRUE.

      ELSE

        WRITE(*,*) ' Location number out of range...'
        WRITE(*,*) ' Branch...',Branch
        WRITE(*,*) ' Requested location number = ',LocationNumber
        WRITE(*,*) ' Available range = 1',
     #             ' to',NumberOfCompLocations(Branch)
        CheckChannelCompLocationRange = .FALSE.

      END IF

      RETURN
      END



*== Public (KeepCx) ====================================================

      LOGICAL FUNCTION KeepCx()

      IMPLICIT NONE

*   Purpose: Get an index for including or not including user cross
*            sections intermediate to branch extremity cross sections
*            as computational cross sections:
*              [.TRUE.] include all cross sections.
*              [.FALSE.] include only branch extremity cross sections.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chconnec.inc'

*   Local variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          January 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      KeepCx = KeepAllCx(Branch)

      RETURN
      END



*== Public (ChApprox) ==================================================

      INTEGER FUNCTION ChApprox()

      IMPLICIT NONE

*   Purpose: Get the initial-condition approximation index
*            indicating initial conditions are to be aproximated from :
*              [0] user input (schematic data file)
*              [1] normal depth
*              [2] steady state
*              [3] normal depth w/adverse slopes removed
*              [4] maximum ws elevation for branch
*              [5] model dependent file
*              [6] no approximation attempted

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chconnec.inc'

*   Local variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          January 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      ChApprox = InitialApprox(Branch)

      RETURN
      END



*== Public (GetUserStreamLocationIDs) ==================================

      SUBROUTINE GetUserStreamLocationIDs(
     I              Num,
     O          CurrentIDs
     #                                   )

      IMPLICIT NONE

*   Purpose:  Return an array of current  user stream-location
*             identifiers for the current channel.  Dimension of
*             returned identifier array must be large enough and is
*             not checked by this routine.

*   Arguments:
      INTEGER Num
      CHARACTER*16 CurrentIDs(Num)

*   Argument definitions:
*     Num - dimension of identifier array.
*     CurrentIDs(i) - array of identifiers.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chnluser.inc'

*   Local Variables:
      INTEGER I, J

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          January 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      J = 0
      DO 100 I=UpUserPointer(Branch),DownUserPointer(Branch)
        J = J + 1
        CurrentIDs(J) = UserLocationID(I)
 100  CONTINUE

      RETURN
      END



*== Public (GetUserStreamFlow) =========================================

      SUBROUTINE GetUserStreamFlow(
     I              Num,
     O          Streamflow
     #                                    )

      IMPLICIT NONE

*   Purpose:  Return an array of initial streamflow values at user
*             locations within the current channel.  Dimension of
*             returned array must be large enough and is not
*             checked by this routine.

*   Arguments:
      INTEGER Num
      REAL    Streamflow(Num)

*   Argument definitions:
*     Num - dimension of identifier array.
*     Streamflow(i) - array of streamflow values.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chnluser.inc'

*   Local Variables:
      INTEGER I, J

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          January 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      J = 0
      DO 100 I=UpUserPointer(Branch),DownUserPointer(Branch)
        J = J + 1
        Streamflow(J) = UserQ(I)
 100  CONTINUE

      RETURN
      END



*== Public (GetUserStreamSurfaceElevation) =============================

      SUBROUTINE GetUserStreamSurfaceElevation(
     I              Num,
     O          StreamElevation
     #                                    )

      IMPLICIT NONE

*   Purpose:  Return an array of initial water surface elevations at
*             user locations within the current channel.  Dimension of
*             returned array must be large enough and is not
*             checked by this routine.

*   Arguments:
      INTEGER Num
      REAL    StreamElevation(Num)

*   Argument definitions:
*     Num - dimension of identifier array.
*     StreamElevation(i) - array of stream-surface-elevation values.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chnluser.inc'

*   Local Variables:
      INTEGER I, J

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          January 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      J = 0
      DO 100 I=UpUserPointer(Branch),DownUserPointer(Branch)
        J = J + 1
        StreamElevation(J) = UserWS(I)
 100  CONTINUE

      RETURN
      END



*== Public (UpstreamCode) ==============================================

      INTEGER FUNCTION UpstreamCode()

      IMPLICIT NONE

*   Purpose:  Return upstream boundary condition code for the current channel.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chconnec.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      UpstreamCode = UpBoundaryCode(Branch)

      RETURN
      END



*== Public (UpstreamConnections) =======================================

      INTEGER FUNCTION UpstreamConnections()

      IMPLICIT NONE

*   Purpose:  Return number of connections to upstream end
*             of current channel.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chconnec.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      UpstreamConnections = UpNumberOfConnections(Branch)

      RETURN
      END



*== Public (DownstreamCode) ============================================

      INTEGER FUNCTION DownstreamCode()

      IMPLICIT NONE

*   Purpose:  Return downstream boundary condition code
*             for the current channel.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chconnec.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      DownstreamCode = DownBoundaryCode(Branch)

      RETURN
      END



*== Public (DownstreamConnections) =====================================

      INTEGER FUNCTION DownstreamConnections()

      IMPLICIT NONE

*   Purpose:  Return number of connections to downstream end
*             of current channel.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chconnec.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      DownstreamConnections = DownNumberOfConnections(Branch)

      RETURN
      END



*== Public (UpstreamConnect) ==========================================

      INTEGER FUNCTION UpstreamConnect(I)

      IMPLICIT NONE

*   Purpose:  Return channel number of the Ith upstream connection to the
*             current channel.  The number is positive if is the downstream
*             end of the connecting channel that is connected, negative if
*             it is the upstream end.

*   Arguments:
      INTEGER I

*   Argument definitions:
*     I - sequential number of connection.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chconnec.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: August   1992
*


*-- Implementation -----------------------------------------------------

      UpstreamConnect = UpConnection(
     #            (Branch-1) * MaxConnectingChannels + I
     #                              )

      RETURN
      END



*== Public (DownstreamConnect) ========================================

      INTEGER FUNCTION DownstreamConnect(I)

      IMPLICIT NONE

*   Purpose:  Return channel number of the Ith downstream connection to the
*             current channel.  The number is positive if is the downstream
*             end of the connecting channel that is connected, negative if
*             it is the upstream end.

*   Arguments:
      INTEGER I

*   Argument definitions:
*     I - sequential number of connection.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chconnec.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: August   1992
*


*-- Implementation -----------------------------------------------------

      DownstreamConnect = DownConnection(
     #            (Branch-1) * MaxConnectingChannels + I
     #                              )

      RETURN
      END


*==== EOF chschmt =========================================================
