*==== BOF chstatus ======================================================

************************************************************************
*
*    This file is a FORTRAN module containing 1-D channel flow values,
*    including density, for networks of open channels.
*
*
*     Module note: Only functions or subroutines marked "public" should be
*                   used outside of this module as those marked "private"
*                   may not be supported by future revisions of this module
*                   or replacement modules.  Likewise, no data or common
*                   blocks contained within this module should be accessed
*                   by routines outside of this module accept through the
*                   use of "public" functions or subroutines contained
*                   within this module.
*
*     Non-standard usage: Symbolic names in this module may be represented by
*                         as many as 31 characters in order to provide better
*                         definition directly in the code.  Standard FORTRAN
*                         allows only 6 characters, but this restriction is
*                         generally extended to 32 characters by most compilers.
*
*
*   Public functions:
*
*     REAL FUNCTION StreamFlow(LocationNumber)
*            - returns volumetric discharge.
*
*     LOGICAL FUNCTION SetStreamFlow(LocationNumber, Value)
*            - store volumetric discharge.
*
*     REAL FUNCTION StreamDepth(LocationNumber)
*            - return depth of flow.
*
*     LOGICAL FUNCTION SetStreamDepth(LocationNumber, Value)
*            - store depth of flow.
*
*     REAL FUNCTION StreamSurfaceElevation(LocationNumber)
*            - return watersurface elevation.
*
*     LOGICAL FUNCTION SetStreamSurfaceElevation(LocationNumber, Value)
*            - store watersurface elevation.
*
*     LOGICAL FUNCTION InitializeNetworkFlowValues()
*            - set initial flow values.
*
*      REAL FUNCTION OldStreamDensity(LocationNumber)
*             - returns water density at LocationNumber,
*               at the beginning of the current time step.
*
*      REAL FUNCTION NewStreamDensity(LocationNumber)
*             - returns water density at LocationNumber,
*               at the end of the current time step.
*
*      REAL FUNCTION EstOldStreamDensity(DownStreamDistance)
*             - returns interpolated density at DownStreamDistance,
*               at the begining of the current time step.
*
*      REAL FUNCTION EstOldStreamDensity(DownStreamDistance)
*             - returns interpolated density at DownStreamDistance,
*               at the begining of the current time step.
*
*      LOGICAL FUNCTION SetNewLinearStreamDensity(
*                         UpstreamDensity, DownstreamDensity
*                                             )
*             - sets density at computational locations, at the
*               end of the current time step.
*
*      LOGICAL FUNCTION SetConstantStreamDensity()
*             - sets stream density to 1.0.
*
*      LOGICAL FUNCTION SetNewLinearStreamDensity(
*                         UpstreamDensity, DownstreamDensity
*                                             )
*             - sets density at computational locations, at the
*               end of the current time step.
*
*
*   Arguments:
*     LocationNumber - computational location index, begining with
*                      1 in the current channel.
*     Value          - value of the specific variable to be stored.
*     UpstreamDensity - density at the upstream end of current channel.
*     DownstreamDensity - density at downstream end of current channel.
*     DownstreamDistance - distance from upstream end of current channel.
*     LocationNumber - computational cross-section sequence number,
*                       begining with 1 at upstream end of current
*                       channel.
*
*
*   Module data:
*
*    'network.inc'
*     MaxChannels - maximum number of channels.
*     NumCh - current number of channels.
*     Branch - current selected or active channel.
*     MaxLocations - maximum number of locations (computational or user).
*
*    'chstatus.inc'
*   Definitions:
*     WS(i) - water surface elevation at computational location "i".
*     Q(i) - volumetric discharge at computational location "i".
*     H(i) - depth of flow at computational location "i".
*     Rho(i) - density at computational location "i".
*
*
************************************************************************






*== Public (WriteNetworkRestartFile) ===================================

      LOGICAL FUNCTION WriteNetworkRestartFile()

      IMPLICIT NONE

*   Purpose:  Write current values of dependent and independent
*             variables to a file that may be used as initial
*             conditions to restart the model.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER      fUnit, I, J
      CHARACTER*12 FileName
      LOGICAL      OK

*   Routines by module:

***** Network control:
      LOGICAL  NetworkRestart
      EXTERNAL NetworkRestart

***** Master file:
      INTEGER GetFileUnit
      CHARACTER*12 GetFileName
      EXTERNAL GetFileName, GetFileUnit

***** File utilities:
      LOGICAL    OpenNewText
      EXTERNAL   OpenNewText

***** Channel schematic:
      INTEGER  NumberOfStreamLocations
      EXTERNAL NumberOfStreamLocations

      REAL     StreamDistance
      EXTERNAL StreamDistance
      
      REAL     StreamSurfaceElevation, StreamFlow
      EXTERNAL StreamSurfaceElevation, StreamFlow

      INTEGER  NumberOfChannels
      EXTERNAL NumberOfChannels

      LOGICAL  OpenChannel, CloseChannel
      EXTERNAL OpenChannel, CloseChannel

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Dec   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      WriteNetworkRestartFile = .FALSE.

      IF( NetworkRestart() ) THEN
      
*---- Open restart file.

      FileName = GetFileName( 'netrstrt.dat' )
      fUnit = GetFileUnit( 'netrstrt.dat' )
      IF( OpenNewText( fUnit, FileName ) )  THEN
      ELSE
        WRITE(*,*) ' ####Error(WriteNetworkRestartFile)'
        WRITE(*,*) ' Failed to open retsart file...'
        RETURN
      END IF

      WRITE(fUnit,*) NumberOfChannels(),'/Channels'


*---- Loop on channels.

      DO 200 I=1,NumberOfChannels()

        IF( OpenChannel(I) ) THEN
	
          WRITE(fUnit,*) I, '/Channel'
          WRITE(fUnit,*) NumberOfStreamLocations(),'/Locations'
          
          DO 100 J=1,NumberOfStreamLocations()

            WRITE(fUnit,'(3(F16.3,1X),A20)') 
     #      StreamDistance(J), StreamSurfaceElevation(J), StreamFlow(J),
     #      '/Distance, WSElev, Q'

 100      CONTINUE
 
          OK = CloseChannel()
	
        ELSE
          WRITE(*,*) ' ####Error(WriteNetworkRestartFile)'
          WRITE(*,*) ' Could not open channel ',I
          RETURN
        END IF

 200  CONTINUE
 
      CLOSE( fUnit )

      END IF

      WriteNetworkRestartFile = .TRUE.

      RETURN
      END




*== Private (ReadNetworkInitialConditions) ==============================

      LOGICAL FUNCTION ReadNetworkInitialConditions()

      IMPLICIT NONE

*   Purpose:  Read initial values of dependent variables.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chinitcd.inc'

*   Local Variables:
      INTEGER      Channels
      INTEGER      fUnit, I, J, K
      CHARACTER*12 FileName

*   Routines by module:

***** Local:

***** Master file:
      INTEGER GetFileUnit
      CHARACTER*12 GetFileName
      EXTERNAL GetFileName, GetFileUnit

***** File utilities:
      LOGICAL    OpenOldText
      EXTERNAL   OpenOldText

***** Channel schematic:
      INTEGER  NumberOfChannels, NumberOfStreamLocations
      EXTERNAL NumberOfChannels, NumberOfStreamLocations


*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Dec   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      ReadNetworkInitialConditions = .FALSE.
      
*---- Initialize channel-number cross-reference numbers.

      DO 50 I=1,MaxChannels
        InitialConditionIndex(I) = 0
 50   CONTINUE

*---- Open initial-condition file.

      FileName = GetFileName( 'initcond.dat' )
      fUnit = GetFileUnit( 'initcond.dat' )
      IF( OpenOldText( fUnit, FileName ) ) THEN
      ELSE
        WRITE(*,*) ' ####Error(ReadNetworkInitialConditions)'
        WRITE(*,*) ' Could not open file...',FileName
        RETURN
      END IF
      

*---- Begin reading of data.

      READ(fUnit,*) Channels
      IF( Channels .GT. MaxChannels ) THEN
        WRITE(*,*) ' ####Error(ReadNetworkInitialConditions)'
        WRITE(*,*) ' Channels in initial conditions',
     #              'exceed configured channels...'
        WRITE(*,*) ' ', Channels,' > ', NumberOfChannels()
        RETURN
      END IF

      K = 0
      DO 200 I=1,Channels

        READ(fUnit,*) ChannelNumber(I)
        
        IF( ChannelNumber(I) .LE. NumberOfChannels()
     #                       .AND.
     #      ChannelNumber(I) .GT. 0 ) THEN
     
          
          READ(fUnit,*) Locations(I)
          
*-------- Check for sufficient number of locations, 2.

          IF( Locations(I) .LT. 2 ) THEN
            WRITE(*,*) ' ####Error(ReadNetworkInitialConditions)'
            WRITE(*,*) ' At a minimum, initial values (if supplied)'
            WRITE(*,*) ' must be supplied for ustream and downstream'
            WRITE(*,*) ' extremities of a channel.'
            WRITE(*,*) ' Minimum number of locations is 2.'
            WRITE(*,*) ' Number of locations indicated ...',Locations(I)
          END IF
          
          IF( ( K + Locations(I) ) .LE. MaxLocations ) THEN  
          
            FirstLocation(I) = K + 1
            DO 100 J=1,Locations(I)

              K = K + 1
              READ(fUnit,*) 
     #        InitialX(K), InitialWS(K), InitialQ(K)

 100        CONTINUE
     
          ELSE
            WRITE(*,*) ' ####Error(ReadNetworkInitialConditions)'
            WRITE(*,*) ' Reading initial conditions for channel...',
     #                   ChannelNumber(I)
            WRITE(*,*) ' Maximum number of loactions exceeded.'
            WRITE(*,*) ' Attempted...', K + Locations(I)
            WRITE(*,*) ' Allowed.....', MaxLocations
            RETURN
          END IF
 
	
        ELSE
          WRITE(*,*) ' ####Error(ReadNetworkInitialConditions)'
          WRITE(*,*) ' Read channel number ... ', ChannelNumber(I)
          WRITE(*,*) '  must fall on or between 1 and ',
     #                  NumberOfChannels()
          RETURN
        END IF

        InitialConditionIndex( ChannelNumber(I) ) = I


 200  CONTINUE


      ReadNetworkInitialConditions = .TRUE.

      RETURN
      END



*== Private (ApproxReadInitialConditions) ==============================

      LOGICAL FUNCTION ApproxReadInitialConditions()

      IMPLICIT NONE

*   Purpose:  Approximate initial values of dependent variables
*             for current channel from values read from an 
*             initial-condition file.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chinitcd.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER I, J, K
      REAL    CompLocation(MaxLocations)
      LOGICAL UpstreamFlag, DownstreamFlag

*   Routines by module:

***** Local:
      LOGICAL  ReadNetworkInitialConditions
      EXTERNAL ReadNetworkInitialConditions



***** Channel schematic:
      INTEGER  NumberOfStreamLocations
      EXTERNAL NumberOfStreamLocations
      
      REAL     StreamDistance
      EXTERNAL StreamDistance

      INTEGER  UpstreamPointer, DownstreamPointer
      EXTERNAL UpstreamPointer, DownstreamPointer

***** Network control:
      INTEGER  CurrentChannel
      EXTERNAL CurrentChannel

***** Channel properties:
      REAL     BtmElev
      EXTERNAL BtmElev

***** Linear interpolation utilities:
      EXTERNAL Linear1D

*   Intrinsics:
      INTEGER   INT
      INTRINSIC INT

*   Programmed by: Lew DeLong
*   Date:          Dec   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      ApproxReadInitialConditions = .FALSE.
      
*---- Read initial conditions from file if they haven't been read.

      IF( .NOT. InitCndInitialized ) THEN
        IF( ReadNetworkInitialConditions() ) THEN
          InitCndInitialized = .TRUE.
        ELSE
          WRITE(*,*) ' ####Error(ApproxReadInitialConditions)'
          WRITE(*,*) ' Reading of initial conditions failed...'
          RETURN
        END IF
      END IF

*---- Set sequence number for initial conditions, check validity.

      I = InitialConditionIndex( CurrentChannel() )
      
      IF( I .GT. 0 ) THEN
      ELSE
        WRITE(*,*) ' ####Error(ApproxReadInitialConditions)'
        WRITE(*,*) ' Initial conditions not available for channel',
     #              CurrentChannel()
        RETURN
      END IF
      
*---- Check that initial conditions exist for first and last
*     cross section of channel.

      IF( INT( 100.0 * StreamDistance(1) ) 
     #                  .EQ.
     #    INT( 100.0 * InitialX( FirstLocation(I) ) ) )  THEN
       
          UpstreamFlag = .TRUE.
          
       ELSE
          UpstreamFlag = .FALSE.
       END IF

      IF( INT( 100.0 * StreamDistance(NumberOfStreamLocations() ) ) 
     #                  .EQ.
     #    INT( 100.0 * InitialX( FirstLocation(I)+Locations(I)-1) ) )
     #                 THEN
       
          DownstreamFlag = .TRUE.
          
       ELSE
          DownstreamFlag = .FALSE.
       END IF


*---- If bad match, report errors and RETURN.

      IF( .NOT. UpstreamFlag .OR. .NOT. DownstreamFlag ) THEN
      
        WRITE(*,*) ' ####Error(ApproxReadInitialConditions)'
        WRITE(*,*) ' Location of channel extremities do not match.'
        WRITE(*,*) ' Channel...', CurrentChannel()
        
        IF( .NOT. UpstreamFlag ) THEN
          WRITE(*,*) ' Initial values for upstream end of channel',
     #               ' not found.'
          WRITE(*,*) ' Expected location...',
     #                 StreamDistance(1)
          WRITE(*,*) ' Found location......',
     #                 InitialX( FirstLocation(I) )
        END IF

        IF( .NOT. DownstreamFlag ) THEN
          WRITE(*,*) ' Initial values for downstream end of channel',
     #               ' not found.'
          WRITE(*,*) ' Expected location...',
     #                 StreamDistance(NumberOfStreamLocations())
          WRITE(*,*) ' Found location......',
     #                 InitialX( FirstLocation(I)+Locations(I)-1 )
        END IF
        
        RETURN
      
      END IF

*-------- Get computational stream locations.

          K = 0
          DO 50 J=UpstreamPointer(),DownstreamPointer()
            K = K + 1
            CompLocation(J) = StreamDistance(K)
 50       CONTINUE


*---------- Approximate streamflow.


            CALL Linear1D(
     I         NumberOfStreamLocations(),
     I         CompLocation(UpstreamPointer()),
     I         Locations(I), InitialX( FirstLocation(I) ),
     I         InitialQ( FirstLocation(I) ),
     O         Q(UpstreamPointer())
     #                   )

*---------- Approximate watersurface elevation.

            CALL Linear1D(
     I         NumberOfStreamLocations(),
     I         CompLocation(UpstreamPointer()),
     I         Locations(I), InitialX( FirstLocation(I) ),
     I         InitialWS( FirstLocation(I) ),
     O         WS(UpstreamPointer())
     #                   )

*---------- Approximate depth of flow.

            DO 100 J=UpstreamPointer(),DownstreamPointer()

               H( J ) = WS( J ) - BtmElev( CompLocation( J ) )

 100        CONTINUE

      
      ApproxReadInitialConditions = .TRUE.

      RETURN
      END



*== Private (SetNetworkInitCndRead) ================================================

      LOGICAL FUNCTION SetNetworkInitCndRead( State )

      IMPLICIT NONE

*   Purpose:  Set state of network initial-condition reading.
*             [.TRUE. ] if read, or
*             [.FALSE.] if not read.

*   Arguments:
      LOGICAL State

*   Argument definitions:
*     State - .TRUE. if read, .FALSE. otherwise.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chinitcd.inc'
      
*   Local Variables:

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Dec   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      InitCndInitialized = State
      SetNetworkInitCndRead = .TRUE.

      RETURN
      END



*== Public (StreamFlow) ================================================

      REAL FUNCTION StreamFlow(LocationNumber)

      IMPLICIT NONE

*   Purpose:  Return current value of stream flow in the current channel
*             at a location corresponding to the index LocationNumber.

*   Arguments:
      INTEGER LocationNumber

*   Argument definitions:
*     LocationNumber - computational-location sequence number within
*                      current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER J

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer, CurrentChannel
      EXTERNAL UpstreamPointer, CurrentChannel

      LOGICAL  CheckChannelCompLocationRange
      EXTERNAL CheckChannelCompLocationRange

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      J = UpstreamPointer() + LocationNumber - 1
      IF( CheckChannelCompLocationRange( LocationNumber ) ) THEN

        StreamFlow = Q( J )

      ELSE

        WRITE(*,*) ' Range error...(StreamFlow)'
        WRITE(*,*) ' Channel ',CurrentChannel(),'...'
        WRITE(*,*) ' Abnormal program end.'
        STOP

      END IF

      RETURN
      END



*== Public (GlobalStreamFlow) ==========================================

      REAL FUNCTION GlobalStreamFlow(LocationNumber)

      IMPLICIT NONE

*   Purpose:  Return current value of stream flow
*             at a location corresponding to the
*             index LocationNumber.

*   Arguments:
      INTEGER LocationNumber

*   Argument definitions:
*     LocationNumber - computational-location sequence number within
*                      current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

        GlobalStreamFlow = Q( LocationNumber )

      RETURN
      END



*== Public (SetStreamFlow) =============================================

      LOGICAL FUNCTION SetStreamFlow(LocationNumber, Value)

      IMPLICIT NONE

*   Purpose:  Set current value of stream flow in the current channel
*             at a location corresponding to the index LocationNumber.

*   Arguments:
      INTEGER LocationNumber
      REAL    Value

*   Argument definitions:
*     LocationNumber - computational-location sequence number within
*                      current channel.
*     Value  - value to be stored. 
*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER J

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer
      EXTERNAL UpstreamPointer

      LOGICAL  CheckChannelCompLocationRange
      EXTERNAL CheckChannelCompLocationRange

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      J = UpstreamPointer() + LocationNumber - 1
      IF( CheckChannelCompLocationRange( LocationNumber ) ) THEN

        Q( J ) = Value
        SetStreamFlow = .TRUE.

      ELSE

        WRITE(*,*) ' Range error...(SetStreamFlow)'
        SetStreamFlow = .False.

      END IF

      RETURN
      END



*== Public (StreamDepth) ===============================================

      REAL FUNCTION StreamDepth(LocationNumber)

      IMPLICIT NONE

*   Purpose:  Return current value of depth of flow in the current
*             channel at a location corresponding to the index
*             LocationNumber.

*   Arguments:
      INTEGER LocationNumber

*   Argument definitions:
*     LocationNumber - computational-location number (index) within
*                      current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER J

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer
      EXTERNAL UpstreamPointer

      LOGICAL  CheckChannelCompLocationRange
      EXTERNAL CheckChannelCompLocationRange

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      J = UpstreamPointer() + LocationNumber - 1
      IF( CheckChannelCompLocationRange( LocationNumber ) ) THEN

        StreamDepth = H( J )

      ELSE

        WRITE(*,*) ' Range error...(StreamDepth)'
        WRITE(*,*) ' Abnormal program end.'
        STOP

      END IF

      RETURN
      END



*== Public (GlobalStreamDepth) =========================================

      REAL FUNCTION GlobalStreamDepth(LocationNumber)

      IMPLICIT NONE

*   Purpose:  Return current value of depth of flow
*             at a location corresponding to the index
*             LocationNumber.

*   Arguments:
      INTEGER LocationNumber

*   Argument definitions:
*     LocationNumber - computational-location number (index) within
*                      current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

        GlobalStreamDepth = H( LocationNumber )


      RETURN
      END



*== Public (SetStreamDepth) ============================================

      LOGICAL FUNCTION SetStreamDepth(LocationNumber, Value)

      IMPLICIT NONE

*   Purpose:  Set current value of depth of flow in the current
*             channel at a location corresponding to the index
*             LocationNumber.

*   Arguments:
      INTEGER LocationNumber
      REAL    Value

*   Argument definitions:
*     LocationNumber - computational-location number (index) within
*                      current channel.
*     Value - current value to be set.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER J

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer
      EXTERNAL UpstreamPointer

      LOGICAL  CheckChannelCompLocationRange
      EXTERNAL CheckChannelCompLocationRange

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      J = UpstreamPointer() + LocationNumber - 1
      IF( CheckChannelCompLocationRange( LocationNumber ) ) THEN

        H( J ) = Value

        SetStreamDepth = .TRUE.

      ELSE
        WRITE(*,*) ' Range error...(SetStreamDepth)'
        SetStreamDepth = .FALSE.
      END IF

      RETURN
      END



*== Public (StreamSurfaceElevation) ====================================

      REAL FUNCTION StreamSurfaceElevation(LocationNumber)

      IMPLICIT NONE

*   Purpose:  Return current value of water-surface elevation in the
*             current channel at a location corresponding to the index
*             LocationNumber.

*   Arguments:
      INTEGER LocationNumber

*   Argument definitions:
*     LocationNumber - computational-location number (index) within
*                      current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER J

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer
      EXTERNAL UpstreamPointer

      LOGICAL  CheckChannelCompLocationRange
      EXTERNAL CheckChannelCompLocationRange

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      J = UpstreamPointer() + LocationNumber - 1

      IF( CheckChannelCompLocationRange( LocationNumber ) ) THEN

        StreamSurfaceElevation = WS( J )

      ELSE
        WRITE(*,*) ' Range error...(StreamSurfaceElevation)'
        WRITE(*,*) ' Abnormal program end.'
        STOP
      END IF

      RETURN
      END



*== Public (GlobalStreamSurfaceElevation) ==============================

      REAL FUNCTION GlobalStreamSurfaceElevation(LocationNumber)

      IMPLICIT NONE

*   Purpose:  Return current value of water-surface elevation
*             at a location corresponding to the index
*             LocationNumber.

*   Arguments:
      INTEGER LocationNumber

*   Argument definitions:
*     LocationNumber - computational-location number (index).

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      GlobalStreamSurfaceElevation = WS( LocationNumber )

      RETURN
      END



*== Public (SetStreamSurfaceElevation) =================================

      LOGICAL FUNCTION SetStreamSurfaceElevation
     #                 (LocationNumber, Value)

      IMPLICIT NONE

*   Purpose:  Set current value of water-surface elevation in the
*             current channel at a location corresponding to the index
*             LocationNumber.  Also, set corresponding depth of flow.

*   Arguments:
      INTEGER LocationNumber
      REAL    Value

*   Argument definitions:
*     LocationNumber - computational-location number (index) within
*                      current channel.
*     Value  - value to be stored. 

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER J

*   Routines by module:

***** Channel properties:
      REAL     BtmElev
      EXTERNAL BtmElev

***** Channel schematic:
      INTEGER  UpstreamPointer
      EXTERNAL UpstreamPointer

      REAL     StreamDistance
      EXTERNAL StreamDistance

      LOGICAL  CheckChannelCompLocationRange
      EXTERNAL CheckChannelCompLocationRange

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      J = UpstreamPointer() + LocationNumber - 1
      IF( CheckChannelCompLocationRange( LocationNumber ) ) THEN

        WS( J ) = Value
        H( J ) = Value - BtmElev( StreamDistance( LocationNumber ) )

        SetStreamSurfaceElevation = .TRUE.

      ELSE
        WRITE(*,*) ' Range error...(SetStreamSurfaceElevation)'
        SetStreamSurfaceElevation = .FALSE.
      END IF

      RETURN
      END



*== Public (InitializeNetworkFlowValues) ===============================

      LOGICAL FUNCTION InitializeNetworkFlowValues()

      IMPLICIT NONE

*   Purpose:  Set initial values of water-surface elevation and flow
*             in a network of channels.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER I, J, K, L
      INTEGER PrintUnit
      INTEGER ChannelNumber, Approx, UserLocations
      REAL    Velocity, dX, CrNo, dT, G, BackWater, Slope, FrNo
      REAL    CompLocation(MaxLocations)
      REAL    WidthRatio, Width, WSSlope, WSSlopeRatio, WSSlopeChange
      REAL    DummyArray(MaxLocations), DummyArray2(MaxLocations)
      REAL    Check, Depth, Flow, X
      CHARACTER*16 DummyCharArray(MaxLocations)
      CHARACTER*12 InternalFileName
      CHARACTER*6 Blat
      LOGICAL OK

*     CompLocation(j) - downstream distance coordinate.
*     DummyArray(j) - a REAL value dependent upon context.
*     DummyArray2(j) - a REAL value dependent upon context.
*     DummyCharArray(j) - a CHARACTER value dependent upon context.

*   Routines by module:

***** Local:
      REAL     Norm, GlobalStreamSurfaceSlope
      EXTERNAL Norm, GlobalStreamSurfaceSlope
      
      LOGICAL  SetNetworkInitCndRead
      EXTERNAL SetNetworkInitCndRead
      
      LOGICAL  ApproxReadInitialConditions
      EXTERNAL ApproxReadInitialConditions
      
      LOGICAL  Bernie, SpForce
      EXTERNAL Bernie, SpForce

***** Channel properties:
      REAL     BtmElev, CxArea, BtmSlope, ChannelWidth
      EXTERNAL BtmElev, CxArea, BtmSlope, ChannelWidth
      
      REAL     Conveyance, dConveyance
      EXTERNAL Conveyance, dConveyance

      EXTERNAL UserStreamLocations

***** Channel schematic:
      INTEGER  NumberOfUserStreamLocations, NumberOfStreamLocations
      INTEGER  UpstreamPointer, DownstreamPointer
      REAL     StreamDistance
      EXTERNAL NumberOfUserStreamLocations, NumberOfStreamLocations
      EXTERNAL UpstreamPointer, DownstreamPointer
      EXTERNAL StreamDistance

      INTEGER  NumberOfChannels, ChApprox
      EXTERNAL NumberOfChannels, ChApprox

      LOGICAL  OpenChannel, CloseChannel
      EXTERNAL OpenChannel, CloseChannel

      EXTERNAL GetUserStreamflow, GetUserStreamSurfaceElevation

***** Linear interpolation utilities:
      EXTERNAL Linear1D
      
***** Network control:
      INTEGER  NetworkPrintLevel, NetworkTimeIncrement
      EXTERNAL NetworKPrintLevel, NetworkTimeIncrement
      REAL     AccelerationGravity
      EXTERNAL AccelerationGravity

***** Master File:
      INTEGER  GetFileUnit
      EXTERNAL GetFileUnit
      
*   Intrinsics:
      REAL      FLOAT, SQRT, ABS
      INTRINSIC FLOAT, SQRT, ABS

*   Programmed by: Lew DeLong
*   Date:          Nov   1990
*   Modified by:   Lew DeLong
*   Last modified: August   1993
*


*-- Implementation -----------------------------------------------------

      InitializeNetworkFlowValues = .FALSE.
      
      Blat = '  ....'
      
      InternalFileName = 'netprint.dat'
      PrintUnit = GetFileUnit(InternalFileName)
      
      IF( NetworkPrintLevel() .GT. 0 ) THEN
        WRITE(*,*) ' Initializing network flow values...'
      END IF
      
      dT = FLOAT( NetworkTimeIncrement() )
      G = AccelerationGravity()

*---- Set initilal-condition read flag to "not initialized".

      OK = .FALSE.
      OK = SetNetworkInitCndRead( OK )


*---- Loop on channels.

      DO 200 I = 1, NumberOfChannels()

        ChannelNumber = I
C       WRITE(*,*) ' Channel',ChannelNumber,'...'
        IF( OpenChannel( ChannelNumber )  ) THEN

*-------- Get computational stream locations.

          K = 0
          DO 50 J=UpstreamPointer(),DownstreamPointer()
            K = K + 1
            CompLocation(J) = StreamDistance(K)
 50       CONTINUE


*-------- Make first estimate of initial values.

          Approx = ChApprox()

*---------- Initial conditions approximated from user input.


            UserLocations = NumberOfUserStreamLocations()

*---------- Get user cross-section IDs and stream distances,
*            store in DummyArray.

            CALL UserStreamLocations(
     #             UserLocations,
     #             DummyCharArray, DummyArray
     #                              )

*---------- Approximate streamflow linearly from user input.

            CALL GetUserStreamFlow( UserLocations, DummyArray2 )

            CALL Linear1D(
     #         NumberOfStreamLocations(),
     #         CompLocation(UpstreamPointer()),
     #         UserLocations, DummyArray, DummyArray2,
     #         Q(UpstreamPointer())
     #                   )

*---------- Approximate watersurface elevation linearly from user input.

            CALL GetUserStreamSurfaceElevation(
     #                     UserLocations, DummyArray2
     #                                        )

            CALL Linear1D(
     #         NumberOfStreamLocations(),
     #         CompLocation(UpstreamPointer()),
     #         UserLocations, DummyArray, DummyArray2,
     #         WS(UpstreamPointer())
     #                   )

*---------- Approximate depth of flow.

            DO 100 J=UpstreamPointer(),DownstreamPointer()

               H( J ) = WS( J ) - BtmElev( CompLocation( J ) )

 100        CONTINUE


*-------- Branch to requested initial-values approximation.

          IF(Approx .EQ. 0) THEN

          ELSE IF(Approx .EQ. 1) THEN
C           WRITE(*,*) '   Initial ws_elev approximated',
C    #                 ' by normal depth...'

            DO 110 J=UpstreamPointer(),DownstreamPointer()
              H( J ) = Norm( CompLocation(J), Q( J ) )
              WS( J ) = H( J ) + BtmElev( CompLocation( J ) )
 110        CONTINUE

          ELSE IF(Approx .EQ. 2) THEN
            WRITE(*,*) '   Initial ws_elev computed',
     #                  ' assuming steady state...'
            WRITE(*,*) '   (not implemented)...'
          ELSE IF(Approx .EQ. 3) THEN
C           WRITE(*,*) '   Initial ws_elev approximated',
C    #                 ' by normal depth...'
C           WRITE(*,*) ' Adverse slopes are  flattened...'
            DO 120 J=UpstreamPointer(),DownstreamPointer()
              H( J ) = Norm( CompLocation(J), Q( J ) )
              WS( J ) = H( J ) + BtmElev( CompLocation( J ) )
 120        CONTINUE
            L = DownstreamPointer()
            DO 122 J=UpstreamPointer(),DownstreamPointer()-1
              L = L - 1
              IF( WS(L) .LT. WS(L+1) ) THEN
                WS( L ) = WS( L+1 )
                H( L ) = WS( L ) - BtmElev( CompLocation( L ) )
              END IF
 122        CONTINUE
          ELSE IF(Approx .EQ. 4) THEN
            WRITE(*,*) '   Initial ws_elev approximated',
     #                 ' equal to maximum for channel...'
            WRITE(*,*) '   (not implemented)...'
          ELSE IF(Approx .EQ. 5) THEN
C           WRITE(*,*) ' Initial conditions approximated ',
C    #                 ' from model-dependent file...'
            OK = ApproxReadInitialConditions()
          ELSE IF(Approx .EQ. 6) THEN
*           Initial linear approximation retained...
          ELSE IF(Approx .EQ. 7) THEN
            IF( Bernie() ) THEN
            END IF
              IF( ConvergedSteady(ChannelNumber) ) THEN
                WRITE(*,*) ' Steady-state approximation',
     #                   ' converged, Channel...',
     #                     ChannelNumber
              ELSE
                WRITE(*,*)  ' Steady-state approximation',
     #                      ' failed to converge, Channel...',
     #                     ChannelNumber
              END IF
              WRITE(*,*) ' smallest dX...',dXBernie(ChannelNumber)
          ELSE IF(Approx .EQ. 8) THEN
            IF( SpForce() ) THEN
            END IF
              IF( ConvergedSteady(ChannelNumber) ) THEN
                WRITE(*,*) ' Steady-state approximation',
     #                   ' converged, Channel...',
     #                     ChannelNumber
              ELSE
                WRITE(*,*)  ' Steady-state approximation',
     #                      ' failed to converge, Channel...',
     #                     ChannelNumber
              END IF
              WRITE(*,*) ' smallest dX...',dXForce(ChannelNumber)
          ELSE
            WRITE(*,*) ' Initial-condition index ',Approx,
     #               ' not supported...'
          END IF



*-------- Write initial conditions to the print file.

          WRITE(PrintUnit,*) ' '
          WRITE(PrintUnit,*) 'Channel...',Branch
          WRITE(PrintUnit,*)
     #             'Initial conditions at computational locations...'
          IF( Approx .EQ. 7 ) THEN
              IF( ConvergedSteady(ChannelNumber) ) THEN
                WRITE(PrintUnit,*) ' Steady-state approximation',
     #                   ' converged, Channel...',
     #                     ChannelNumber
              ELSE
                WRITE(PrintUnit,*)  ' Steady-state approximation',
     #                      ' failed to converge...'
              END IF
              WRITE(PrintUnit,*) ' smallest dX...',
     #                         dXBernie(ChannelNumber)
          END IF

          IF( Approx .EQ. 8 ) THEN
              IF( ConvergedSteady(ChannelNumber) ) THEN
                WRITE(PrintUnit,*) ' Steady-state approximation',
     #                   ' converged, Channel...',
     #                     ChannelNumber
              ELSE
                WRITE(PrintUnit,*)  ' Steady-state approximation',
     #                      ' failed to converge...'
              END IF
              WRITE(PrintUnit,*) ' smallest dX...',
     #                         dXForce(ChannelNumber)
          END IF


          WRITE(PrintUnit,*) ' '
          WRITE(PrintUnit,*)
     #       '                                          ',
     #       'Depth     WS     Top                  ',
     #       '     Slope      Slope  Width'
          WRITE(PrintUnit,*)
     #       '  CxNo Location       Q    WS_Elev  Depth ',
     #       ' -Hn    Slope   Width   Vel  CrNo FdNo',
     #       '    Change      Ratio  Ratio'
          WRITE(PrintUnit,*) ' '

*-------- Upstream end of channel.

          K = 1
          J = UpstreamPointer()
          Velocity = Q(J) / CxArea( CompLocation(J), H(J) )
          dX = CompLocation(J+1) - CompLocation(J)
          WSSlope = ( WS(J+1) - WS(J) ) / dX
          FrNo = Velocity / SQRT( G * H(J) )
          CrNo = dT * ( Velocity + SQRT( G * H(J) ) ) / dX 
          Slope = BtmSlope( CompLocation(J) )
          Width = ChannelWidth( CompLocation(J), H(J) )
          IF( Slope .LT. -0.00001 .AND. Q(J) .GT. 0.0 ) THEN
            BackWater = H(J) - Norm( CompLocation(J), Q(J) )
              WRITE(PrintUnit,'(I4,2X,2F9.0,F10.2,F6.2,F6.2,
     #        F10.6,F6.0,F7.2,F6.1,F5.2)')
     #        K,CompLocation(J),Q(J),WS(J),H(J),BackWater,
     #        WSSlope, Width, Velocity, CrNo, FrNo
          ELSE
              WRITE(PrintUnit,'(I4,2X,2F9.0,F10.2,F6.2,A6,
     #        F10.6,F6.0,F7.2,F6.1,F5.2)')
     #        K,CompLocation(J),Q(J),WS(J),H(J),Blat,
     #        WSSlope, Width, Velocity, CrNo, FrNo
          END IF

*-------- Intervening cross sections.

          IF( (DownstreamPointer() - UpstreamPointer()) .GT. 2) THEN
          DO 150 J=UpstreamPointer()+1,DownstreamPointer()-1
            K = K + 1
            Velocity = Q(J) / CxArea( CompLocation(J), H(J) )
            dX = ( CompLocation(J+1) - CompLocation(J-1) )
            WSSlope = ( WS(J+1) - WS(J-1) ) / dX
            CrNo = dT * ( Velocity + SQRT( G * H(J) ) ) / ( 0.5 * dX )
            FrNo = Velocity / SQRT( G * H(J) )
            dX   = CompLocation(J+1) - CompLocation(J)
            Slope = BtmSlope( CompLocation(J) )
            IF( ABS( ( WS(J) - WS(J-1) ) ) .GT. 1.0E-10 ) THEN 
              WSSlopeRatio = ( WS(J+1) - WS(J) ) / ( WS(J) - WS(J-1) )
     #                     * ( CompLocation(J) - CompLocation(J-1) )
     #                     / ( CompLocation(J+1) - CompLocation(J) )
            ELSE
              WSSlopeRatio = 0.0
            END IF
            WSSlopeChange = ( WS(J+1) - WS(J) )
     #                    / ( CompLocation(J+1) - CompLocation(J) )
     #                    - ( WS(J) - WS(J-1) )
     #                    / ( CompLocation(J) - CompLocation(J-1) )
            Width = ChannelWidth( CompLocation(J), H(J) )
            WidthRatio = ChannelWidth( CompLocation(J+1), H(J+1) )
     #                 / ChannelWidth( CompLocation(J-1), H(J-1) )
            IF( Slope .LT. -0.00001 .AND. Q(J) .GT. 0.0 ) THEN
              BackWater = H(J) - Norm( CompLocation(J), Q(J) )
              WRITE(PrintUnit,'(I4,2X,2F9.0,F10.2,F6.2,F6.2,
     #        F10.6,F6.0,F7.2,F6.1,F5.2, F10.6,F10.2,F6.2)')
     #        K,CompLocation(J),Q(J),WS(J),H(J),BackWater,
     #        WSSlope, Width, Velocity, CrNo, FrNo
     #        , WSSlopeChange, WSSlopeRatio, WidthRatio
          ELSE
              WRITE(PrintUnit,'(I4,2X,2F9.0,F10.2,F6.2,A6,
     #        F10.6,F6.0,F7.2,F6.1,F5.2, F10.6,F10.2,F6.2)')
     #        K,CompLocation(J),Q(J),WS(J),H(J),Blat,
     #        WSSlope, Width, Velocity, CrNo, FrNo
     #        , WSSlopeChange, WSSlopeRatio, WidthRatio
          END IF
 150      CONTINUE
          END IF

*-------- Downstream end of channel.

          K = K + 1
          J = DownstreamPointer()
          Velocity = Q(J) / CxArea( CompLocation(J), H(J) )
          dX = CompLocation(J) - CompLocation(J-1)
          FrNo = Velocity / SQRT( G * H(J) )
          CrNo = dT * ( Velocity + SQRT( G * H(J) ) ) / dX 
          Slope = BtmSlope( CompLocation(J) )
          WSSlope = ( WS(J) - WS(J-1) ) / dX
          Width = ChannelWidth( CompLocation(J), H(J) )
          IF( Slope .LT. -0.00001 .AND. Q(J) .GT. 0.0 ) THEN
            BackWater = H(J) - Norm( CompLocation(J), Q(J) )
              WRITE(PrintUnit,'(I4,2X,2F9.0,F10.2,F6.2,F6.2,
     #        F10.6,F6.0,F7.2,F6.1,F5.2)')
     #        K,CompLocation(J),Q(J),WS(J),H(J),BackWater,
     #        WSSlope, Width, Velocity, CrNo, FrNo
          ELSE
              WRITE(PrintUnit,'(I4,2X,2F9.0,F10.2,F6.2,A6,
     #        F10.6,F6.0,F7.2,F6.1,F5.2)')
     #        K,CompLocation(J),Q(J),WS(J),H(J),Blat,
     #        WSSlope, Width, Velocity, CrNo, FrNo
          END IF
          WRITE(PrintUnit,*) ' '

*-------- Check friction / WS slope / dX relation.

          IF( NetworkPrintLevel() .GT. 1 ) THEN
            WRITE(*,*) ' Checking friction / WS slope / dX relation...'
          
          DO 160 J=UpstreamPointer()+1,DownstreamPointer()
            Depth = 0.5 * ( H(J) + H(J-1) )
            X = 0.5 * ( CompLocation(J) + CompLocation(J-1) )
            dX = CompLocation(J) - CompLocation(J-1)
            Flow = 0.5 * ( Q(J) + Q(J-1) )
            Check = Flow**2 / Conveyance(X,Depth)**3
     #                  * dConveyance(X,Depth) * dX
            WRITE(*,*) J, CompLocation(J), Check
            
 160      CONTINUE
          END IF

          OK = CloseChannel()

        ELSE
          WRITE(*,*) ' Could not open channel...',ChannelNumber
          WRITE(*,*) ' (InitializeNetworkFlowValues)'
          RETURN
        END IF



 200  CONTINUE

      InitializeNetworkFlowValues = .TRUE.

      RETURN
      END



*== Public (OldStreamDensity) ================================================

      REAL FUNCTION OldStreamDensity(LocationNumber)

      IMPLICIT NONE

*   Purpose:  Return current value of stream density in the current
*             channel at a location corresponding to the index 
*             LocationNumber, at the begining of the current time step.

*   Arguments:
      INTEGER LocationNumber

*   Argument definitions:
*     LocationNumber - computational-location sequence number within
*                      current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER J

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer, CurrentChannel
      EXTERNAL UpstreamPointer, CurrentChannel

      LOGICAL  CheckChannelCompLocationRange
      EXTERNAL CheckChannelCompLocationRange

*   Programmed by: Lew DeLong
*   Date:          October 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      J = UpstreamPointer() + LocationNumber - 1
      IF( CheckChannelCompLocationRange( LocationNumber ) ) THEN

        OldStreamDensity = Rho1( J )

      ELSE

        WRITE(*,*) ' Range error...(OldStreamDensity)'
        WRITE(*,*) ' Channel ',CurrentChannel(),'...'
        WRITE(*,*) ' Abnormal program end.'
        STOP

      END IF

      RETURN
      END



*== Public (NewStreamDensity) ================================================

      REAL FUNCTION NewStreamDensity(LocationNumber)

      IMPLICIT NONE

*   Purpose:  Return current value of stream density in the current
*             channel at a location corresponding to the index 
*             LocationNumber, at the end of the current time step.

*   Arguments:
      INTEGER LocationNumber

*   Argument definitions:
*     LocationNumber - computational-location sequence number within
*                      current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER J

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer, CurrentChannel
      EXTERNAL UpstreamPointer, CurrentChannel

      LOGICAL  CheckChannelCompLocationRange
      EXTERNAL CheckChannelCompLocationRange

*   Programmed by: Lew DeLong
*   Date:          October 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      J = UpstreamPointer() + LocationNumber - 1
      IF( CheckChannelCompLocationRange( LocationNumber ) ) THEN

        NewStreamDensity = Rho2( J )

      ELSE

        WRITE(*,*) ' Range error...(NewStreamDensity)'
        WRITE(*,*) ' Channel ',CurrentChannel(),'...'
        WRITE(*,*) ' Abnormal program end.'
        STOP

      END IF

      RETURN
      END



*== Public (SetOldStreamDensity) ================================================

      LOGICAL FUNCTION SetOldStreamDensity(LocationNumber, Value)

      IMPLICIT NONE

*   Purpose:  Set current value of stream density in the current
*             channel at a location corresponding to the index 
*             LocationNumber, at the begining of the current time step.

*   Arguments:
      INTEGER LocationNumber
      REAL    Value

*   Argument definitions:
*     LocationNumber - computational-location sequence number within
*                      current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER J

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer, CurrentChannel
      EXTERNAL UpstreamPointer, CurrentChannel

      LOGICAL  CheckChannelCompLocationRange
      EXTERNAL CheckChannelCompLocationRange

*   Programmed by: Lew DeLong
*   Date:          October 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      J = UpstreamPointer() + LocationNumber - 1
      IF( CheckChannelCompLocationRange( LocationNumber ) ) THEN

        Rho1( J ) = Value

      ELSE

        WRITE(*,*) ' Range error...(SetOldStreamDensity)'
        WRITE(*,*) ' Channel ',CurrentChannel(),'...'
        WRITE(*,*) ' Abnormal program end.'
        STOP

      END IF

      SetOldStreamDensity = .TRUE.
      
      RETURN
      END



*== Public (SetNewStreamDensity) ================================================

      LOGICAL FUNCTION SetNewStreamDensity(LocationNumber, Value)

      IMPLICIT NONE

*   Purpose:  Set current value of stream density in the current
*             channel at a location corresponding to the index 
*             LocationNumber, at the end of the current time step.

*   Arguments:
      INTEGER LocationNumber
      REAL    Value

*   Argument definitions:
*     LocationNumber - computational-location sequence number within
*                      current channel.
*     Value - value to be set.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER J

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer, CurrentChannel
      EXTERNAL UpstreamPointer, CurrentChannel

      LOGICAL  CheckChannelCompLocationRange
      EXTERNAL CheckChannelCompLocationRange

*   Programmed by: Lew DeLong
*   Date:          October 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      J = UpstreamPointer() + LocationNumber - 1
      IF( CheckChannelCompLocationRange( LocationNumber ) ) THEN

        Rho2( J ) = Value

      ELSE

        WRITE(*,*) ' Range error...(SetNewStreamDensity)'
        WRITE(*,*) ' Channel ',CurrentChannel(),'...'
        WRITE(*,*) ' Abnormal program end.'
        STOP

      END IF
      
      SetNewStreamDensity = .TRUE.

      RETURN
      END



*== Public (EstOldStreamDensity) ================================================

      REAL FUNCTION EstOldStreamDensity(DownstreamDistance)

      IMPLICIT NONE

*   Purpose:  Return current value of stream density in the current
*             channel at a  downstream distance of 
*             DownStreamDistance, at the begining of the current
*             time step.

*   Arguments:
      REAL DownstreamDistance

*   Argument definitions:
*     DownstreamDistance - downstream distance to location within
*                           current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER I, N
      REAL    XUp, dX, Shape

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer, DownstreamPointer, CurrentChannel
      REAL     StreamDistance
      EXTERNAL UpstreamPointer, DownstreamPointer, CurrentChannel
      EXTERNAL StreamDistance

*   Programmed by: Lew DeLong
*   Date:          October 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      N = 0
      DO 100 I=UpstreamPointer(),DownstreamPointer()
        N = N + 1
        IF(StreamDistance(N) .LE. DownstreamDistance ) THEN
        ELSE
          GO TO 102
        END IF
 100  CONTINUE
      WRITE(*,*) ' Range error...(EstOldStreamDensity)'
      WRITE(*,*) ' Channel ',CurrentChannel(),'...'
      WRITE(*,*) ' Downstream distance ...', DownstreamDistance
 102  CONTINUE
      
      XUp = StreamDistance(N-1)
      dX = StreamDistance(N) - XUp
      Shape = (DownstreamDistance - XUp) / dX
      EstOldStreamDensity = Rho1(N) * Shape + Rho1(N-1) * (1.0 - Shape)
      
      RETURN
      END



*== Public (EstNewStreamDensity) ================================================

      REAL FUNCTION EstNewStreamDensity(DownstreamDistance)

      IMPLICIT NONE

*   Purpose:  Return current value of stream density in the current
*             channel at a location at a downstream distance of 
*             DownStreamDistance, at the end of the current
*             time step.

*   Arguments:
      REAL DownstreamDistance

*   Argument definitions:
*     DownstreamDistance - downstream distance to location within
*                           current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER I, N
      REAL    XUp, dX, Shape

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer, DownstreamPointer, CurrentChannel
      REAL     StreamDistance
      EXTERNAL UpstreamPointer, DownstreamPointer, CurrentChannel
      EXTERNAL StreamDistance

*   Programmed by: Lew DeLong
*   Date:          October 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      N = 0
      DO 100 I=UpstreamPointer(),DownstreamPointer()
        N = N + 1
        IF(StreamDistance(N) .LE. DownstreamDistance ) THEN
        ELSE
          GO TO 102
        END IF
 100  CONTINUE
      WRITE(*,*) ' Range error...(EstNewStreamDensity)'
      WRITE(*,*) ' Channel ',CurrentChannel(),'...'
      WRITE(*,*) ' Downstream distance ...', DownstreamDistance
 102  CONTINUE
      
      XUp = StreamDistance(N-1)
      dX = StreamDistance(N) - XUp
      Shape = (DownstreamDistance - XUp) / dX
      EstNewStreamDensity = Rho2(N) * Shape + Rho2(N-1) * (1.0 - Shape)
      
      RETURN
      END






*== Public (SetConstantStreamDensity) ====================================

      LOGICAL FUNCTION SetConstantStreamDensity()

      IMPLICIT NONE

*   Purpose:  Set stream density to 1.0 for all channels.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:
      INTEGER I, M, Channel
      LOGICAL  OK

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer, DownstreamPointer
      EXTERNAL UpstreamPointer, DownstreamPointer
      INTEGER  NumberOfChannels
      EXTERNAL NumberOfChannels
      LOGICAL  OpenChannel, CloseChannel
      EXTERNAL OpenChannel, CloseChannel

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          October  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------


      DO 200 M=1,NumberOfChannels()
      
        Channel = M
        IF( OpenChannel( Channel ) ) THEN
          DO 100 I=UpstreamPointer(),DownstreamPointer()
            Rho1(I) = 1.0
            Rho2(I) = 1.0
 100      CONTINUE
        ELSE
          WRITE(*,*) ' ####error(SetConstantStreamDensity)'
          WRITE(*,*) ' could not open channel...',Channel
          WRITE(*,*) ' Abnormal program end.'
          STOP
        END IF
        
        OK = CloseChannel()
        
 200  CONTINUE
      
      
      SetConstantStreamDensity = .TRUE.

      RETURN
      END



*== Public (Norm) ======================================================

      REAL FUNCTION Norm(X, Q)

      IMPLICIT NONE

*   Purpose:  Estimate normal depth in the current channel,
*             at X downstream distance, for discharge Q.

*   Arguments:
      REAL X, Q

*   Argument Definitions:
*     X      - downstream distance.
*     Q      - streamflow

*   Local variables:
      INTEGER I, Attempts
      INTEGER MaxAttempts
      PARAMETER( MaxAttempts = 50 )
      REAL    K, NewK,NewH,dH, LowH, HighH
      REAL    Slope, Tolerance
      PARAMETER( Tolerance = 0.01 )

*   Local Definitions:
*     Slope   - water-surface slope, assumed = bottom slope.

*   Routines by model:

***** Channel properties:
      REAL     Conveyance, dConveyance, BtmSlope
      EXTERNAL Conveyance, dConveyance, BtmSlope

***** Channel schematic:
      INTEGER  CurrentChannel
      EXTERNAL CurrentChannel

*   Intrinsics:
        REAL      ABS,SQRT
        INTRINSIC ABS,SQRT

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:   Lew DeLong
*   Last modified: July     1993
*


*-- Implementation ------------------------------------------------------------

      Slope = ABS( BtmSlope( X ) )
      
*---- Return arbitrary depth if slope is flat.

      IF( Slope .LT. 0.00001 ) THEN
        Norm = 42.0
        WRITE(*,*) '  ***Error(Norm) flat slope...'
        WRITE(*,*) ' Channel = ',CurrentChannel(),' X = ',
     #    X,' assigned H = ',Norm
        WRITE(*,*) ' Q = ',Q,'  Slope = ',Slope
        RETURN
      END IF

      K = Q/SQRT(Slope)
      NewH = 1.0
      LowH = 0.0
      
*---- Bracket normal depth.

      Attempts = 0
 50   CONTINUE
 
      IF( Attempts .LT. MaxAttempts ) THEN
        Attempts = Attempts + 1
        NewK = Conveyance(X,NewH)
        IF( NewK .LT. K ) THEN
          NewH = 2.0 * NewH
          GO TO 50
        END IF
      ELSE
        WRITE(*,*) ' *** Error (Norm)'
        WRITE(*,*) ' Failed to bracket normal depth.'
        WRITE(*,*) ' Channel = ',CurrentChannel(),
     #  ' X = ',X,' Last H = ',NewH
        WRITE(*,*) ' Q = ',Q,'  Slope = ',Slope
        Norm = NewH
        RETURN
      END IF
      
*---- Locate normal depth between brackets.

      Attempts = 0
      HighH = NewH
 60   CONTINUE
      IF( Attempts .LT. MaxAttempts ) THEN

        NewH = 0.5 * (LowH + HighH)
        NewK = Conveyance(X,NewH)
        
        IF( NewK .GT. K ) THEN
          HighH = NewH
        ELSE
          LowH = NewH
        END IF
        
        IF( ABS( HighH - LowH ) .GT. Tolerance ) THEN
          GO TO 60
        ELSE
          Norm = NewH
          RETURN
        END IF
          

      ELSE
        WRITE(*,*) ' ***Warning(Norm)'
        WRITE(*,*) ' Failed to accurately locate normal depth'
        WRITE(*,*) ' by a bisection method..., continuing...'
        WRITE(*,*) ' Channel = ',CurrentChannel(),
     #  ' X = ',X,' Last H = ',NewH
        WRITE(*,*) ' Q = ',Q,'  Slope = ',Slope
       END IF      
      


      DO 100 I=1,MaxAttempts

        dH = (K-Conveyance(X,NewH))/dConveyance(X,NewH)
        NewH  = NewH+dH
*       WRITE(*,*) I,'...',NewH
        IF(ABS(dH).LT.0.01) GO TO 102

 100  CONTINUE

      WRITE(*,*) '  ***Warning(Norm) did not close...'
      WRITE(*,*) ' Channel = ',CurrentChannel(),' X = ',X,' H = ',NewH
      WRITE(*,*) ' Q = ',Q,'  Slope = ',Slope

 102  CONTINUE

      Norm = NewH

      RETURN
      END



*== Public (Bernie) ================================================

      LOGICAL FUNCTION Bernie()

      IMPLICIT NONE

*   Purpose:  Roughly approximate steady flow within a 1-D channel,
*             using Bernoulli's equation (specific energy).  Lateral
*             flow is ignored, and alpha is assumed equal to 1.

*   Arguments:

*   Argument definitions:

*   Module data:

*   Local Variables:
      INTEGER I, J, CxNo, Subdivisions, Channel
      INTEGER MaxIterations, Iterations
      REAL    X, Xd, Z, Zd, Q, e0, c, dZ
      REAL    Tolerance, dX, Change
      REAL    Xup, Xdn, Zup, Zdn, e0i, WSSlope
      REAL    SmallestdX
      LOGICAL OK, Print, CheckConvergence, PrintMore

*   Routines by module:

***** Local:
      REAL     StreamEnergy, dStreamEnergydZ, StreamResistance
      EXTERNAL StreamEnergy, dStreamEnergydZ, StreamResistance
      REAL     dStreamResistancedZu
      EXTERNAL dStreamResistancedZu
      LOGICAL  SetdXBernie
      EXTERNAL SetdXBernie
      
***** Channel status:
      REAL     StreamFlow, StreamSurfaceElevation
      EXTERNAL StreamFlow, StreamSurfaceElevation
      LOGICAL  SetStreamSurfaceElevation
      EXTERNAL SetStreamSurfaceElevation
      REAL     StreamDepth
      EXTERNAL StreamDepth
      
***** Channel schematic:
      INTEGER  NumberOfStreamLocations, CurrentChannel
      EXTERNAL NumberOfStreamLocations, CurrentChannel
      REAL     StreamDistance
      EXTERNAL StreamDistance

***** Network control:
      REAL     ToleranceStreamZ
      EXTERNAL ToleranceStreamZ
      INTEGER  NetworkPrintLevel, MaxNetworkIterations
      EXTERNAL NetworkPrintLevel, MaxNetworkIterations
      
*   Intrinsics:
      INTEGER   INT
      REAL      ABS, FLOAT
      INTRINSIC INT, ABS, FLOAT
      

*   Programmed by: Lew DeLong
*   Date:          Sept  1993
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      Bernie = .FALSE.

      MaxIterations = MaxNetworkIterations()
      Tolerance = ToleranceStreamZ()
      Channel   = CurrentChannel()

      IF( NetworkPrintLevel() .GT. 1  ) THEN
        Print = .TRUE.
        PrintMore = .TRUE.
      ELSE IF( NetworkPrintLevel() .GT. 0  ) THEN
        PrintMore = .FALSE.
        Print = .TRUE.
      ELSE     
        Print = .FALSE.
        PrintMore = .FALSE.
      END IF
      
      IF( Print ) THEN
        WRITE(*,*) ' '
        WRITE(*,*) ' Steady approximation, Channel...',Channel
      END IF
      
*---- Compute energy at downstream end of channel.

      CxNo = NumberOfStreamLocations()
      Xd = StreamDistance(CxNo)
      Q  = StreamFlow(CxNo)
      Zd = StreamSurfaceElevation(CxNo)
      

      IF( Q .GT. 0.0 ) THEN
        e0 = StreamEnergy(Xd,Q,Zd)
      ELSE
        WRITE(*,*) ' #### Warning(Bernie)...'
        WRITE(*,*) ' Channel...',Channel
        WRITE(*,*) ' Steady-state approximation not implemented for',
     #             ' negative flow.'
        RETURN
      END IF
      
      SmallestdX = Xd - StreamDistance(CxNo-1)

*---- Set initial dX.

      dX = SmallestdX
      
*---- Begin computational-location loop, at downstream end of channel.

      DO 300 I=2,NumberOfStreamLocations()
      
*------ Set reference distance and WSElev at adjacent upstream
*       computational location.

        CxNo = CxNo -1
      
        X = StreamDistance(CxNo)
        Z = StreamSurfaceElevation(CxNo)
        
*------ Begin spatial-convergence loop.
        
        CheckConvergence = .FALSE.

  50    CONTINUE
  
*------ Estimate initial water-surface slope.

        IF( I .GT. 2 ) THEN
          WSSlope = ( StreamSurfaceElevation(CxNo+2) - Zd )
     #                  /( StreamDistance(CxNo+2) - Xd)

        ELSE
          WSSlope = (Zd - Z) / (Xd - X)
        END IF

*------ Determine number of subdivisions and corresponding dX.

        Subdivisions = INT( (Xd-X) / dX )
        IF( Subdivisions .LT. 1 ) THEN
          Subdivisions = 1
        END IF
        dX = (Xd-X) / FLOAT(Subdivisions)
        
*------ Initialize values for subdivision loop.  Note, different
*       variables are used in the subdivision loop to simplify 
*       restarting the loop from the same initial values when required.
        
        Xup = Xd
        Zup = Zd
        e0i = e0
        
*------ Begin subdivision loop.

        DO 200 J=1,Subdivisions
        
*-------- Initialize values for the iteration loop.

          Xdn = Xup
          Xup = Xup - dX
          Zdn = Zup
          Zup = Zdn - dX * WSSlope
          Iterations = 0

*-------- Begin iteration loop within the subdivision loop.

 100      CONTINUE
          Iterations = Iterations + 1

*-------- Begin computation of coefficients.

          c = dStreamEnergydZ(Xup,Q,Zup)
     #      - dStreamResistancedZu(Xup,Xdn,Zup,Zdn,Q)
        
*-------- Update estimate of Z at the upstream end
*         of current the subdivision.

          dZ = ( 
     #             e0i-StreamEnergy(Xup,Q,Zup)
     #           + StreamResistance(Xup,Xdn,Zup,Zdn,Q)
     #         )  / c

          Zup = Zup + dZ

*-------- Check for closure of iteration.

          IF( ABS( dZ ) .GT. Tolerance ) THEN
            IF( Iterations .LT. MaxIterations ) THEN
              GO TO 100
            ELSE

*------------ Subdivide dX, if iteration hasn't closed.

              dX = 0.5 * dX
              IF( dX .GT. 1.0 ) THEN
                GO TO 50
              ELSE
                WRITE(*,*) ' Giving up on channel...',Channel
                WRITE(*,*) ' X...',Xup,'Z...',Zup,
     #                 'Iterations...',Iterations
                IF( dX .LT. SmallestdX ) THEN
                  SmallestdX = dX
                END IF
                OK = SetdXBernie( -SmallestdX )
                RETURN
              END IF

            END IF
          END IF
*-------- End of iteration loop within subdivision loop.

*-------- Estimate new water-surface slope and total energy
*         for use in next subdivision.

          WSSlope = (Zdn - Zup) / (Xdn - Xup)
          e0i = e0i + StreamResistance(Xup,Xdn,Zup,Zdn,Q)
          
 200    CONTINUE
*------ End of subdivision loop.
                  
*------ Assign resulting final WSElev from the subdivision loop
*       to the adjacent upstream computational location.

        Z = Zup
        
*------ Check for convergence.

        Change = ABS(StreamSurfaceElevation(CxNo)-Z)
        OK = SetStreamSurfaceElevation( CxNo, Z )
        
        IF( CheckConvergence ) THEN
        ELSE
          CheckConvergence = .TRUE.
          dX = 0.5 * dX
          GO TO 50
        END IF

        IF( Change .LE. Tolerance ) THEN

*-------- If this is a successful test for convergence, the preceding
*         dX was sufficient ... change dX back to the preceding value.

          IF( CheckConvergence ) THEN
            dX = 2.0 * dX
            IF( dX .LT. SmallestdX ) THEN
              SmallestdX = dX
            END IF
            IF( PrintMore ) THEN
              WRITE(*,*) ' X...',Xup,'  Z...',Zup,'  dX...',dX,
     #                 '  Iterations last step...',Iterations
            END IF
          END IF

        ELSE

*-------- Convergence has not been obtained, reduce dX.

          dX = 0.5 * dX
          IF( dX .GT. 1.0 ) THEN
            GO TO 50
          ELSE
            WRITE(*,*) ' X...', X
            WRITE(*,*) ' Steady solution failed to converge...'
            WRITE(*,*) ' change, last iteration, ...',Change
            WRITE(*,*) ' giving up on this channel...'
            WRITE(*,*) ' '
            OK = SetdXBernie( -dX )
            RETURN
          END IF

        END IF
*------ End of spatial-convergence loop.

*------ Reassign values for the next computational location.

        e0 = e0i
        Xd = X
        Zd = Z
        

 300  CONTINUE
*---- End of computational-location loop.

      
      Bernie = .TRUE.
      
      OK = SetdXBernie( SmallestdX )

      RETURN
      END



*== Public (StreamEnergy) ================================================

      REAL FUNCTION StreamEnergy( X, Q, Z )

      IMPLICIT NONE

*   Purpose:  Approximate specific energy in the current channel at X,
*             given flow (Q) and water-surface elevation (Z).

*   Arguments:
      REAL X, Q, Z
      
*   Argument definitions:
*     X - downstream distance in current channel.
*     Q - discharge.
*     Z - water-surface elevation.

*   Module data:

*   Local Variables:
      REAL Depth, Velocity, G

*   Routines by module:

***** Local:

***** Network control:
      REAL     AccelerationGravity
      EXTERNAL AccelerationGravity
        
***** Channel properties:
      REAL     CxArea, BtmElev
      EXTERNAL CxArea, BtmElev

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Sept  1993
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      Depth = Z - BtmElev( X )
      IF( Depth .GT. 0.0 ) THEN
        G = AccelerationGravity()
        Velocity = Q / CxArea( X, Depth )
        StreamEnergy = Z + Velocity**2 / (2.0 * G)
      ELSE
        StreamEnergy = 0.0
      END IF

      RETURN
      END



*== Public (dStreamEnergydZ) ================================================

      REAL FUNCTION dStreamEnergydZ( X, Q, Z )

      IMPLICIT NONE

*   Purpose:  Approximate the gradient of specific energy
*             with depth of flow.

      REAL X, Q, Z
      
*   Argument definitions:
*     X - downstream distance in current channel.
*     Q - discharge.
*     Z - water-surface elevation.

*   Module data:

*   Local Variables:
      REAL G, A, Depth, Velocity

*   Routines by module:

***** Local:

***** Network control:
      REAL     AccelerationGravity
      EXTERNAL AccelerationGravity
        
***** Channel properties:
      REAL     CxArea, ChannelWidth, BtmElev
      EXTERNAL CxArea, ChannelWidth, BtmElev

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Sept  1993
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      Depth = Z - BtmElev( X )
      IF( Depth .GT. 0.0 ) THEN
        G = AccelerationGravity()
        A = CxArea( X, Depth )
        Velocity = Q / A
        dStreamEnergydZ = 1.0
     #        - ChannelWidth( X,Depth ) * Velocity**2 /(A*G)
      ELSE
        dStreamEnergydZ = 0.0
      END IF


      RETURN
      END



*== Public (StreamResistance) ================================================

      REAL FUNCTION StreamResistance( Xu, Xd, Zu, Zd, Q )

      IMPLICIT NONE

*   Purpose:  Approximate head loss due to flow resistance between
*             an upstream and downstream cross section.

*   Arguments:
      REAL Xu, Xd, Zu, Zd, Q
      
*   Argument definitions:
*     Xu - down stream distance to upstream cross section.
*     Xd - down stream distance to downstream cross section.
*     Zu - upstream water-surface elevation.
*     Zd - downstream water-surface elevation.
*     Q  - discharge.

*   Module data:
      INCLUDE 'network.inc'

*   Local Variables:
      INTEGER K, QuadPts
      REAL N( MaxQuadPts )
      REAL Hd, Hu
      REAL H, X, QuadWt, QuadPt, fric

*   Routines by module:

***** Local:
        
***** Channel properties:
      REAL     BtmElev, Conveyance
      EXTERNAL BtmElev, Conveyance
      
***** Network control:
      INTEGER  NetworkQuadPts
      EXTERNAL NetworkQuadPts, NetworkQuadPtWt

*   Intrinsics:


*   Programmed by: Lew DeLong
*   Date:          Sept  1993
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      Hu = Zu - BtmElev( Xu )
      Hd = Zd - BtmElev( Xd )
      
      fric = 0.0
      QuadPts = NetworkQuadPts()

      DO 100 K=1,QuadPts

*-------- Estimate quadrature-point values.

            CALL NetworkQuadPtWt( K, QuadPt, QuadWt )

*---------- Interpolation functions.
            N(1) = 1.0 - QuadPt 
            N(2) = QuadPt

*---------- Location of quadrature point.
            X = N(1) * Xu + N(2) * Xd

*---------- Dependent variables.
            H = N(1) * Hu + N(2) * Hd
            
        IF( H .GT. 0.0 ) THEN
          fric = fric + QuadWt / ( Conveyance(X,H) ** 2 )
        END IF
      
 100  CONTINUE
 
      StreamResistance = Q * ABS( Q ) * fric * ( Xd - Xu )

      RETURN
      END



*== Public (dStreamResistancedZu) ================================================

      REAL FUNCTION dStreamResistancedZu( Xu, Xd, Zu, Zd, Q )

      IMPLICIT NONE

*   Purpose:  Approximate gradient, with respect to a change in upstream
*             water-surface elevation, of head loss due to flow
*             resistance between an upstream and downstream cross section.

*   Arguments:
      REAL Xu, Xd, Zu, Zd, Q
      
*   Argument definitions:
*     Xu - down stream distance to upstream cross section.
*     Xd - down stream distance to downstream cross section.
*     Zu - upstream water-surface elevation.
*     Zd - downstream water-surface elevation.
*     Q  - discharge.

*   Module data:
      INCLUDE 'network.inc'

*   Local Variables:
      INTEGER K, QuadPts
      REAL N( MaxQuadPts )
      REAL Hd, Hu
      REAL H, X, QuadWt, QuadPt, fric

*   Routines by module:

***** Local:
        
***** Channel properties:
      REAL     BtmElev, Conveyance, dConveyance
      EXTERNAL BtmElev, Conveyance, dConveyance
      
***** Network control:
      INTEGER  NetworkQuadPts
      EXTERNAL NetworkQuadPts, NetworkQuadPtWt

*   Intrinsics:


*   Programmed by: Lew DeLong
*   Date:          Sept  1993
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      Hu = Zu - BtmElev( Xu )
      Hd = Zd - BtmElev( Xd )
      
      fric = 0.0
      QuadPts = NetworkQuadPts()

      DO 100 K=1,QuadPts

*-------- Estimate quadrature-point values.

            CALL NetworkQuadPtWt( K, QuadPt, QuadWt )

*---------- Interpolation functions.
            N(1) = 1.0 - QuadPt 
            N(2) = QuadPt

*---------- Location of quadrature point.
            X = N(1) * Xu + N(2) * Xd

*---------- Dependent variables.
            H = N(1) * Hu + N(2) * Hd

*------ Integrate friction term.
            
        IF( H .GT. 0.0 ) THEN
          fric = fric - QuadWt * N(1) *  2.0 * dConveyance(X,H) 
     #                         / ( Conveyance(X,H) ** 3 )
        END IF
      
 100  CONTINUE
 
      dStreamResistancedZu = Q * ABS( Q ) * fric * ( Xd - Xu )

      RETURN
      END


*== Public (GlobalStreamSurfaceSlope) ====================================

      REAL FUNCTION GlobalStreamSurfaceSlope(LocationNumber)

      IMPLICIT NONE

*   Purpose:  Return current value of water-surface slope
*             at global location corresponding to LocationNumbe.

*   Arguments:
      INTEGER LocationNumber

*   Argument definitions:
*     LocationNumber - global computational-location number.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:

*   Routines by module:

***** Local:
      REAL     GlobalStreamDistance
      EXTERNAL GlobalStreamDistance
      
***** Channel schematic:
      INTEGER  UpstreamPointer, DownstreamPointer
      EXTERNAL UpstreamPointer, DownstreamPointer

*   Programmed by: Lew DeLong
*   Date:          September 1993
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------


      IF( LocationNumber .EQ. UpstreamPointer() ) THEN

*------ Upstream end of channel.

          GlobalStreamSurfaceSlope =
     #      ( WS( LocationNumber + 1 )  - WS( LocationNumber ) )
     #                             /
     #      (
     #          GlobalStreamDistance( LocationNumber + 1 )
     #        - GlobalStreamDistance( LocationNumber     )
     #      )


       ELSE IF( LocationNumber .EQ.  DownstreamPointer() ) THEN

*------ Downstream end of channel.

          GlobalStreamSurfaceSlope =
     #                         (
     #        WS( LocationNumber     )
     #      - WS( LocationNumber - 1 )
     #                         )  /  (
     #        GlobalStreamDistance( LocationNumber     )
     #      - GlobalStreamDistance( LocationNumber - 1 )
     #                               )
          
      ELSE
        
*-------- Intervening reaches of channel.

          GlobalStreamSurfaceSlope =
     #                         (
     #        WS( LocationNumber + 1 )
     #      - WS( LocationNumber - 1 )
     #                         )  /  (
     #        GlobalStreamDistance( LocationNumber + 1 )
     #      - GlobalStreamDistance( LocationNumber - 1 )
     #                               )
     
      END IF


      RETURN
      END



*== Public (SetdXBernie) ============================================

      LOGICAL FUNCTION SetdXBernie( Value )

      IMPLICIT NONE

*   Purpose:  Record value of incremental stream distance required
*             for solution of Bernouilli's equation for steady flow
*             in the current channel.

*   Arguments:
      REAL    Value

*   Argument definitions:
*     Value - incremental stream distance required
*             for solution of Bernouilli's equation for steady flow
*             in the current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:

*   Routines by module:

***** Channel schematic:
      INTEGER  CurrentChannel
      EXTERNAL CurrentChannel


*   Programmed by: Lew DeLong
*   Date:          Sept 1993
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      IF( Value .GT. 0.0 ) THEN
        dXBernie( CurrentChannel() ) = Value
        ConvergedSteady(CurrentChannel()) = .TRUE.
      ELSE
        dXBernie( CurrentChannel() ) = -Value
        ConvergedSteady(CurrentChannel()) = .FALSE.
      END IF
      
      SetdXBernie = .TRUE.

      RETURN
      END



*== Public (SetdXForce) ============================================

      LOGICAL FUNCTION SetdXForce( Value )

      IMPLICIT NONE

*   Purpose:  Record last value of incremental stream distance used
*             for solution of specific-force equation for steady flow
*             in the current channel.

*   Arguments:
      REAL    Value

*   Argument definitions:
*     Value - last incremental stream distance used
*             for solution of specific-force equation for steady flow
*             in the current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chstatus.inc'

*   Local Variables:

*   Routines by module:

***** Channel schematic:
      INTEGER  CurrentChannel
      EXTERNAL CurrentChannel


*   Programmed by: Lew DeLong
*   Date:          Sept 1993
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      IF( Value .GT. 0.0 ) THEN
        dXForce( CurrentChannel() ) = Value
        ConvergedSteady(CurrentChannel()) = .TRUE.
      ELSE
        dXForce( CurrentChannel() ) = -Value
        ConvergedSteady(CurrentChannel()) = .FALSE.
      END IF
      
      SetdXForce = .TRUE.

      RETURN
      END



*== Public (SpForce) ================================================

      LOGICAL FUNCTION SpForce()

      IMPLICIT NONE

*   Purpose:  Roughly approximate steady flow within a 1-D channel,
*             using specific-force equation.  Lateral flow is
*             ignored.

*   Arguments:

*   Argument definitions:

*   Module data:

*   Local Variables:
      INTEGER I, J, CxNo, Subdivisions, Channel
      INTEGER MaxIterations, Iterations
      REAL    X, Xd, Z, Zd, Q, e0, c, dZ
      REAL    Tolerance, dX, Change
      REAL    Xup, Xdn, Zup, Zdn, e0i, WSSlope
      LOGICAL Errors, OK, Print, CheckConvergence

*   Routines by module:

***** Local:
      REAL     StreamMoFlux, dStreamMoFluxdZ, gAdZdX, gASf
      EXTERNAL StreamMoFlux, dStreamMoFluxdZ, gAdZdX, gASf
      REAL     dgAdZdXdZu, dgASfdZu
      EXTERNAL dgAdZdXdZu, dgASfdZu
      LOGICAL  SetdXForce
      EXTERNAL SetdXForce
      
***** Channel status:
      REAL     StreamFlow, StreamSurfaceElevation
      EXTERNAL StreamFlow, StreamSurfaceElevation
      LOGICAL  SetStreamSurfaceElevation
      EXTERNAL SetStreamSurfaceElevation
      REAL     StreamDepth
      EXTERNAL StreamDepth
      
***** Channel schematic:
      INTEGER  NumberOfStreamLocations, CurrentChannel
      EXTERNAL NumberOfStreamLocations, CurrentChannel
      REAL     StreamDistance
      EXTERNAL StreamDistance

***** Network control:
      REAL     ToleranceStreamZ
      EXTERNAL ToleranceStreamZ
      INTEGER  NetworkPrintLevel, MaxNetworkIterations
      EXTERNAL NetworkPrintLevel, MaxNetworkIterations
      
*   Intrinsics:
      INTEGER   INT
      REAL      ABS, FLOAT
      INTRINSIC INT, ABS, FLOAT
      

*   Programmed by: Lew DeLong
*   Date:          Sept  1993
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      SpForce = .FALSE.
      Errors = .FALSE.

      MaxIterations = MaxNetworkIterations()
      Tolerance = ToleranceStreamZ()
      Channel   = CurrentChannel()

      IF( NetworkPrintLevel() .GT. 0  ) THEN
        Print = .TRUE.
      ELSE
        Print = .FALSE.
      END IF
      
      IF( Print ) THEN
        WRITE(*,*) ' '
        WRITE(*,*) ' Steady approximation, Channel...',Channel
      END IF
      
*---- Compute momentum flux at downstream end of channel.

      CxNo = NumberOfStreamLocations()
      Xd = StreamDistance(CxNo)
      Q  = StreamFlow(CxNo)
      Zd = StreamSurfaceElevation(CxNo)
      
      IF( Q .GT. 0.0 ) THEN
        e0 = StreamMoFlux(Xd,Q,Zd)
      ELSE
        WRITE(*,*) ' #### Warning(SpForce)...'
        WRITE(*,*) ' Channel...',Channel
        WRITE(*,*) ' Steady-state approximation not implemented for',
     #             ' negative flow.'
        RETURN
      END IF
      
*---- Compute initial dX.

      dX = Xd - StreamDistance(CxNo-1)
      
*---- Begin computational-location loop, at downstream end of channel.

      DO 300 I=2,NumberOfStreamLocations()
      
*------ Set reference distance and WSElev at adjacent upstream
*       computational location.

        CxNo = CxNo -1
        X = StreamDistance(CxNo)
        Z = StreamSurfaceElevation(CxNo)
        
*------ Begin spatial-convergence loop.
        
        CheckConvergence = .FALSE.

  50    CONTINUE
  
*------ Estimate initial water-surface slope.

        IF( I .GT. 2 ) THEN
          WSSlope = ( StreamSurfaceElevation(CxNo+2) - Zd )
     #                  /( StreamDistance(CxNo+2) - Xd)

        ELSE
          WSSlope = (Zd - Z) / (Xd - X)
        END IF

*------ Determine number of subdivisions and corresponding dX.

        Subdivisions = INT( (Xd-X) / dX )
        IF( Subdivisions .LT. 1 ) THEN
          Subdivisions = 1
        END IF
        dX = (Xd-X) / FLOAT(Subdivisions)
        
*------ Initialize values for subdivision loop.  Note, different
*       variables are used in the subdivision loop to simplify 
*       restarting the loop from the same initial values when required.
        
        Xup = Xd
        Zup = Zd
        e0i = e0
        
*------ Begin subdivision loop.

        DO 200 J=1,Subdivisions
        
*-------- Initialize values for the iteration loop.

          Xdn = Xup
          Xup = Xup - dX
          Zdn = Zup
          Zup = Zdn - dX * WSSlope
          Iterations = 0

*-------- Begin iteration loop within the subdivision loop.

 100      CONTINUE
          Iterations = Iterations + 1

*-------- Begin computation of coefficients.

          c = dStreamMoFluxdZ(Xup,Q,Zup)
     #      - dgASfdZu(Xup,Xdn,Zup,Zdn,Q)
     #      - dgAdZdXdZu(Xup,Xdn,Zup,Zdn)
        
*-------- Update estimate of Z at the upstream end
*         of current the subdivision.

          dZ = ( 
     #             e0i-StreamMoFlux(Xup,Q,Zup)
     #           + gASf(Xup,Xdn,Zup,Zdn,Q)
     #           + gAdZdX(Xup,Xdn,Zup,Zdn)
     #         )  / c

      write(*,*) ' X, Q, Zup...',X, Q, Zup
      write(*,*) ' Xup, Xdn, Zdn...',Xup, Xdn, Zdn
      write(*,*) ' e0i...',e0i
      write(*,*) ' gASf...',gASf(Xup,Xdn,Zup,Zdn,Q)
      write(*,*) ' gAdZdX...',gAdZdX(Xup,Xdn,Zup,Zdn)
      write(*,*) ' StreamMoFlux...',StreamMoFlux(Xup,Q,Zup)
      write(*,*) ' c...',c

          Zup = Zup + dZ

*-------- Check for closure of iteration.

          IF( ABS( dZ ) .GT. Tolerance ) THEN
            IF( Iterations .LT. MaxIterations ) THEN
              GO TO 100
            ELSE
              Errors = .TRUE.

*------------ Subdivide dX, if iteration hasn't closed.

              dX = 0.5 * dX
              IF( dX .GT. 1.0 ) THEN
                GO TO 50
              ELSE
                WRITE(*,*) ' Giving up on channel...',Channel
                OK = SetdXForce( -dX )
                RETURN
              END IF

            END IF
          END IF
*-------- End of iteration loop within subdivision loop.

*-------- Estimate new water-surface slope and momentum flux
*         for use in next subdivision.

          WSSlope = (Zdn - Zup) / (Xdn - Xup)
          e0i = e0i + gASf(Xup,Xdn,Zup,Zdn,Q)
     #              + gAdZdX(Xup,Xdn,Zup,Zdn)
          
 200    CONTINUE
*------ End of subdivision loop.
                  
*------ Assign resulting final WSElev from the subdivision loop
*       to the adjacent upstream computational location.

        Z = Zup
        
*------ Check for convergence.

        Change = ABS(StreamSurfaceElevation(CxNo)-Z)
        OK = SetStreamSurfaceElevation( CxNo, Z )
        
        IF( CheckConvergence ) THEN
        ELSE
          CheckConvergence = .TRUE.
          dX = 0.5 * dX
          GO TO 50
        END IF

        IF( Change .LE. Tolerance ) THEN

*-------- If this is a successful test for convergence, the preceding
*         dX was sufficient ... change dX back to the preceding value.

          IF( CheckConvergence ) THEN
            dX = 2.0 * dX
          END IF

        ELSE

*-------- Convergence has not been obtained, reduce dX.

          Errors = .TRUE.
          dX = 0.5 * dX
          IF( dX .GT. 1.0 ) THEN
            GO TO 50
          ELSE
            WRITE(*,*) ' X...', X
            WRITE(*,*) ' Steady solution failed to converge...'
            WRITE(*,*) ' change, last iteration, ...',Change
            WRITE(*,*) ' giving up on this channel...'
            WRITE(*,*) ' '
            OK = SetdXForce( -dX )
            RETURN
          END IF

        END IF
*------ End of spatial-convergence loop.

*------ Reassign values for the next computational location.

        e0 = e0i
        Xd = X
        Zd = Z
        

 300  CONTINUE
*---- End of computational-location loop.

      IF( Errors ) THEN
        IF( Print ) THEN
          WRITE(*,*)
     #    ' For steady solution, initial dX reduced to ...',dX
        END IF
      END IF
      
      SpForce = .TRUE.
      
      OK = SetdXForce( dX )

      RETURN
      END



*== Public (StreamMoFlux) ================================================

      REAL FUNCTION StreamMoFlux( X, Q, Z )

      IMPLICIT NONE

*   Purpose:  Approximate momentum flux in the current channel at X,
*             given flow (Q) and water-surface elevation (Z).

*   Arguments:
      REAL X, Q, Z
      
*   Argument definitions:
*     X - downstream distance in current channel.
*     Q - discharge.
*     Z - water-surface elevation.

*   Module data:

*   Local Variables:
      REAL H

*   Routines by module:

***** Local:
        
***** Channel properties:
      REAL     CxArea, BtmElev, Beta
      EXTERNAL CxArea, BtmElev, Beta

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Sept  1993
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      H = Z - BtmElev( X )
      IF( H .GT. 0.0 ) THEN
        StreamMoFlux = Beta(X,H) * Q ** 2 / CxArea(X,H)
      ELSE
        StreamMoFlux = 0.0
      END IF

      RETURN
      END



*== Public (dStreamMoFluxdZ) ================================================

      REAL FUNCTION dStreamMoFluxdZ( X, Q, Z )

      IMPLICIT NONE

*   Purpose:  Approximate the gradient of momentum flux
*             with depth of flow.

      REAL X, Q, Z
      
*   Argument definitions:
*     X - downstream distance in current channel.
*     Q - discharge.
*     Z - water-surface elevation.

*   Module data:

*   Local Variables:
      REAL A, H

*   Routines by module:

***** Local:
        
***** Channel properties:
      REAL     CxArea, ChannelWidth, BtmElev, Beta, dBeta
      EXTERNAL CxArea, ChannelWidth, BtmElev, Beta, dBeta

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Sept  1993
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      H = Z - BtmElev( X )
      IF( H .GT. 0.0 ) THEN
        A = CxArea( X, H )
        dStreamMoFluxdZ = Q**2 / A * (
     #        - ChannelWidth( X,H ) * Beta(X,H) / A
     #        + dBeta(X,H)
     #                               )
      ELSE
        dStreamMoFluxdZ = 0.0
      END IF


      RETURN
      END



*== Public (gASf) ================================================

      REAL FUNCTION gASf( Xu, Xd, Zu, Zd, Q )

      IMPLICIT NONE

*   Purpose:  Approximate momentum loss due to flow resistance between
*             an upstream and downstream cross section.

*   Arguments:
      REAL Xu, Xd, Zu, Zd, Q
      
*   Argument definitions:
*     Xu - down stream distance to upstream cross section.
*     Xd - down stream distance to downstream cross section.
*     Zu - upstream water-surface elevation.
*     Zd - downstream water-surface elevation.
*     Q  - discharge.

*   Module data:
      INCLUDE 'network.inc'

*   Local Variables:
      INTEGER K, QuadPts
      REAL N( MaxQuadPts )
      REAL Hd, Hu, G
      REAL H, X, QuadWt, QuadPt, fric

*   Routines by module:

***** Local:
        
***** Channel properties:
      REAL     BtmElev, Conveyance, CxArea
      EXTERNAL BtmElev, Conveyance, CxArea
      
***** Network control:
      INTEGER  NetworkQuadPts
      EXTERNAL NetworkQuadPts, NetworkQuadPtWt

      REAL     AccelerationGravity
      EXTERNAL AccelerationGravity

*   Intrinsics:


*   Programmed by: Lew DeLong
*   Date:          Sept  1993
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      Hu = Zu - BtmElev( Xu )
      Hd = Zd - BtmElev( Xd )
      
      G = AccelerationGravity()
      
      fric = 0.0
      QuadPts = NetworkQuadPts()

      DO 100 K=1,QuadPts

*-------- Estimate quadrature-point values.

            CALL NetworkQuadPtWt( K, QuadPt, QuadWt )

*---------- Interpolation functions.
            N(1) = 1.0 - QuadPt 
            N(2) = QuadPt

*---------- Location of quadrature point.
            X = N(1) * Xu + N(2) * Xd

*---------- Dependent variables.
            H = N(1) * Hu + N(2) * Hd
            
        IF( H .GT. 0.0 ) THEN
          fric = fric + QuadWt * CxArea(X,H) / ( Conveyance(X,H) ** 2 )
        END IF
      
 100  CONTINUE
 
      gASf = G * Q * ABS( Q ) * fric * ( Xd - Xu )

      RETURN
      END



*== Public (dgASfdZu) ================================================

      REAL FUNCTION dgASfdZu( Xu, Xd, Zu, Zd, Q )

      IMPLICIT NONE

*   Purpose:  Approximate gradient, with respect to a change in upstream
*             water-surface elevation, of momentum loss due to flow
*             resistance between an upstream and downstream cross section.

*   Arguments:
      REAL Xu, Xd, Zu, Zd, Q
      
*   Argument definitions:
*     Xu - down stream distance to upstream cross section.
*     Xd - down stream distance to downstream cross section.
*     Zu - upstream water-surface elevation.
*     Zd - downstream water-surface elevation.
*     Q  - discharge.

*   Module data:
      INCLUDE 'network.inc'

*   Local Variables:
      INTEGER K, QuadPts
      REAL N( MaxQuadPts )
      REAL Hd, Hu, G
      REAL H, X, QuadWt, QuadPt, fric
      REAL Conv

*   Routines by module:

***** Local:
        
***** Channel properties:
      REAL     BtmElev, Conveyance, dConveyance
      EXTERNAL BtmElev, Conveyance, dConveyance
      
      REAL     CxArea, ChannelWidth
      EXTERNAL CxArea, ChannelWidth
      
***** Network control:
      INTEGER  NetworkQuadPts
      EXTERNAL NetworkQuadPts, NetworkQuadPtWt

      REAL     AccelerationGravity
      EXTERNAL AccelerationGravity

*   Intrinsics:


*   Programmed by: Lew DeLong
*   Date:          Sept  1993
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      Hu = Zu - BtmElev( Xu )
      Hd = Zd - BtmElev( Xd )
      
      G = AccelerationGravity()
      
      fric = 0.0
      QuadPts = NetworkQuadPts()

      DO 100 K=1,QuadPts

*-------- Estimate quadrature-point values.

            CALL NetworkQuadPtWt( K, QuadPt, QuadWt )

*---------- Interpolation functions.
            N(1) = 1.0 - QuadPt 
            N(2) = QuadPt

*---------- Location of quadrature point.
            X = N(1) * Xu + N(2) * Xd

*---------- Dependent variables.
            H = N(1) * Hu + N(2) * Hd

*------ Integrate friction term.
            
        IF( H .GT. 0.0 ) THEN
          Conv = Conveyance(X,H)
          fric = fric + QuadWt * N(1) *  (
     #          - 2.0 * dConveyance(X,H) * CxArea(X,H) / Conv
     #          + ChannelWidth(X,H)
     #                                   )
     #                         / ( Conv ** 2 )

        END IF
      
 100  CONTINUE
 
      dgASfdZu = G * Q * ABS( Q ) * fric * ( Xd - Xu )

      RETURN
      END



*== Public (gAdZdX) ================================================

      REAL FUNCTION gAdZdX( Xu, Xd, Zu, Zd )

      IMPLICIT NONE

*   Purpose:  Approximate momentum gain due to WS slope between
*             an upstream and downstream cross section.

*   Arguments:
      REAL Xu, Xd, Zu, Zd
      
*   Argument definitions:
*     Xu - down stream distance to upstream cross section.
*     Xd - down stream distance to downstream cross section.
*     Zu - upstream water-surface elevation.
*     Zd - downstream water-surface elevation.

*   Module data:
      INCLUDE 'network.inc'

*   Local Variables:
      INTEGER K, QuadPts
      REAL N( MaxQuadPts )
      REAL dNdX( MaxQuadPts )
      REAL Hd, Hu, G
      REAL H, X, QuadWt, QuadPt, Slope, A

*   Routines by module:

***** Local:
        
***** Channel properties:
      REAL     BtmElev, CxArea
      EXTERNAL BtmElev, CxArea
      
***** Network control:
      INTEGER  NetworkQuadPts
      EXTERNAL NetworkQuadPts, NetworkQuadPtWt

      REAL     AccelerationGravity
      EXTERNAL AccelerationGravity

*   Intrinsics:


*   Programmed by: Lew DeLong
*   Date:          Sept  1993
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      Hu = Zu - BtmElev( Xu )
      Hd = Zd - BtmElev( Xd )
            
      G = AccelerationGravity()
      
      A = 0.0
      QuadPts = NetworkQuadPts()

      DO 100 K=1,QuadPts

*-------- Estimate quadrature-point values.

            CALL NetworkQuadPtWt( K, QuadPt, QuadWt )

*---------- Interpolation functions.
            N(1) = 1.0 - QuadPt 
            N(2) = QuadPt
            dNdX(1) = -1.0 / (Xu - Xd)
            dNdX(2) = - dNdX(1)

*---------- Location of quadrature point.
            X = N(1) * Xu + N(2) * Xd

*---------- Dependent variables.
            H = N(1) * Hu + N(2) * Hd
            
*---------- WS Slope.

            Slope = dNdX(1) * Zu + dNdX(2) * Zd

        IF( H .GT. 0.0 ) THEN
          A = A + QuadWt * CxArea(X,H) * Slope 
        END IF
      
 100  CONTINUE
 
      gAdZdX = G * A * (Xd - Xu)

      RETURN
      END



*== Public (dgAdZdXdZu) ================================================

      REAL FUNCTION dgAdZdXdZu( Xu, Xd, Zu, Zd )

      IMPLICIT NONE

*   Purpose:  Approximate gradient, with respect to a change in upstream
*             water-surface elevation, of momentum gain due to WS
*             slope between an upstream and downstream cross section.

*   Arguments:
      REAL Xu, Xd, Zu, Zd
      
*   Argument definitions:
*     Xu - down stream distance to upstream cross section.
*     Xd - down stream distance to downstream cross section.
*     Zu - upstream water-surface elevation.
*     Zd - downstream water-surface elevation.

*   Module data:
      INCLUDE 'network.inc'

*   Local Variables:
      INTEGER K, QuadPts
      REAL N( MaxQuadPts )
      REAL dNdX( MaxQuadPts )
      REAL Hd, Hu, G
      REAL H, X, QuadWt, QuadPt, A, Slope

*   Routines by module:

***** Local:
        
***** Channel properties:
      REAL     BtmElev
      EXTERNAL BtmElev
      
      REAL     CxArea, ChannelWidth
      EXTERNAL CxArea, ChannelWidth
      
***** Network control:
      INTEGER  NetworkQuadPts
      EXTERNAL NetworkQuadPts, NetworkQuadPtWt

      REAL     AccelerationGravity
      EXTERNAL AccelerationGravity

*   Intrinsics:


*   Programmed by: Lew DeLong
*   Date:          Sept  1993
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      Hu = Zu - BtmElev( Xu )
      Hd = Zd - BtmElev( Xd )
      
      G = AccelerationGravity()
      
      QuadPts = NetworkQuadPts()

      A = 0.0
      DO 100 K=1,QuadPts

*-------- Estimate quadrature-point values.

            CALL NetworkQuadPtWt( K, QuadPt, QuadWt )

*---------- Interpolation functions.
            N(1) = 1.0 - QuadPt 
            N(2) = QuadPt
            dNdX(1) = -1.0 / (Xd - Xu)
            dNdX(2) = - dNdX(1)
            
            

*---------- Location of quadrature point.
            X = N(1) * Xu + N(2) * Xd

*---------- Dependent variables.
            H = N(1) * Hu + N(2) * Hd
            
*---------- WS Slope.

            Slope = dNdX(1) * Zu + dNdX(2) * Zd

*------ Integrate friction term.
            
        IF( H .GT. 0.0 ) THEN
          A = A + QuadWt * (
     #          N(1) * ChannelWidth(X,H) * Slope 
     #        + dNdX(1) * CxArea(X,H)
     #                     )

        END IF
      
 100  CONTINUE
 
      dgAdZdXdZu = G * A * (Xd - Xu)

      RETURN
      END


*==== EOF chstatus ======================================================
