*==== BOF fileutil ============================================================
*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
*
*     This file contains utility functions for manipulating FORTRAN files.
*
*
*     Function Name                 Description
*     -------------                 -----------
*
*     (OpenNewText) - Open a new text file for output and delete the old file
*                       if one exists.
*     (OpenNewBin)  - Open a new binary file for output and delete the old file
*                       if one exists.
*     (OpenDirect)  - Open a direct access file and create it if it doesn't
*                       exist.
*     (OpenOldText) - Open an old text file for input.
*     (OpenOldBin)  - Open an old binary file for input.
*     (OpenAppText) - Open an existing sequential text file for output, place
*                       new output at the end of the file.
*     (OpenAppBin)  - Open an existing sequential binary file for output, place
*                       new output at the end of the file.
*     (DeleteFile)  - Delete an existing file.
*     (FileExist)   - Does the file name exist?
*
*     * = not written yet.
*
*
*
*     Function Name     Type
*     -------------     ----
*
*     OpenNewText   -  LOGICAL
*     OpenNewBin    -  LOGICAL
*     OpenDirect    -  LOGICAL
*     OpenOldText   -  LOGICAL
*     OpenOldBin    -  LOGICAL
*     OpenAppText   -  LOGICAL
*     OpenAppBin    -  LOGICAL
*     DeleteFile    -  LOGICAL
*     FileExist     -  LOGICAL
*
*
*     Input:
*     ------
*
*     The open functions expect the following arguments:
*
*       UnitNumber  - an INTEGER FORTRAN unit number, and
*       FileName    - a variable dimension character string containing the
*                       name of the file to be manipulated.
*
*     In addition, the direct access file opening function expects:
*
*       RecordLength - an INTEGER logical record length, and
*       Formatted    - a LOGICAL variable indicating whether the file is
*                      to be FORMATTED (.TRUE.) or UNFORMATTED (.FALSE.;
*                      binary).
*
*     DeleteFile and FileExist expect only one argument:
*
*        FileName - a variable dimension character string containing the
*                   name of the file to be deleted or tested for existence.
*
*
*     Output:
*     -------
*
*     Each function returns .TRUE. if the operation was successful and
*       .FALSE. if unsuccessful.
*
*
*     Function Prototypes:
*     --------------------
*
*     LOGICAL FUNCTION OpenNewText (
*    I                              UnitNumber, FileName
*    *                             )
*
*     LOGICAL FUNCTION OpenNewBin (
*    I                             UnitNumber, FileName
*    *                            )
*
*     LOGICAL FUNCTION OpenDirect (
*    I                             UnitNumber, FileName, RecordLength,
*    I                             Formatted
*    *                            )
*
*     LOGICAL FUNCTION OpenOldText (
*    I                              UnitNumber, FileName
*    *                             )
*
*     LOGICAL FUNCTION OpenAppText (
*    I                              UnitNumber, FileName
*    *                             )
*
*     LOGICAL FUNCTION OpenAppBin (
*    I                             UnitNumber, FileName
*    *                            )
*
*     LOGICAL FUNCTION OpenOldBin (
*    I                             UnitNumber, FileName
*    *                            )
*
*     LOGICAL FUNCTION DeleteFile (
*    I                             FileName
*    *                            )
*
*     LOGICAL FUNCTION FileExist (
*    I                              FileName
*    *                           )
*
*
*     Notes:
*     ------
*
*     These utilities have not been tested on the Prime mini as of
*       Fri 11-16-1990.  Therefore, there may be undetermined bugs to be
*       corrected for successful operations on the Prime.  In addition, it is
*       not clear that these routines will function on platforms other than
*       PCompatibles using MickeySoft FORTRAN.  Tests will be required.
*
*   Programmed by: Dave Thompson
*   Date:          Mon  12-17-1990
*   Modified by:   Dave Thompson
*   Last modified: Tue  05-21-1991
*
*
*   History:
*   --------
*
*   Tue  05-21-1991  -- Added IMPLICIT NONE to all subprograms in this unit.
*
*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

C== Public (OpenNewText) ======================================================

      LOGICAL   FUNCTION OpenNewText (
     I                                UnitNumber, FileName
     *                               )

      IMPLICIT NONE

C   Purpose:  Open a new text file, deleting any existing file with the
C             same name.

C   Arguments:

      INTEGER       UnitNumber
      CHARACTER*(*) FileName
C   Argument Definitions:
C        UnitNumber  - an INTEGER FORTRAN unit number, and
C        FileName    - a variable dimension character string containing the
C                       name of the file to be manipulated.
    
C   Local Variables:

      INTEGER   IOStatus

C   Functions:

      LOGICAL   DeleteFile
      EXTERNAL  DeleteFile

C   Programmed by: Dave Thompson
C   Date:          September 1989
C   Modified by:   Dave Thompson
C   Last modified: Wed  12-12-1990
*

C-- Implementation ------------------------------------------------------------

C---- Debugging information

C     WRITE(*,*) ' In routine OpenNewText'
C     WRITE(*,*) ' Incoming file = ', FileName
C     WRITE(*,*) ' Unit number = ', UnitNumber

C---- Try to delete the file (assuming it exists).

      IF (DeleteFile(FileName)) THEN

C       WRITE(*,*) ' IOStatus = ',IOS

        OPEN (
     *        UNIT   = UnitNumber,
     *        FILE   = FileName,
     *        ACCESS = 'SEQUENTIAL',
     *        FORM   = 'FORMATTED',
     *        IOSTAT = IOStatus
     *       )
        IF     (IOStatus .EQ. 0) THEN

          OpenNewText = .TRUE.

        ELSEIF (IOStatus .NE. 0) THEN

          OpenNewText = .FALSE.

        ENDIF

      ELSE

        WRITE(*,*) ' File not deleted (probably doesn''t exist)'
C       WRITE(*,*) ' IOStatus = ',IOS

        OpenNewText = .FALSE.

      ENDIF

      RETURN
      END

C== Public (OpenNewBin) =======================================================

      LOGICAL   FUNCTION OpenNewBin (
     I                               UnitNumber, FileName
     *                              )

      IMPLICIT NONE

C   Purpose:  Open a new binary file, deleting any existing file with the
C               same name.

C   Arguments:

      INTEGER       UnitNumber
      CHARACTER*(*) FileName
      
C   Argument Definitions:
C        UnitNumber  - an INTEGER FORTRAN unit number, and
C       FileName    - a variable dimension character string containing the
C                       name of the file to be manipulated.
C
C   Local Variables:

      INTEGER IOStatus

C   Functions:

      LOGICAL   DeleteFile
      EXTERNAL  DeleteFile

C   Programmed by: Dave Thompson
C   Date:          April 1990
C   Modified by:   Dave Thompson
C   Last modified: Wed  12-12-1990
*

C-- Implementation ------------------------------------------------------------

C---- Debugging information

C     WRITE(*,*) ' In routine OpenNewBin'
C     WRITE(*,*) ' Incoming file = ', FileName
C     WRITE(*,*) ' Unit number   = ', UnitNumber

C---- Try to delete the file (assuming it exists).

      IF (DeleteFile(FileName)) THEN

C       WRITE(*,*) ' File deleted'
C       WRITE(*,*) ' IOStatus = ',IOStatus

        OPEN (
     *        UNIT   = UnitNumber,
     *        FILE   = FileName,
     *        FORM   = 'UNFORMATTED',
     *        ACCESS = 'SEQUENTIAL',
     *        IOSTAT = IOStatus
     *       )
        IF     (IOStatus .EQ. 0) THEN

          OpenNewBin = .TRUE.

        ELSEIF (IOStatus .NE. 0) THEN

          OpenNewBin = .FALSE.

        ENDIF

      ELSE

C       WRITE(*,*) ' File not deleted (probably doesn''t exist)'
C       WRITE(*,*) ' IOStatus = ',IOS

        OpenNewBin = .FALSE.

      ENDIF

      RETURN
      END

C== Public (OpenDirect) =======================================================

      LOGICAL   FUNCTION OpenDirect (
     I                               UnitNumber, FileName, RecordLength,
     I                               Formatted
     *                              )

      IMPLICIT NONE

C   Purpose:  Open a direct access file and specify whether the output is to
C             be formatted or unformatted.

C   Arguments:

      INTEGER       UnitNumber, RecordLength
      CHARACTER*(*) FileName
      LOGICAL       Formatted

C   Argument Definitions:
C        UnitNumber  - an INTEGER FORTRAN unit number, and
C        FileName    - a variable dimension character string containing the
C                       name of the file to be manipulated.
C        RecordLength - an INTEGER logical record length, and
C        Formatted    - a LOGICAL variable indicating whether the file is
C                      to be FORMATTED (.TRUE.) or UNFORMATTED (.FALSE.;
C                      binary).

C   Local Variables:

      INTEGER      IOStatus
      CHARACTER*11 FormatString

C   Programmed by: Dave Thompson
C   Date:          May 1990
C   Modified by:   Dave Thompson
C   Last modified: Wed  12-12-1990
*

C-- Implementation ------------------------------------------------------------

C---- Debugging information

C     WRITE (*,*) 'In routine OpenDirect'
C     WRITE (*,*) 'Incoming file            = ', FileName
C     WRITE (*,*) 'Unit number              = ', UnitNumber
C     WRITE (*,*) 'Record length            = ', RecordLength
C     WRITE (*,*) 'Formatting specification = ', Formatted

C---- Initialize

      IF     (Formatted) THEN

        FormatString = 'FORMATTED  '

      ELSEIF (.NOT. Formatted) THEN

        FormatString = 'UNFORMATTED'

      ENDIF

C---- Now, open the file.

      OPEN (
     *      UNIT   = UnitNumber,
     *      FILE   = FileName,
     *      FORM   = FormatString,
     *      ACCESS = 'DIRECT',
     *      RECL   = RecordLength,
     *      IOSTAT = IOStatus
     *     )


C---- Exit with .FALSE. value if the open was unsuccessful.

      IF     (IOStatus .EQ. 0) THEN

        OpenDirect = .TRUE.

      ELSEIF (IOStatus .NE. 0) THEN

        OpenDirect = .FALSE.

      ENDIF

      RETURN
      END

C== Public (OpenOldText) ======================================================

      LOGICAL   FUNCTION OpenOldText (
     I                                UnitNumber, FileName
     *                               )

      IMPLICIT NONE

C   Purpose:  Open an existing flat file for input.

C   Arguments:

      INTEGER       UnitNumber
      CHARACTER*(*) FileName

C   Argument Definitions:
C       UnitNumber  - an INTEGER FORTRAN unit number, and
C       FileName    - a variable dimension character string containing the
C                       name of the file to be manipulated.
  
C   Local Variables:

      INTEGER   IOStatus

C   Programmed by: Dave Thompson
C   Date:          September 1989
C   Modified by:   Dave Thompson
C   Last modified: Wed  12-12-1990
*

C-- Implementation ------------------------------------------------------------

C---- Debugging information

C     WRITE(*,*) ' In routine OpenOldText'
C     WRITE(*,*) ' Incoming file = ', FileName
C     WRITE(*,*) ' Unit number   = ', UnitNumber

C---- Open the existing file.

      OPEN (
     *      UNIT   = UnitNumber,
     *      FILE   = FileName,
     *      STATUS = 'OLD',
     *      ACCESS = 'SEQUENTIAL',
     *      FORM   = 'FORMATTED',
     *      IOSTAT = IOStatus
     *     )

      IF     (IOStatus .EQ. 0) THEN

        OpenOldText = .TRUE.

      ELSEIF (IOStatus .NE. 0) THEN

        OpenOldText = .FALSE.

      ENDIF

      RETURN
      END

C== Public (OpenAppText) ======================================================

      LOGICAL   FUNCTION OpenAppText (
     I                                UnitNumber, FileName
     *                               )

      IMPLICIT NONE

C   Purpose:  Open an existing sequential text file in append mode.

C   Arguments:

      INTEGER       UnitNumber
      CHARACTER*(*) FileName
      
C   Argument Definitions:
C        UnitNumber  - an INTEGER FORTRAN unit number, and
C        FileName    - a variable dimension character string containing the
C                       name of the file to be manipulated.


C   Functions:

      LOGICAL    OpenOldText, FileExist
      EXTERNAL   OpenOldText, FileExist

C   Programmed by: Dave Thompson
C   Date:          21 February 1989
C   Modified by:   Dave Thompson
C   Last modified: Thu  11-15-1990
*

C-- Implementation ------------------------------------------------------------

C---- Debugging information

C     WRITE(*,*) ' In routine OpenAppText'
C     WRITE(*,*) ' Incoming file = ', FileName
C     WRITE(*,*) ' Unit number = ', UnitNumber

C---- Does the file exist?

      IF (FileExist(FileName)) THEN

C------ Open the file and read to end of file.

        IF (OpenOldText(UnitNumber, FileName)) THEN

          OpenAppText = .TRUE.

C-------- The file exists and is open.  Now, read to the end of the file.

   10     CONTINUE

            READ (UnitNumber, END = 20)

          GOTO 10

   20     CONTINUE

        ENDIF

      ELSE

        OpenAppText = .FALSE.

      ENDIF

      RETURN
      END

C== Public (OpenAppBin) =======================================================

      LOGICAL   FUNCTION OpenAppBin (
     I                               UnitNumber, FileName
     *                              )

      IMPLICIT NONE

C   Purpose:  Open an existing sequential binary file in append mode.

C   Arguments:

      INTEGER       UnitNumber
      CHARACTER*(*) FileName

C   Argument Definitions:
C        UnitNumber  - an INTEGER FORTRAN unit number, and
C        FileName    - a variable dimension character string containing the
C                       name of the file to be manipulated.

C   Functions:

      LOGICAL    OpenOldBin, FileExist
      EXTERNAL   OpenOldBin, FileExist

C   Programmed by: Dave Thompson
C   Date:          Thu  11-15-1990
C   Modified by:
C   Last modified:
*

C-- Implementation ------------------------------------------------------------

C---- Debugging information

C     WRITE(*,*) ' In routine OpenAppBin'
C     WRITE(*,*) ' Incoming file = ', FileName
C     WRITE(*,*) ' Unit number = ', UnitNumber

C---- Does the file exist?

      IF (FileExist(FileName)) THEN

C------ Open the file and read to end of file.

        IF (OpenOldBin(UnitNumber, FileName)) THEN

          OpenAppBin = .TRUE.

C-------- The file exists and is open.  Now, read to the end of the file.

   10     CONTINUE

            READ (UnitNumber, END = 20)

          GOTO 10

   20     CONTINUE

        ENDIF

      ELSE

        OpenAppBin = .FALSE.

      ENDIF

      RETURN
      END

C== Public (OpenOldBin) =======================================================

      LOGICAL   FUNCTION OpenOldBin (
     I                               UnitNumber, FileName
     *                              )

      IMPLICIT NONE

C   Purpose:  Open an existing binary file for input.

C   Arguments:

      INTEGER       UnitNumber
      CHARACTER*(*) FileName

C   Argument Definitions:
C       UnitNumber  - an INTEGER FORTRAN unit number, and
C       FileName    - a variable dimension character string containing the
C                       name of the file to be manipulated.

C   Local Variables:

      INTEGER   IOStatus

C   Programmed by: Dave Thompson
C   Date:          May 1990
C   Modified by:   Dave Thompson
C   Last modified: Wed  12-12-1990
*

C-- Implementation ------------------------------------------------------------

C---- Debugging information

C     WRITE (*,*) ' In routine OpenOldBin'
C     WRITE (*,*) ' Incoming file = ', FileName
C     WRITE (*,*) ' Unit number   = ', UnitNumber

C---- Open the existing file.

      OPEN (
     *      UNIT   = UnitNumber,
     *      FILE   = FileName,
     *      STATUS = 'OLD',
     *      ACCESS = 'SEQUENTIAL',
     *      FORM   = 'UNFORMATTED',
     *      IOSTAT = IOStatus
     *     )

      IF     (IOStatus .EQ. 0) THEN

        OpenOldBin = .TRUE.

      ELSEIF (IOStatus .NE. 0) THEN

        OpenOldBin = .FALSE.

      ENDIF

      RETURN
      END

C== Public (DeleteFile) =======================================================

      LOGICAL   FUNCTION DeleteFile (
     I                               FileName
     *                              )

      IMPLICIT NONE

C   Purpose:  Delete an existing file.  This routine contains system
C             specific calls.

C   Arguments:

      CHARACTER*(*) FileName
      
C   Argument Definitions:
C      FileName    - a variable dimension character string containing the
C                      name of the file to be manipulated.


C   Local Variables:

      INTEGER      DefaultUnit
      LOGICAL      PC, Prime

* NOTE: when running in UNIX, DefaultUnit must be no bigger than 2 digits.

      PARAMETER   (
     *             DefaultUnit = 99,
     *             PC          = .TRUE.,
     *             Prime       = .FALSE.
C    *             PC          = .FALSE.,
C    *             Prime       = .TRUE.
     *            )

      INTEGER      IOStatus
      LOGICAL      Exists

      LOGICAL      FileExist
      EXTERNAL     FileExist

***** Uncomment the following for Prime applications only!!!!!

C     INTEGER       Code
C     INCLUDE      'SYSCOM>A$KEYS.INS.FTN'
C     EXTERNAL      FIL$DL

***** End of Prime specific code.

C   Programmed by: Dave Thompson
C   Date:          September 1989
C   Modified by:   Dave Thompson
C   Last modified: Wed  12-12-1990
*

C-- Implementation ------------------------------------------------------------

C---- Debugging information

C---- Check for file existence.

      Exists = FileExist(FileName)

C---- If it doesn't exist, no need to continue.

      IF     (.NOT. Exists) THEN

        DeleteFile = .TRUE.

C---- Otherwise, test for the type of file and take appropriate action.

      ELSEIF (Exists .AND. PC) THEN

        OPEN  (
     *         UNIT   = DefaultUnit,
     *         FILE   = FileName,
     *         STATUS = 'OLD'
     *        )

        CLOSE (
     *         UNIT   = DefaultUnit,
     *         STATUS = 'DELETE',
     *         IOSTAT = IOStatus
     *        )

C------ Was the file deleted?

        IF     (IOStatus .EQ. 0) THEN

          DeleteFile = .TRUE.

        ELSEIF (IOStatus .NE. 0) THEN

          DeleteFile = .FALSE.

        ENDIF

      ELSEIF (Exists .AND. Prime) THEN

***** Uncomment the following for Prime applications only!!!!!
*
*       CALL FIL$DL (
*    I               FileName,
*    O               Code
*    *              )
*
*       IF (Code .NE. 0) THEN
*
*         DeleteFile = .FALSE.
*
*       ELSE
*
*         DeleteFile = .TRUE.
*
*       ENDIF
*
***** End of Prime specific code.

      ENDIF

      RETURN
      END

C== Public (FileExist) =======================================================

      LOGICAL   FUNCTION FileExist (
     I                              FileName
     *                             )

      IMPLICIT NONE

C   Purpose:  Determine if file exists.  This routine contains system
C             specific calls.

C   Arguments:

      CHARACTER*(*)  FileName

C   Argument Definitions:
C       FileName    - a variable dimension character string containing the
C                       name of the file to be manipulated.

C   Local Variables:

      INTEGER    UnitNumber
      INTEGER    IOStatus
      LOGICAL    PC, Prime
      
* NOTE: when running in UNIX, UnitNumber must be no bigger than 2 digits.

      PARAMETER (
     *           UnitNumber = 99,
     *           PC         = .TRUE.,
     *           Prime      = .FALSE.
C    *           PC         = .FALSE.,
C    *           Prime      = .TRUE.
     *          )

***** Uncomment the following for Prime applications only!!!!!

C     INTEGER       FUnit, Code
C     INCLUDE      'SYSCOM>A$KEYS.INS.FTN'
C     EXTERNAL      SRCH$$

***** End of Prime specific code.

C   Programmed by: Dave Thompson
C   Date:          June 1990
C   Modified by:   Dave Thompson
C   Last modified: Wed  12-12-1990
*

C-- Implementation ------------------------------------------------------------

C---- Debugging information

C     WRITE (*,*) ' FileExist Target = ', FileName

      IF (PC) THEN

C------ Try to open the file.  If it exists, report success.
C         Otherwise, the file doesn't exist, so report failure.

        OPEN (
     *        UNIT   = UnitNumber,
     *        FILE   = FileName,
     *        IOSTAT = IOStatus
     *       )
     
        IF     (IOStatus .EQ. 0) THEN

          FileExist = .TRUE.

        ELSEIF (IOStatus .NE. 0) THEN

          FileExist = .FALSE.

        ENDIF

        CLOSE (UnitNumber)

      ELSEIF (Prime) THEN

***** Uncomment the following for Prime applications only!!!!!

C       CALL SRCH$$ (
C    I               K$EXST, FileName, 12, 0,
C    O               Type,   Code
C    *              )
C
C       IF (Code .EQ. E$FNTF) THEN
C
C         FileExist = .FALSE.
C
C       ELSE
C
C         FileExist = .TRUE.
C
C       ENDIF

***** End of Prime specific code.

      ENDIF

      RETURN
      END

*==== EOF fileutil ============================================================
