*==== BOF floweq1d =====================================================


*== Public (DynamicWaveEq) =============================================

      LOGICAL FUNCTION DynamicWaveEq(Up, Down)

      IMPLICIT NONE

*   Purpose:  Compute and store coefficients of an integral form
*             of the dynamic-wave equation (for computing
*             1-D unsteady flow in open channels).

*   Program notes:
*     Variables are integrated over time using an average weighted
*     by Theta.  Equal weight would be given to new and old time steps
*     when Theta = 0.5.  A Newton-Raphson technique is used in the
*     computation of equation coefficients with the exception of the
*     momentum coefficient Beta which is estimated by successive
*     substitution.  Integration in space is accomplished by numerical
*     quadrature.  Maximum number of quadrature points is currently
*     set by the parameter MaxQuadPts.

*     This routine allows solution with constant coefficient matrix
*     when Forward is .FALSE..

*   Arguments:
      INTEGER Up, Down

*   Argument definitions:
*     Up - upstream location number.
*     Down - downstream location number.

*   Module data:

*   Local Variables:

      INTEGER J, J2, K
      INTEGER Nodes, Iteration, Four, QuadPts
      PARAMETER (Nodes = 2, Four = 4)
      REAL G
      REAL N(Nodes), DNDX(Nodes)
      REAL QuadWt, QuadPt
      REAL X1, X2, Q1, Q2, Z1, Z2, H1, H2
      REAL Width1, Width2, Area1, Area2
      REAL Velocity1, Velocity2
      REAL BetaVelocity1, BetaVelocity2
      REAL Coef(8), DX, DT, Theta, OneMinusTheta
      REAL CoefMassKnown, CoefMassAdjust
      REAL CoefDynmKnown, CoefDynmAdjust
      REAL X, Q, H, DZDX
      REAL Width, Area, Conv, ConvSq
      REAL DXxDT, AbsQxAreaByConvSq, AbsQxQbyConvSq, DConvDZ
      LOGICAL OK, Forward

*   Routines by module:

***** Channel flow status:
      REAL     StreamFlow, StreamDepth
      EXTERNAL StreamFlow, StreamDepth

      REAL     StreamSurfaceElevation
      EXTERNAL StreamSurfaceElevation

***** Channel properties:
      REAL     ChannelWidth, CxArea, Beta, Conveyance, dConveyance
      EXTERNAL ChannelWidth, CxArea, Beta, Conveyance, dConveyance

***** Channel schematic:
      REAL     StreamDistance
      EXTERNAL StreamDistance

***** Network control:
      INTEGER  NetworkIteration
      INTEGER  NetworkQuadPts
      REAL     AccelerationGravity
      EXTERNAL NetworkQuadPts, NetworkQuadPtWt
      EXTERNAL NetworkIteration, AccelerationGravity

      INTEGER  NetworkTimeIncrement
      REAL     NetworkTheta
      EXTERNAL NetworkTheta, NetworkTimeIncrement

***** Solver:
      LOGICAL  StoreMassEq, StoreMassKnown, StoreMassAdjust
      EXTERNAL StoreMassEq, StoreMassKnown, StoreMassAdjust

      LOGICAL  StoreDynamicEq, StoreDynmKnown, StoreDynmAdjust
      EXTERNAL StoreDynamicEq, StoreDynmKnown, StoreDynmAdjust

      LOGICAL  ForwardElim
      EXTERNAL ForwardElim

*   Intrinsics:
      REAL      ABS, FLOAT
      INTRINSIC ABS, FLOAT

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: April 1992
*


*-- Implementation -----------------------------------------------------

      Iteration = NetworkIteration()
      Forward = ForwardElim()
      
      G = AccelerationGravity()

*---- Current flow variables.

*          upstream face
      Q1 = StreamFlow(Up)
      Z1 = StreamSurfaceElevation(Up)
      H1 = StreamDepth(Up)

*          downstream face
      Q2 = StreamFlow(Down)
      Z2 = StreamSurfaceElevation(Down)
      H2 = StreamDepth(Down)

*---- Current channel properties.

*          upstream face
      X1 = StreamDistance(Up)
      Width1 = ChannelWidth(X1,H1)
      Area1 = CxArea(X1,H1)
      Velocity1 = Q1 / Area1
      BetaVelocity1 = Beta(X1,H1) * Velocity1

*          downstream face
      X2 = StreamDistance(Down)
      Width2 = ChannelWidth(X2,H2)
      Area2 = CxArea(X2,H2)
      Velocity2 = Q2 / Area2
      BetaVelocity2 = Beta(X2,H2) * Velocity2

*---- Time increment and weighting.
      DT = FLOAT( NetworkTimeIncrement() )
      Theta = NetworkTheta()
      OneMinusTheta = 1.0 - Theta

*---- Spatial increment.
      DX = X2 - X1

*---- Water-surface slope, assumed constant from X1 to X2.
      DZDX = (Z2 - Z1) / DX

      IF( Forward ) THEN


*------ Mass equation.

*  Coef(1)*Q1 + Coef(2)*Z1 + Coef(3)*Q2 + Coef(4)*Z2 = MassConst + MassAdjust


*------ Dynamic equation.

*  Coef(5)*Q1 + Coef(6)*Z1 + Coef(7)*Q2 + Coef(8)*Z2 = DynmConst + DynmAdjust


*------ Contribution from upstream and downstream faces,
*        integrated in time.

*------ mass-equation coefficients for change in Q1, Z1, Q2, & Z2.
        Coef(1) = - Theta * DT
        Coef(2) =   0.0
        Coef(3) =   Theta * DT
        Coef(4) =   0.0


*------ dynamic equation coefficients for change in Q1, Z1, Q2, & Z2.
        Coef(5) = - Theta * 2.0                * BetaVelocity1 * DT
        Coef(6) =   Theta * Velocity1 * Width1 * BetaVelocity1 * DT
        Coef(7) =   Theta * 2.0                * BetaVelocity2 * DT
        Coef(8) = - Theta * Velocity2 * Width2 * BetaVelocity2 * DT

      END IF  
        
        IF( Iteration .GT. 1 ) THEN
        ELSE

          CoefMassKnown = OneMinusTheta * ( Q1 - Q2 ) * DT

          CoefDynmKnown = OneMinusTheta * (
     #                     Q1 * BetaVelocity1
     #                   - Q2 * BetaVelocity2
     #                                    ) * DT

        END IF

        CoefMassAdjust = Theta * ( Q1 - Q2 ) * DT

        CoefDynmAdjust = Theta * (
     #                     Q1 * BetaVelocity1
     #                   - Q2 * BetaVelocity2
     #                           ) * DT



*------ Contribution from spatial dimension,
*        integrated in space or time and space.

*---------- Derivative of interpolation functions with respect
*            to space, constant across subdomain.
            DNDX(1) = -1.0 / DX
            DNDX(2) = -DNDX(1)

      QuadPts = NetworkQuadPts()

      DO 200 K=1,QuadPts

*-------- Estimate quadrature-point values.

            CALL NetworkQuadPtWt( K, QuadPt, QuadWt )

*---------- Interpolation functions.
            N(1) = 1.0 - QuadPt 
            N(2) = QuadPt

*---------- Location of quadrature point.
            X = N(1) * X1 + N(2) * X2

*---------- Dependent variables.
            Q = N(1) * Q1 + N(2) * Q2
            H = N(1) * H1 + N(2) * H2

*---------- Channel variables.
            Width = ChannelWidth( X, H )
            Area = CxArea( X, H )

*---------- Friction-slope terms.
            Conv = Conveyance(X,H)
            ConvSq = Conv * Conv
            DConvDZ = dConveyance(X,H)

            AbsQxQbyConvSq =    ABS(Q) / ConvSq
            AbsQxAreaByConvSq = AbsQxQbyConvSq * Area
            AbsQxQbyConvSq =    AbsQxQbyConvSq * Q

*---------- Product of space and time increments.
            DXxDT = DX * DT


          IF( Forward ) THEN
          DO 100 J=1,2

            J2 = J*2

*---------- mass equation, coefficients for change in Z,
*              coefficient numbers 2 & 4.

            Coef(J2) = Coef(J2) + QuadWt
     #                  * DX * Width * N(J)

*---------- dynamic equation, coefficients for change in Q,
*              coefficient numbers 5 and 7.

            Coef(J2+3) = Coef(J2+3) + QuadWt *    (
     #                     DX
     #           + Theta * DXxDT * G * 2.0 * AbsQxAreaByConvSq
     #                                            ) * N(J)

*---------- dynamic equation, coefficients for change in Z
*              coefficient numbers 6 and 8.

            Coef(J2+4) = Coef(J2+4)
     #                 + QuadWt * Theta * G * DXxDT *    (

     #                   Width * DZDX * N(J)
     #                 + Area   *    DNDX(J)

     #                 + (
     #                     Q * AbsQxAreaByConvSq * (
     #                   Width / Area - 2.0 * DConvDZ / Conv
     #                                             )
     #                   )            * N(J)

     #                                                   )

 100      CONTINUE
          END IF

          IF( Iteration .NE. 1 ) THEN
          ELSE

            CoefMassKnown = CoefMassKnown + QuadWt * Area * DX

            CoefDynmKnown = CoefDynmKnown + QuadWt *    (
     #                      Q * DX
     #                    - OneMinusTheta * DXxDT * G * Area * (
     #                       DZDX + AbsQxQbyConvSq
     #                                                         )
     #                                                  )

          END IF

          CoefMassAdjust = CoefMassAdjust - QuadWt * Area * DX

          CoefDynmAdjust = CoefDynmAdjust + QuadWt *    (
     #                   - Q * DX
     #                   - Theta * DXxDT * G * Area * (
     #                       DZDX + AbsQxQbyConvSq
     #                                                )
     #                                                  )


 200    CONTINUE


*---- Store computed coefficients.

      IF( Forward ) THEN
        OK = StoreMassEq( Up, Four, Coef(1) )
        OK = StoreDynamicEq( Up, Four, Coef(5) )
      END IF

      OK = StoreMassAdjust( Up, CoefMassAdjust )
      OK = StoreDynmAdjust( Up, CoefDynmAdjust )


      IF( Iteration .GT. 1 ) THEN
      ELSE

        OK = StoreMassKnown( Up, CoefMassKnown )
        OK = StoreDynmKnown( Up, CoefDynmKnown )

      END IF



      DynamicWaveEq = .TRUE.

      RETURN
      END



*== Public (DiffusionWaveEq) ===========================================

      LOGICAL FUNCTION DiffusionWaveEq(Up, Down)

      IMPLICIT NONE

*   Purpose:  Compute and store coefficients of an integral form
*             of the diffusion-wave equation (for computing
*             1-D unsteady flow in open channels).

*   Program notes:
*     Variables are integrated over time using an average weighted
*     by Theta.  Equal weight would be given to new and old time steps
*     when Theta = 0.5.  A Newton-Raphson technique is used in the
*     computation of equation coefficients with the exception of the
*     momentum coefficient Beta which is estimated by successive
*     substitution.  Integration in space is accomplished by numerical
*     quadrature.  Maximum number of quadrature points is currently
*     set by the parameter MaxQuadPts.

*     This routine allows solution with constant coefficient matrix
*     when Forward is .FALSE..

*   Arguments:
      INTEGER Up, Down

*   Argument definitions:
*     Up - upstream location number.
*     Down - downstream location number.

*   Module data:

*   Local Variables:
      INTEGER J, J2, K
      INTEGER Nodes, Iteration, Four, QuadPts
      PARAMETER (Nodes = 2, Four = 4)
      REAL G
      REAL N(Nodes), DNDX(Nodes)
      REAL QuadWt, QuadPt
      REAL X1, X2, Q1, Q2, Z1, Z2, H1, H2
      REAL Coef(8), DX, DT, Theta, OneMinusTheta
      REAL CoefMassKnown, CoefMassAdjust
      REAL CoefDynmKnown, CoefDynmAdjust
      REAL X, Q, H, DZDX
      REAL Width, Area, Conv, ConvSq
      REAL DXxDT, AbsQxAreaByConvSq, AbsQxQbyConvSq, DConvDZ
      LOGICAL OK, Forward
      
*  Routines by module:

***** Channel flow status:
      REAL     StreamFlow, StreamDepth
      EXTERNAL StreamFlow, StreamDepth

      REAL     StreamSurfaceElevation
      EXTERNAL StreamSurfaceElevation

***** Channel properties:
      REAL     ChannelWidth, CxArea, Beta, Conveyance, dConveyance
      EXTERNAL ChannelWidth, CxArea, Beta, Conveyance, dConveyance

***** Channel schematic:
      REAL     StreamDistance
      EXTERNAL StreamDistance

***** Network control:
      INTEGER  NetworkIteration
      REAL     AccelerationGravity
      EXTERNAL NetworkIteration, AccelerationGravity

      INTEGER  NetworkTimeIncrement
      REAL     NetworkTheta
      EXTERNAL NetworkTheta, NetworkTimeIncrement

      INTEGER  NetworkQuadPts
      EXTERNAL NetworkQuadPts, NetworkQuadPtWt

***** Solver:
      LOGICAL  StoreMassEq, StoreMassKnown, StoreMassAdjust
      EXTERNAL StoreMassEq, StoreMassKnown, StoreMassAdjust

      LOGICAL  StoreDynamicEq, StoreDynmKnown, StoreDynmAdjust
      EXTERNAL StoreDynamicEq, StoreDynmKnown, StoreDynmAdjust

      LOGICAL  ForwardElim
      EXTERNAL ForwardElim

*   Intrinsics:
      REAL      ABS, FLOAT
      INTRINSIC ABS, FLOAT

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      Iteration = NetworkIteration()
      Forward = ForwardElim()
      
      G = AccelerationGravity()


*---- Current channel properties.

*          upstream face
      X1 = StreamDistance(Up)

*          downstream face
      X2 = StreamDistance(Down)


*---- Current flow variables.

*          upstream face
      Q1 = StreamFlow(Up)
      Z1 = StreamSurfaceElevation(Up)
      H1 = StreamDepth(Up)

*          downstream face
      Q2 = StreamFlow(Down)
      Z2 = StreamSurfaceElevation(Down)
      H2 = StreamDepth(Down)


*---- Time increment and weighting.
      DT = FLOAT( NetworkTimeIncrement() )
      Theta = NetworkTheta()
      OneMinusTheta = 1.0 - Theta

*---- Spatial increment.
      DX = X2 - X1

*---- Water-surface slope, assumed constant from X1 to X2.
      DZDX = (Z2 - Z1) / DX

      IF( Forward ) THEN

*------ Mass equation.

*  Coef(1)*Q1 + Coef(2)*Z1 + Coef(3)*Q2 + Coef(4)*Z2 = MassConst + MassAdjust


*------ Dynamic equation.

*  Coef(5)*Q1 + Coef(6)*Z1 + Coef(7)*Q2 + Coef(8)*Z2 = DynmConst + DynmAdjust


*------ Contribution from upstream and downstream faces,
*        integrated in time.

*------ mass-equation coefficients for change in Q1, Z1, Q2, & Z2.
        Coef(1) = - Theta * DT
        Coef(2) =   0.0
        Coef(3) =   Theta * DT
        Coef(4) =   0.0


*------ dynamic equation coefficients for change in Q1, Z1, Q2, & Z2,
*         flux terms dropped in diffusion-wave equation.
        Coef(5) = 0.0
        Coef(6) = 0.0
        Coef(7) = 0.0
        Coef(8) = 0.0

      END IF
      
        IF( Iteration .GT. 1 ) THEN
        ELSE

          CoefMassKnown = OneMinusTheta * ( Q1 - Q2 ) * DT

          CoefDynmKnown = 0.0

        END IF

        CoefMassAdjust = Theta * ( Q1 - Q2 ) * DT

        CoefDynmAdjust = 0.0



*------ Contribution from spatial dimension,
*        integrated in space or time and space.

*---------- Derivative of interpolation functions with respect
*            to space, constant across subdomain.
            DNDX(1) = -1.0 / DX
            DNDX(2) = -DNDX(1)

        QuadPts = NetworkQuadPts()

        DO 200 K=1,QuadPts

*-------- Estimate quadrature-point values.

            CALL NetworkQuadPtWt( K, QuadPt, QuadWt )

*---------- Interpolation functions.
            N(1) = 1.0 - QuadPt 
            N(2) = QuadPt

*---------- Location of quadrature point.
            X = N(1) * X1 + N(2) * X2

*---------- Dependent variables.
            Q = N(1) * Q1 + N(2) * Q2
            H = N(1) * H1 + N(2) * H2

*---------- Channel variables.
            Width = ChannelWidth( X, H )
            Area = CxArea( X, H )

*---------- Friction-slope terms.
            Conv = Conveyance(X,H)
            ConvSq = Conv * Conv
            DConvDZ = dConveyance(X,H)

            AbsQxQbyConvSq =    ABS(Q) / ConvSq
            AbsQxAreaByConvSq = AbsQxQbyConvSq * Area
            AbsQxQbyConvSq =    AbsQxQbyConvSq * Q

*---------- Product of space and time increments.
            DXxDT = DX * DT


          IF( Forward ) THEN
          DO 100 J=1,2

            J2 = J*2

*---------- mass equation, coefficients for change in Z,
*              coefficient numbers 2 & 4.

            Coef(J2) = Coef(J2) + QuadWt
     #                  * DX * Width * N(J)

*---------- dynamic equation, coefficients for change in Q,
*              coefficient numbers 5 and 7.

            Coef(J2+3) = Coef(J2+3) + QuadWt *    (

     #           + Theta * DXxDT * G * 2.0 * AbsQxAreaByConvSq
     #                                            ) * N(J)

*---------- dynamic equation, coefficients for change in Z
*              coefficient numbers 6 and 8.

            Coef(J2+4) = Coef(J2+4)
     #                 + QuadWt * Theta * G * DXxDT *    (

     #                   Width * DZDX * N(J)
     #                 + Area   *    DNDX(J)

     #                 + (
     #                     Q * AbsQxAreaByConvSq * (
     #                   Width / Area - 2.0 * DConvDZ / Conv
     #                                             )
     #                   )            * N(J)

     #                                                   )

 100      CONTINUE
          END IF
          
          IF( Iteration .NE. 1 ) THEN
          ELSE

            CoefMassKnown = CoefMassKnown + QuadWt * Area * DX

            CoefDynmKnown = CoefDynmKnown + QuadWt *    (

     #                    - OneMinusTheta * DXxDT * G * Area * (
     #                       DZDX + AbsQxQbyConvSq
     #                                                         )
     #                                                  )

          END IF

          CoefMassAdjust = CoefMassAdjust - QuadWt * Area * DX

          CoefDynmAdjust = CoefDynmAdjust + QuadWt *    (

     #                   - Theta * DXxDT * G * Area * (
     #                       DZDX + AbsQxQbyConvSq
     #                                                )
     #                                                  )


 200    CONTINUE


*---- Store computed coefficients.

      IF( Forward ) THEN
        OK = StoreMassEq( Up, Four, Coef(1) )
        OK = StoreDynamicEq( Up, Four, Coef(5) )
      END IF

      OK = StoreMassAdjust( Up, CoefMassAdjust )
      OK = StoreDynmAdjust( Up, CoefDynmAdjust )


      IF( Iteration .GT. 1 ) THEN
      ELSE

        OK = StoreMassKnown( Up, CoefMassKnown )
        OK = StoreDynmKnown( Up, CoefDynmKnown )

      END IF



      DiffusionWaveEq = .TRUE.

      RETURN
      END



*== Public (KinematicWaveEq) ===========================================

      LOGICAL FUNCTION KinematicWaveEq(Up, Down)

      IMPLICIT NONE

*   Purpose:  Compute and store coefficients of an integral form
*             of the kinematic-wave equation (for computing
*             1-D unsteady flow in open channels).

*   Program notes:
*     Variables are integrated over time using an average weighted
*     by Theta.  Equal weight would be given to new and old time steps
*     when Theta = 0.5.  A Newton-Raphson technique is used in the
*     computation of equation coefficients with the exception of the
*     momentum coefficient Beta which is estimated by successive
*     substitution.  Integration in space is accomplished by numerical
*     quadrature.  Maximum number of quadrature points is currently
*     set by the parameter MaxQuadPts.

*   Arguments:
      INTEGER Up, Down

*   Argument definitions:
*     Up - upstream location number.
*     Down - downstream location number.

*   Module data:

*   Local Variables:
      INTEGER J, J2, K
      INTEGER Nodes, Iteration, Four, QuadPts
      PARAMETER (Nodes = 2, Four = 4)
      REAL G
      REAL N(Nodes), DNDX(Nodes)
      REAL QuadWt, QuadPt
      REAL X1, X2, Q1, Q2, Z1, Z2, H1, H2
      REAL Coef(8), DX, DT, Theta, OneMinusTheta
      REAL CoefMassKnown, CoefMassAdjust
      REAL CoefDynmKnown, CoefDynmAdjust
      REAL X, Q, H, DZDX
      REAL Width, Area, Conv, ConvSq
      REAL DXxDT, AbsQxAreaByConvSq, AbsQxQbyConvSq, DConvDZ
      LOGICAL OK

*   Routines by module:

***** Channel flow status:
      REAL     StreamFlow, StreamDepth
      EXTERNAL StreamFlow, StreamDepth

      EXTERNAL StreamSurfaceElevation
      REAL     StreamSurfaceElevation

***** Channel properties:
      REAL     BtmSlope
      EXTERNAL BtmSlope

      REAL     ChannelWidth, CxArea, Beta, Conveyance, dConveyance
      EXTERNAL ChannelWidth, CxArea, Beta, Conveyance, dConveyance

***** Channel schematic:
      REAL     StreamDistance
      EXTERNAL StreamDistance

***** Network control:
      INTEGER  NetworkIteration
      REAL     AccelerationGravity
      EXTERNAL NetworkIteration, AccelerationGravity

      INTEGER  NetworkTimeIncrement
      REAL     NetworkTheta
      EXTERNAL NetworkTheta, NetworkTimeIncrement

      INTEGER  NetworkQuadPts
      EXTERNAL NetworkQuadPts, NetworkQuadPtWt

***** Solver:
      LOGICAL  StoreMassEq, StoreMassKnown, StoreMassAdjust
      EXTERNAL StoreMassEq, StoreMassKnown, StoreMassAdjust

      LOGICAL  StoreDynamicEq, StoreDynmKnown, StoreDynmAdjust
      EXTERNAL StoreDynamicEq, StoreDynmKnown, StoreDynmAdjust

*   Intrinsics:
      REAL      ABS, FLOAT
      INTRINSIC ABS, FLOAT

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      Iteration = NetworkIteration()
      
      G = AccelerationGravity()

*---- Current channel properties.

*          upstream face
      X1 = StreamDistance(Up)

*          downstream face
      X2 = StreamDistance(Down)


*---- Current flow variables.

*          upstream face
      Q1 = StreamFlow(Up)
      Z1 = StreamSurfaceElevation(Up)
      H1 = StreamDepth(Up)

*          downstream face
      Q2 = StreamFlow(Down)
      Z2 = StreamSurfaceElevation(Down)
      H2 = StreamDepth(Down)

*---- Time increment and weighting.
      DT = FLOAT( NetworkTimeIncrement() )
      Theta = NetworkTheta()
      OneMinusTheta = 1.0 - Theta

*---- Spatial increment.
      DX = X2 - X1

*------ Mass equation.

*  Coef(1)*Q1 + Coef(2)*Z1 + Coef(3)*Q2 + Coef(4)*Z2 = MassConst + MassAdjust


*------ Dynamic equation.

*  Coef(5)*Q1 + Coef(6)*Z1 + Coef(7)*Q2 + Coef(8)*Z2 = DynmConst + DynmAdjust


*------ Contribution from upstream and downstream faces,
*        integrated in time.

*------ mass-equation coefficients for change in Q1, Z1, Q2, & Z2.
        Coef(1) = - Theta * DT
        Coef(2) =   0.0
        Coef(3) =   Theta * DT
        Coef(4) =   0.0




*------ dynamic equation coefficients for change in Q1, Z1, Q2, & Z2,
*         flux terms dropped in kinematic-wave equation.
        Coef(5) = 0.0
        Coef(6) = 0.0
        Coef(7) = 0.0
        Coef(8) = 0.0

        IF( Iteration .GT. 1 ) THEN
        ELSE

          CoefMassKnown = OneMinusTheta * ( Q1 - Q2 ) * DT

          CoefDynmKnown = 0.0

        END IF

        CoefMassAdjust = Theta * ( Q1 - Q2 ) * DT

        CoefDynmAdjust = 0.0



*------ Contribution from spatial dimension,
*        integrated in space or time and space.

*---------- Derivative of interpolation functions with respect
*            to space, constant across subdomain.
            DNDX(1) = -1.0 / DX
            DNDX(2) = -DNDX(1)

        QuadPts = NetworkQuadPts()

        DO 200 K=1,QuadPts

*-------- Estimate quadrature-point values.

            CALL NetworkQuadPtWt( K, QuadPt, QuadWt )

*---------- Interpolation functions.
            N(1) = 1.0 - QuadPt 
            N(2) = QuadPt

*---------- Location of quadrature point.
            X = N(1) * X1 + N(2) * X2

*---------- Dependent variables.
            Q = N(1) * Q1 + N(2) * Q2
            H = N(1) * H1 + N(2) * H2

*---------- Channel variables.
            Width = ChannelWidth( X, H )
            Area = CxArea( X, H )

*---------- Kinematic approximation, assume water-surface slope
*             is same as bottom slope.
            DZDX = BtmSlope(X)

*---------- Friction-slope terms.
            Conv = Conveyance(X,H)
            ConvSq = Conv * Conv
            DConvDZ = dConveyance(X,H)

            AbsQxQbyConvSq =    ABS(Q) / ConvSq
            AbsQxAreaByConvSq = AbsQxQbyConvSq * Area
            AbsQxQbyConvSq =    AbsQxQbyConvSq * Q

*---------- Product of space and time increments.
            DXxDT = DX * DT


          DO 100 J=1,2

            J2 = J*2

*---------- mass equation, coefficients for change in Z,
*              coefficient numbers 2 & 4.

            Coef(J2) = Coef(J2) + QuadWt
     #                  * DX * Width * N(J)

*---------- dynamic equation, coefficients for change in Q,
*              coefficient numbers 5 and 7.

            Coef(J2+3) = Coef(J2+3) + QuadWt *    (

     #           + Theta * DXxDT * G * 2.0 * AbsQxAreaByConvSq
     #                                            ) * N(J)

*---------- dynamic equation, coefficients for change in Z
*              coefficient numbers 6 and 8.

            Coef(J2+4) = Coef(J2+4)
     #                 + QuadWt * Theta * G * DXxDT *    (

     #                   Width * DZDX * N(J)
     #                 + Area   *    DNDX(J)

     #                 + (
     #                     Q * AbsQxAreaByConvSq * (
     #                   Width / Area - 2.0 * DConvDZ / Conv
     #                                             )
     #                   )            * N(J)

     #                                                   )

 100      CONTINUE

          IF( Iteration .NE. 1 ) THEN
          ELSE

            CoefMassKnown = CoefMassKnown + QuadWt * Area * DX

            CoefDynmKnown = CoefDynmKnown + QuadWt *    (

     #                    - OneMinusTheta * DXxDT * G * Area * (
     #                       DZDX + AbsQxQbyConvSq
     #                                                         )
     #                                                  )

          END IF

          CoefMassAdjust = CoefMassAdjust - QuadWt * Area * DX

          CoefDynmAdjust = CoefDynmAdjust + QuadWt *    (

     #                   - Theta * DXxDT * G * Area * (
     #                       DZDX + AbsQxQbyConvSq
     #                                                )
     #                                                  )


 200    CONTINUE


*---- Store computed coefficients.

      OK = StoreMassEq( Up, Four, Coef(1) )
      OK = StoreDynamicEq( Up, Four, Coef(5) )

      OK = StoreMassAdjust( Up, CoefMassAdjust )
      OK = StoreDynmAdjust( Up, CoefDynmAdjust )


      IF( Iteration .GT. 1 ) THEN
      ELSE

        OK = StoreMassKnown( Up, CoefMassKnown )
        OK = StoreDynmKnown( Up, CoefDynmKnown )

      END IF



      KinematicWaveEq = .TRUE.

      RETURN
      END



*== Public (DynamicWaveEqDS) =============================================

      LOGICAL FUNCTION DynamicWaveEqDS(Up, Down)

      IMPLICIT NONE

*   Purpose:  Compute and store coefficients of an integral form
*             of the dynamic-wave equation (for computing
*             1-D unsteady flow in open channels).

*   Program notes:
*     Variables are integrated over time using an average weighted
*     by Theta.  Equal weight would be given to new and old time steps
*     when Theta = 0.5.  A Newton-Raphson technique is used in the
*     computation of equation coefficients with the exception of the
*     momentum coefficient Beta which is estimated by successive
*     substitution.  Integration in space is accomplished by numerical
*     quadrature.  Maximum number of quadrature points is currently
*     set by the parameter MaxQuadPts.  Water density is allowed to vary
*     with time and distance.  Channel sinuosity is allowed to vary
*     with depth of flow and distance.

*     This routine allows solution with constant coefficient matrix
*     when Forward is .FALSE..

*   Arguments:
      INTEGER Up, Down

*   Argument definitions:
*     Up - upstream location number.
*     Down - downstream location number.

*   Module data:

*   Local Variables:

      INTEGER J, J2, K
      INTEGER Nodes, Iteration, Four, QuadPts
      PARAMETER (Nodes = 2, Four = 4)
      REAL G
      REAL N(Nodes), DNDX(Nodes)
      REAL QuadWt, QuadPt
      REAL X1, X2, Q1, Q2, Z1, Z2, H1, H2
      REAL Width1, Width2, Area1, Area2
      REAL Velocity1, Velocity2
      REAL BetaVelocity1, BetaVelocity2
      REAL OldRho1, NewRho1, OldRho2, NewRho2, OldRho, NewRho
      REAL OlddRhodX, NewdRhodX
      REAL ZBar, dZBardX
      REAL Coef(8), DX, DT, Theta, OneMinusTheta
      REAL CoefMassKnown, CoefMassAdjust
      REAL CoefDynmKnown, CoefDynmAdjust
      REAL X, Q, H, DZDX
      REAL Ma, dMadH, Mq, dMqdH
      REAL Width, Area, Conv, ConvSq
      REAL DXxDT, AbsQxAreaByConvSq, AbsQxQbyConvSq, DConvDZ
      LOGICAL OK, Forward

*   Routines by module:

***** Channel flow status:
      REAL     StreamFlow, StreamDepth
      REAL     OldStreamDensity, NewStreamDensity
      EXTERNAL StreamFlow, StreamDepth
      EXTERNAL OldStreamDensity, NewStreamDensity

      REAL     StreamSurfaceElevation
      EXTERNAL StreamSurfaceElevation

***** Channel properties:
      REAL     ChannelWidth, CxArea, Beta, Conveyance, dConveyance
      REAL     AreaWtSinuosity,  QWtSinuosity
      REAL     dAreaWtSinuosity, dQWtSinuosity
      REAL     CxCentroid, dCxCentroid
      EXTERNAL CxCentroid, dCxCentroid
      EXTERNAL ChannelWidth, CxArea, Beta, Conveyance, dConveyance
      EXTERNAL AreaWtSinuosity,  QWtSinuosity
      EXTERNAL dAreaWtSinuosity, dQWtSinuosity

***** Channel schematic:
      REAL     StreamDistance
      EXTERNAL StreamDistance

***** Network control:
      INTEGER  NetworkIteration
      REAL     AccelerationGravity
      EXTERNAL NetworkIteration, AccelerationGravity

      INTEGER  NetworkTimeIncrement
      REAL     NetworkTheta
      EXTERNAL NetworkTheta, NetworkTimeIncrement

      INTEGER  NetworkQuadPts
      EXTERNAL NetworkQuadPts, NetworkQuadPtWt

***** Solver:
      LOGICAL  StoreMassEq, StoreMassKnown, StoreMassAdjust
      EXTERNAL StoreMassEq, StoreMassKnown, StoreMassAdjust

      LOGICAL  StoreDynamicEq, StoreDynmKnown, StoreDynmAdjust
      EXTERNAL StoreDynamicEq, StoreDynmKnown, StoreDynmAdjust

      LOGICAL  ForwardElim
      EXTERNAL ForwardElim

*   Intrinsics:
      REAL      ABS, FLOAT
      INTRINSIC ABS, FLOAT

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      Iteration = NetworkIteration()

      G = AccelerationGravity()
      
      Forward = ForwardElim()

*---- Current flow variables.

*          upstream face
      Q1 = StreamFlow(Up)
      Z1 = StreamSurfaceElevation(Up)
      H1 = StreamDepth(Up)
      NewRho1 = NewStreamDensity(Up)

*          downstream face
      Q2 = StreamFlow(Down)
      Z2 = StreamSurfaceElevation(Down)
      H2 = StreamDepth(Down)
      NewRho2 = NewStreamDensity(Down)

*---- Current channel properties.

*          upstream face
      X1 = StreamDistance(Up)
      Width1 = ChannelWidth(X1,H1)
      Area1 = CxArea(X1,H1)
      Velocity1 = Q1 / Area1
      BetaVelocity1 = Beta(X1,H1) * Velocity1

*          downstream face
      X2 = StreamDistance(Down)
      Width2 = ChannelWidth(X2,H2)
      Area2 = CxArea(X2,H2)
      Velocity2 = Q2 / Area2
      BetaVelocity2 = Beta(X2,H2) * Velocity2

*---- Time increment and weighting.
      DT = FLOAT( NetworkTimeIncrement() )
      Theta = NetworkTheta()
      OneMinusTheta = 1.0 - Theta

*---- Spatial increment.
      DX = X2 - X1

*---- Water-surface slope, assumed constant from X1 to X2.
      DZDX = (Z2 - Z1) / DX

      IF( Forward ) THEN


*------ Mass equation.

*  Coef(1)*Q1 + Coef(2)*Z1 + Coef(3)*Q2 + Coef(4)*Z2 = MassConst + MassAdjust


*------ Dynamic equation.

*  Coef(5)*Q1 + Coef(6)*Z1 + Coef(7)*Q2 + Coef(8)*Z2 = DynmConst + DynmAdjust


*------ Contribution from upstream and downstream faces,
*        integrated in time.

*------ mass-equation coefficients for change in Q1, Z1, Q2, & Z2.
        Coef(1) = - Theta * NewRho1 * DT
        Coef(2) =   0.0
        Coef(3) =   Theta * NewRho2 * DT
        Coef(4) =   0.0


*------ dynamic equation coefficients for change in Q1, Z1, Q2, & Z2.
        Coef(5) =-Theta * 2.0 *     NewRho1       * BetaVelocity1 * DT
        Coef(6) = Theta *Velocity1* NewRho1* Width1*BetaVelocity1 * DT
        Coef(7) = Theta * 2.0 *     NewRho2       * BetaVelocity2 * DT
        Coef(8) =-Theta *Velocity2* NewRho2* Width2*BetaVelocity2 * DT

      END IF

        IF( Iteration .GT. 1 ) THEN
        ELSE

          OldRho1 = OldStreamDensity(Up)
          OldRho2 = OldStreamDensity(Down)

          CoefMassKnown = OneMinusTheta* (OldRho1*Q1 - OldRho2*Q2) * DT

          CoefDynmKnown = OneMinusTheta * (
     #                     OldRho1 * Q1 * BetaVelocity1
     #                   - OldRho2 * Q2 * BetaVelocity2
     #                                    ) * DT

        END IF

        CoefMassAdjust = Theta * (NewRho1*Q1 - NewRho2*Q2 ) * DT

        CoefDynmAdjust = Theta * (
     #                     NewRho1 * Q1 * BetaVelocity1
     #                   - NewRho2 * Q2 * BetaVelocity2
     #                           ) * DT



*------ Contribution from spatial dimension,
*        integrated in space or time and space.

*---------- Derivative of interpolation functions with respect
*            to space, constant across subdomain.
            DNDX(1) = -1.0 / DX
            DNDX(2) = -DNDX(1)

        QuadPts = NetworkQuadPts()

        DO 200 K=1,QuadPts

*-------- Estimate quadrature-point values.

            CALL NetworkQuadPtWt( K, QuadPt, QuadWt )

*---------- Interpolation functions.
            N(1) = 1.0 - QuadPt 
            N(2) = QuadPt

*---------- Location of quadrature point.
            X = N(1) * X1 + N(2) * X2

*---------- Dependent variables.
            Q = N(1) * Q1 + N(2) * Q2
            H = N(1) * H1 + N(2) * H2

*---------- Channel geometry variables.
            Width = ChannelWidth( X, H )
            Area = CxArea( X, H )

*---------- Density and density gradient.
            NewRho = N(1) * NewRho1 + N(2) * NewRho2
            NewdRhodX = DNDX(1) * NewRho1 + DNDX(2) * NewRho2

*---------- Distance from centroid to water surface.
            ZBar = CxCentroid(X,H)
            dZBardX = dCxCentroid(X,H)

*---------- Friction-slope terms.
            Conv = Conveyance(X,H)
            ConvSq = Conv * Conv
            DConvDZ = dConveyance(X,H)

            AbsQxQbyConvSq =    ABS(Q) / ConvSq
            AbsQxAreaByConvSq = AbsQxQbyConvSq * Area
            AbsQxQbyConvSq =    AbsQxQbyConvSq * Q

*---------- Sinuosity.
            Ma = AreaWtSinuosity(X,H)
            dMadH = dAreaWtSinuosity(X,H)
            Mq = QWtSinuosity(X,H)
            dMqdH = dQWtSinuosity(X,H)

*---------- Product of space and time increments.
            DXxDT = DX * DT


          IF( Forward ) THEN
          DO 100 J=1,2

            J2 = J*2

*---------- mass equation, coefficients for change in Z,
*              coefficient numbers 2 & 4.

            Coef(J2) = Coef(J2) + QuadWt *    (
     #               Ma  * Width + Area * dMadH
     #                                        ) * NewRho * DX * N(J)

*---------- dynamic equation, coefficients for change in Q,
*              coefficient numbers 5 and 7.

            Coef(J2+3) = Coef(J2+3) + QuadWt *    (
     #                     NewRho * Mq * DX
     #       + Theta * NewRho * DXxDT * G * 2.0 * AbsQxAreaByConvSq
     #                                            ) * N(J)

*---------- dynamic equation, coefficients for change in Z
*              coefficient numbers 6 and 8.

            Coef(J2+4) = Coef(J2+4) + QuadWt *            (
     #                     NewRho * Q * dMqdH
     #                         +    Theta * G * DXxDT * (

     #                    NewRho * (
     #                   Width * DZDX * N(J)
     #                 + Area   *    DNDX(J)
     #                             )

     #             + (
     #                    NewdRhodX * (
     #                    ZBar * Width + Area * dZBardX
     #                               )

     #                 + (
     #            NewRho * Q * AbsQxAreaByConvSq * (
     #                   Width / Area - 2.0 * DConvDZ / Conv
     #                                             )
     #                   )
     #                )  * N(J)

     #                                                   )
     #                                                      )

 100      CONTINUE
          END IF
          
          IF( Iteration .NE. 1 ) THEN
          ELSE
            
            OldRho = N(1) * OldRho1 + N(2) * OldRho2
            OlddRhodX = DNDX(1) * OldRho1 + DNDX(2) * OldRho2

            CoefMassKnown = CoefMassKnown + QuadWt * 
     #                             OldRho * Area * Ma * DX

            CoefDynmKnown = CoefDynmKnown + QuadWt *    (
     #                      OldRho * Q * Mq * DX
     #                    - OneMinusTheta * DXxDT * G * Area * (
     #                        OldRho * (DZDX + AbsQxQbyConvSq)
     #                            + ZBar * OlddRhodX
     #                                                         )

     #                    
     #                                                  )

          END IF

          CoefMassAdjust = CoefMassAdjust - QuadWt *
     #                        NewRho * Area * Ma * DX

          CoefDynmAdjust = CoefDynmAdjust + QuadWt *    (
     #                   - NewRho * Q * Mq * DX
     #                   - Theta * DXxDT * G * Area * (
     #                     NewRho * (DZDX + AbsQxQbyConvSq)
     #                          + ZBar * NewdRhodX
     #                     
     #                                                )
     #                                                  )


 200    CONTINUE


*---- Store computed coefficients.

      IF( Forward ) THEN
        OK = StoreMassEq( Up, Four, Coef(1) )
        OK = StoreDynamicEq( Up, Four, Coef(5) )
      END IF

      OK = StoreMassAdjust( Up, CoefMassAdjust )
      OK = StoreDynmAdjust( Up, CoefDynmAdjust )


      IF( Iteration .GT. 1 ) THEN
      ELSE

        OK = StoreMassKnown( Up, CoefMassKnown )
        OK = StoreDynmKnown( Up, CoefDynmKnown )

      END IF



      DynamicWaveEqDS = .TRUE.

      RETURN
      END



*== Public (DiffusionWaveEqS) ===========================================

      LOGICAL FUNCTION DiffusionWaveEqS(Up, Down)

      IMPLICIT NONE

*   Purpose:  Compute and store coefficients of an integral form
*             of the diffusion-wave equation (for computing
*             1-D unsteady flow in open channels).  Sinuosity is 
*             allowed to vary with flow depth.

*   Program notes:
*     Variables are integrated over time using an average weighted
*     by Theta.  Equal weight would be given to new and old time steps
*     when Theta = 0.5.  A Newton-Raphson technique is used in the
*     computation of equation coefficients with the exception of the
*     momentum coefficient Beta which is estimated by successive
*     substitution.  Integration in space is accomplished by numerical
*     quadrature.  Maximum number of quadrature points is currently
*     set by the parameter MaxQuadPts.

*     This routine allows solution with constant coefficient matrix
*     when Forward is .FALSE..

*   Arguments:
      INTEGER Up, Down

*   Argument definitions:
*     Up - upstream location number.
*     Down - downstream location number.

*   Module data:

*   Local Variables:
      INTEGER J, J2, K
      INTEGER Nodes, Iteration, Four, QuadPts
      PARAMETER (Nodes = 2, Four = 4)
      REAL G
      REAL N(Nodes), DNDX(Nodes)
      REAL QuadWt, QuadPt
      REAL X1, X2, Q1, Q2, Z1, Z2, H1, H2
      REAL Coef(8), DX, DT, Theta, OneMinusTheta
      REAL CoefMassKnown, CoefMassAdjust
      REAL CoefDynmKnown, CoefDynmAdjust
      REAL X, Q, H, DZDX
      REAL Width, Area, Conv, ConvSq
      REAL Ma, dMadH
      REAL DXxDT, AbsQxAreaByConvSq, AbsQxQbyConvSq, DConvDZ
      LOGICAL OK, Forward

*  Routines by module:

***** Channel flow status:
      REAL     StreamFlow, StreamDepth
      EXTERNAL StreamFlow, StreamDepth

      REAL     StreamSurfaceElevation
      EXTERNAL StreamSurfaceElevation

***** Channel properties:
      REAL     ChannelWidth, CxArea, Beta, Conveyance, dConveyance
      REAL     AreaWtSinuosity, dAreaWtSinuosity
      EXTERNAL ChannelWidth, CxArea, Beta, Conveyance, dConveyance
      EXTERNAL AreaWtSinuosity, dAreaWtSinuosity

***** Channel schematic:
      REAL     StreamDistance
      EXTERNAL StreamDistance

***** Network control:
      INTEGER  NetworkIteration
      REAL     AccelerationGravity
      EXTERNAL NetworkIteration, AccelerationGravity

      INTEGER  NetworkTimeIncrement
      REAL     NetworkTheta
      EXTERNAL NetworkTheta, NetworkTimeIncrement

      INTEGER  NetworkQuadPts
      EXTERNAL NetworkQuadPts, NetworkQuadPtWt

***** Solver:
      LOGICAL  StoreMassEq, StoreMassKnown, StoreMassAdjust
      EXTERNAL StoreMassEq, StoreMassKnown, StoreMassAdjust

      LOGICAL  StoreDynamicEq, StoreDynmKnown, StoreDynmAdjust
      EXTERNAL StoreDynamicEq, StoreDynmKnown, StoreDynmAdjust

      LOGICAL  ForwardElim
      EXTERNAL ForwardElim

*   Intrinsics:
      REAL      ABS, FLOAT
      INTRINSIC ABS, FLOAT

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      Iteration = NetworkIteration()
      Forward = ForwardElim()
      
      G = AccelerationGravity()

*---- Current channel properties.

*          upstream face
      X1 = StreamDistance(Up)

*          downstream face
      X2 = StreamDistance(Down)


*---- Current flow variables.

*          upstream face
      Q1 = StreamFlow(Up)
      Z1 = StreamSurfaceElevation(Up)
      H1 = StreamDepth(Up)

*          downstream face
      Q2 = StreamFlow(Down)
      Z2 = StreamSurfaceElevation(Down)
      H2 = StreamDepth(Down)


*---- Time increment and weighting.
      DT = FLOAT( NetworkTimeIncrement() )
      Theta = NetworkTheta()
      OneMinusTheta = 1.0 - Theta

*---- Spatial increment.
      DX = X2 - X1

*---- Water-surface slope, assumed constant from X1 to X2.
      DZDX = (Z2 - Z1) / DX

      IF( Forward ) THEN

*------ Mass equation.

*  Coef(1)*Q1 + Coef(2)*Z1 + Coef(3)*Q2 + Coef(4)*Z2 = MassConst + MassAdjust


*------ Dynamic equation.

*  Coef(5)*Q1 + Coef(6)*Z1 + Coef(7)*Q2 + Coef(8)*Z2 = DynmConst + DynmAdjust


*------ Contribution from upstream and downstream faces,
*        integrated in time.

*------ mass-equation coefficients for change in Q1, Z1, Q2, & Z2.
        Coef(1) = - Theta * DT
        Coef(2) =   0.0
        Coef(3) =   Theta * DT
        Coef(4) =   0.0


*------ dynamic equation coefficients for change in Q1, Z1, Q2, & Z2,
*         flux terms dropped in diffusion-wave equation.
        Coef(5) = 0.0
        Coef(6) = 0.0
        Coef(7) = 0.0
        Coef(8) = 0.0

      END IF

        IF( Iteration .GT. 1 ) THEN
        ELSE

          CoefMassKnown = OneMinusTheta * ( Q1 - Q2 ) * DT

          CoefDynmKnown = 0.0

        END IF

        CoefMassAdjust = Theta * ( Q1 - Q2 ) * DT

        CoefDynmAdjust = 0.0



*------ Contribution from spatial dimension,
*        integrated in space or time and space.

*---------- Derivative of interpolation functions with respect
*            to space, constant across subdomain.
            DNDX(1) = -1.0 / DX
            DNDX(2) = -DNDX(1)

        QuadPts = NetworkQuadPts()

        DO 200 K=1,QuadPts

*-------- Estimate quadrature-point values.

            CALL NetworkQuadPtWt( K, QuadPt, QuadWt )

*---------- Interpolation functions.
            N(1) = 1.0 - QuadPt 
            N(2) = QuadPt

*---------- Location of quadrature point.
            X = N(1) * X1 + N(2) * X2

*---------- Dependent variables.
            Q = N(1) * Q1 + N(2) * Q2
            H = N(1) * H1 + N(2) * H2

*---------- Channel variables.
            Width = ChannelWidth( X, H )
            Area = CxArea( X, H )

*---------- Friction-slope terms.
            Conv = Conveyance(X,H)
            ConvSq = Conv * Conv
            DConvDZ = dConveyance(X,H)

            AbsQxQbyConvSq =    ABS(Q) / ConvSq
            AbsQxAreaByConvSq = AbsQxQbyConvSq * Area
            AbsQxQbyConvSq =    AbsQxQbyConvSq * Q

*---------- Sinuosity.
            Ma = AreaWtSinuosity(X,H)
            dMadH = dAreaWtSinuosity(X,H)

*---------- Product of space and time increments.
            DXxDT = DX * DT


          IF( Forward ) THEN
          DO 100 J=1,2

            J2 = J*2

*---------- mass equation, coefficients for change in Z,
*              coefficient numbers 2 & 4.

            Coef(J2) = Coef(J2) + QuadWt *    (
     #               Ma  * Width + Area * dMadH
     #                                        ) * DX * N(J)

*---------- dynamic equation, coefficients for change in Q,
*              coefficient numbers 5 and 7.

            Coef(J2+3) = Coef(J2+3) + QuadWt *    (

     #           + Theta * DXxDT * G * 2.0 * AbsQxAreaByConvSq
     #                                            ) * N(J)

*---------- dynamic equation, coefficients for change in Z
*              coefficient numbers 6 and 8.

            Coef(J2+4) = Coef(J2+4)
     #                 + QuadWt * Theta * G * DXxDT *    (

     #                   Width * DZDX * N(J)
     #                 + Area   *    DNDX(J)

     #                 + (
     #                     Q * AbsQxAreaByConvSq * (
     #                   Width / Area - 2.0 * DConvDZ / Conv
     #                                             )
     #                   )            * N(J)

     #                                                   )

 100      CONTINUE
          END IF
          
          IF( Iteration .NE. 1 ) THEN
          ELSE

            CoefMassKnown = CoefMassKnown + QuadWt * Area * Ma * DX

            CoefDynmKnown = CoefDynmKnown + QuadWt *    (

     #                    - OneMinusTheta * DXxDT * G * Area * (
     #                       DZDX + AbsQxQbyConvSq
     #                                                         )
     #                                                  )

          END IF

          CoefMassAdjust = CoefMassAdjust - QuadWt * Area * DX

          CoefDynmAdjust = CoefDynmAdjust + QuadWt *    (

     #                   - Theta * DXxDT * G * Area * (
     #                       DZDX + AbsQxQbyConvSq
     #                                                )
     #                                                  )


 200    CONTINUE


*---- Store computed coefficients.

      IF( Forward ) THEN
        OK = StoreMassEq( Up, Four, Coef(1) )
        OK = StoreDynamicEq( Up, Four, Coef(5) )
      END IF

      OK = StoreMassAdjust( Up, CoefMassAdjust )
      OK = StoreDynmAdjust( Up, CoefDynmAdjust )


      IF( Iteration .GT. 1 ) THEN
      ELSE

        OK = StoreMassKnown( Up, CoefMassKnown )
        OK = StoreDynmKnown( Up, CoefDynmKnown )

      END IF



      DiffusionWaveEqS = .TRUE.

      RETURN
      END


*==== EOF floweq1d =====================================================
