*==== BOF fourpt =======================================================



*== Public  (FourPt) ===================================================

      PROGRAM FourPt

      IMPLICIT NONE

*   Purpose:  Compute 1-dimensional streamflow in a network of open
*             channels, in terms of discharge and water-surface
*             elevation.
*
*             Optionally density is allowed to vary with time and 
*             distance.  Currently density at both ends of each channel
*             must be read from a user-supplied file if this option is
*             used.
*
*             Optionally sinuosity is allowed to vary with depth of flow
*             and distance.  Sinuosity information is supplied through
*             channel-properties tables.


*                      Lew DeLong
*                      U.S.Geological Survey
*                      Water Resources Division
*                      Bldg. 2101, Mail Stop 1515
*                      Stennis Space Center, MS, 39529
*                      lldelong@qmsbs2.er.usgs.GOV
*                      
*
*                      David Thompson, formerly with U.S.G.S., currently
*                      with Texas Tech University, is primarily responsible
*                      for adding file utilities, buffered output of
*                      time-series and space-series results, string
*                      utilities, the master-file look-up module for
*                      relating user-supplied file names and unit numbers
*                      with internal defaults.
*
*                      Janice Fulford contributed routines necessary
*                      to represent general 3-parameter ratings
*                      capable of representing hydraulic structures.
*
*                      Other persons involved in the coding or
*                      modification of code used by FourPt include
*                      Barry Wicktom, Jenifer Johnson, Victoria Israel.


*     Non-standard usage: Symbolic names in this module may be represented by
*                         as many as 31 characters in order to provide better
*                         definition directly in the code.  Standard FORTRAN
*                         allows only 6 characters, but this restriction is
*                         generally extended to 32 characters by most compilers.
*

*   Local variables:
      INTEGER   Step, PrintUnit
      LOGICAL   OK
      CHARACTER*12 InternalFileName

*   Routines by module:

***** Buffered output:
      LOGICAL  FlushSeries
      EXTERNAL FlushSeries

***** Network control:
      INTEGER  NetworkTimeSteps, TotalNetworkIterations
      EXTERNAL NetworkTimeSteps, TotalNetworkIterations

      LOGICAL  InitializeNetworkControl
      EXTERNAL InitializeNetworkControl
      
***** Master file:  
      INTEGER  GetFileUnit
      LOGICAL  OpenMaster
      LOGICAL  CloseMaster
      EXTERNAL OpenMaster, CloseMaster, GetFileUnit

***** Local:
      LOGICAL  DefineNetwork, UpdateNetwork
      EXTERNAL DefineNetwork, UpdateNetwork
      
***** Network volume and mass balance:
      LOGICAL  InitNetBalance, UpdateNetBalance
      EXTERNAL InitNetBalance, UpdateNetBalance
      LOGICAL  ReportNetBalance
      EXTERNAL ReportNetBalance
      
***** Channel status:
      LOGICAL  WriteNetworkRestartFile
      EXTERNAL WriteNetworkRestartFile

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: December 1992
*

      CHARACTER*64 versn
      versn = 
     & '$Id: fourpt.f,v 1.2 1995/09/06 14:38:19 rsregan Exp rsregan $'
      versn = 
     & '@(#)FOURPT - One-Dimensional, Unsteady, Open-Channel Flow Model'
      versn =
     &   '@(#)FOURPT - USGS WRIR Report 97-4016 DeLong, Thompson, & Lee'
      versn = '@(#)FOURPT - Contact: h2osoft@usgs.gov'
      versn = '@(#)FOURPT - Version: 95.01 1.2x 1995/09/06'

*-- Implementation -----------------------------------------------------

      OK = OpenMaster()

      InternalFileName = 'netprint.dat'
      PrintUnit = GetFileUnit(InternalFileName)

      IF( InitializeNetworkControl() ) THEN

        IF( DefineNetwork() ) THEN

          OK = InitNetBalance()
          
          DO 100 Step = 1,NetworkTimeSteps()

            IF( UpdateNetwork() ) THEN

              OK = UpdateNetBalance()
            
            ELSE
              WRITE(*,*) ' Network update failed,'
              WRITE(*,*) ' Abnormal program end.'
              STOP
            END IF

100       CONTINUE

*-------- Write network restart file.
          OK = WriteNetworkRestartFile()

*-------- Write space- and time-series network results.
          OK = FlushSeries()
          
*-------- Compute and report final volume and mass balances.
          OK = ReportNetBalance()

          WRITE(*,*) ' '
          WRITE(*,*) '   -----------------------------'
          WRITE(*,*) ' '
          WRITE(*,*) ' ',
     #      TotalNetworkIterations(),' total network iterations...'

          WRITE(PrintUnit,*) ' Normal program end.'

          WRITE(PrintUnit,*) ' '
          WRITE(PrintUnit,*)
     #              '   -----------------------------'
          WRITE(PrintUnit,*) ' '
          WRITE(PrintUnit,*)
     #      TotalNetworkIterations(),'  total network iterations...'
          WRITE(PrintUnit,*) ' '
          WRITE(PrintUnit,*)
     #              '   -----------------------------'

          WRITE(*,*) ' '
          WRITE(*,*) '   Normal program end.'
          WRITE(*,*) ' '
          WRITE(*,*) '   -----------------------------'
          WRITE(*,*) ' '

        ELSE
          WRITE(*,*) ' System definition not complete,'
          WRITE(*,*) ' Abnormal program end.'
          STOP
        END IF

      ELSE
        WRITE(*,*) ' Failed to read network control information...'
        WRITE(*,*) ' Abnormal program end.'
        STOP
      END IF
      
      OK = CloseMaster()
     
      STOP
      END



*== Public (DefineNetwork) =============================================

      LOGICAL FUNCTION DefineNetwork()

      IMPLICIT NONE

*   Purpose:  Define a network of channels in terms of downstream locations
*             within each channel and corresponding initial values of depth
*             of flow and flow.

*   Arguments:

*   Argument definitions:

*   Module data:

*   Local variables:
      LOGICAL OK

*   Routines by module:

***** Buffered output:
      LOGICAL  InitSpaceTimeSeries
      EXTERNAL InitSpaceTimeSeries

***** Channel schematic:
      LOGICAL  InitializeChannelNetwork
      EXTERNAL InitializeChannelNetwork

***** Solver:
      LOGICAL  InitializeSolver
      EXTERNAL InitializeSolver
      
***** Network control:
      INTEGER  NetworkPrintLevel
      EXTERNAL NetworkPrintLevel

***** Network boundary values:
      LOGICAL  InitializeNetworkBoundaryVal
      EXTERNAL InitializeNetworkBoundaryVal
      
*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: December 1992
*


*-- Implementation -----------------------------------------------------

      DefineNetwork = .FALSE.
      IF( NetworkPrintLevel() .GT. 0 ) THEN
        WRITE(*,*) ' Defining network...'
      END IF

      IF( InitializeChannelNetwork() ) THEN

        OK = InitializeNetworkBoundaryVal()
        
        IF( InitSpaceTimeSeries() ) THEN

          IF( InitializeSolver() ) THEN
          ELSE
           WRITE(*,*)
     #            ' Initialization of Skyline-matrix solver failed...'
           STOP
          END IF

        ELSE

          WRITE(*,*) ' Initialization of space- and time-series',
     #                ' buffers failed...'
          RETURN

        END IF

      ELSE
      WRITE(*,*) ' Initialization of the network of channels failed...'
        RETURN
      END IF

      DefineNetwork = .TRUE.

      RETURN
      END



*== Public (UpdateNetwork) =============================================

      LOGICAL FUNCTION UpdateNetwork()

      IMPLICIT NONE

*   Purpose:  Determine values of discharge and watersurface elevation
*             within a network of open channels at a new time.

*   Arguments:

*   Argument Definitions:

*   Module data:

*   Local variables:
      INTEGER ChannelNumber
      LOGICAL OK

*   Routines by module:

***** Channel schematic:
      INTEGER  NumberOfChannels
      EXTERNAL NumberOfChannels

      LOGICAL  OpenChannel, CloseChannel
      EXTERNAL OpenChannel, CloseChannel

***** Network control:
      LOGICAL  OpenNetworkIteration, CloseNetworkIteration
      LOGICAL  VariableStreamDensity
      EXTERNAL VariableStreamDensity
      EXTERNAL OpenNetworkIteration, CloseNetworkIteration

      LOGICAL  IncrementNetworkIteration
      EXTERNAL IncrementNetworkIteration

      LOGICAL  IncrementNetworkTimeStep, SetNetworkBoundaryEqValues
      EXTERNAL IncrementNetworkTimeStep, SetNetworkBoundaryEqValues
      
      LOGICAL  SetNewNetworkDensity, SetUserNetBndValues
      EXTERNAL SetNewNetworkDensity, SetUserNetBndValues

***** Solver:
      LOGICAL  SolveFourPt
      EXTERNAL SolveFourPt

***** Locals:
      LOGICAL  UpdateChannel, NetworkClosure
      EXTERNAL UpdateChannel, NetworkClosure

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: December 1992
*


*-- Implementation -----------------------------------------------------

      UpdateNetwork = .FALSE.

*     WRITE(*,*) ' Updating network...'

      OK = IncrementNetworkTimeStep()
      OK = SetNetworkBoundaryEqValues()
      OK = SetUserNetBndValues()
      IF( VariableStreamDensity() ) THEN
        OK = SetNewNetworkDensity()
      END IF

      IF( OpenNetworkIteration() ) THEN

 100    CONTINUE

*------ Begin iteration loop.
        OK = IncrementNetworkIteration()

        DO 200 ChannelNumber = 1, NumberOfChannels()

          IF( OpenChannel(ChannelNumber) ) THEN

            IF( UpdateChannel() ) THEN

              OK = CloseChannel()

            ELSE
              WRITE(*,*) ' Update of channel',ChannelNumber,' failed...'
              RETURN
            END IF

          ELSE
            OK = CloseChannel()
          END IF

 200    CONTINUE


*------ Solve for incremental change in dependent variables.

        IF( SolveFourPt() ) THEN
        ELSE
          WRITE(*,*) ' *** Error (UpdateNetwork)'
          WRITE(*,*) ' Failed to solve matrix...'
          RETURN
        END IF

*------ Update dependent variables and check for closure.

        IF( .Not. NetworkClosure() ) THEN

          GO TO 100

        END IF
        
*------ End iteration loop.

        OK = CloseNetworkIteration()

      ELSE
      END IF

      UpdateNetwork = .TRUE.

      RETURN
      END



*== Public (UpdateChannel) =============================================

      LOGICAL FUNCTION UpdateChannel()

      IMPLICIT NONE

*   Purpose:  Update dependent variables defining a channel.  In this
*             implementation, dependent variables are flow and water-
*             surface elevation.

*   Arguments:

*   Argument definitions:

*   Module data:

*   Local variables:
      INTEGER Node, I, Terms

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer, DownstreamPointer
      EXTERNAL UpstreamPointer, DownstreamPointer

***** Flow equations:
      LOGICAL  DynamicWaveEq, DiffusionWaveEq, KinematicWaveEq
      LOGICAL  DynamicWaveEqDS, DiffusionWaveEqS
      EXTERNAL DynamicWaveEq, DiffusionWaveEq, KinematicWaveEq
      EXTERNAL DynamicWaveEqDS, DiffusionWaveEqS

***** Network control:
      INTEGER  Terms1D
      LOGICAL  VariableStreamSinuosity
      EXTERNAL Terms1D
      EXTERNAL VariableStreamSinuosity

***** Locals:
      LOGICAL  SetUpstreamConstraint, SetDownstreamConstraint
      EXTERNAL SetUpstreamConstraint, SetDownstreamConstraint

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      UpdateChannel = .TRUE.

*---- Compute and store constraint-equation coefficients for
*      upstream end of channel.

      IF( SetUpstreamConstraint() ) THEN
      ELSE
        UpdateChannel = .FALSE.
        WRITE(*,*) ' *** Error (UpdateChannel)'
        WRITE(*,*) ' Attempt to set upstream constraint failed...'
      END IF


*---- Compute and store intermediate general-equation coeficients for ...

      Node = 0
      Terms = Terms1D()

      IF(Terms .EQ. 1 ) THEN

*------ Dynamic wave equation.

        IF( .NOT. VariableStreamSinuosity() ) THEN

*-------- Constant sinuosity and water density.

          DO 100 I=UpstreamPointer(),DownstreamPointer()-1
            Node = Node + 1

            IF( DynamicWaveEq(Node, Node+1) ) THEN
            ELSE
              UpdateChannel = .FALSE.
              WRITE(*,*) ' *** Error (UpdateChannel)'
              WRITE(*,*) ' Failed to update node...',Node
            END IF

 100      CONTINUE


        ELSE

*-------- Variable sinuosity and water density.

          DO 150 I=UpstreamPointer(),DownstreamPointer()-1
            Node = Node + 1

            IF( DynamicWaveEqDS(Node, Node+1) ) THEN
            ELSE
              UpdateChannel = .FALSE.
              WRITE(*,*) ' *** Error (UpdateChannel)'
              WRITE(*,*) ' Failed to update node...',Node
            END IF
            
 150      CONTINUE

        END IF


      ELSE IF(Terms .EQ. 2) THEN

*------ Diffusion-wave equation.

        IF( .NOT. VariableStreamSinuosity() ) THEN

*-------- Constant sinuosity and water density.

          DO 200 I=UpstreamPointer(),DownstreamPointer()-1
            Node = Node + 1

            IF( DiffusionWaveEq(Node, Node+1) ) THEN
            ELSE
              UpdateChannel = .FALSE.
              WRITE(*,*) ' *** Error (UpdateChannel)'
              WRITE(*,*) ' Failed to update node...',Node
            END IF

 200      CONTINUE

        ELSE
          
*-------- Variable sinuosity but constant water density.

          DO 250 I=UpstreamPointer(),DownstreamPointer()-1
            Node = Node + 1

            IF( DiffusionWaveEqS(Node, Node+1) ) THEN
            ELSE
              UpdateChannel = .FALSE.
              WRITE(*,*) ' *** Error (UpdateChannel)'
              WRITE(*,*) ' Failed to update node...',Node
            END IF

 250      CONTINUE

        END IF


      ELSE IF(Terms .EQ. 3) THEN

*------ Kinematic-wave equation, constant sinuosity and density.

        DO 300 I=UpstreamPointer(),DownstreamPointer()-1
          Node = Node + 1

          IF( KinematicWaveEq(Node, Node+1) ) THEN
          ELSE
            UpdateChannel = .FALSE.
            WRITE(*,*) ' *** Error (UpdateChannel)'
            WRITE(*,*) ' Failed to update node...',Node
          END IF

 300    CONTINUE

      ELSE
        UpdateChannel = .FALSE.
        WRITE(*,*) ' *** Error (UpdateChannel)'
        WRITE(*,*) ' Terms = ',Terms,'  not supported...'
      END IF


*---- Compute and store constraint-equation coefficients for
*      downstream end of channel.

      IF( SetDownstreamConstraint() ) THEN
      ELSE
        UpdateChannel = .FALSE.
        WRITE(*,*) ' *** Error (UpdateChannel)'
        WRITE(*,*) ' Attempt to set downstream constraint failed...'
      END IF

      RETURN
      END



*== Public (SetUpstreamConstraint) =====================================

      LOGICAL FUNCTION SetUpstreamConstraint()

      IMPLICIT NONE

*   Purpose:  Compute and store a channel's upstream constraint-equation
*             coefficients in coefficient matrix.

*   Arguments:

*   Argument definitions:

*   Module data:

*   Local Variables:
      INTEGER Code, Extremity

*   Routines by module:

***** Channel constraint:
      LOGICAL   ForceStreamSurface, ForceStreamFlow
      EXTERNAL  ForceStreamSurface, ForceStreamFlow

      LOGICAL   ForceSumStreamFlow, ForceEqualStreamSurface
      EXTERNAL  ForceSumStreamFlow, ForceEqualStreamSurface

      LOGICAL   ForceStreamSurfaceSlope
      EXTERNAL  ForceStreamSurfaceSlope
      
      LOGICAL   Force3ParmStreamSurface, Force3ParmStreamFlow
      EXTERNAL  Force3ParmStreamSurface, Force3ParmStreamFlow
      
      LOGICAL   ForceUserStreamConstraint
      EXTERNAL  ForceUserStreamConstraint

***** Channel schematic:
      INTEGER   UpstreamCode
      EXTERNAL  UpstreamCode
      
*   Intrinsics:
      INTEGER   MOD
      INTRINSIC MOD

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: December 1992
*


*-- Implementation -----------------------------------------------------

      SetUpstreamConstraint = .FALSE.

*---- Boundary-condition code          Use...
*
*       water surface
*          1                        explicitly known
*          4                        self-setting (downstream only)
*         11                        equal to another water surface
* ........31                        3-parameter relation
* ........41                        (reserved for four-parameter rating)
* ........51                        user-defined
*
*       flow
*          2                        explicitly known
*         12                        sum of flows equal zero
* ........32                        3-parameter relation
* ........42                        (reserved for four-parameter rating)
* ........52                        user-defined

*---- Set channel-extremity code to upstream.
      Extremity = +1

*---- Get upstream boundary-condition code.
      Code = MOD( UpstreamCode(), 100 )

*---- Set appropriate constraint.
      IF     ( Code .EQ. 1 ) THEN
        IF( ForceStreamSurface(Extremity) ) THEN
        ELSE
          RETURN
        END IF

      ELSE IF( Code .EQ. 2 ) THEN
        IF( ForceStreamFlow(Extremity) ) THEN
        ELSE
          RETURN
        END IF

      ELSE IF (Code .EQ. 12 ) THEN
        IF( ForceSumStreamFlow(Extremity) ) THEN
        ELSE
          RETURN
        END IF

      ELSE IF (Code .EQ. 11 ) THEN
        IF( ForceEqualStreamSurface(Extremity) ) THEN
        ELSE
          RETURN
        END IF

      ELSE IF( Code .EQ. 4 ) THEN
        IF( ForceStreamSurfaceSlope(Extremity) ) THEN
        ELSE
          RETURN
        END IF

      ELSE IF( Code .EQ. 31 ) THEN
        IF( Force3ParmStreamSurface(Extremity) ) THEN
        ELSE
          RETURN
        END IF

      ELSE IF( Code .EQ. 32 ) THEN
        IF( Force3ParmStreamFlow(Extremity) ) THEN
        ELSE
          RETURN
        END IF

      ELSE IF( Code .EQ. 51
     #              .OR.
     #         Code .EQ. 52 ) THEN
        IF( ForceUserStreamConstraint(Extremity) ) THEN
        ELSE
          RETURN
        END IF
      
      ELSE
        WRITE(*,*) ' *** Error (SetUpstreamConstraint)'
        WRITE(*,*) ' Condition code ',Code,' not implemented.'
        RETURN

      END IF


      SetUpstreamConstraint = .TRUE.


      RETURN
      END



*== Public (SetDownstreamConstraint) ===================================

      LOGICAL FUNCTION SetDownstreamConstraint()

      IMPLICIT NONE

*   Purpose:  Compute and store a channel's downstream constraint-equation
*             coefficients in coefficient matrix.

*   Arguments:

*   Arguments definitions:

*   Module data:

*   Local Variables:
      INTEGER Code, Extremity

*   Routines by module:

***** Channel constraint:
      LOGICAL   ForceStreamSurface, ForceStreamFlow
      EXTERNAL  ForceStreamSurface, ForceStreamFlow

      LOGICAL   ForceSumStreamFlow, ForceEqualStreamSurface
      EXTERNAL  ForceSumStreamFlow, ForceEqualStreamSurface

      LOGICAL   ForceStreamSurfaceSlope
      EXTERNAL  ForceStreamSurfaceSlope
      
      LOGICAL   Force3ParmStreamSurface, Force3ParmStreamFlow
      EXTERNAL  Force3ParmStreamSurface, Force3ParmStreamFlow
      
      LOGICAL   ForceUserStreamConstraint
      EXTERNAL  ForceUserStreamConstraint

***** Channel schematic:
      INTEGER   DownstreamCode
      EXTERNAL  DownstreamCode
      
*   Intrinsics:
      INTEGER   MOD
      INTRINSIC MOD

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: December 1992
*


*-- Implementation -----------------------------------------------------

      SetDownstreamConstraint = .FALSE.

*---- Boundary-condition code          Use...
*
*       water surface
*          1                        explicitly known
*          4                        self-setting (downstream only)
*         11                        equal to another water surface
* ........31                        3-parameter relation
* ........41                        (reserved for four-parameter rating)
* ........51                        user-defined
*
*       flow
*          2                        explicitly known
*         12                        sum of flows equal zero
* ........32                        3-parameter relation
* ........42                        (reserved for four-parameter rating)
* ........52                        user-defined

*---- Set channel-extremity code to downstream.
      Extremity = -1

*---- Get downstream boundary-condition code.
      Code = Mod( DownstreamCode(), 100 )

*---- Set appropriate constraint.
      IF     ( Code .EQ. 1 ) THEN
        IF( ForceStreamSurface(Extremity) ) THEN
        ELSE
          RETURN
        END IF

      ELSE IF( Code .EQ. 2 ) THEN
        IF( ForceStreamFlow(Extremity) ) THEN
        ELSE
          RETURN
        END IF

      ELSE IF( Code .EQ. 12 ) THEN
        IF( ForceSumStreamFlow(Extremity) ) THEN
        ELSE
          RETURN
        END IF

      ELSE IF( Code .EQ. 11 ) THEN
        IF( ForceEqualStreamSurface(Extremity) ) THEN
        ELSE
          RETURN
        END IF

      ELSE IF( Code .EQ. 4 ) THEN
        IF( ForceStreamSurfaceSlope(Extremity) ) THEN
        ELSE
          RETURN
        END IF

      ELSE IF( Code .EQ. 31 ) THEN
        IF( Force3ParmStreamSurface(Extremity) ) THEN
        ELSE
          RETURN
        END IF

      ELSE IF( Code .EQ. 32 ) THEN
        IF( Force3ParmStreamFlow(Extremity) ) THEN
        ELSE
          RETURN
        END IF

      ELSE IF( Code .EQ. 51
     #              .OR.
     #         Code .EQ. 52 ) THEN
        IF( ForceUserStreamConstraint(Extremity) ) THEN
        ELSE
          RETURN
        END IF
      
      ELSE
        WRITE(*,*) ' *** Error (SetDownstreamConstraint)'
        WRITE(*,*) ' Condition code ',Code,' not implemented.'
        RETURN
      END IF


      SetDownstreamConstraint = .TRUE.


      RETURN
      END



*== Public (NetworkClosure) ============================================

      LOGICAL FUNCTION NetworkClosure()

      IMPLICIT NONE

*   Purpose:  Check for closure of iteration on a set of equations
*             describing flow in a network of open channels, and
*             update dependent variables.

*   Arguments:

*   Argument definitions:

*   Module data:

*   Local Variables:
      INTEGER I, J
      INTEGER ChannelQ, NodeQ, ChannelZ, NodeZ
      INTEGER ChannelNumber, Node
      REAL    SmallQ
      PARAMETER (SmallQ = 100.0)
      REAL    Change, CurrentValue, CurrentRatioQ, CurrentChangeZ
      REAL    MaxChangeZ, MaxRatioQ
      LOGICAL OK

*   Routines by module:

***** Channel flow status:
      LOGICAL  SetStreamSurfaceElevation, SetStreamFlow
      EXTERNAL SetStreamSurfaceElevation, SetStreamFlow

      REAL     StreamFlow, StreamSurfaceElevation
      EXTERNAL StreamFlow, StreamSurfaceElevation

      LOGICAL  StoreNetworkClosure
      EXTERNAL StoreNetworkClosure

***** Channel schematic:
      INTEGER  NumberOfStreamLocations
      EXTERNAL NumberOfStreamLocations

      INTEGER  NumberOfChannels, MaxNetworkIterations
      EXTERNAL NumberOfChannels, MaxNetworkIterations

      LOGICAL  OpenChannel, CloseChannel
      EXTERNAL OpenChannel, CloseChannel

***** Solver:
      REAL     IncrementalZ, IncrementalQ
      EXTERNAL IncrementalZ, IncrementalQ

***** Network control:
      LOGICAL  IntermediateNetworkResults
      EXTERNAL IntermediateNetworkResults
      
*   Intrinsics:
      REAL       ABS
      INTRINSIC  ABS

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: December 1992
*


*-- Implementation -----------------------------------------------------


      MaxRatioQ  = 0.0
      MaxChangeZ = 0.0

      DO 200 I=1,NumberOfChannels()

        ChannelNumber = I

        IF( OpenChannel(ChannelNumber) ) Then

          DO 100 J = 1,NumberOfStreamLocations()
            Node = J

*---------- Check for closure on flow.

            Change = IncrementalQ(Node)
            CurrentValue = StreamFlow(Node)

            IF( ABS(CurrentValue) .GT. SmallQ  ) THEN
*------------ Flow is greater than a small flow.
              CurrentRatioQ = ABS(Change/CurrentValue)
            ELSE
*------------ Flow is smaller than a small flow.
              CurrentRatioQ = ABS(Change/SmallQ)
            END IF

            IF( CurrentRatioQ .GT. MaxRatioQ ) THEN
              MaxRatioQ = CurrentRatioQ
              NodeQ = Node
              ChannelQ = ChannelNumber
            END IF

*---------- Update flow.
            OK = SetStreamFlow( Node, CurrentValue + Change )


*---------- Check for closure on water-surface elevation.

            Change = IncrementalZ(Node)
            CurrentChangeZ = ABS(Change)
            CurrentValue = StreamSurfaceElevation(Node)

            IF( CurrentChangeZ .GT. MaxChangeZ ) THEN
              MaxChangeZ = CurrentChangeZ
              NodeZ = Node
              ChannelZ = ChannelNumber
            END IF

*---------- Update water-surface elevation.
            OK = SetStreamSurfaceElevation(Node, CurrentValue + Change)

 100      CONTINUE

        OK = CloseChannel()

      ELSE
      END IF

 200  CONTINUE


*---- Check for overall closure and store closure information.

      IF( StoreNetworkClosure(
     #                 MaxRatioQ, ChannelQ, NodeQ,
     #                 MaxChangeZ, ChannelZ, NodeZ
     #                       )
     #  ) THEN

        NetworkClosure = .TRUE.
      ELSE
        NetworkClosure = .FALSE.
      END IF

*---- Write intermediate results.

      OK = IntermediateNetworkResults()

      RETURN
      END



*==  Public (ReportNetworkStatus) ======================================

      LOGICAL FUNCTION ReportNetworkStatus()

      IMPLICIT NONE

*   Purpose:  Report current status of flow in a network of open channels.

*   Arguments:

*   Argument definitions:

*   Module data:

*   Local Variables:
      INTEGER PrintUnit
      INTEGER I, J, Node, ChannelNumber, Channels
      LOGICAL OK
      CHARACTER*12 InternalFileName

*   Routines by module:

***** Channel flow status:
      REAL     StreamFlow, StreamDepth, StreamDistance
      EXTERNAL StreamFlow, StreamDepth, StreamDistance

      REAL     StreamSurfaceElevation
      EXTERNAL StreamSurfaceElevation

***** Channel schematic:
      INTEGER  NumberOfChannels, CurrentChannel, NumberOfStreamLocations
      EXTERNAL NumberOfChannels, CurrentChannel, NumberOfStreamLocations

      LOGICAL  OpenChannel, CloseChannel
      EXTERNAL OpenChannel, CloseChannel
      
***** Master file:  
      INTEGER  GetFileUnit
      EXTERNAL GetFileUnit

*   Programmed by: Lew DeLong
*   Date:          November 1990
*   Modified by:   Barry Wicktom (use of master file names added)
*   Last modified: Tues  01-07-1992
*


*-- Implementation -----------------------------------------------------


      InternalFileName = 'netprint.dat'
      PrintUnit = GetFileUnit(InternalFileName)

      ReportNetworkStatus = .FALSE.

      Channels = NumberOfChannels()

      DO 200 I=1,Channels

        ChannelNumber = I

        IF( Channels .GT. 1 ) THEN
          WRITE(PrintUnit,*) ' '
          WRITE(PrintUnit,*) '        Channel...',ChannelNumber
        END IF

        IF( OpenChannel( ChannelNumber ) ) THEN

          WRITE(PrintUnit,*) ' '
          WRITE(PrintUnit,*)
     #       'CompCxNo  Location   Discharge   WS_Elev     Depth'

          DO 100 J=1,NumberOfStreamLocations()
            Node = J

            WRITE(PrintUnit,'(I4,2X,4F11.2)')
     #           Node, StreamDistance(Node),
     #           StreamFlow(Node), StreamSurfaceElevation(Node),
     #           StreamDepth(Node)

 100      CONTINUE

          ELSE

            WRITE(*,*) ' ***Error (ReportSystemStatus)'
            WRITE(*,*) ' Failed to open channel ',ChannelNumber
            WRITE(PrintUnit,*) '***Error (ReportSystemStatus)'
            WRITE(PrintUnit,*) 'Failed to open channel ',ChannelNumber
            STOP

          END IF

          OK = CloseChannel()

 200    CONTINUE

      ReportNetworkStatus = .TRUE.

      RETURN
      END


*==== EOF fourpt =======================================================
