*==== BOF netcntr1 =====================================================

*   Module data:
*
*    'network.inc'
*     MaxChannels - maximum number of channels.
*     NumCh - current number of channels.
*     Branch - current selected or active channel.
*     MaxLocations - maximum number of locations (computational or user).
*
*
*
*    'netcntrl.inc'
*     MaxQuadPts - maximum number of quadrature points allowed
*                  for numerical spatial integration between
*                  adjacent computational cross sections.
*     MaxBndEq - maximum number of equation-type boundaries.
*     MaxTS - Maximum number of locations at which timeseries
*             output may be accumulated.

*     Terms - index indicating form of 1-D flow equations.
*              [1] full dynamic-wave equation.
*              [2] diffusion-wave equation.
*              [3] kinematic-wave equation.
*
*     VariableSinuosity - .TRUE. if allowed to vary, .FALSE. otherwise.
*     VariableDensity - .TRUE. if allowed to vary, .FALSE. otherwise.
*     Perturbation - .TRUE. if perturbation of input is requested,
*                    otherwise .FALSE. .
*
*     Gravity - acceleration due to gravity.
*
*     Seed - seed for random-number generator.
*
*     MaxTimeSteps - maximum number of time steps allowed.
*     TimeStep - current time step.
*     MaxIterations - Maximum number of iterations per time step.
*     Iteration - current iteration number.
*     TotalIterations - total iterations.
*     LuInc - interval of complete forward eliminations, in terms of
*             number of iterations during a time step.
*
*     PrintLevel - level of printing activity, 0 to 9, activity
*                   increasing with increasing number.
*     PrintCount - counter for printed output.
*     PrintInc - increment, in time steps, for printed output.
*     TimeSeriesCount - counter for time-series output.
*     TimeSeriesInc - increment, in time steps, for time-series output.
*     SpatialSeriesCount - counter for spatial-series output.
*     SpatialSeriesInc - increment for spatial-series output.
*
*     TimeSeriesLocation(i) - global computational location sequence number
*                              for the "i"th time series.
*     TSChannel(i) - channel number coreesponding to TimeSeriesLocation(i).
*     TSX(i) - downstream distance corresponding to TimeSeriesLocation(i).
*
*     TimeSeries - total number of time series requested.
*
*
*
*                 Equation-type boundary values ...
*
*       value = EqBase + for each component [
*
*         Amplitude * cos( TwoPI* (Hours+PhaseAngle) / Period )
*
*                                           ]
*
*
*
*             ( time unit for following constants is hours... )
*
*     BoundaryEquations - number of user-supplied boundary equations.
*     EqNumber(k+MaxChannels+1) - equation number for channel k
*      (upstream end of channel k>0, downstream end of channel k<0).
*     EqChannelNumber(j) - channel number, + upsteam, - downstream, equation j.
*     EqComponents(j) - number of harmonic components in equation j.
*     MaxEqComponents - maximum harmonic components per equation.
*     EqBase(j) - base value, equation j.
*     Amplitude1(i,j) - amplitude of component i, equation j.
*     Period(i,j) - period, in hours, of component i, equation j.
*     PhaseAngle(i,j) - phase angle, in hours, of component i, equation j.
*     EqStart(j) - elapse time, in hours, at which equation becomes effective.
*     EqEnd(j) - elapse time, in hours, time at which equation is no longer effective.
*     j - sequential storage index.
*     StreamBndValue(k) - current value computed from equation, possibly
*                         one upstream and one downstream value,
*                         sequentially, odd "k" are upstream,
*                         even are downstream.
*
*
*
*          Standard deviations of normal distributions of errors, used to
*          perturb boundary-equation values, follow. Perturbation will be active
*          when the file "perturb.dat" exists and may be opened and read
*          successfully.  Standard deviations are initialized to zero.
*          Non-zero values are read from file "perturb.dat" if it exists.
*
*
*          group 1: applied each time step and held constant over a time step.
*
*     EqBaseStdDev(j) - standard deviation of a random normal distribution
*                          of errors in base value for equation j.
*     AmplitudeStdDev(j,i) - standard deviation of a random normal distribution
*                          of errors in amplitude for component i, equation j.
*     PeriodStdDev(j,i) - standard deviation of a random normal distribution
*                          of errors in period for component i, equation j.
*     PhaseAngleStdDev(j,i) - standard deviation of a random normal distribution
*                          of errors in phase angle for component i, equation j.
*
*          group 2: applied only once, at the beginning of a model run,
*                   and held constant through out the entire run.
*
*     EqBaseBias(j,i) - standard deviation of a random normal distribution
*                          of errors in base value for equation j.
*     AmplitudeBias(j,i) - standard deviation of a random normal distribution
*                          of errors in amplitude for component i, equation j.
*     PeriodBias(j,i) - standard deviation of a random normal distribution
*                          of errors in period for component i, equation j.
*     PhaseAngleBias(j,i) - standard deviation of a random normal distribution
*                          of errors in phase angle for component i, equation j.
*
*
*
*     DT - time increment, in seconds.
*     Theta - time-weighting factor.
*     ToleranceQ - closure tolerance for discharge.
*     ToleranceZ - closure tolerance for water-surface elevation.
*     RatioQ - current maximum ratio of change in Q to prior Q (flow).
*     ChangeZ - current maximum change in Z (water-surface elevation).
*     ChannelRatioQ - channel in which RatioQ occurs.
*     NodeRatioQ - location within ChannelRatioQ that RatioQ occurs.
*     ChannelChangeZ - channel in which ChangeZ occurs.
*     NodeChangeZ - location within ChannelChangeZ that ChangeZ occurs.
*
*     CloseQ - .TRUE. if closure criteria for discharge has been
*                determined to currently be satisfied, .FALSE. otherwise.
*     CloseZ - .TRUE. if closure criteria for ws_elev has been determined
*                to currently be satisfied, .FALSE. otherwise.
*     EndIteration - .TRUE. if it has determined that iterations are to
*                      stop, .FALSE. otherwise.
*
*
*     MaxQuadPts - Maximum number of quadrature points.
*     QuadPts - current number of quadrature points.
*     QuadPt(i) - location of quadrature point "i",
*                  in local coordinate ( 0 to 1 ).
*     QuadWt(i) - weight, corresponding to quadrature point "i",
*                  for numerical integration.
*
*     Seconds - elapse time, in seconds



*== Public (InitializeNetworkControl) ==================================

      LOGICAL FUNCTION InitializeNetworkControl()

      IMPLICIT NONE

*   Purpose:  Initialize NetworkControl module.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:
      LOGICAL  OK

*   Routines by module:

***** Local:
      INTEGER  NetworkPrintLevel
      LOGICAL  ReadNetworkControl, PerturbBoundaryEq
      EXTERNAL NetworkPrintLevel
      EXTERNAL ReadNetworkControl, PerturbBoundaryEq

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      IF( NetworkPrintLevel() .GT. 0 ) THEN
        WRITE(*,*) ' Initializing NetworkControl module...'
      END IF

*---- Initialize time-step counter.
      TimeStep = 0

      IF( ReadNetworkControl()) THEN

        InitializeNetworkControl = .TRUE.
        
        IF( Perturbation ) THEN
          OK = PerturbBoundaryEq()
        END IF

      ELSE

        InitializeNetworkControl = .FALSE.
        WRITE(*,*) ' Reading of program control data failed...'

      END IF

      RETURN
      END



*== Private (ReadNetworkControl) =======================================

      LOGICAL FUNCTION ReadNetworkControl()

      IMPLICIT NONE

*   Purpose:  Read network program control data.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:
      INTEGER   TwoTo31Minus1
      PARAMETER(TwoTo31Minus1=2147483647)
      INTEGER I, P, K
      INTEGER PrintUnit, ControlUnit
      INTEGER ReadBnd
      REAL    TotalWeight
      LOGICAL OK
      CHARACTER*12 PRTOUT
      CHARACTER*12 CNTRL, InternalFileName1
      CHARACTER*12 InternalFileName3
      CHARACTER*80 HeaderMessage

*   Routines by module:

***** Master file:
      INTEGER GetFileUnit
      CHARACTER*12 GetFileName
      EXTERNAL GetFileName, GetFileUnit

***** File utilities:
      LOGICAL*4  OpenNewText, OpenOldText
      EXTERNAL   OpenNewText, OpenOldText
      
***** Boundary-value:
      LOGICAL  SetUserNetBndValueIndex
      EXTERNAL SetUserNetBndValueIndex

*   Intrinsics:
      INTEGER   IABS
      REAL       ABS, FLOAT
      INTRINSIC IABS, ABS, FLOAT

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Barry Wicktom (use of master file names added)
*   Date modified: 1/6/92
*   Modified by:   Lew DeLong
*   Date Modified: AUG   1992


*-- Implementation -----------------------------------------------------


*   Output formats:
 1000 FORMAT(
     #   /
     #   10X,'                   .... FourPt ....'/
     #   /
     #   10X,'    Computation of unsteady, 1-D open-channel flow,'/
*    #   10X,'          using a four-point-implicit scheme.'/
     #   /
     #   10X,'              Version 95.01, February 1995'/
     #   ' '
     #   /
     #      )


      ReadNetworkControl = .FALSE.
      
*                          123456789012
      InternalFileName1 = 'netprint.dat'
      InternalFileName3 = 'control.dat '
      
      PrintUnit = GetFileUnit(InternalFileName1)
      ControlUnit = GetFileUnit(InternalFileName3)
      PRTOUT = GetFileName(InternalFileName1)
      CNTRL = GetFileName(InternalFileName3)

      
      WRITE(*,*) ' Reading program-control data...'
      WRITE(*,*) ' '
      WRITE(*,1000)

      P = PrintUnit

      IF(OpenNewText(PrintUnit,PRTOUT)) THEN
        WRITE(PrintUnit,1000)
       ELSE
        WRITE(*,*) ' Failed to open print file.'
        WRITE(*,*) ' File name : ', PRTOUT
        WRITE(*,*) ' Abnormal program end.'
        STOP
      END IF
      IF(OpenOldText(ControlUnit,CNTRL)) THEN


        READ(ControlUnit,'(A80)') HeaderMessage
        WRITE(PrintUnit,*) ' '
        WRITE(PrintUnit,'(1X,A80)') HeaderMessage
        WRITE(*,*) ' '
        WRITE(*,'(1X,A80)') HeaderMessage
        READ(ControlUnit,'(A80)') HeaderMessage
        WRITE(PrintUnit,'(1X,A80)') HeaderMessage
        WRITE(*,'(1X,A80)') HeaderMessage

        READ(ControlUnit  ,*) Terms
        WRITE(*,*) ' '
        WRITE(P,*) ' '
        WRITE(P,*) ' '
        WRITE(P,*) ' Flow-equation particulars:'
        WRITE(P,*) ' '
        IF(Terms.EQ.1) THEN
          WRITE(P,*) ' Full dynamic-wave solution...'
          WRITE(*,*) ' Full dynamic-wave solution...'
        ELSE IF(Terms.EQ.2) THEN
          WRITE(P,*) ' Diffusion-wave solution...'
          WRITE(*,*) ' Diffusion-wave solution...'
        ELSE IF(Terms.EQ.3) THEN
          WRITE(P,*) ' Kinematic-wave solution...'
          WRITE(*,*) ' Kinematic-wave solution...'
        ELSE
          WRITE(P,*) ' Control-data input...'
          WRITE(P,*) ' Terms = ',Terms,'  not supported...'
          WRITE(P,*) ' Dynamic wave assumed.'
          WRITE(*,*) ' Control-data input...'
          WRITE(*,*) ' Terms = ',Terms,'  not supported...'
          WRITE(*,*) ' Dynamic wave assumed...'
          Terms = 1
        END IF
        
        READ(ControlUnit,*) I
        IF( I .EQ. 0 ) THEN
          VariableDensity = .FALSE.
          WRITE(P,*) ' Water density is assumed constant.'
        ELSE IF( I .EQ. 1 ) THEN
          VariableDensity = .TRUE.
          WRITE(P,*) ' Water density is allowed to vary.'
        ELSE
          VariableDensity = .FALSE.
          WRITE(*,*) ' ####warning, density index should be'
          WRITE(*,*) '    0 or 1.  Constant density assumed.'
        END IF
        
        READ(ControlUnit,*) I
        IF( I .EQ. 0 ) THEN
          VariableSinuosity = .FALSE.
        ELSE IF( I .EQ. 1 ) THEN
          VariableSinuosity = .TRUE.
        ELSE
          VariableSinuosity = .FALSE.
          WRITE(*,*) ' ####warning, sinuosity index should be'
          WRITE(*,*) '    0 or 1.  Constant sinuosity assumed.'
        END IF
        

*------ Set sinuosity and density program control flags.
*       Variable density is allowed only in the dynamic-wave
*       equation.  Variable sinuosity is allowed in either the
*       dynamic-wave or difussion-wave equations.

        IF( Terms .EQ. 1 ) THEN
        
          IF( VariableDensity ) THEN
            WRITE(P,*) ' Water density is allowed to vary with',
     #                 ' time and distance.'
            WRITE(P,*) ' Sinuosity is allowed to vary with',
     #                 ' depth of flow and distance.'

            IF( .NOT. VariableSinuosity ) THEN
              WRITE(P,*) ' (This option was invoked as a result',
     #                   '  variable density option.'
              VariableSinuosity = .TRUE.
            END IF
          
          ELSE IF( VariableSinuosity ) THEN
            WRITE(P,*) ' Sinuosity is allowed to vary with',
     #                 ' depth of flow and distance.'
          END IF
          
        ELSE IF( Terms .EQ. 2 ) THEN
        
          IF( VariableSinuosity ) THEN
            WRITE(P,*) ' Sinuosity is allowed to vary with',
     #                 ' depth of flow and distance.'
          END IF
          
          IF( VariableDensity ) THEN
            WRITE(P,*) ' Water density held constant with',
     #                 ' difussion-wave option.'
            VariableDensity = .FALSE.
          END IF

        ELSE IF( Terms .EQ. 3 ) THEN
        
          IF( VariableDensity .OR. VariableSinuosity ) THEN
            WRITE(P,*) ' Water density and sinuosity held',
     #                 ' constant with kinematic-wave option.'
            VariableDensity = .FALSE.
            VariableSinuosity = .FALSE.
          END IF
        
        END IF
        
        READ(ControlUnit  ,*) ReadBnd
        OK = SetUserNetBndValueIndex(ReadBnd)        
        
        READ(ControlUnit  ,*) I
        IF( I .EQ. 1 ) THEN
          Perturbation = .TRUE.
        ELSE
          Perturbation = .FALSE.
        END IF
        
        READ(ControlUnit ,*) Gravity
        WRITE(P,*)  ' Acceleration due to gravity = ',Gravity
        

        READ(ControlUnit  ,*) MaxTimeSteps
        WRITE(P,*) ' Maximum time steps = ', MaxTimeSteps

        READ(ControlUnit  ,*) NetStartTime
        WRITE(P,*) ' Starting elapse time, in seconds, = ',NetStartTime

*------ Set initial elapse time.
        Seconds = NetStartTime
	
        READ(ControlUnit  ,*) DT
        WRITE(P,*) ' Time increment, in seconds, = ',DT
        
*------ Check for range on INTEGER Seconds.
        IF( ( FLOAT(MaxTimeSteps) * FLOAT(DT) + FLOAT(Seconds) )
     #                        .GT.
     #                       FLOAT(TwoTo31Minus1) ) THEN
          WRITE(P,*) ' ####error(ReadNetworkControl)'
          WRITE(P,*) ' Range of allowable elapse seconds exceeded...'
          WRITE(P,*) ' maximum allowable...', TwoTo31Minus1
          WRITE(P,*) ' Abnormal program end.'
          WRITE(*,*) ' ####error(ReadNetworkControl)'
          WRITE(*,*) ' Range of allowable elapse seconds exceeded...'
          WRITE(*,*) ' maximum allowable...', TwoTo31Minus1
          WRITE(*,*) ' Abnormal program end.'
          STOP
        END IF

        READ(ControlUnit  ,*) Theta
        WRITE(P,*) ' Time-weighting factor = ',Theta

        DO 70 I=1,MaxQuadPts
          QuadPt(I) = -1.0
 70     CONTINUE
        READ(ControlUnit  ,*) (QuadPt(I), QuadWt(I), I=1,MaxQuadPts)
        QuadPts = 0
        TotalWeight = 0.0
        DO 80 I=1,MaxQuadPts
          IF( QuadPt(I) .LT. 0.0 ) GO TO 82
          QuadPts = QuadPts + 1
          TotalWeight = TotalWeight + QuadWt(I)
          IF(QuadPt(I) .GT. 1.0 ) THEN
            WRITE(P,*) ' Control-data input...'
            WRITE(P,*) ' Quadrature location > 1.0'
            WRITE(P,*) I,QuadPt(I)
            WRITE(P,*) ' Abnormal program end.'
          END IF
 80     CONTINUE
 82     CONTINUE
        IF(QuadPts .EQ. 0 ) THEN
          QuadPts = 1
          QuadPt(1) = 0.5
          QuadWt(1) = 1.0
        ELSE IF( ABS( TotalWeight - 1.0 ) .GT. 1.0E-6 ) THEN
          WRITE(P,*) ' Control-data input...'
          WRITE(P,*) ' Warning....'
          WRITE(P,*) ' Total quadrature weights = ', TotalWeight
          WRITE(P,*) '  ( should be 1.0 )'
        END IF
        WRITE(P,*) ' Number of quadrature points = ',QuadPts
        WRITE(P,*) '  local coordinates..... weights...'
        WRITE(P,'(2F15.6)') (QuadPt(I), QuadWt(I), I=1,QuadPts)


        READ(ControlUnit  ,*) MaxIterations
        WRITE(P,*)
     #  ' Maximum number of iterations per time step = ',
     #    MaxIterations

        READ(ControlUnit  ,*) LuInc
        WRITE(P,*)
     #  ' Interval for complete forward eliminations = ', LuInc

        READ(ControlUnit,*) ToleranceQ
        WRITE(P,*)
     #  ' Tolerance for closure on discharge = ', ToleranceQ
        READ(ControlUnit,*) ToleranceZ
        WRITE(P,*)
     #  ' Tolerance for closure on water-surface elevation = ',
     #   ToleranceZ


        WRITE(P,*) ' '
        READ(ControlUnit,*) PrintLevel
        WRITE(P,*) ' Level of printing activity = ',
     #   PrintLevel
        READ(ControlUnit,*) PrintCount
        WRITE(P,*) ' Initial value of print counter = ',
     #   PrintCount
        READ(ControlUnit  ,*) PrintInc
        WRITE(P,*) ' Print increment = ', PrintInc

        READ(ControlUnit  ,*) TimeSeriesCount
        WRITE(P,*)
     #  ' Initial value of time-series output counter =',
     #    TimeSeriesCount
        READ(ControlUnit  ,*) TimeSeriesInc
        WRITE(P,*) ' Time-series increment = ', TimeSeriesInc

*------ Determine locations for which time-series output is requested.
        DO 100 I=1,MaxTS
          TSChannel(I) = 0
 100    CONTINUE
        READ(ControlUnit,*) (TSChannel(I),TSX(I),I=1,MaxTS)

*------ Determine number of requested time series.
        TimeSeries = 0
        DO 200 I=1,MaxTS
          IF( TSChannel(I) .GT. 0 ) THEN
            TimeSeries = TimeSeries + 1
          ELSE
            GO TO 202
          END IF
 200    CONTINUE
          WRITE(*,*) ' Control-data input...'
          WRITE(*,*) ' Warning... maximum output time series is ',
     #      MaxTS,' ...'
          WRITE(P,*) ' Warning... maximum output time series is ',
     #      MaxTS,' ...'
 202    CONTINUE

        IF(TimeSeries.GT.0) THEN
          WRITE(P,*) ' '
          WRITE(P,*) ' Time-series output selected for',
     #                        TimeSeries,'  Location(s):'
          WRITE(P,*) ' Channel    Location'
          WRITE(P,'(I5,1X,F12.2)') (TSChannel(I),TSX(I),I=1,TimeSeries)
        ELSE
          WRITE(P,*) ' No requested output of time series.'
        END IF

        READ(ControlUnit,*) SpatialSeriesCount
        WRITE(P,*) ' '
        WRITE(P,*)
     #  ' Initial value of spatial-series output counter =',
     #   SpatialSeriesCount
        READ(ControlUnit  ,*) SpatialSeriesInc
        WRITE(P,*) ' Spatial-series output increment = ',
     #   SpatialSeriesInc

*------ Restart file output?

        READ(ControlUnit  ,*) I
        IF( I .EQ. 1 ) THEN
          Restart = .TRUE.
          WRITE(P,*) ' Network restart file is requested.'
        ELSE
          Restart = .FALSE.
        END IF


*------ Initialize boundary-equation number array.
        DO 210 I=1,2*MaxChannels+1
          EqNumber(I) = 0
 210    CONTINUE


*------ Read boundary-equation coefficients and parameters.
        BoundaryEquations = 0
        DO 300 I=1,MaxBndEq

          READ(ControlUnit,*,END=302) EqChannelNumber(I)
          
*-------- Record equation number for extremity of channel.
          EqNumber( EqChannelNumber(I) + MaxChannels + 1 ) = I

          READ(ControlUnit,*,END=302) EqComponents(I)
          
          IF( EqComponents(I) .GT. MaxComponents
     #                        .OR.
     #        EqComponents(I) .LT. 0 )  THEN
             WRITE(P,*) ' **** Error(ReadNetworkControl)'
             WRITE(P,*) ' Number of boundary-equation components.'
             WRITE(P,*) ' Out of range.'
             WRITE(P,*) ' Read...',EqComponents(I)
             WRITE(P,*) ' Range... 0 to ',MaxComponents
             WRITE(P,*) ' Abnormal program end.'
             STOP
          
          END IF

          BoundaryEquations = BoundaryEquations + 1

          EqBase(I) = 0.0

          READ(ControlUnit,*,END=302) EqBase(I)

          DO 250 K=1,MaxComponents
            Amplitude(K,I) = 0.0
            Period(K,I) = 0.0
            PhaseAngle(K,I) = 0.0
 250      CONTINUE
          
          DO 260 K=1,EqComponents(I)
            READ(ControlUnit,*,END=302) Amplitude(K,I)
            READ(ControlUnit,*,END=302) Period(K,I)
            READ(ControlUnit,*,END=302) PhaseAngle(K,I)
 260      CONTINUE

          EqStart(I) = 0.0
          EqEnd(I) = 0.0
          READ(ControlUnit,*,END=302) EqStart(I)
          READ(ControlUnit,*,END=302) EqEnd(I)



 300    CONTINUE

 302    CONTINUE

        IF( BoundaryEquations .GT. 0 ) THEN

          WRITE(P,*) ' '
          WRITE(P,*) ' Number of user-supplied boundary equations ...',
     #     BoundaryEquations
     
          WRITE(P,*) ' '
          WRITE(P,*) ' Note, period and phase angles are in hours,'
          WRITE(P,*) ' start and end times are in simulated hours,'
          WRITE(P,*) ' elapsed since the beginning of the current'
          WRITE(P,*) ' model execution.'
          WRITE(P,*) ' '
          
          DO 400 I=1,BoundaryEquations
            WRITE(P,*) ' '
            IF( EqChannelNumber(I) .GT. 0 ) THEN
              WRITE(P,*) '  Upstream end, channel ... ',
     #                         EqChannelNumber(I)
            ELSE IF( EqChannelNumber(I) .LT. 0 ) THEN
              WRITE(P,*) '  Downstream end, channel... ',
     #                    IABS(EqChannelNumber(I))
            ELSE
              WRITE(P,*) ' Control-data input...'
              WRITE(P,*) ' *** Error (ReadNetworkControl)'
              WRITE(P,*) ' Reading boundary equations...'
              WRITE(P,*) ' "0" channel number not recognized...'
              WRITE(P,*) ' Abnormal program end.'
              STOP
            END IF

          WRITE(P,*) '    Components ......',EqComponents(I)
          WRITE(P,*) '    Base ............',EqBase(I)

          DO 350 K=1,EqComponents(I)
            WRITE(P,*) ' '
            WRITE(P,*) '    Component .......',K
            WRITE(P,*) '    Amplitude .......',Amplitude(K,I)
            WRITE(P,*) '    Period ..........',Period(K,I)
            WRITE(P,*) '    Phase angle .....',PhaseAngle(K,I)
            WRITE(P,*) ' '
 350      CONTINUE
          
          WRITE(P,*) '    Start time ......',EqStart(I)
          WRITE(P,*) '    End time ........',EqEnd(I)

 400      CONTINUE
          WRITE(P,*) ' '
          WRITE(P,*) ' '

        END IF

      ELSE
        WRITE(P,*) ' Control-data input...'
        WRITE(P,*) ' Failed to open control input file.'
        WRITE(*,*) ' Failed to open control input file.'
        WRITE(P,*) ' File name : ', CNTRL
        WRITE(*,*) ' File name : ', CNTRL
        WRITE(P,*) ' Abnormal program end.'
        STOP
      END IF



      ReadNetworkControl = .TRUE.

      RETURN
      END



*== Public (SetNetworkTimeSeriesLocations) =============================

      LOGICAL FUNCTION SetNetworkTimeSeriesLocations()

      IMPLICIT NONE

*   Purpose:  Determine global, computational-cross-section numbers
*             of locations at which output of time-series results
*             is requested.


*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:
      INTEGER I, J, M, Channels
      INTEGER Ch, XLoc, XComp, PastXComp

*   Routines by module:

***** Channel schematic:
      INTEGER  StreamEndNode, NumberOfChannels
      EXTERNAL StreamEndNode, NumberOfChannels

      REAL     GlobalStreamDistance
      EXTERNAL GlobalStreamDistance

***** Network control:
      INTEGER  NetworkPrintLevel
      EXTERNAL NetworkPrintLevel

*   Intinsics:
      INTEGER   INT
      INTRINSIC INT

*   Programmed by: Lew DeLong
*   Date:          May 1991
*   Modified by:   Lew DeLong
*   Last modified: July 3, 91
*


*-- Implementation -----------------------------------------------------

      SetNetworkTimeSeriesLocations = .FALSE.

      Channels = NumberOfChannels() 
      I = 0
      DO 200 M=1,TimeSeries

        Ch   = TSChannel(M)
        XLoc = INT( TSX(M) )

*------ Check for valid channel.

        IF( Ch .LE. Channels ) THEN
          I = I + 1
        ELSE
          GO TO 200
        END IF
        
        
*------ Check and adjust for sign convention on downstream distance.

        IF(  GlobalStreamDistance(StreamEndNode(Ch)) .LE. 0.0
     #                    .AND.
     #       GlobalStreamDistance(-StreamEndNode(-Ch)) .LT. 0.0
     #                    .AND.
     #               XLoc .GT. 0
     #    ) THEN
                     XLoc = - XLoc
        END IF

        IF(  GlobalStreamDistance(StreamEndNode(Ch)) .GE. 0.0
     #                    .AND.
     #       GlobalStreamDistance(-StreamEndNode(-Ch)) .GT. 0.0
     #                    .AND.
     #               XLoc .LT. 0
     #    ) THEN
                     XLoc = - XLoc
        END IF



*------ Match requested locations with computational locations.

        J = StreamEndNode(Ch)
        PastXComp = INT( GlobalStreamDistance( J ) )

        IF( XLoc .NE. PastXComp ) THEN

*-------- doesn't match upstream location.

          DO 100 J=StreamEndNode(Ch)+1,-StreamEndNode(-Ch)

            XComp = INT( GlobalStreamDistance(J) )
            IF( XLoc .EQ. XComp ) THEN

*------------ an integer match.
              TimeSeriesLocation(I) = J
              GO TO 102

            ELSE IF( XLoc .LT. XComp ) THEN

*------------ no exact integer match, find closest.

              IF( (XLoc-PastXComp) .GT. (XComp-XLoc) ) THEN
                TimeSeriesLocation(I) = J
              ELSE
                TimeSeriesLocation(I) = J - 1
              END IF

              GO TO 102

            END IF

            PastXComp = XComp

 100      CONTINUE

*--------- default, force match with downstream location.
           TimeSeriesLocation(I) = -StreamEndNode(-Ch)

 102    CONTINUE

        ELSE

*-------- does match upstream location.
          TimeSeriesLocation(I) = J

        END IF

 200  CONTINUE

*---- Assign valid number of time series.

      TimeSeries = I


      IF(NetworkPrintLevel() .GT. 1 ) THEN

        WRITE(*,*) ' '
        WRITE(*,*) ' Requested time series at global locations:'
        WRITE(*,*) (TimeSeriesLocation(I),I=1,TimeSeries)

      END IF
      
      SetNetworkTimeSeriesLocations = .TRUE.

      RETURN
      END



*== Public (NetworkTimeSteps) ==========================================

      INTEGER FUNCTION NetworkTimeSteps()

      IMPLICIT NONE

*   Purpose:  Return requested number of time steps for network.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      NetworkTimeSteps = MaxTimeSteps

      RETURN
      END



*== Public (Terms1D) ===================================================

      INTEGER FUNCTION Terms1D()

      IMPLICIT NONE

*   Purpose: Return an index indicating which terms are included in the
*            unsteady 1-D open-channel flow equations.

*              [1] full dynamic-wave equation.
*              [2] diffusion-wave equation.
*              [3] kinematic-wave equation.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      Terms1D = Terms

      RETURN
      END



*== Public (MaxNetworkIterations) ======================================

      INTEGER FUNCTION MaxNetworkIterations()

      IMPLICIT NONE

*   Purpose: Return the maximum number of iterations allowed for each
*            time step executed by an open-channel network model.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      MaxNetworkIterations = MaxIterations

      RETURN
      END



*== Public (NetworkTimeIncrement) ======================================

      INTEGER FUNCTION NetworkTimeIncrement()

      IMPLICIT NONE

*   Purpose: Return the time increment used by an
*            open-channel network model.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      NetworkTimeIncrement = DT

      RETURN
      END



*== Public (NetworkTheta) ==============================================

      REAL FUNCTION NetworkTheta()

      IMPLICIT NONE

*   Purpose:  Return the time-weighting factor used by an
*             open-channel network model.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      NetworkTheta = Theta

      RETURN
      END



*== Public (AccelerationGravity) ==============================================

      REAL FUNCTION AccelerationGravity()

      IMPLICIT NONE

*   Purpose:  Return the value of the acceleration due to gravity.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          May   1992
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      AccelerationGravity = Gravity

      RETURN
      END



*== Public (ToleranceStreamZ) ==========================================

      REAL FUNCTION ToleranceStreamZ()

      IMPLICIT NONE

*   Purpose:  Return the closure tolerance for water-surface elevation.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      ToleranceStreamZ = ToleranceZ

      RETURN
      END



*== Public (ToleranceStreamQ) ==========================================

      REAL FUNCTION ToleranceStreamQ()

      IMPLICIT NONE

*   Purpose:  Return the closure tolerance for stream flow.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      ToleranceStreamQ = ToleranceQ

      RETURN
      END



*== Public (NetworkPrintLevel) =========================================

      INTEGER FUNCTION NetworkPrintLevel()

      IMPLICIT NONE

*   Purpose:  Return level of printing activity.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      NetworkPrintLevel = PrintLevel

      RETURN
      END


*== Public (NetworkQuadPts) =========================================

      INTEGER FUNCTION NetworkQuadPts()

      IMPLICIT NONE

*   Purpose:  Return current number of quadrature points to be used
*             in numerical integration.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: May      1992
*


*-- Implementation -----------------------------------------------------

      NetworkQuadPts = QuadPts

      RETURN
      END




*== Public (NetworkQuadPts) ============================================

      SUBROUTINE NetworkQuadPtWt(
     I                              K,
     O                              Pt, Wt
     #                          )

      IMPLICIT NONE

*   Purpose:  Return location, in local coordinate ( 0 to 1 ), and weight
*             of quadrature-point K.

*   Arguments:
      INTEGER K
      REAL    Pt, Wt

*   Argument definitions:
*     K - quadrature-point sequence number.
*     Pt - location of quadrature point K,
*              in local coordinate ( 0 to 1 ).
*     Wt - weight, corresponding to quadrature point "K",
*              for numerical integration.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: May      1992
*


*-- Implementation -----------------------------------------------------

      Pt = QuadPt(K)
      Wt = QuadWt(K)

      RETURN
      END



*== Public (NetworkTimeStep) ===========================================

      INTEGER FUNCTION NetworkTimeStep()

      IMPLICIT NONE

*   Purpose:  Return current time step number.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      NetworkTimeStep = TimeStep

      RETURN
      END



*== Public (NetworkSeconds) ============================================

      INTEGER FUNCTION NetworkSeconds()

      IMPLICIT NONE

*   Purpose:  Return current network elapse time in seconds.
*             Seconds = 0 at begining of current execution.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      NetworkSeconds = Seconds

      RETURN
      END



*== Public (IncrementNetworkTimeStep) ==================================

      LOGICAL FUNCTION IncrementNetworkTimeStep()

      IMPLICIT NONE

*   Purpose:  Increment current time step number.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      TimeStep = TimeStep + 1

      Seconds = Seconds + DT

      IF( TimeStep .LE. MaxTimeSteps ) THEN
        IncrementNetworkTimeStep = .TRUE.
      ELSE
        IncrementNetworkTimeStep = .FALSE.
      END IF

      RETURN
      END



*== Public (IncrementNetworkIteration) =================================

      LOGICAL FUNCTION IncrementNetworkIteration()

      IMPLICIT NONE

*   Purpose:  Increment current iteration number.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:
      LOGICAL OK

*   Routines by module:

***** Solver:
      LOGICAL  InitializeMatrix
      EXTERNAL InitializeMatrix

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      Iteration = Iteration + 1

      IF( Iteration .LE. MaxIterations ) THEN

        OK = InitializeMatrix()

        IncrementNetworkIteration = .TRUE.
      ELSE
        IncrementNetworkIteration = .FALSE.
      END IF

      RETURN
      END



*== Public (StoreNetworkClosure) =======================================

      LOGICAL FUNCTION StoreNetworkClosure(
     #                   MaxRatioQ, ChannelQ, NodeQ,
     #                   MaxChangeZ, ChannelZ, NodeZ
     #                                    )

      IMPLICIT NONE

*   Purpose:  Store current network closure information, and check if
*              current information satisfies criteria.

*   Arguments:
      INTEGER ChannelQ, ChannelZ, NodeQ, NodeZ
      REAL    MaxRatioQ, MaxChangeZ

*   Argument definitions:
*     MaxRatioQ - maximum ratio of change in flow over the last
*                  iteration to the current value of flow.
*     ChannelQ - channel number in which MaxRatioQ occurred.
*     NodeQ - local location number at which MaxRatioQ occurred.

*     MaxChangeZ - maximum change in water-surface elevation over
*                   the last iteration.
*     ChannelZ - channel number in which MaxChangeZ ocurred.
*     NodeZ - local location number at which MaxChangeZ occurred.


*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

*---- Store current values.

      RatioQ         = MaxRatioQ
      ChannelRatioQ  = ChannelQ
      NodeRatioQ     = NodeQ

      ChangeZ        = MaxChangeZ
      ChannelChangeZ = ChannelZ
      NodeChangeZ    = NodeZ


*---- Test for overall closure.

      IF( ChangeZ .GT. ToleranceZ ) THEN
        StoreNetworkClosure = .FALSE.
        CloseZ = .FALSE.
      ELSE
        CloseZ = .TRUE.
      END IF


      IF( RatioQ .GT. ToleranceQ ) THEN
        StoreNetworkClosure = .FALSE.
        CloseQ = .FALSE.
      ELSE
        CloseQ = .TRUE.
      END IF

      IF( CloseZ .AND. CloseQ ) THEN
        StoreNetworkClosure = .TRUE.
      ELSE

*------ Check for exceedence of maximum number of iterations.
        IF( Iteration .GE. MaxIterations ) THEN
          StoreNetworkClosure = .TRUE.
        END IF

      END IF

      EndIteration = StoreNetworkClosure

      RETURN
      END



*== Public (OpenNetworkIteration) ======================================

      LOGICAL FUNCTION OpenNetworkIteration()

      IMPLICIT NONE

*   Purpose:  Initialize for an iteration.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:
      LOGICAL OK

*   Routines by module:

***** Solver:
      LOGICAL  InitializeRightSide
      EXTERNAL InitializeRightSide

*   Programmed by: Lew DeLong
*   Date:          January 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

*---- Set iteration counter to zero.

      Iteration = 0

*---- Set all rows of right-hand side load vector to zero.

      OK = InitializeRightSide()
      
      OpenNetworkIteration = .TRUE.

      RETURN
      END



*== Public (CloseNetworkIteration) =====================================

      LOGICAL FUNCTION CloseNetworkIteration()

      IMPLICIT NONE

*   Purpose:  Close out the current iteration.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:
      LOGICAL OK

*   Routines by module:

***** Buffered output:
      LOGICAL  WriteNetworkTimeSeries, WriteNetworkSpatialSeries
      EXTERNAL WriteNetworkTimeSeries, WriteNetworkSpatialSeries

***** Network control:
      LOGICAL  ReportNetworkIterationStatus
      EXTERNAL ReportNetworkIterationStatus

***** Network model:
      LOGICAL  ReportNetworkStatus
      EXTERNAL ReportNetworkStatus

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

*---- Increment output counters.

      PrintCount = PrintCount + 1
      TimeSeriesCount = TimeSeriesCount + 1
      SpatialSeriesCount = SpatialSeriesCount + 1


*---- Report network status if requested.

      IF( PrintCount .GE. PrintInc
     #               .AND.
     #    PrintLevel .LT. 3 ) THEN

          OK = ReportNetworkIterationStatus()
          OK = ReportNetworkStatus()

      END IF

*---- Reset print counter.

      IF( PrintCount .GE. PrintInc )  THEN
        PrintCount = 0
      END IF

*---- Write Time-series info.

      IF( TimeSeriesCount .GE. TimeSeriesInc ) THEN

        TimeSeriesCount = 0
        OK = WriteNetworkTimeSeries()

      END IF

*---- Write spatial-series info.

      IF( SpatialSeriesCount .GE. SpatialSeriesInc ) THEN

        SpatialSeriesCount = 0
        OK = WriteNetworkSpatialSeries()

      END IF

*---- Sum total iterations.

      TotalIterations = TotalIterations + Iteration


      CloseNetworkIteration = .TRUE.

      RETURN
      END



*== Public (TotalNetworkIterations) ====================================

      INTEGER FUNCTION TotalNetworkIterations()

      IMPLICIT NONE

*   Purpose:  Return level of printing activity.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      TotalNetworkIterations = TotalIterations

      RETURN
      END



*== Private (ReportNetworkIterationStatus) =============================

      LOGICAL FUNCTION ReportNetworkIterationStatus()

      IMPLICIT NONE

*   Purpose:  Write network iteration information to print file.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:
      INTEGER PrintUnit
      CHARACTER*1 Star, Blank
      PARAMETER ( Star = '*', Blank = ' ' )
      CHARACTER*12 InternalFileName

*   Routines by module:

***** Master file:  
      INTEGER  GetFileUnit
      EXTERNAL GetFileUnit

*   Programmed by: Lew DeLong
*   Date:          March 1991
*   Modified by:  Barry Wicktom (use of master file names added)
*   Last modified: 1/7/92
*


*-- Implementation -----------------------------------------------------

      InternalFileName = 'netprint.dat'
      PrintUnit = GetFileUnit(InternalFileName)

*   Output formats:

 2000 FORMAT(2I5,2I8,A1,4X,2I8,A1)

      IF( CloseQ ) THEN
        IF( CloseZ ) THEN

          WRITE(PrintUnit,2000) TimeStep, Iteration,
     #     ChannelRatioQ, NodeRatioQ, Blank,
     #     ChannelChangeZ, NodeChangeZ, Blank

        ELSE

          WRITE(PrintUnit,2000) TimeStep, Iteration,
     #     ChannelRatioQ, NodeRatioQ, Blank,
     #     ChannelChangeZ, NodeChangeZ, Star

        END IF

      ELSE IF( CloseZ ) THEN

          WRITE(PrintUnit,2000) TimeStep, Iteration,
     #     ChannelRatioQ, NodeRatioQ, Star,
     #     ChannelChangeZ, NodeChangeZ, Blank

      ELSE

          WRITE(PrintUnit,2000) TimeStep, Iteration,
     #     ChannelRatioQ, NodeRatioQ, Star,
     #     ChannelChangeZ, NodeChangeZ, Star

      END IF


      ReportNetworkIterationStatus = .TRUE.

      RETURN
      END



*== Private (ScreenNetworkStatus) ======================================

      LOGICAL FUNCTION ScreenNetworkStatus()

      IMPLICIT NONE

*   Purpose:  Write network iteration/timestep information to screen.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:
      CHARACTER*1 Star, Blank
      PARAMETER ( Star = '*', Blank = ' ' )

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          March 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

*   Output formats:

 2000 FORMAT(2I5,2I8,A1,4X,2I8,A1)

      IF( CloseQ ) THEN
        IF( CloseZ ) THEN

          WRITE(*,2000) TimeStep, Iteration,
     #     ChannelRatioQ, NodeRatioQ, Blank,
     #     ChannelChangeZ, NodeChangeZ, Blank

        ELSE

          WRITE(*,2000) TimeStep, Iteration,
     #     ChannelRatioQ, NodeRatioQ, Blank,
     #     ChannelChangeZ, NodeChangeZ, Star

        END IF

      ELSE IF( CloseZ ) THEN

          WRITE(*,2000) TimeStep, Iteration,
     #     ChannelRatioQ, NodeRatioQ, Star,
     #     ChannelChangeZ, NodeChangeZ, Blank

      ELSE

          WRITE(*,2000) TimeStep, Iteration,
     #     ChannelRatioQ, NodeRatioQ, Star,
     #     ChannelChangeZ, NodeChangeZ, Star

      END IF


      ScreenNetworkStatus = .TRUE.

      RETURN
      END



*== Public (IntermediateNetworkResults) ================================

      LOGICAL FUNCTION IntermediateNetworkResults()

      IMPLICIT NONE

*   Purpose:  Report values of variables, current iteration.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:
      INTEGER PrintUnit
      LOGICAL OK
      CHARACTER*12 InternalFileName

*   Routines by module:

***** Master file:  
      INTEGER GetFileUnit
      EXTERNAL GetFileUnit

***** Network control:
      INTEGER  NetworkPrintLevel
      LOGICAL  ReportNetworkIterationStatus
      EXTERNAL ReportNetworkIterationStatus, NetworkPrintLevel

      LOGICAL  ScreenNetworkStatus
      EXTERNAL ScreenNetworkStatus

***** Network model:
      LOGICAL  ReportNetworkStatus
      EXTERNAL ReportNetworkStatus

*   Intrinsics:
      INTEGER MOD
      INTRINSIC MOD

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Barry Wicktom (use of master file names added)
*   Last modified: 1/7/92
*


*-- Implementation -----------------------------------------------------

      InternalFileName = 'netprint.dat'
      PrintUnit = GetFileUnit(InternalFileName)

*   Output formats:
 2001 FORMAT(////'  Time          ....Q....            ....Z....'/
     #        '  Step Iter. Channel  Section     Channel  Section')


*---- Write iteration-status header to the print file.

      IF( PrintLevel .GT. 0
     #               .AND.
     #    PrintCount .EQ. 0
     #               .AND.
     #    Iteration  .EQ. 1 ) THEN

        WRITE(PrintUnit,2001)

      ELSE IF( PrintLevel .GE. 3) THEN

        WRITE(PrintUnit,2001)

      END IF


*---- Write network status to print file if requested.

      IF( PrintLevel .GE. 3 ) THEN

          OK = ReportNetworkIterationStatus()
          OK = ReportNetworkStatus()

      END IF

*---- Write iteration-status header to screen.

      IF( NetworkPrintLevel() .GT. 0 ) THEN
        IF(
     #      EndIteration
     #         .AND.
     #     MOD( TimeStep, 20 ) .EQ. 1
     #    )      THEN

          WRITE(*,2001)
          WRITE(*,*) ' '

        END IF
      END IF

*---- Write iteration status to screen.
      IF( EndIteration ) THEN
        IF( NetworkPrintLevel() .GT. 0 ) THEN
          OK = ScreenNetworkStatus()
        END IF
      END IF


      IntermediateNetworkResults = .TRUE.

      RETURN
      END



*== Public (NetworkIteration) ==========================================

      INTEGER FUNCTION NetworkIteration()

      IMPLICIT NONE

*   Purpose: Return current iteration number.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      NetworkIteration = Iteration

      RETURN
      END



*== Public (ForwardElimInt) ==========================================

      INTEGER FUNCTION ForwardElimInt()

      IMPLICIT NONE

*   Purpose: Return interval at which complete forward eliminations
*            are to be performed on the solution matrix, in terms of
*            number of iterations during a time step.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          April 1992
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      ForwardElimInt = LuInc

      RETURN
      END



*== Public (NetworkRestart) ==========================================

      LOGICAL FUNCTION NetworkRestart()

      IMPLICIT NONE

*   Purpose: Return .TRUE. if network restart file has been requested,
*            otherwise return .FALSE..

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          December 1992
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      NetworkRestart = Restart

      RETURN
      END



*== Public (NetworkTimeSeries) =========================================

      INTEGER FUNCTION NetworkTimeSeries()

      IMPLICIT NONE

*   Purpose:  Return number of requested time series.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          March 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      NetworkTimeSeries = TimeSeries

      RETURN
      END



*== Public (NetworkTimeSeriesLocation) =================================

      INTEGER FUNCTION NetworkTimeSeriesLocation( Series )

      IMPLICIT NONE

*   Purpose:  Return computational location number corresponding to the
*             time-series number "Series".

*   Arguments:
      INTEGER Series

*   Argument definitions:
*     Series - requested time-series number.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          March 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      NetworkTimeSeriesLocation = TimeSeriesLocation( Series )

      RETURN
      END



*==== EOF netcntr1 =====================================================
