*===== BOF netdense =====================================================

*== Public (InitializeNetworkDensity) ===================================

      LOGICAL FUNCTION InitializeNetworkDensity()

      IMPLICIT NONE

*   Purpose:  Initialize water density in a network of open channels. In
*             this version, density is assumed to vary linearly over the
*             length of each channel and thus, is determined entirely by
*             values of density at channel extremities.  These values
*             are simply read from a file.  Density is assumed to vary
*             linearly with time within a density time-series increment.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netdense.inc'

*   Local Variables:
        LOGICAL OK
        CHARACTER*12 FileName, InternalFileName

*   Routines by module:

***** Local:
      LOGICAL  ReadNetworkDensity, SetNewNetworkDensity
      EXTERNAL ReadNetworkDensity, SetNewNetworkDensity

***** Network schmatic:
      INTEGER  NumberOfChannels
      EXTERNAL NumberOfChannels
      
***** Network control:
      INTEGER  NetworkPrintLevel, NetworkSeconds
      EXTERNAL NetworkPrintLevel, NetworkSeconds

***** File utilities:
      LOGICAL  OpenOldText
      EXTERNAL OpenOldText
        
***** Master file:
      INTEGER GetFileUnit
      CHARACTER*12 GetFileName
      EXTERNAL GetFileName, GetFileUnit

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          October  1991
*   Modified by:   Barry Wicktom (use of master file names added)
*   Last modified: 1/8/92
*

*-- Implementation -----------------------------------------------------

      
      InitializeNetworkDensity = .FALSE.

*---- Set file name and unit number.
      
      InternalFileName = 'density.dat  '

      FortranUnit = GetFileUnit(InternalFileName)
      FileName =  GetFileName(InternalFileName)

        IF(OpenOldText(FortranUnit,FileName)) THEN
          READ(FortranUnit,*) StartSeconds
          READ(FortranUnit,*) dT
          IF( NetworkPrintLevel() .GT. 0 ) THEN
            WRITE(*,*) ' Water density read from ', FileName
            WRITE(*,*) ' Density starting elapse time...',
     #                 StartSeconds
            WRITE(*,*) ' Density time increment...', dT
          END IF
        ELSE
          WRITE(*,*) ' Could not open density file ...', FileName
          RETURN
        END IF

      IF( NetworkSeconds() .GE. StartSeconds ) THEN

        NewTime = StartSeconds - dT
        OK = ReadNetworkDensity()

      ELSE
        WRITE(*,*) ' ####error(InitializeNetworkDensity)'
        WRITE(*,*) ' Starting network elapse time is smaller than'
        WRITE(*,*) ' starting elapse time for density records.'
        WRITE(*,*) ' ',NetworkSeconds(),' < ',StartSeconds
        WRITE(*,*) ' Abnormal program end.'
        STOP
      END IF
     
      IF( SetNewNetworkDensity() ) THEN
      ELSE
        WRITE(*,*) ' ####Error(InitializeNetworkDensity)'
        WRITE(*,*) ' Failed to set starting network water density.'
      END IF
     
      InitializeNetworkDensity = .TRUE.

      RETURN
      END



*== Private (ReadNetworkDensity) ===================================

      LOGICAL FUNCTION ReadNetworkDensity()

      IMPLICIT NONE

*   Purpose:  Read density values at channel ends.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netdense.inc'

*   Local Variables:
        INTEGER      I

*   Routines by module:

***** Network schmatic:
      INTEGER  NumberOfChannels
      EXTERNAL NumberOfChannels


*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          October  1991
*   Modified by:   Barry Wicktom (use of master file names added)
*   Last modified: 1/8/92
*

*-- Implementation -----------------------------------------------------

      
      ReadNetworkDensity = .FALSE.

        OldTime = NewTime
        NewTime = NewTime + dT
        DO 100 I=1,2*NumberOfChannels()
          Old(I) = New(I)
 100    CONTINUE
        
C       WRITE(*,*) ' '
C       WRITE(*,*) ' Reading density...'
C       WRITE(*,*) ' OldTime....', OldTime
C       WRITE(*,*) ' NewTime....', NewTime
C       WRITE(*,*) ' Old values.', (Old(I),I=1,2*NumberOfChannels())
        
        READ(FortranUnit,*,END=200) (New(I),I=1,2*NumberOfChannels())
C       WRITE(*,*) ' New values.',  (New(I),I=1,2*NumberOfChannels())

        GO TO 202
 200    CONTINUE
        WRITE(*,*) ' ####error(ReadNetworkDensity)'
        WRITE(*,*) ' New values.',  (New(I),I=1,2*NumberOfChannels())
        WRITE(*,*) ' Unexpected end of file.'
        WRITE(*,*) ' Possibly channel...',(I+1)/2
        WRITE(*,*) ' Abnormal program end.'
        STOP
 202    CONTINUE
     
     
      ReadNetworkDensity = .TRUE.

      RETURN
      END



*== Public (SetNewNetworkDensity) ================================================

      LOGICAL FUNCTION SetNewNetworkDensity()

      IMPLICIT NONE

*   Purpose:  Set water density in a network of open channels.
*             Density is assumed to vary linearly with time
*             (over a density time increment) and with space
*             (from one end of a channel to the other).

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netdense.inc'

*   Local Variables:
      INTEGER I, Channel
      REAL UpstreamDensity, DownstreamDensity
      REAL Shape1, Shape2
      LOGICAL OK

*   Routines by module:

***** Local:
      LOGICAL  ReadNetworkDensity
      EXTERNAL ReadNetworkDensity
      LOGICAL  SetNewLinearStreamDensity
      EXTERNAL SetNewLinearStreamDensity
      
***** Channel schematic:
      LOGICAL  OpenChannel, CloseChannel
      INTEGER  NumberOfChannels
      EXTERNAL NumberOfChannels, OpenChannel, CloseChannel
        
      
***** Channel control:
      INTEGER  NetworkSeconds
      EXTERNAL NetworkSeconds
      


*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          October  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      
      SetNewNetworkDensity = .FALSE.
      
      CurrentTime = NetworkSeconds()
      
 100  CONTINUE

*---- Compute current density values at channel ends.

C     WRITE(*,*) ' '
C     WRITE(*,*) ' SetNewNetworkDensity...'
C     WRITE(*,*) ' OldTime.......',OldTime
C     WRITE(*,*) ' CurrentTime...',CurrentTime
C     WRITE(*,*) ' NewTime.......',NewTime
      IF( CurrentTime .GT. OldTime ) THEN
        IF( CurrentTime .LT. NewTime ) THEN

*-------- Interpolate current values.

          Shape1 = FLOAT(NewTime - CurrentTime)/
     #             FLOAT(NewTime - OldTime)
          Shape2 = 1.0 - Shape1
          
          DO 200 I=1,2*NumberOfChannels()
            Current(I) = Shape1*Old(I) + Shape2*New(I)
C           WRITE(*,*) I,Current(I)
 200      CONTINUE

        ELSE IF( CurrentTime .EQ. NewTime ) THEN

*-------- Assign current values.

          DO 300 I=1,2*NumberOfChannels()
            Current(I) = New(I)
C           WRITE(*,*) I,Current(I)
 300      CONTINUE

        ELSE
        
*-------- Read new values.
          
          OK = ReadNetworkDensity()

          GO TO 100

        END IF
      ELSE IF(CurrentTime .EQ. OldTime ) THEN
        
*------ Read new values.

        OK = ReadNetworkDensity()

        GO TO 100
        
      ELSE
      
*------ Shouldn't ever get here!

        WRITE(*,*) ' ####error(SetNewNetDensity)'
        WRITE(*,*) ' Current time < available from boundary values.'
        WRITE(*,*) ' CurrentTime...',CurrentTime
        WRITE(*,*) ' OldTime.......',OldTime
        WRITE(*,*) ' Abnormal program end.'
        STOP
      END IF
      

      
      DO 400 I=1,NumberOfChannels()

        Channel = I
        OK = OpenChannel(Channel)

        UpstreamDensity = Current(2*I-1)
        DownstreamDensity = Current(2*I)
        
        IF( SetNewLinearStreamDensity(
     #        UpstreamDensity, DownstreamDensity)
     #                               ) THEN
        ELSE
        
          WRITE(*,*) ' ####Error(SetNetworkDensity)'
          WRITE(*,*) ' Water density not set, channel...',Channel
          RETURN
        
        END IF
        
        OK = CloseChannel()

 400  CONTINUE


      SetNewNetworkDensity = .TRUE.

      RETURN
      END



*== Private (SetNewLinearStreamDensity) ====================================

      LOGICAL FUNCTION SetNewLinearStreamDensity(
     #                   UpstreamDensity, DownstreamDensity
     #                                       )

      IMPLICIT NONE

*   Purpose:  Set stream density to values linearly interpolated from
*              values supplied at extremities of current channel, at 
*              the end of the current time step.

*   Arguments:
      REAL UpstreamDensity, DownstreamDensity

*   Argument definitions:
*     UpstreamDensity - density at the upstream end of current channel.
*     DownstreamDensity - density at downstream end of current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netdense.inc'

*   Local Variables:
      INTEGER I, N
      REAL    XUp, dX, Shape, Rho
      LOGICAL OK

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer, DownstreamPointer
      INTEGER  NumberOfStreamLocations
      REAL     StreamDistance
      EXTERNAL UpstreamPointer, DownstreamPointer
      EXTERNAL NumberOfStreamLocations
      EXTERNAL StreamDistance
      
***** Channel status:
      REAL     NewStreamDensity
      LOGICAL  SetOldStreamDensity, SetNewStreamDensity
      EXTERNAL NewStreamDensity
      EXTERNAL SetOldStreamDensity, SetNewStreamDensity
*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          October  1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      XUp = StreamDistance(1)
      dX = StreamDistance( NumberOfStreamLocations() ) - XUp

      N = 0
      DO 100 I=UpstreamPointer(),DownstreamPointer()
        N = N + 1

        OK = SetOldStreamDensity( N, NewStreamDensity(N) )
        
        Shape = ( StreamDistance(N) - XUp ) / dX
        Rho = (1.0 - Shape) * UpstreamDensity 
     #         + Shape * DownstreamDensity
        OK = SetNewStreamDensity( N, Rho )
        
 100  CONTINUE
      
      
      SetNewLinearStreamDensity = .TRUE.

      RETURN
      END



*===== EOF netdense =====================================================
