************************************************************************
*
*     This file is a FORTRAN module useful in the accumulation and solution
*     of sets of linear equations such as those that may be obtained by
*     linearization of the 1-D shallow-water flow equations over a single
*     time step.  Gaussian elimination is performed on an asymetric, sparse
*     coeficient matrix efficiently stored in "skyline" form in a 1-D array.
*     Routines are included for assemblying coefficients for specific model
*     formulations such as the four-point-implicit finite-difference used in
*     the 1-D surface water flow model FourPt.
*
*     Solver routines are found in the following files:
*
*          solveutl.f - required for all configurations.
*
*          solvefpt.f - required specifically for the four-point-implicit scheme.
*
*
*          Either, but not both, of the following files are required:
*
*            solvestd.f - required for standard skyline solution, uses less
*                         memory than the indexed solution and is probably as
*                         fast for small networks of fewer than 50 channels.
*
*            solveidx.f - required for indexed skyline solution, uses more
*                         memory than the standard solution but is increasingly
*                         faster as the number of channels increases.
*
*
*
*     Source file             Rerquired module-data files
*
*      solveutl.f           network.inc, skyline1.inc, skyline2.inc,
*                           skyline3.inc
*
*      solvefpt.f           network.inc, skyline1.inc, skyline2.inc
*
*      solvestd.f           network.inc, skyline1.inc, skyline2.inc
*                           skyline3.inc
*
*      solveidx.f           network.inc, skyline1.inc, skyline2.inc,
*                           skyline3.inc, skyline5.inc, skyline6.inc
*
*
*
*     Program note: Only functions or subroutines marked "public" should be
*                   used outside of this module as those marked "private"
*                   may not be supported by future revisions of this module
*                   or replacement modules.  Likewise, no data or common
*                   blocks contained within this module should be accessed
*                   by routines outside of this module accept through the
*                   use of "public" functions or subroutines contained
*                   within this module.
*
*
*     Non-standard usage: Symbolic names in this module may be represented by
*                         as many as 31 characters in order to provide better
*                         definition directly in the code.  Standard FORTRAN
*                         allows only 6 characters, but this restriction is
*                         generally extended to 32 characters by most compilers.
*
*                    Programmed by:   Lew DeLong
*                    Date programmed: July, 1990
*                    Modified by:     Lew DeLong
*                    Last modified:   April 1992
*
*



*== Public (InitializeRightSide) ==========================================

      LOGICAL FUNCTION InitializeRightSide()

      IMPLICIT NONE

*   Purpose:  Initialize right-hand side vector to zero.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline1.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:
      INTEGER I

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          January 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      InitializeRightSide = .FALSE.

      DO 200 I = 1,Equations
        XOld(I) = 0.0
        XAdj(I) = 0.0
 200  CONTINUE

      InitializeRightSide = .TRUE.

      RETURN
      END



*== Private (Reorder) ==================================================

      LOGICAL FUNCTION Reorder()

      IMPLICIT NONE

*   Purpose:  Determine key to subscripts of 1-D skyline array.

*       Note: When this routine finishes, IDIAG(I) represents the subscript
*             from the 1-d floating point storage array, A(__), indicating
*             the diagonal of the Ith row. LE(I) represents the subscript
*             of the left extent of the Ith row.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline1.inc'

*   Local Variables:
      INTEGER   I

*   Routines by module:

***** Channel schematic:
      INTEGER  TotalStreamLocations
      EXTERNAL TotalStreamLocations

*   Programmed by: Lew DeLong
*   Date:          January 1990
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      Reorder = .FALSE.
      NUMFLT=0

      DO 200 I=1,Equations
        NUMFLT   = NUMFLT+1
        NUMFLT   = NUMFLT+I-LE(I)
        LE(I)    = NUMFLT
        NUMFLT   = NUMFLT+I-IDIAG(I)
        IDIAG(I) = NUMFLT
 200  CONTINUE

      Reorder = .TRUE.

      RETURN
      END



*== Private (SkyScript) ================================================

      INTEGER   FUNCTION SkyScript(I,K)

      IMPLICIT NONE

*   Purpose:  Compute skyline-storage subscript from 2-D matrix row (I)
*             and offset from diagonal (K).

*   Arguments:
      INTEGER   I,K
*   Argument definitions:
*     I - row
*     K - offset from diagonal

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline1.inc'

*   Programmed by:   Lew DeLong
*   Date programmed: Dark ages
*   Modified by:     Lew DeLong
*   Last modified:   July, 1990
*


*-- Implementation -----------------------------------------------------

      IF (K.GT.0) THEN
        SkyScript = LE(I+K)-K
      ELSE
        SkyScript = IDIAG(I)+K
      END IF

      RETURN
      END



*== Public (StoreMassKnown) ============================================

      LOGICAL FUNCTION StoreMassKnown(EqNum, Value)

      IMPLICIT NONE

*   Purpose:  Store known part of load vector, unchanging over
*             iteration, contributed by mass equation.

*   Arguments:
      INTEGER EqNum
      REAL    Value

*   Argument definitions:
*     EqNum - mass-equation sequence number, beginning in the network
*              adjacent to the lowest cross-section number and
*              progressing in the direction of increasing cross-section
*              numbers.
*              This number is local to the current channel and the sequence 
*              does not include boundary equations.
*     Value - value to be stored.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline2.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:
      INTEGER K

*   Routines by module:

***** Local:
      INTEGER  SkyScript
      EXTERNAL SkyScript

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------


      K = MASSEQ(EqPointer(Branch) + EqNum - 1)
      XOld(K) = Value

      StoreMassKnown = .TRUE.

      RETURN
      END



*== Public (StoreMassAdjust) ===========================================

      LOGICAL FUNCTION StoreMassAdjust(EqNum, Value)

      IMPLICIT NONE

*   Purpose:  Store adjustment part of load vector, changing over
*             iteration, contributed by mass equation.

*   Arguments:
      INTEGER EqNum
      REAL    Value

*   Argument definitions:
*     EqNum - mass-equation sequence number, beginning in the network
*             adjacent to the lowest cross-section number and
*             progressing in the direction of increasing cross-section
*             numbers.
*              This number is local to the current channel and the sequence 
*              does not include boundary equations.
*     Value - value to be stored.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline2.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:
      INTEGER K

*   Routines by module:

***** Local:
      INTEGER  SkyScript
      EXTERNAL SkyScript

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      K = MASSEQ(EqPointer(Branch) + EqNum - 1)
      XAdj(K) = Value

      StoreMassAdjust = .TRUE.

      RETURN
      END



*== Public (StoreDynmKnown) ============================================

      LOGICAL FUNCTION StoreDynmKnown(EqNum, Value)

      IMPLICIT NONE

*   Purpose:  Store known part of load vector, unchanging over
*             iteration, contributed by dynamic equation.


*   Arguments:
      INTEGER EqNum
      REAL    Value

*   Argument definitions:
*     EqNum - dynamic-equation sequence number, beginning in the network
*              adjacent to the lowest cross-section number and
*              progressing in the direction of increasing cross-section
*              numbers.
*     Value - value to be stored.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline2.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:
      INTEGER K

*   Routines by module:

***** Local:
      INTEGER  SkyScript
      EXTERNAL SkyScript

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      K = DYNMEQ(EqPointer(Branch) + EqNum - 1)
      XOld(K) = Value

      StoreDynmKnown = .TRUE.

      RETURN
      END



*== Public (StoreDynmAdjust) ===========================================

      LOGICAL FUNCTION StoreDynmAdjust(EqNum, Value)

      IMPLICIT NONE

*   Purpose:  Store adjustment part of load vector, changing over
*             iteration, contributed by dynamic equation.

*   Arguments:
      INTEGER EqNum
      REAL    Value

*   Argument definitions:
*     EqNum - dynamic-equation sequence number, beginning in the network
*             adjacent to the lowest cross-section number and
*             progressing in the direction of increasing cross-section
*             numbers.
*     Value - value to be stored.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline2.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:
      INTEGER K

*   Routines by module:

***** Local:
      INTEGER  SkyScript
      EXTERNAL SkyScript

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      K = DYNMEQ(EqPointer(Branch) + EqNum - 1)
      XAdj(K) = Value

      StoreDynmAdjust = .TRUE.

      RETURN
      END



*== Public (IncrementalQ) ==============================================

      REAL FUNCTION IncrementalQ(LocationNumber)

      IMPLICIT NONE

*   Purpose:  Return the incremental change in flow over the last
*             iteration.

*   Arguments:
      INTEGER LocationNumber

*   Argument definitions:
*     LocationNumber - sequential location number within the current
*                       channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:
      INTEGER J

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer
      EXTERNAL UpstreamPointer

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

        J = UpstreamPointer() + LocationNumber - 1
        IncrementalQ = X( 2 * J - 1 )

      RETURN
      END



*== Public (IncrementalZ) ==============================================

      REAL FUNCTION IncrementalZ(LocationNumber)

      IMPLICIT NONE

*   Purpose:  Return the incremental change in stream-surface elevation
*             over the last iteration.

*   Arguments:
      INTEGER LocationNumber

*   Argument definitions:
*     LocationNumber - sequential location number within the current
*                      channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:
      INTEGER J

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamPointer
      EXTERNAL UpstreamPointer

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

        J = UpstreamPointer() + LocationNumber - 1
        IncrementalZ = X( 2 * J )

      RETURN
      END



*== Public (ForwardElim) ================================================

      LOGICAL FUNCTION ForwardElim()

      IMPLICIT NONE

*   Purpose:  Return .TRUE. if complete forward elimination will be
*             performed on the coefficient matrix in the curren iteration,
*             otherwise .FALSE..

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      
*   Local Variables:
      INTEGER Iteration, IterBack

*   Routines by module:

***** Network control:
        INTEGER  NetworkIteration
        INTEGER  ForwardElimInt
        EXTERNAL NetworkIteration
        EXTERNAL ForwardElimInt

***** Local:

*   Intrinsics:
      INTEGER   MOD
      INTRINSIC MOD

*   Programmed by: Lew DeLong
*   Date:          April 1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IterBack = ForwardElimInt()
      
      IF( IterBack .GT. 1  ) THEN
        
        Iteration = NetworkIteration()
        
        IF( Iteration .GT. 1) THEN
        
          IF( MOD(Iteration-1,IterBack) .EQ. 0 ) THEN
            ForwardElim = .TRUE.
          ELSE
            ForwardElim = .FALSE.
          END IF
          
        ELSE
          ForwardElim = .TRUE.
        END IF

      ELSE
        ForwardElim = .TRUE.
      END IF
        
          

      RETURN
      END



*== Public (StoreAtRow) ================================================

      LOGICAL FUNCTION StoreAtRow(Row, Value)

      IMPLICIT NONE

*   Purpose:  Store a value in the load vector at a location
*             corresponding to Row.

*   Arguments:
      INTEGER Row
      REAL    Value

*   Argument definitions:
*     Row   - row number in the load vector.
*     Value - value to be stored.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      XAdj(Row) = Value
      XOld(Row) = 0.0

      StoreAtRow = .TRUE.

      RETURN
      END



*== Public (AddAtRow) ==================================================

      LOGICAL FUNCTION AddAtRow(Row, Value)

      IMPLICIT NONE

*   Purpose:  Add a value to the load vector at a location
*             corresponding to Row.

*   Arguments:
      INTEGER Row
      REAL    Value

*   Argument definitions:
*     Row   - row number in the load vector.
*     Value - value to be stored.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      XAdj(Row) = XAdj(Row) + Value

      AddAtRow = .TRUE.

      RETURN
      END



*== Public (UpstreamConstraintRow) =====================================

      INTEGER FUNCTION UpstreamConstraintRow()

      IMPLICIT NONE

*   Purpose:  Return row number for the upstream constraint equation
*             for the current channel.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline2.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      UpstreamConstraintRow = UpConstraintEq(Branch)

      RETURN
      END



*== Public (DownstreamConstraintRow) ===================================

      INTEGER FUNCTION DownstreamConstraintRow()

      IMPLICIT NONE

*   Purpose:  Return row number for the downstream constraint equation
*             for the current channel.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline2.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      DownstreamConstraintRow = DownConstraintEq(Branch)

      RETURN
      END
